@@ -19,9 +19,7 @@
 #include <map>
 #include <string>
 
-#if defined (WIN32) || defined (LINUX) || defined(IOS)
-
-#else
+#if defined (PSP)
 	#include <pspiofilemgr.h>
 	#include <pspiofilemgr_fcntl.h>
 #endif
@@ -117,6 +115,9 @@ public:
   // Returns a string prefixed with the resource path
   string GetResourceFile(string filename);
 
+    // Manually Clear the zip cache
+  void clearZipCache();
+    
 protected:
 	JFileSystem();
 	~JFileSystem();
@@ -129,11 +130,12 @@ private:
 	string mZipFileName;
 	char *mPassword;
 	bool mZipAvailable;
-  void preloadZip(string filename);
-#if defined (WIN32) || defined (LINUX) || defined(IOS)
-	FILE *mFile;
-#else
+  	void preloadZip(const string& filename);
+
+#if defined (PSP)
 	SceUID mFile;
+#else
+	FILE *mFile;
 #endif
 	unzFile mZipFile;
 	int mFileSize;
