@@ -18,7 +18,6 @@
 
 #include "JTypes.h"
 
-
 #ifdef USE_PHONON
 #include <phonon/AudioOutput>
 #include <phonon/MediaObject>
@@ -27,10 +26,7 @@
 
 	#include <windows.h>
 #define WITH_FMOD
-#elif defined (LINUX) || defined (IOS)
-
-#else
-
+#elif defined (PSP)
 	#include <pspgu.h>
 	#include <pspkernel.h>
 	#include <pspdisplay.h>
@@ -61,7 +57,7 @@ class JMusic
 #endif
 public:
 	JMusic();
-        ~JMusic();
+  ~JMusic();
   void Update();
   int getPlayTime();
 
@@ -71,18 +67,13 @@ public:
   public slots:
   void seekAtTheBegining();
 
-#else
-
-#if defined (WIN32) || defined (LINUX) || defined (IOS)
-#ifdef WITH_FMOD
-  FSOUND_SAMPLE* mTrack;		// MP3 needed to be of "sample" type for FMOD, FMUSIC_MODULE is for MODs
-  #else
-  void* mTrack;
-#endif
-#else
+#elif defined (PSP)
   JMP3* mTrack;
-#endif
-#endif
+#elif defined WITH_FMOD
+  FSOUND_SAMPLE* mTrack;		// MP3 needed to be of "sample" type for FMOD, FMUSIC_MODULE is for MODs
+#else
+  void* mTrack;
+#endif  //WITH_FMOD
 
 };
 
@@ -94,22 +85,17 @@ class JSample
   JSample();
   ~JSample();
 
-  int mVoice;
-
   unsigned long fileSize();
-#if defined (WIN32) || defined (LINUX) || defined (IOS)
-#ifdef WITH_FMOD
+
+#if defined (PSP)
+  WAVDATA *mSample;
+#elif defined  (WITH_FMOD)
   FSOUND_SAMPLE *mSample;
-  #else
-#ifdef USE_PHONON
+#elif defined (USE_PHONON)
   Phonon::AudioOutput* mOutput;
   Phonon::MediaObject* mMediaObject;
 #endif
   void* mSample;
-  #endif
-#else
-  WAVDATA *mSample;
-#endif
 };
 
 
