@@ -323,6 +323,10 @@ void JRenderer::BeginScene()
 		//if (mMode3D)
 		sceGuClear(GU_DEPTH_BUFFER_BIT|GU_COLOR_BUFFER_BIT);
 	}
+    else
+    {
+        sceGuClear(GU_COLOR_BUFFER_BIT);
+    }
 
 	sceGuTexMode(TEXTURE_FORMAT, 0, 0, mSwizzle);
   mCurrentTextureFormat = TEXTURE_FORMAT;
@@ -362,10 +366,13 @@ void JRenderer::EnableVSync(bool flag)
 
 void JRenderer::ClearScreen(PIXEL_TYPE color)
 {
-	sceGuClearColor(color);
-	//sceGuClearStencil( 0 );
-	sceGuClear(GU_COLOR_BUFFER_BIT);
-	//sceGuClear( GU_COLOR_BUFFER_BIT | GU_STENCIL_BUFFER_BIT );
+    static PIXEL_TYPE previousColor = 0xFFFFFFFF;
+    if (previousColor != color)
+    {
+	    sceGuClearColor(color);
+	    sceGuClear(GU_COLOR_BUFFER_BIT);
+        previousColor = color;
+    }
 }
 
 
