@@ -15,9 +15,11 @@ class GameObserver;
 class DeckStat
 {
 public:
+    DeckStat(int _nbgames = 0, int _victories = 0, string manaColorIndex = "");
+
     int nbgames;
     int victories;
-    DeckStat(int _nbgames = 0, int _victories = 0);
+    string manaColorIndex;
     int percentVictories();
 };
 
@@ -26,14 +28,15 @@ class DeckStats
 protected:
     static DeckStats * mInstance;
 public:
-    map<string, DeckStat *> stats;
+    string currentDeck;
+    map<string, map<string, DeckStat*> > masterDeckStats;
+    
     static DeckStats * GetInstance();
+    static void EndInstance();
     void saveStats(Player * player, Player * opponent, GameObserver * game);
-    void save(const char * filename);
-    void save(Player * player);
-    void load(const char * filename);
-    void load(Player * player);
-    void cleanStats();
+    void save(const std::string& filename);
+    void load(const std::string& filename);
+
     ~DeckStats();
     int percentVictories(string opponentsDeckFile);
     int percentVictories();
@@ -45,9 +48,9 @@ public:
 
 class StatsWrapper
 {
-private: 
+private:
     void initValues();
-    
+
 public:
     StatsWrapper(int deckId);
     StatsWrapper(string filename);
@@ -91,7 +94,8 @@ public:
     int countNonLandProducersPerColor[Constants::MTG_NB_COLORS + 1];
     int totalCostPerColor[Constants::MTG_NB_COLORS + 1];
     int totalColoredSymbols;
-
+    
+    string getManaColorIndex();
     void updateStats(string filename, MTGAllCards * collection);
     void updateStats(DeckDataWrapper *mtgDeck);
     int countCardsByType(const char * _type, DeckDataWrapper * myDeck);
