@@ -3,24 +3,34 @@
 
 #include <vector>
 #include "Counters.h"
+#include "ObjectAnalytics.h"
+
 using std::vector;
 
 class TargetChooser;
 class MTGCardInstance;
 class MTGAbility;
 
-class ExtraCost{
+class ExtraCost
+#ifdef TRACK_OBJECT_USAGE
+    : public InstanceCounter<ExtraCost>
+#endif
+{
 public:
   TargetChooser * tc;
   MTGCardInstance * source;
   MTGCardInstance * target;
-  MTGCardInstance * targets[20];
   std::string mCostRenderString;
 
   ExtraCost(const std::string& inCostRenderString, TargetChooser *_tc = NULL);
   virtual ~ExtraCost();
+  
   virtual int setPayment(MTGCardInstance * card);
-  virtual int isPaymentSet() { return (target != NULL && targets != NULL); }
+  virtual int isPaymentSet()
+  {
+      return (target != NULL);
+  }
+
   virtual int canPay() { return 1; }
   virtual int doPay() = 0;
   virtual void Render();
@@ -28,11 +38,13 @@ public:
   virtual ExtraCost* clone() const = 0;
 };
 
-class ExtraCosts{
+class ExtraCosts
+{
 public:
   vector<ExtraCost *>costs;
   MTGCardInstance * source;
   MTGAbility * action;
+
   ExtraCosts();
   ~ExtraCosts();
   void Render();
@@ -46,7 +58,8 @@ public:
   ExtraCosts * clone() const;
 };
 
-class SacrificeCost: public ExtraCost{
+class SacrificeCost : public ExtraCost
+{
 public:
   SacrificeCost(TargetChooser *_tc = NULL);
   virtual int doPay();
@@ -54,16 +67,29 @@ public:
 };
 
 //life cost 
-class LifeCost: public ExtraCost{
+class LifeCost : public ExtraCost
+{
 public:
   LifeCost(TargetChooser *_tc = NULL);
-
+  virtual int canPay();
   virtual int doPay();
   virtual LifeCost * clone() const;
 };
 
+//pyrhaixa mana
+class LifeorManaCost : public ExtraCost
+{
+public:
+    LifeorManaCost(TargetChooser *_tc = NULL,string manaType = "");
+    string manaType;
+    virtual int canPay();
+    virtual int doPay();
+    virtual LifeorManaCost * clone() const;
+};
+
 //Discard a random card cost 
-class DiscardRandomCost: public ExtraCost{
+class DiscardRandomCost : public ExtraCost
+{
 public:
   DiscardRandomCost(TargetChooser *_tc = NULL);
   virtual int canPay();
@@ -72,7 +98,8 @@ public:
 };
 
 //a choosen discard
-class DiscardCost: public ExtraCost{
+class DiscardCost : public ExtraCost
+{
 public:
   DiscardCost(TargetChooser *_tc = NULL);
   virtual int doPay();
@@ -80,7 +107,8 @@ public:
 };
 
 //tolibrary cost 
-class ToLibraryCost: public ExtraCost{
+class ToLibraryCost : public ExtraCost
+{
 public:
   ToLibraryCost(TargetChooser *_tc = NULL);
   virtual int doPay();
@@ -88,7 +116,8 @@ public:
 };
 
 //Millyourself cost 
-class MillCost: public ExtraCost{
+class MillCost : public ExtraCost
+{
 public:
   MillCost(TargetChooser *_tc = NULL);
   virtual int canPay();
@@ -97,14 +126,38 @@ public:
 };
 
 //Mill to exile yourself cost 
-class MillExileCost: public MillCost{
+class MillExileCost : public MillCost
+{
 public:
 	MillExileCost(TargetChooser *_tc = NULL);
   virtual int doPay();
 };
 
+//tap  cost
+class TapCost : public ExtraCost
+{
+public:
+    TapCost();
+    virtual int isPaymentSet();
+    virtual int canPay();
+    virtual int doPay();
+    virtual TapCost * clone() const;
+};
+
+//untap  cost
+class UnTapCost : public ExtraCost
+{
+public:
+    UnTapCost();
+    virtual int isPaymentSet();
+    virtual int canPay();
+    virtual int doPay();
+    virtual UnTapCost * clone() const;
+};
+
 //tap other cost
-class TapTargetCost: public ExtraCost{
+class TapTargetCost : public ExtraCost
+{
 public:
     TapTargetCost(TargetChooser *_tc = NULL);
     virtual int isPaymentSet();
@@ -113,7 +166,8 @@ public:
 };
 
 //exile as cost
-class ExileTargetCost: public ExtraCost{
+class ExileTargetCost : public ExtraCost
+{
 public:
   ExileTargetCost(TargetChooser *_tc = NULL);
   virtual int doPay();
@@ -121,7 +175,8 @@ public:
 };
 
 //bounce cost
-class BounceTargetCost: public ExtraCost{
+class BounceTargetCost : public ExtraCost
+{
 public:
   BounceTargetCost(TargetChooser *_tc = NULL);
   virtual int doPay();
@@ -129,7 +184,8 @@ public:
 };
 
 //bounce cost
-class Ninja: public ExtraCost{
+class Ninja : public ExtraCost
+{
 public:
   Ninja(TargetChooser *_tc = NULL);
   virtual int isPaymentSet();
@@ -137,7 +193,8 @@ public:
   virtual Ninja * clone() const;
 };
 
-class CounterCost: public ExtraCost{
+class CounterCost : public ExtraCost
+{
 public:
   Counter * counter;
   int hasCounters;
