@@ -14,7 +14,9 @@
 class TestSuite;
 #endif
 class Credits;
-class Rules;
+#ifdef NETWORK_SUPPORT
+class JNetwork;
+#endif
 
 class GameStateDuel: public GameState, public JGuiListener
 {
@@ -22,11 +24,11 @@ private:
 #ifdef TESTSUITE
     TestSuite * testSuite;
 #endif
+
     Credits * credits;
     int mGamePhase;
     Player * mCurrentPlayer;
     Player * mPlayers[2];
-    MTGPlayerCards * deck[2];
     GameObserver * game;
     DeckMenu * deckmenu;
     DeckMenu * opponentMenu;
@@ -38,10 +40,9 @@ private:
     bool premadeDeck;
     int OpponentsDeckid;
     string musictrack;
-    Rules * rules;
 
     bool MusicExist(string FileName);
-    void loadPlayer(int playerId, int decknb = 0, int isAI = 0);
+    void loadPlayer(int playerId, int decknb = 0, bool isAI = false, bool isNetwork = false);
     void ConstructOpponentMenu(); //loads the opponentMenu if it doesn't exist
     void initScroller();
 
@@ -58,15 +59,21 @@ public:
     virtual void Render();
     void initRand(unsigned seed = 0);
 
+    void OnScroll(int inXVelocity, int inYVelocity);
+
     enum ENUM_DUEL_STATE_MENU_ITEM
     {
         MENUITEM_CANCEL = kCancelMenuID,
         MENUITEM_NEW_DECK = -10,
-        MENUITEM_RANDOM_PLAYER = -11,
-        MENUITEM_RANDOM_AI = -12,
+        MENUITEM_RANDOM_PLAYER = kRandomPlayerMenuID,
+        MENUITEM_RANDOM_AI = kRandomAIPlayerMenuID,
         MENUITEM_MAIN_MENU = -13,
-        MENUITEM_EVIL_TWIN = -14,
+        MENUITEM_EVIL_TWIN = kEvilTwinMenuID,
         MENUITEM_MULLIGAN = -15,
+#ifdef NETWORK_SUPPORT
+        MENUITEM_REMOTE_CLIENT = -16,
+        MENUITEM_REMOTE_SERVER = -17,
+#endif
         MENUITEM_MORE_INFO = kInfoMenuID
     };
 
