@@ -6,6 +6,8 @@ using std::map;
 
 #include "MTGDeck.h"
 #include "MTGCardInstance.h"
+#include "PlayRestrictions.h"
+#include "TargetChooser.h"
 
 #define MTG_MAX_PLAYER_CARDS 100
 
@@ -67,9 +69,12 @@ class MTGGameZone {
    };
 
    Player * owner;
-   //Both cards and cardsMap contain the cards of a zone. The long term objective is to get rid of the array
-   vector<MTGCardInstance *> cards; //[MTG_MAX_PLAYER_CARDS];
+   //Both cards and cardsMap contain the cards of a zone. The vector is used to keep the order, useful for some zones such as the stack, library, etc...
+   vector<MTGCardInstance *> cards;
    map<MTGCardInstance *,int> cardsMap;
+
+    //list of cards that have been through this zone in the current turn
+    vector<MTGCardInstance *> cardsSeenThisTurn;
    int nb_cards;
    MTGGameZone();
    ~MTGGameZone();
@@ -79,19 +84,36 @@ class MTGGameZone {
    MTGCardInstance * removeCard(MTGCardInstance * card, int createCopy = 1);
    MTGCardInstance * hasCard(MTGCardInstance * card);
    void cleanupPhase();
+   void beforeBeginPhase();
+
    int countByType(const char * value);
+   int countByCanTarget(TargetChooser * tc);
    MTGCardInstance * findByName(string name);
-   int hasAbility(int ability); //returns 1 if one of the cards in the  zone has the ability, 0 otherwise
-   int hasType(const char * value); //returns 1 if one of the cards in the zone has the type, 0 otherwise
-	 int hasColor(int value); //returns 1 if one of the cards in the zone has the color, 0 otherwise
-	 int hasX();
+
+    //returns true if one of the cards in the zone has the ability
+   bool hasAbility(int ability);
+
+   //returns true if one of the cards in the zone has the type
+   bool hasType(const char * value);
+   bool hasSpecificType(const char* value, const char* secondvalue);
+   bool hasPrimaryType(const char* value, const char* secondvalue);
+   bool hasTypeButNotType(const char* value, const char* secondvalue);
+   bool hasName(string value);
+   bool hasColor(int value);
+   bool hasX();
+
+   //How many cards matching a TargetChooser have been put in this zone during the turn
+   int seenThisTurn(TargetChooser * tc, int castFilter = Constants::CAST_DONT_CARE); 
+   int seenThisTurn(string s, int castFilter = Constants::CAST_DONT_CARE); 
+
    void setOwner(Player * player);
    MTGCardInstance * lastCardDrawn;
    static MTGGameZone * stringToZone(string zoneName, MTGCardInstance * source, MTGCardInstance * target);
    static int zoneStringToId(string zoneName);
    static MTGGameZone *intToZone(int zoneId, MTGCardInstance * source = NULL,MTGCardInstance * target = NULL);
+   static MTGGameZone *intToZone(int zoneId, Player * source, Player * target = NULL);
    bool needShuffle;
-   virtual const char * getName(){return "zone";}; 
+   virtual const char * getName(){return "zone";};
    virtual ostream& toString(ostream&) const;
 };
 
@@ -139,7 +161,9 @@ class MTGPlayerCards {
  protected:
   void init();
 
- public:
+public:
+    Player * owner;
+    PlayRestrictions * playRestrictions;
   MTGLibrary * library;
   MTGGraveyard * graveyard;
   MTGHand * hand;
@@ -152,6 +176,7 @@ class MTGPlayerCards {
   MTGGameZone * garbage;
   MTGGameZone * temp;
 
+  MTGPlayerCards();
   MTGPlayerCards(int * idList, int idListSize);
   MTGPlayerCards(MTGDeck * deck);
   ~MTGPlayerCards();
@@ -163,14 +188,19 @@ class MTGPlayerCards {
   void showHand();
   void resetLibrary();
   void initDeck(MTGDeck * deck);
+  void beforeBeginPhase();
   MTGCardInstance * putInGraveyard(MTGCardInstance * card);
   MTGCardInstance * putInExile(MTGCardInstance * card);
   MTGCardInstance * putInLibrary(MTGCardInstance * card);
   MTGCardInstance * putInHand(MTGCardInstance * card);
   MTGCardInstance * putInZone(MTGCardInstance * card, MTGGameZone * from, MTGGameZone * to);
   int isInPlay(MTGCardInstance * card);
+  int isInGrave(MTGCardInstance * card);
+  int isInZone(MTGCardInstance * card,MTGGameZone * zone);
 };
 
 ostream& operator<<(ostream&, const MTGGameZone&);
+ostream& operator<<(ostream&, const MTGPlayerCards&);
+istream& operator>>(istream&, MTGPlayerCards&);
 
 #endif
