@@ -43,6 +43,7 @@ public:
 
     int maxtargets; //Set to -1 for "unlimited"
     bool validTargetsExist();
+    int countValidTargets();
     virtual int setAllZones()
     {
         return 0;
@@ -51,6 +52,10 @@ public:
     {
         return false;
     }
+    virtual bool targetsZone(MTGGameZone * z,MTGCardInstance * mSource)
+    {
+        return false;
+    }
     ;
     int ForceTargetListReady();
     int targetsReadyCheck();
@@ -63,7 +68,7 @@ public:
 
     virtual int full()
     {
-        if (maxtargets != -1 && cursor >= maxtargets)
+        if (maxtargets != -1 && ((int) (targets.size())) >= maxtargets)
         {
             return 1;
         }
@@ -75,7 +80,7 @@ public:
     ;
     virtual int ready()
     {
-        return cursor;
+        return (int) (targets.size());
     }
     ;
     virtual ~TargetChooser()
@@ -100,6 +105,7 @@ public:
     int nbzones;
     int init(int * _zones, int _nbzones);
     bool targetsZone(MTGGameZone * z);
+    bool targetsZone(MTGGameZone * z,MTGCardInstance * mSource);
     bool withoutProtections;
     TargetZoneChooser(MTGCardInstance * card = NULL, int _maxtargets = 1, bool other = false);
     TargetZoneChooser(int * _zones, int _nbzones, MTGCardInstance * card = NULL, int _maxtargets = 1, bool other = false);
