@@ -62,6 +62,34 @@ class ThisEquip:public ThisDescriptor{
     ThisEquip(int equipment);
 };
 
+class ThisAuras:public ThisDescriptor{
+ public:
+    virtual int match(MTGCardInstance * card);
+  
+    ThisAuras(int auras);
+};
+
+class ThisOpponentDamageAmount:public ThisDescriptor{
+ public:
+    virtual int match(MTGCardInstance * card);
+  
+    ThisOpponentDamageAmount(int damagecount);
+};
+
+class ThisUntapped:public ThisDescriptor{
+ public:
+    virtual int match(MTGCardInstance * card);
+  
+    ThisUntapped(int untapped);
+};
+
+class ThisTapped:public ThisDescriptor{
+ public:
+    virtual int match(MTGCardInstance * card);
+  
+    ThisTapped(int tapped);
+};
+
 
 class ThisAttacked:public ThisDescriptor{
  public:
@@ -70,6 +98,13 @@ class ThisAttacked:public ThisDescriptor{
 	ThisAttacked(int attack);
 };
 
+class ThisBlocked:public ThisDescriptor{
+ public:
+    virtual int match(MTGCardInstance * card);
+
+	ThisBlocked(int block);
+};
+
 class ThisNotBlocked:public ThisDescriptor{
  public:
     virtual int match(MTGCardInstance * card);
@@ -77,6 +112,13 @@ class ThisNotBlocked:public ThisDescriptor{
 	ThisNotBlocked(int unblocked);
 };
 
+class ThisDamaged:public ThisDescriptor{
+ public:
+    virtual int match(MTGCardInstance * card);
+
+	ThisDamaged(int wasDealtDamage);
+};
+
 class ThisPower:public ThisDescriptor{
  public:
     virtual int match(MTGCardInstance * card);
