@@ -43,7 +43,6 @@ struct WEventZoneChange : public WEvent {
   virtual Targetable * getTarget(int target);
 };
 
-
 struct WEventDamage : public WEvent {
   Damage * damage;
   WEventDamage(Damage * damage);
@@ -52,6 +51,13 @@ struct WEventDamage : public WEvent {
   virtual Targetable * getTarget(int target);
 };
 
+struct WEventLife : public WEvent {
+    Player * player;
+    int amount;
+    WEventLife(Player * player,int amount);
+    virtual Targetable * getTarget(int target);
+};
+
 struct WEventDamageStackResolved : public WEvent {
   WEventDamageStackResolved();
 };
@@ -69,6 +75,23 @@ struct WEventCardUpdate : public WEvent {
   WEventCardUpdate(MTGCardInstance * card);
 };
 
+//creature damaged was killed, triggers effects targetter
+struct WEventVampire : public WEventCardUpdate {
+  MTGCardInstance * card;
+  MTGCardInstance * source;
+  MTGCardInstance * victem;
+  WEventVampire(MTGCardInstance * card,MTGCardInstance * source,MTGCardInstance * victem);
+  virtual Targetable * getTarget(int target);
+};
+
+//creature became the target of a spell or ability
+struct WEventTarget : public WEventCardUpdate {
+  MTGCardInstance * card;
+  MTGCardInstance * source;
+  WEventTarget(MTGCardInstance * card,MTGCardInstance * source);
+  virtual Targetable * getTarget(int target);
+};
+
 //Event when a card gains/looses types
 struct WEventCardChangeType : public WEventCardUpdate {
   int type;
@@ -122,13 +145,15 @@ struct WEventCardAttackedNotBlocked : public WEventCardUpdate {
 
 //event when card attacks but is blocked.
 struct WEventCardAttackedBlocked : public WEventCardUpdate {
-  WEventCardAttackedBlocked(MTGCardInstance * card);
+  WEventCardAttackedBlocked(MTGCardInstance * card,MTGCardInstance * opponent);
+  MTGCardInstance * opponent;
 	virtual Targetable * getTarget(int target);
 };
 
 //event when card blocked.
 struct WEventCardBlocked : public WEventCardUpdate {
-  WEventCardBlocked(MTGCardInstance * card);
+  WEventCardBlocked(MTGCardInstance * card,MTGCardInstance * opponent);
+  MTGCardInstance * opponent;
 	virtual Targetable * getTarget(int target);
 };
 
