@@ -3,9 +3,7 @@
 
 #include <JGE.h>
 
-#if defined (WIN32) || defined (LINUX) || defined (IOS) 
-
-#else
+#if defined (PSP) 
 #include <pspkernel.h>
 #include <pspdisplay.h>
 #include <pspctrl.h>
@@ -29,12 +27,11 @@
 #include <algorithm>
 #include <stdlib.h>
 
-#if defined( WIN32 ) || defined (LINUX)
+#include "DebugRoutines.h"
+
 // enable this define to collect statistics on how many times an ifstream is created for a given file.
 //#define TRACK_FILE_USAGE_STATS
 
-#endif
-
 namespace wagic
 {
 
@@ -46,6 +43,7 @@ namespace wagic
         std::ifstream(inFilename, inMode)
         {
             sFileMap[std::string(inFilename)] += 1;
+            DebugTrace("ifstream opened on file: " << inFilename);
         }
 
         static void Dump()
@@ -89,8 +87,22 @@ std::string join(vector<string>& v, string delim = " ");
 
 std::vector<std::string>& split(const std::string& s, char delim, std::vector<std::string>& elems);
 std::vector<std::string> split(const std::string& s, char delim); //splits a string with "delim" and returns a vector of strings.
+
+// A simple parsing function
+// splits string s by finding the first occurence of start, and the first occurence of stop, and returning
+// a vector of 3 strings. The first string is everything before the first occurence of start, the second string is everything between start and stop
+// the third string is everything after stop.
+// for example, parseBetween ("this is a function(foo) call", "function(", ")") will return: ["this is a ", "foo", " call"];
+//If an error occurs, returns an empty vector.
+// if "stopRequired" is set to false, the function will return a vector of 3 strings even if "stop" is not found in the string.
+std::vector<std::string>&  parseBetween(const std::string& s, string start, string stop, bool stopRequired, std::vector<std::string>& elems);
+std::vector<std::string> parseBetween(const std::string& s, string start, string stop, bool stopRequired = true);
+
 std::string wordWrap(const std::string& s, float width, int fontId);
 
+//basic hash function
+unsigned long hash_djb2(const char *str);
+
 int loadRandValues(string s);
 int filesize(const char * filename);
 int fileExists(const char * filename);
@@ -117,7 +129,7 @@ u32 ramAvailable(void);
 
 #ifdef WIN32
 #include <direct.h>
-#define MAKEDIR(name) mkdir(name)
+#define MAKEDIR(name) _mkdir(name)
 #else
 #include <sys/stat.h>
 #define MAKEDIR(name) mkdir(name, 0777)
