@@ -34,6 +34,9 @@ AIStats::~AIStats()
 void AIStats::updateStatsCard(MTGCardInstance * cardInstance, Damage * damage, float multiplier)
 {
     MTGCard * card = cardInstance->model;
+    if (!card)
+        return; //card can be null because some special cardInstances (such as ExtraRules) don't have a "model"
+
     AIStat * stat = find(card);
     if (!stat)
     {
@@ -42,7 +45,7 @@ void AIStats::updateStatsCard(MTGCardInstance * cardInstance, Damage * damage, f
     }
     if (damage->target == player)
     {
-        stat->value += multiplier * STATS_PLAYER_MULTIPLIER * damage->damage;
+        stat->value += static_cast<int>(multiplier * STATS_PLAYER_MULTIPLIER * damage->damage);
     }
     else if (damage->target->type_as_damageable == DAMAGEABLE_MTGCARDINSTANCE)
     {
@@ -50,7 +53,7 @@ void AIStats::updateStatsCard(MTGCardInstance * cardInstance, Damage * damage, f
         if (target->controller() == player && !target->isInPlay())
         {
             //One of my creatures got lethal damage...
-            stat->value += multiplier * STATS_CREATURE_MULTIPLIER * damage->damage;
+            stat->value += static_cast<int>(multiplier * STATS_CREATURE_MULTIPLIER * damage->damage);
         }
     }
 }
@@ -79,7 +82,7 @@ int AIStats::receiveEvent(WEvent * event)
     GameObserver * g = GameObserver::GetInstance();
     //Lords
     map<MTGCardInstance *, int> lords;
-    for (int i = 1; i < g->mLayers->actionLayer()->mCount; i++)
+    for (size_t i = 1; i < g->mLayers->actionLayer()->mObjects.size(); i++)
     { //0 is not a mtgability...hackish
         MTGAbility * a = ((MTGAbility *) g->mLayers->actionLayer()->mObjects[i]);
         if (ALord * al = dynamic_cast<ALord*>(a))
@@ -143,7 +146,7 @@ AIStat * AIStats::find(MTGCard * source)
 
 void AIStats::load(char * filename)
 {
-    std::ifstream file(filename);
+    wagic::ifstream file(filename);
     std::string s;
 
     if (file)
@@ -154,7 +157,7 @@ void AIStats::load(char * filename)
             std::getline(file, s);
             int value = atoi(s.c_str());
             std::getline(file, s);
-            int direct = atoi(s.c_str());
+            bool direct = atoi(s.c_str()) > 0;
             AIStat * stat = NEW AIStat(cardid, value, 1, direct);
             stats.push_back(stat);
         }
@@ -205,7 +208,7 @@ void AIStats::Render()
         AIStat * stat = *it;
         if (stat->value > 0)
         {
-            MTGCard * card = GameApp::collection->getCardById(stat->source);
+            MTGCard * card = MTGCollection()->getCardById(stat->source);
             if (card)
             {
                 sprintf(buffer, "%s %i", card->data->getName().c_str(), stat->value);
