@@ -8,13 +8,14 @@ The Action Stack contains all information for Game Events that can be interrupte
 #include "CardSelectorSingleton.h"
 #include "Damage.h"
 #include "GameObserver.h"
-#include "GameOptions.h"
 #include "ManaCost.h"
 #include "MTGAbility.h"
 #include "Subtypes.h"
 #include "TargetChooser.h"
 #include "Translate.h"
 #include "WResourceManager.h"
+#include "ModRules.h"
+#include "AllAbilities.h"
 
 #include <typeinfo>
 
@@ -186,7 +187,10 @@ ostream& StackAbility::toString(ostream& out) const
 const string StackAbility::getDisplayName() const
 {
     std::ostringstream stream;
+    if(ability->source)
     stream << "StackAbility.  (Source: " << ability->source->getDisplayName() << ")";
+    else
+    stream << "StackAbility.  (Source: " << ability->getMenuText() << ")";
 
     return stream.str();
 }
@@ -214,10 +218,12 @@ Interruptible(id), tc(tc), cost(_cost), payResult(payResult)
     mHeight = 40;
     type = ACTION_SPELL;
     from = _source->getCurrentZone();
-    for(int i = 0;i < MAX_TARGETS;i++)
+
+    _source->backupTargets.clear();
+    if (tc)
     {
-    if(tc && tc->targets[i] != NULL)
-    _source->backupTargets[i] = tc->targets[i];
+        for(size_t i = 0;i < tc->targets.size();i++)
+            _source->backupTargets.push_back(tc->targets[i]);
     }
 
     // fill information on how the card came into this zone. Right now the quickest way is to do it here, based on how the mana was paid...
@@ -243,14 +249,6 @@ int Spell::computeX(MTGCardInstance * card)
     return x;
 }
 
-int Spell::computeXX(MTGCardInstance * card)
-{
-    ManaCost * c = cost->Diff(card->getManaCost());
-    int xx = c->getCost(Constants::MTG_NB_COLORS) / 2;
-    delete c;
-    return xx;
-}
-
 bool Spell::FullfilledAlternateCost(const int &costType)
 {
     bool hasFullfilledAlternateCost = false;
@@ -365,7 +363,7 @@ int Spell::getNbTargets()
 {
     if (!tc)
         return 0;
-    return tc->cursor;
+    return (int) (tc->targets.size());
 }
 
 void Spell::Render()
@@ -499,9 +497,9 @@ void LifeAction::Render()
     mFont->SetBase(0);
     mFont->SetScale(DEFAULT_MAIN_FONT_SCALE);
     char buffer[200];
-    if(amount >= 0)
+    if(amount > 0)
         sprintf(buffer, _("Player gains %i life").c_str(), amount);
-    else if(amount >= 0)
+    else if(amount < 0)
         sprintf(buffer, _("Player loses %i life").c_str(), amount);
     else
         sprintf(buffer, _("Nothing happened").c_str(), amount);
@@ -516,14 +514,14 @@ ostream& LifeAction::toString(ostream& out) const
 /* The Action Stack itself */
 int ActionStack::addPutInGraveyard(MTGCardInstance * card)
 {
-    PutInGraveyard * death = NEW PutInGraveyard(mCount, card);
+    PutInGraveyard * death = NEW PutInGraveyard(mObjects.size(), card);
     addAction(death);
     return 1;
 }
 
 int ActionStack::addAbility(MTGAbility * ability)
 {
-    StackAbility * stackAbility = NEW StackAbility(mCount, ability);
+    StackAbility * stackAbility = NEW StackAbility(mObjects.size(), ability);
     int result = addAction(stackAbility);
     if (!game->players[0]->isAI() && ability->source->controller() == game->players[0] && 0
         == options[Options::INTERRUPTMYABILITIES].number)
@@ -533,20 +531,14 @@ int ActionStack::addAbility(MTGAbility * ability)
 
 int ActionStack::addDraw(Player * player, int nb_cards)
 {
-    DrawAction * draw = NEW DrawAction(mCount, player, nb_cards);
+    DrawAction * draw = NEW DrawAction(mObjects.size(), player, nb_cards);
     addAction(draw);
-    GameObserver *g = GameObserver::GetInstance();
-		for(int i = nb_cards; i > 0;i--)
-		{
-    WEvent * e = NEW WEventcardDraw(player, nb_cards);
-    g->receiveEvent(e);
-		}
     return 1;
 }
 
 int ActionStack::addLife(Damageable * _target, int amount)
 {
-    LifeAction * life = NEW LifeAction(mCount, _target, amount);
+    LifeAction * life = NEW LifeAction(mObjects.size(), _target, amount);
     addAction(life);
     return 1;
 }
@@ -564,14 +556,11 @@ int ActionStack::AddNextGamePhase()
 {
     if (getNext(NULL, NOT_RESOLVED))
         return 0;
-    NextGamePhase * next = NEW NextGamePhase(mCount);
+
+    NextGamePhase * next = NEW NextGamePhase(mObjects.size());
     addAction(next);
-    int playerId = 0;
     game->currentActionPlayer = game->GetInstance()->currentActionPlayer;
-    if (game->currentActionPlayer == game->players[1])
-    {
-        playerId = 1;
-    }
+    int playerId = (game->currentActionPlayer == game->players[1]) ? 1 : 0;
     interruptDecision[playerId] = 1;
     return 1;
 }
@@ -580,23 +569,25 @@ int ActionStack::AddNextCombatStep()
 {
     if (getNext(NULL, NOT_RESOLVED))
         return 0;
-    NextGamePhase * next = NEW NextGamePhase(mCount);
+
+    NextGamePhase * next = NEW NextGamePhase(mObjects.size());
     addAction(next);
     return 1;
 }
 
 int ActionStack::setIsInterrupting(Player * player)
 {
-    if (player == game->players[0])
-    {
-        interruptDecision[0] = -1;
-    }
-    else
-    {
-        interruptDecision[1] = -1;
-    }
-    game->isInterrupting = player;
     askIfWishesToInterrupt = NULL;
+
+    if (!gModRules.game.canInterrupt())
+    {
+        cancelInterruptOffer();
+        return 0;
+    }
+
+    int playerId = (player == game->players[1]) ? 1 : 0;
+    interruptDecision[playerId] = -1;
+    game->isInterrupting = player;
     return 1;
 }
 
@@ -620,11 +611,7 @@ Spell * ActionStack::addSpell(MTGCardInstance * _source, TargetChooser * tc, Man
     {
         mana = NULL;
     }
-    if (mana < 0)
-    {
-        mana = 0;
-    }
-    Spell * spell = NEW Spell(mCount, _source, tc, mana, payResult);
+    Spell * spell = NEW Spell(mObjects.size(), _source, tc, mana, payResult);
     addAction(spell);
     if (!game->players[0]->isAI() && _source->controller() == game->players[0] && 0 == options[Options::INTERRUPTMYSPELLS].number)
         interruptDecision[0] = DONT_INTERRUPT;
@@ -634,8 +621,8 @@ Spell * ActionStack::addSpell(MTGCardInstance * _source, TargetChooser * tc, Man
 Interruptible * ActionStack::getAt(int id)
 {
     if (id < 0)
-        id = mCount + id;
-    if (id > mCount - 1)
+        id = mObjects.size() + id;
+    if (id > (int)(mObjects.size()) - 1 || id < 0)
         return NULL;
     return (Interruptible *) mObjects[id];
 }
@@ -658,17 +645,11 @@ ActionStack::ActionStack(GameObserver* game)
         pspIcons[i] = WResourceManager::Instance()->RetrieveQuad("iconspsp.png", (float) i * 32, 0, 32, 32, stream.str(), RETRIEVE_MANAGE);
         pspIcons[i]->SetHotSpot(16, 16);
     }
-
-    // fix for translation.
-    kInterruptMessageString = _(kInterruptMessageString);
-    kInterruptString = _(kInterruptString);
-    kNoString = _(kNoString);
-    kNoToAllString = _(kNoToAllString);
 }
 
 int ActionStack::has(MTGAbility * ability)
 {
-    for (int i = 0; i < mCount; i++)
+    for (size_t i = 0; i < mObjects.size(); i++)
     {
         if (((Interruptible *) mObjects[i])->type == ACTION_ABILITY)
         {
@@ -682,7 +663,7 @@ int ActionStack::has(MTGAbility * ability)
 
 int ActionStack::has(Interruptible * action)
 {
-    for (int i = 0; i < mCount; i++)
+    for (size_t i = 0; i < mObjects.size(); i++)
     {
         if (mObjects[i] == action)
             return 1;
@@ -742,7 +723,7 @@ int ActionStack::getPreviousIndex(Interruptible * next, int type, int state, int
     int found = 0;
     if (!next)
         found = 1;
-    for (int i = mCount - 1; i >= 0; i--)
+    for (int i = (int)(mObjects.size()) - 1; i >= 0; i--)
     {
         Interruptible * current = (Interruptible *) mObjects[i];
         if (found && (type == 0 || current->type == type) && (state == 0 || current->state == state) && (display
@@ -761,7 +742,7 @@ int ActionStack::getPreviousIndex(Interruptible * next, int type, int state, int
 int ActionStack::count(int type, int state, int display)
 {
     int result = 0;
-    for (int i = 0; i < mCount; i++)
+    for (size_t i = 0; i < mObjects.size(); i++)
     {
         Interruptible * current = (Interruptible *) mObjects[i];
         if ((type == 0 || current->type == type) && (state == 0 || current->state == state) && (display == -1
@@ -773,6 +754,22 @@ int ActionStack::count(int type, int state, int display)
     return result;
 }
 
+int ActionStack::getActionElementFromCard(MTGCardInstance * card)
+{
+
+	if(!card)
+		return NULL;
+    for (size_t i = 0; i < mObjects.size(); i++)
+    {
+        Interruptible * current = (Interruptible *) mObjects[i];
+        if (current->source == card)
+        {
+            return i;
+        }
+    }  
+    return NULL;
+}
+
 Interruptible * ActionStack::getNext(Interruptible * previous, int type, int state, int display)
 {
     int n = getNextIndex(previous, type, state, display);
@@ -786,7 +783,7 @@ int ActionStack::getNextIndex(Interruptible * previous, int type, int state, int
     int found = 0;
     if (!previous)
         found = 1;
-    for (int i = 0; i < mCount; i++)
+    for (size_t i = 0; i < mObjects.size(); i++)
     {
         Interruptible * current = (Interruptible *) mObjects[i];
         if (found && (type == 0 || current->type == type) && (state == 0 || current->state == state) && (display
@@ -804,7 +801,7 @@ int ActionStack::getNextIndex(Interruptible * previous, int type, int state, int
 
 Interruptible * ActionStack::getLatest(int state)
 {
-    for (int i = mCount - 1; i >= 0; i--)
+    for (int i = (int)(mObjects.size()) - 1; i >= 0; i--)
     {
         Interruptible * action = ((Interruptible *) mObjects[i]);
         if (action->state == state)
@@ -816,7 +813,7 @@ Interruptible * ActionStack::getLatest(int state)
 int ActionStack::receiveEventPlus(WEvent * event)
 {
     int result = 0;
-    for (int i = 0; i < mCount; ++i)
+    for (size_t i = 0; i < mObjects.size(); ++i)
     {
         Interruptible * current = (Interruptible *) mObjects[i];
         result += current->receiveEvent(event);
@@ -826,6 +823,12 @@ int ActionStack::receiveEventPlus(WEvent * event)
 
 void ActionStack::Update(float dt)
 {
+
+    //This is a hack to avoid updating the stack while tuto messages are being shown
+    //Ideally, the tuto messages should be moved to a layer above this one
+    if (ATutorialMessage::Current)
+        return;
+
     askIfWishesToInterrupt = NULL;
     //modal = 0;
     GameObserver * game = GameObserver::GetInstance();
@@ -839,7 +842,7 @@ void ActionStack::Update(float dt)
     if (mode == ACTIONSTACK_STANDARD && tc && !checked)
     {
         checked = 1;
-        for (int i = 0; i < mCount; i++)
+        for (size_t i = 0; i < mObjects.size(); i++)
         {
             Interruptible * current = (Interruptible *) mObjects[i];
             if (tc->canTarget(current))
@@ -929,17 +932,18 @@ void ActionStack::Update(float dt)
         // game will wait for ever for the user to make a selection.
         if (options[Options::INTERRUPT_SECONDS].number > 0)
         {
-					int extraTime = 0;
-					//extraTime is a multiplier, it counts the number of unresolved stack actions
-					//then is used to extend the time you have to interupt.
-					//this prevents you from "running out of time" while deciding.
-					//before this int was added, it was possible to run out of time if you had 10 stack actions
-					//and set the timer to 4 secs. BUG FIX //http://code.google.com/p/wagic/issues/detail?id=464
-					extraTime = count(0, NOT_RESOLVED, 0);
-					if (extraTime == 0)
-						extraTime = 1;//we never want this int to be 0.
+            int extraTime = 0;
+            //extraTime is a multiplier, it counts the number of unresolved stack actions
+            //then is used to extend the time you have to interupt.
+            //this prevents you from "running out of time" while deciding.
+            //before this int was added, it was possible to run out of time if you had 10 stack actions
+            //and set the timer to 4 secs. BUG FIX //http://code.google.com/p/wagic/issues/detail?id=464
+            extraTime = count(0, NOT_RESOLVED, 0);
+            if (extraTime == 0)
+	            extraTime = 1;//we never want this int to be 0.
+
             if (timer < 0)
-							timer = options[Options::INTERRUPT_SECONDS].number * extraTime;
+			    timer = static_cast<float>(options[Options::INTERRUPT_SECONDS].number * extraTime);
             timer -= dt;
             if (timer < 0)
                 cancelInterruptOffer();
@@ -949,14 +953,8 @@ void ActionStack::Update(float dt)
 
 void ActionStack::cancelInterruptOffer(int cancelMode)
 {
-    if (game->isInterrupting == game->players[0])
-    {
-        interruptDecision[0] = cancelMode;
-    }
-    else
-    {
-        interruptDecision[1] = cancelMode;
-    }
+    int playerId = (game->isInterrupting == game->players[1]) ? 1 : 0;
+    interruptDecision[playerId] = cancelMode;
     askIfWishesToInterrupt = NULL;
     game->isInterrupting = NULL;
     timer = -1;
@@ -964,14 +962,8 @@ void ActionStack::cancelInterruptOffer(int cancelMode)
 
 void ActionStack::endOfInterruption()
 {
-    if (game->isInterrupting == game->players[0])
-    {
-        interruptDecision[0] = 0;
-    }
-    else
-    {
-        interruptDecision[1] = 0;
-    }
+    int playerId = (game->isInterrupting == game->players[1]) ? 1 : 0;
+    interruptDecision[playerId] = 0;
     game->isInterrupting = NULL;
 }
 
@@ -982,7 +974,7 @@ bool ActionStack::CheckUserInput(JButton key)
     {
         if (askIfWishesToInterrupt)
         {
-            if (JGE_BTN_SEC == key)
+            if (JGE_BTN_SEC == key && gModRules.game.canInterrupt())
             {
                 setIsInterrupting(askIfWishesToInterrupt);
                 return true;
@@ -1070,7 +1062,6 @@ int ActionStack::garbageCollect()
         if (current->state != NOT_RESOLVED)
         {
             iter = mObjects.erase(iter);
-            mCount--;
             SAFE_DELETE(current);
         }
         else
@@ -1096,6 +1087,11 @@ void ActionStack::Fizzle(Interruptible * action)
 
 void ActionStack::Render()
 {
+    //This is a hack to avoid rendering the stack above the tuto messages
+    //Ideally, the tuto messages should be moved to a layer above this one
+    if (ATutorialMessage::Current)
+        return;
+
     static const float kSpacer = 8;
     static const float x0 = 250;
     static const float y0 = 0;
@@ -1108,7 +1104,7 @@ void ActionStack::Render()
         if (!askIfWishesToInterrupt || !askIfWishesToInterrupt->displayStack())
             return;
 
-        for (int i = 0; i < mCount; i++)
+        for (size_t i = 0; i < mObjects.size(); i++)
         {
             Interruptible * current = (Interruptible *) mObjects[i];
             if (current->state == NOT_RESOLVED)
@@ -1121,12 +1117,8 @@ void ActionStack::Render()
         mFont->SetColor(ARGB(255,255,255,255));
         JRenderer * renderer = JRenderer::GetInstance();
 
-        //JQuad * back = WResourceManager::Instance()->GetQuad("interrupt");
-        //float xScale = width / back->mWidth;
-        //float yScale = height / back->mHeight;
         renderer->FillRoundRect(x0 + 16, y0 + 16, width + 2, height + 2, 10, ARGB(128,0,0,0));
         renderer->FillRoundRect(x0 - 5, y0, width + 2, height + 2, 10, ARGB(200,0,0,0));
-        //renderer->RenderQuad(back,x0,y0,0,xScale, yScale);
         renderer->DrawRoundRect(x0 - 5, y0, width + 2, height + 2, 10, ARGB(255,255,255,255));
 
         std::ostringstream stream;
@@ -1137,36 +1129,45 @@ void ActionStack::Render()
         // Mootpoint 01/12/2011: draw the interrupt text first, at the top.  Offset the rest of the 
         // unresolved stack effects down so that they don't collide with the interrupt text.
         if (options[Options::INTERRUPT_SECONDS].number == 0)
-            stream << kInterruptMessageString;
+            stream << _(kInterruptMessageString);
         else
-            stream << kInterruptMessageString << " " << static_cast<int>(timer);
+            stream << _(kInterruptMessageString) << " " << static_cast<int>(timer);
 
         mFont->DrawString(stream.str(), x0 + 5, currenty);
 
         static const float kIconVerticalOffset = 24;
-        if (mCount > 1)
+        static const float kIconHorizontalOffset = 9;
+        static const float kBeforeIconSpace = 10;
+  
+        //Render "interrupt?" text + possible actions
         {
-            renderer->RenderQuad(pspIcons[7].get(), x0 + 10, kIconVerticalOffset, 0, kGamepadIconSize, kGamepadIconSize);
-            mFont->DrawString(kInterruptString, x0 + 19, kIconVerticalOffset - 6);
+            float currentx = x0 + 10;
+      
+            if (gModRules.game.canInterrupt())
+            {
+                renderer->RenderQuad(pspIcons[7].get(), currentx, kIconVerticalOffset, 0, kGamepadIconSize, kGamepadIconSize);
+                currentx+= kIconHorizontalOffset;
+                mFont->DrawString(_(kInterruptString), currentx, kIconVerticalOffset - 6);
+                currentx+= mFont->GetStringWidth(_(kInterruptString).c_str()) + kBeforeIconSpace;
+            }
 
-            renderer->RenderQuad(pspIcons[4].get(), x0 + 97, kIconVerticalOffset, 0, kGamepadIconSize, kGamepadIconSize);
-            mFont->DrawString(kNoString, x0 + 106, kIconVerticalOffset - 6);
+            renderer->RenderQuad(pspIcons[4].get(), currentx, kIconVerticalOffset, 0, kGamepadIconSize, kGamepadIconSize);
+            currentx+= kIconHorizontalOffset;
+            mFont->DrawString(_(kNoString), currentx, kIconVerticalOffset - 6);
+            currentx+= mFont->GetStringWidth(_(kNoString).c_str()) + kBeforeIconSpace;
 
-            renderer->RenderQuad(pspIcons[6].get(), x0 + 145, kIconVerticalOffset, 0, kGamepadIconSize, kGamepadIconSize);
-            mFont->DrawString(kNoToAllString, x0 + 154, kIconVerticalOffset - 6);
-        }
-        else
-        {
-            renderer->RenderQuad(pspIcons[7].get(), x0 + 40, kIconVerticalOffset, 0, kGamepadIconSize, kGamepadIconSize);
-            mFont->DrawString(kInterruptString, x0 + 49, kIconVerticalOffset - 6);
-
-            renderer->RenderQuad(pspIcons[4].get(), x0 + 140, kIconVerticalOffset - 6, 0, kGamepadIconSize, kGamepadIconSize);
-            mFont->DrawString(kNoString, x0 + 146, kIconVerticalOffset - 6);
+            if (mObjects.size() > 1)
+            {
+                renderer->RenderQuad(pspIcons[6].get(), currentx, kIconVerticalOffset, 0, kGamepadIconSize, kGamepadIconSize);
+                currentx+= kIconHorizontalOffset;
+                mFont->DrawString(_(kNoToAllString), currentx, kIconVerticalOffset - 6);
+                currentx+= mFont->GetStringWidth(_(kNoToAllString).c_str()) + kBeforeIconSpace;
+            }
         }
 
         currenty += kIconVerticalOffset + kSpacer;
 
-        for (int i = 0; i < mCount; i++)
+        for (size_t i = 0; i < mObjects.size(); i++)
         {
             Interruptible * current = (Interruptible *) mObjects[i];
             if (current && current->state == NOT_RESOLVED)
@@ -1181,7 +1182,7 @@ void ActionStack::Render()
     }
     else if (mode == ACTIONSTACK_TARGET && modal)
     {
-        for (int i = 0; i < mCount; i++)
+        for (size_t i = 0; i < mObjects.size(); i++)
         {
             Interruptible * current = (Interruptible *) mObjects[i];
             if (current->display)
@@ -1196,13 +1197,13 @@ void ActionStack::Render()
         renderer->FillRect(x0, y0, width, height, ARGB(200,0,0,0));
         renderer->DrawRect(x0 - 1, y0 - 1, width + 2, height + 2, ARGB(255,255,255,255));
 
-        for (int i = 0; i < mCount; i++)
+        for (size_t i = 0; i < mObjects.size(); i++)
         {
             Interruptible * current = (Interruptible *) mObjects[i];
             if (mObjects[i] != NULL && current->display)
             {
                 ((Interruptible *) mObjects[i])->x = x0 + 5;
-                if (i != mCount - 1)
+                if (i != mObjects.size() - 1)
                 {
                     ((Interruptible *) mObjects[i])->y = currenty;
                     currenty += ((Interruptible *) mObjects[i])->mHeight;
@@ -1272,7 +1273,7 @@ void Interruptible::Dump()
 void ActionStack::Dump()
 {
     DebugTrace("=====\nDumping Action Stack=====");
-    for (int i=0;i<mCount;i++)
+    for (size_t i = 0; i < mObjects.size(); i++)
     {
         Interruptible * current = (Interruptible *)mObjects[i];
         current->Dump();
