@@ -4,7 +4,8 @@
 #include "Subtypes.h"
 #include "Counters.h"
 
-CardDescriptor::CardDescriptor() :  MTGCardInstance()
+CardDescriptor::CardDescriptor()
+    :  MTGCardInstance(), mColorExclusions(0)
 {
     init();
     counterName = "";
@@ -17,6 +18,11 @@ CardDescriptor::CardDescriptor() :  MTGCardInstance()
     manacostComparisonMode = COMPARISON_NONE;
     counterComparisonMode = COMPARISON_NONE;
     convertedManacost = -1;
+    compareName ="";
+    nameComparisonMode = COMPARISON_NONE;
+    colorComparisonMode = COMPARISON_NONE;
+    CDopponentDamaged = 0;
+    CDcontrollerDamaged = 0;
 }
 
 int CardDescriptor::init()
@@ -42,6 +48,32 @@ void CardDescriptor::unsecuresetfresh(int k)
     fresh = k;
 }
 
+void CardDescriptor::setisMultiColored(int w)
+{
+    isMultiColored = w;
+}
+
+void CardDescriptor::setisBlackAndWhite(int w)
+{
+    isBlackAndWhite = w;
+}
+void CardDescriptor::setisRedAndBlue(int w)
+{
+    isRedAndBlue = w;
+}
+void CardDescriptor::setisBlackAndGreen(int w)
+{
+    isBlackAndGreen = w;
+}
+void CardDescriptor::setisBlueAndGreen(int w)
+{
+    isBlueAndGreen = w;
+}
+void CardDescriptor::setisRedAndWhite(int w)
+{
+    isRedAndWhite = w;
+}
+    
 void CardDescriptor::setNegativeSubtype(string value)
 {
     int id = Subtypes::subtypesList->find(value);
@@ -70,6 +102,7 @@ bool CardDescriptor::valueInRange(int comparisonMode, int value, int criterion)
     return false;
 }
 
+
 MTGCardInstance * CardDescriptor::match_or(MTGCardInstance * card)
 {
     int found = 1;
@@ -97,29 +130,19 @@ MTGCardInstance * CardDescriptor::match_or(MTGCardInstance * card)
     if (!found)
         return NULL;
 
-    for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    if (colors)
     {
-        if (colors[i] == 1)
-        {
-            found = 0;
-            if (card->hasColor(i))
-            {
-                found = 1;
-                break;
-            }
-        }
-        else if (colors[i] == -1)
-        {
-            found = 0;
-            if (!card->hasColor(i))
-            {
-                found = 1;
-                break;
-            }
-        }
+        found = (colors & card->colors);
+        if (!found)
+            return NULL;
+    }
+
+    if (mColorExclusions)
+    {
+        found = mColorExclusions & card->colors;
+        if (found)
+            return NULL;
     }
-    if (!found)
-        return NULL;
 
     // Quantified restrictions are always AND-ed:
     if (powerComparisonMode && !valueInRange(powerComparisonMode, card->getPower(), power))
@@ -128,7 +151,8 @@ MTGCardInstance * CardDescriptor::match_or(MTGCardInstance * card)
         return NULL;
     if (manacostComparisonMode && !valueInRange(manacostComparisonMode, card->getManaCost()->getConvertedCost(), convertedManacost))
         return NULL;
-
+    if (nameComparisonMode && compareName != card->name)
+        return NULL;
     return card;
 }
 
@@ -152,12 +176,13 @@ MTGCardInstance * CardDescriptor::match_and(MTGCardInstance * card)
             }
         }
     }
-    for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    if ((colors & card->colors) != colors)
+        match = NULL;
+
+    if (mColorExclusions)
     {
-        if ((colors[i] == 1 && !card->hasColor(i)) || (colors[i] == -1 && card->hasColor(i)))
-        {
+        if ((mColorExclusions & card->colors) == mColorExclusions)
             match = NULL;
-        }
     }
 
     if (powerComparisonMode && !valueInRange(powerComparisonMode, card->getPower(), power))
@@ -166,6 +191,8 @@ MTGCardInstance * CardDescriptor::match_and(MTGCardInstance * card)
         match = NULL;
     if (manacostComparisonMode && !valueInRange(manacostComparisonMode, card->getManaCost()->getConvertedCost(), convertedManacost))
         match = NULL;
+    if(nameComparisonMode && compareName != card->name)
+        match = NULL;
 
     return match;
 }
@@ -178,20 +205,20 @@ MTGCardInstance * CardDescriptor::match(MTGCardInstance * card)
     {
         match = match_and(card);
     }
-    else
+    else if (mode == CD_OR)
     {
         match = match_or(card);
     }
 
     //Abilities
-    for (map<int, int>::const_iterator it = basicAbilities.begin(); it != basicAbilities.end(); ++it)
-    {
-        int j = it->first;
-        if ((basicAbilities[j] == 1 && !card->basicAbilities[j]) || (basicAbilities[j] == -1 && card->basicAbilities[j]))
-        {
-            match = NULL;
-        }
-    }
+    BasicAbilitiesSet set = basicAbilities & card->basicAbilities;
+    if (set != basicAbilities)
+        return NULL;
+    
+    BasicAbilitiesSet excludedSet = mAbilityExclusions & card->basicAbilities;
+    if (excludedSet.any())
+        return NULL;
+
 
     if ((tapped == -1 && card->isTapped()) || (tapped == 1 && !card->isTapped()))
     {
@@ -203,10 +230,57 @@ MTGCardInstance * CardDescriptor::match(MTGCardInstance * card)
         match = NULL;
     }
 
-    if ((isToken == -1 && card->isToken) || (isToken == 1 && !card->isToken))
+    if ((isMultiColored == -1 && card->isMultiColored) || (isMultiColored == 1 && !card->isMultiColored))
     {
         match = NULL;
     }
+        if ((isBlackAndWhite == -1 && card->isBlackAndWhite) || (isBlackAndWhite == 1 && !card->isBlackAndWhite))
+    {
+        match = NULL;
+    }
+        if ((isRedAndBlue == -1 && card->isRedAndBlue) || (isRedAndBlue == 1 && !card->isRedAndBlue))
+    {
+        match = NULL;
+    }
+        if ((isBlackAndGreen == -1 && card->isBlackAndGreen) || (isBlackAndGreen == 1 && !card->isBlackAndGreen))
+    {
+        match = NULL;
+    }
+        if ((isBlueAndGreen == -1 && card->isBlueAndGreen) || (isBlueAndGreen == 1 && !card->isBlueAndGreen))
+        {
+            match = NULL;
+        }
+        if ((isRedAndWhite == -1 && card->isRedAndWhite) || (isRedAndWhite == 1 && !card->isRedAndWhite))
+        {
+            match = NULL;
+        }
+        if ((isLeveler == -1 && card->isLeveler) || (isLeveler == 1 && !card->isLeveler))
+        {
+            match = NULL;
+        }
+        if ((CDenchanted == -1 && card->enchanted) || (CDenchanted == 1 && !card->enchanted))
+        {
+            match = NULL;
+        }
+        if ((CDdamaged == -1 && card->wasDealtDamage) || (CDdamaged == 1 && !card->wasDealtDamage))
+        {
+            match = NULL;
+        }
+        Player * p = controller()->opponent();
+        if ((CDopponentDamaged == -1 && card->damageToOpponent && card->controller() == p) || (CDopponentDamaged == 1 && !card->damageToOpponent && card->controller() == p)
+            || (CDopponentDamaged == -1 && card->damageToController && card->controller() == p->opponent()) || (CDopponentDamaged == 1 && !card->damageToController && card->controller() == p->opponent()))
+        {
+            match = NULL;
+        }
+        if ((CDcontrollerDamaged == -1 && card->damageToController && card->controller() == p) || (CDcontrollerDamaged == 1 && !card->damageToController && card->controller() == p)
+            || (CDcontrollerDamaged == -1 && card->damageToOpponent && card->controller() == p->opponent()) || (CDcontrollerDamaged == 1 && !card->damageToOpponent && card->controller() == p->opponent()))
+        {
+            match = NULL;
+        }
+        if ((isToken == -1 && card->isToken) || (isToken == 1 && !card->isToken))
+        {
+            match = NULL;
+        }
     if (attacker == 1)
     {
         if (defenser == &AnyCard)
@@ -314,3 +388,11 @@ MTGCardInstance * CardDescriptor::nextmatch(MTGGameZone * zone, MTGCardInstance
     }
     return NULL;
 }
+
+void CardDescriptor::SetExclusionColor(int _color, int removeAllOthers)
+{
+    if (removeAllOthers)
+        mColorExclusions = 0;
+
+    mColorExclusions |= ConvertColorToBitMask(_color);
+}
\ No newline at end of file
