@@ -1,7 +1,7 @@
 /*
- * CardGui.cpp
- * This class is in charge of rendering Cards on the screen
- */
+* CardGui.cpp
+* This class is in charge of rendering Cards on the screen
+*/
 
 #include "PrecompiledHeader.h"
 
@@ -33,14 +33,26 @@ namespace
     {
         return cosf(2 * M_PI * (value - 35) / 256.0f);
     }
+
+    void FormatText(std::string inText, std::vector<string>& outFormattedText)
+    {
+        std::string::size_type found = inText.find_first_of("{}");
+        while (found != string::npos)
+        {
+            inText[found] = '/';
+            found = inText.find_first_of("{}", found + 1);
+        }
+        WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::MAGIC_FONT);
+        mFont->FormatText(inText, outFormattedText);
+    }
 }
 
-CardGui::CardGui(MTGCardInstance* card, float x, float y) :
-    PlayGuiObject(Height, x, y, false), card(card)
+CardGui::CardGui(MTGCardInstance* card, float x, float y)
+	: PlayGuiObject(Height, x, y, 0, false), card(card)
 {
 }
-CardGui::CardGui(MTGCardInstance* card, const Pos& ref) :
-    PlayGuiObject(Height, ref, false), card(card)
+CardGui::CardGui(MTGCardInstance* card, const Pos& ref)
+	: PlayGuiObject(Height, ref, 0, false), card(card)
 {
 }
 
@@ -61,8 +73,8 @@ float CardView::GetCenterY()
 }
 
 
-CardView::CardView(const SelectorZone owner, MTGCardInstance* card, float x, float y) :
-    CardGui(card, x, y), owner(owner)
+CardView::CardView(const SelectorZone owner, MTGCardInstance* card, float x, float y)
+	: CardGui(card, x, y), owner(owner)
 {
     const Pos* ref = card->view;
     while (card)
@@ -73,8 +85,8 @@ CardView::CardView(const SelectorZone owner, MTGCardInstance* card, float x, flo
     }
 }
 
-CardView::CardView(const SelectorZone owner, MTGCardInstance* card, const Pos& ref) :
-    CardGui(card, ref), owner(owner)
+CardView::CardView(const SelectorZone owner, MTGCardInstance* card, const Pos& ref)
+	: CardGui(card, ref), owner(owner)
 {
     const Pos* r = card->view;
     while (card)
@@ -89,7 +101,7 @@ CardView::~CardView()
 {
     if (card)
     {
-        const Pos* r = card->view;
+        const Pos* r = this;
         while (card)
         {
             if (r == card->view)
@@ -140,12 +152,9 @@ void CardGui::Render()
         tc = game->getCurrentTargetChooser();
 
     bool alternate = true;
-    JQuad * quad = WResourceManager::Instance()->RetrieveCard(card, CACHE_THUMB);
-#if defined (WIN32) || defined (LINUX)
-    //On pcs we render the big image if the thumbnail is not available
-    if (!quad) quad = WResourceManager::Instance()->RetrieveCard(card);
-#endif
-    if (quad)
+    JQuadPtr quad = WResourceManager::Instance()->RetrieveCard(card, CACHE_THUMB);
+
+    if (quad.get())
         alternate = false;
     else
         quad = AlternateThumbQuad(card);
@@ -153,18 +162,26 @@ void CardGui::Render()
     float cardScale = quad ? 40 / quad->mHeight : 1;
     float scale = actZ * cardScale;
 
-    JQuad* shadow = NULL;
+    JQuadPtr shadow;
     if (actZ > 1)
     {
         shadow = WResourceManager::Instance()->GetQuad("shadow");
         shadow->SetColor(ARGB(static_cast<unsigned char>(actA)/2,255,255,255));
-        renderer->RenderQuad(shadow, actX + (actZ - 1) * 15, actY + (actZ - 1) * 15, actT, 28 * actZ / 16, 40 * actZ / 16);
+        renderer->RenderQuad(shadow.get(), actX + (actZ - 1) * 15, actY + (actZ - 1) * 15, actT, 28 * actZ / 16, 40 * actZ / 16);
+    }
+
+    JQuadPtr extracostshadow;
+    if (card->isExtraCostTarget)
+    {
+        extracostshadow = WResourceManager::Instance()->GetQuad("extracostshadow");
+        extracostshadow->SetColor(ARGB(static_cast<unsigned char>(actA)/2,100,0,0));
+        renderer->RenderQuad(extracostshadow.get(), actX + (actZ - 1) * 15, actY + (actZ - 1) * 15, actT, 28 * actZ / 16, 40 * actZ / 16);
     }
 
     if (quad)
     {
         quad->SetColor(ARGB(static_cast<unsigned char>(actA),255,255,255));
-        renderer->RenderQuad(quad, actX, actY, actT, scale, scale);
+        renderer->RenderQuad(quad.get(), actX, actY, actT, scale, scale);
     }
 
     if (alternate)
@@ -174,7 +191,7 @@ void CardGui::Render()
         mFont->DrawString(_(card->getName()), actX - actZ * Width / 2 + 1, actY - actZ * Height / 2 + 1);
         mFont->SetScale(DEFAULT_MAIN_FONT_SCALE);
 
-        JQuad * icon = NULL;
+        JQuadPtr icon;
         if (card->hasSubtype("plains"))
             icon = WResourceManager::Instance()->GetQuad("c_white");
         else if (card->hasSubtype("swamp"))
@@ -186,14 +203,22 @@ void CardGui::Render()
         else if (card->hasSubtype("island"))
             icon = WResourceManager::Instance()->GetQuad("c_blue");
 
-        if (icon)
+        if (icon.get())
         {
             icon->SetColor(ARGB(static_cast<unsigned char>(actA),255,255,255));
-            renderer->RenderQuad(icon, actX, actY, 0);
+            renderer->RenderQuad(icon.get(), actX, actY, 0);
             icon->SetColor(ARGB(255,255,255,255)); //Putting color back as this quad is shared
         }
 
     }
+    JQuadPtr mor;
+    if(card->isMorphed && !alternate)
+    {
+        mor = WResourceManager::Instance()->GetQuad("morph");
+        mor->SetColor(ARGB(255,255,255,255));
+        renderer->RenderQuad(mor.get(), actX, actY, actT,scale, scale);
+    }
+
     //draws the numbers power/toughness
     if (card->isCreature())
     {
@@ -201,7 +226,7 @@ void CardGui::Render()
         char buffer[200];
         sprintf(buffer, "%i/%i", card->power, card->life);
         renderer->FillRect(actX - (12 * actZ), actY + 6 * actZ, 25 * actZ, 12 * actZ,
-                        ARGB(((static_cast<unsigned char>(actA))/2),0,0,0));
+            ARGB(((static_cast<unsigned char>(actA))/2),0,0,0));
         mFont->SetColor(ARGB(static_cast<unsigned char>(actA),255,255,255));
         mFont->SetScale(actZ);
         mFont->DrawString(buffer, actX - 10 * actZ, actY + 8 * actZ);
@@ -228,21 +253,38 @@ void CardGui::Render()
             mFont->SetScale(1);
         }
     }
-
     if (tc && !tc->canTarget(card))
     {
         if (!shadow)
             shadow = WResourceManager::Instance()->GetQuad("shadow");
         shadow->SetColor(ARGB(200,255,255,255));
-        renderer->RenderQuad(shadow, actX, actY, actT, (28 * actZ + 1) / 16, 40 * actZ / 16);
+        renderer->RenderQuad(shadow.get(), actX, actY, actT, (28 * actZ + 1) / 16, 40 * actZ / 16);
     }
 
+    // Render a mask over the card, if set
+    if (mask && quad)
+        JRenderer::GetInstance()->FillRect(actX - (scale * quad->mWidth / 2),actY - (scale * quad->mHeight / 2), scale * quad->mWidth, scale* quad->mHeight, mask);
+
+    if (tc && tc->alreadyHasTarget(card))//paint targets red.
+    {
+        if (card->isTapped())
+        {
+            renderer->FillRect(actX - (scale * quad->mWidth / 2)-7,actY - (scale * quad->mHeight / 2)+7,scale* quad->mHeight,scale * quad->mWidth, ARGB(128,255,0,0));
+        }
+        else
+        {
+            renderer->FillRect(actX - (scale * quad->mWidth / 2),actY - (scale * quad->mHeight / 2), scale * quad->mWidth, scale* quad->mHeight, ARGB(128,255,0,0));
+        }
+    }
+    if(tc && tc->source && tc->source->view->actZ >= 1.3)//paint the source green while infocus.
+        renderer->FillRect(tc->source->view->actX - (scale * quad->mWidth / 2),tc->source->view->actY - (scale * quad->mHeight / 2), scale*quad->mWidth, scale*quad->mHeight, ARGB(128,0,255,0));
+
     PlayGuiObject::Render();
 }
 
-JQuad * CardGui::AlternateThumbQuad(MTGCard * card)
+JQuadPtr CardGui::AlternateThumbQuad(MTGCard * card)
 {
-    JQuad * q;
+    JQuadPtr q;
 
     if (card->data->countColors() > 1)
     {
@@ -287,51 +329,51 @@ void CardGui::AlternateRender(MTGCard * card, const Pos& pos)
 {
     // Draw the "unknown" card model
     JRenderer * renderer = JRenderer::GetInstance();
-    JQuad * q;
+    JQuadPtr q;
 
     float x = pos.actX;
 
     if (card->data->countColors() > 1)
     {
-        q = WResourceManager::Instance()->RetrieveTempQuad("gold.jpg");
+        q = WResourceManager::Instance()->RetrieveTempQuad("gold.jpg", TEXTURE_SUB_5551);
     }
     else
     {
         switch (card->data->getColor())
         {
         case Constants::MTG_COLOR_ARTIFACT:
-            q = WResourceManager::Instance()->RetrieveTempQuad("artifact.jpg");
+            q = WResourceManager::Instance()->RetrieveTempQuad("artifact.jpg", TEXTURE_SUB_5551);
             break;
         case Constants::MTG_COLOR_GREEN:
-            q = WResourceManager::Instance()->RetrieveTempQuad("green.jpg");
+            q = WResourceManager::Instance()->RetrieveTempQuad("green.jpg", TEXTURE_SUB_5551);
             break;
         case Constants::MTG_COLOR_BLUE:
-            q = WResourceManager::Instance()->RetrieveTempQuad("blue.jpg");
+            q = WResourceManager::Instance()->RetrieveTempQuad("blue.jpg", TEXTURE_SUB_5551);
             break;
         case Constants::MTG_COLOR_RED:
-            q = WResourceManager::Instance()->RetrieveTempQuad("red.jpg");
+            q = WResourceManager::Instance()->RetrieveTempQuad("red.jpg", TEXTURE_SUB_5551);
             break;
         case Constants::MTG_COLOR_BLACK:
-            q = WResourceManager::Instance()->RetrieveTempQuad("black.jpg");
+            q = WResourceManager::Instance()->RetrieveTempQuad("black.jpg", TEXTURE_SUB_5551);
             break;
         case Constants::MTG_COLOR_WHITE:
-            q = WResourceManager::Instance()->RetrieveTempQuad("white.jpg");
+            q = WResourceManager::Instance()->RetrieveTempQuad("white.jpg", TEXTURE_SUB_5551);
             break;
         case Constants::MTG_COLOR_LAND:
-            q = WResourceManager::Instance()->RetrieveTempQuad("land.jpg");
+            q = WResourceManager::Instance()->RetrieveTempQuad("land.jpg", TEXTURE_SUB_5551);
             break;
         default:
-            q = WResourceManager::Instance()->RetrieveTempQuad("gold.jpg");
+            q = WResourceManager::Instance()->RetrieveTempQuad("gold.jpg", TEXTURE_SUB_5551);
             break;
         }
     }
-    if (q && q->mTex)
+    if (q.get() && q->mTex)
     {
         q->SetHotSpot(static_cast<float> (q->mTex->mWidth / 2), static_cast<float> (q->mTex->mHeight / 2));
 
         float scale = pos.actZ * 250 / q->mHeight;
         q->SetColor(ARGB((int)pos.actA,255,255,255));
-        renderer->RenderQuad(q, x, pos.actY, pos.actT, scale, scale);
+        renderer->RenderQuad(q.get(), x, pos.actY, pos.actT, scale, scale);
     }
     // Write the title
     WFont * font = WResourceManager::Instance()->GetWFont(Fonts::MAGIC_FONT);
@@ -351,7 +393,10 @@ void CardGui::AlternateRender(MTGCard * card, const Pos& pos)
     // Write the description
     {
         font->SetScale(kWidthScaleFactor * pos.actZ);
-        const std::vector<string> txt = card->data->formattedText();
+
+        std::vector<string> txt;
+        FormatText(card->data->getText(), txt);
+
         unsigned i = 0;
         unsigned h = neofont ? 14 : 11;
         for (std::vector<string>::const_iterator it = txt.begin(); it != txt.end(); ++it, ++i)
@@ -381,21 +426,21 @@ void CardGui::AlternateRender(MTGCard * card, const Pos& pos)
 
             if (scale < 0)
             {
-                renderer->RenderQuad(manaIcons[h->color1], x + (-12 * j + 75 + 3 * SineHelperFunction((float) t)) * pos.actZ,
-                                pos.actY + (yOffset + 3 * CosineHelperFunction((float) t)) * pos.actZ, 0, 0.4f + scale, 0.4f
-                                                + scale);
-                renderer->RenderQuad(manaIcons[h->color2], x + (-12 * j + 75 + 3 * SineHelperFunction((float) v)) * pos.actZ,
-                                pos.actY + (yOffset + 3 * CosineHelperFunction((float) v)) * pos.actZ, 0, 0.4f - scale, 0.4f
-                                                - scale);
+                renderer->RenderQuad(manaIcons[h->color1].get(), x + (-12 * j + 75 + 3 * SineHelperFunction((float) t)) * pos.actZ,
+                    pos.actY + (yOffset + 3 * CosineHelperFunction((float) t)) * pos.actZ, 0, 0.4f + scale, 0.4f
+                    + scale);
+                renderer->RenderQuad(manaIcons[h->color2].get(), x + (-12 * j + 75 + 3 * SineHelperFunction((float) v)) * pos.actZ,
+                    pos.actY + (yOffset + 3 * CosineHelperFunction((float) v)) * pos.actZ, 0, 0.4f - scale, 0.4f
+                    - scale);
             }
             else
             {
-                renderer->RenderQuad(manaIcons[h->color2], x + (-12 * j + 75 + 3 * SineHelperFunction((float) v)) * pos.actZ,
-                                pos.actY + (yOffset + 3 * CosineHelperFunction((float) v)) * pos.actZ, 0, 0.4f - scale, 0.4f
-                                                - scale);
-                renderer->RenderQuad(manaIcons[h->color1], x + (-12 * j + 75 + 3 * SineHelperFunction((float) t)) * pos.actZ,
-                                pos.actY + (yOffset + 3 * CosineHelperFunction((float) t)) * pos.actZ, 0, 0.4f + scale, 0.4f
-                                                + scale);
+                renderer->RenderQuad(manaIcons[h->color2].get(), x + (-12 * j + 75 + 3 * SineHelperFunction((float) v)) * pos.actZ,
+                    pos.actY + (yOffset + 3 * CosineHelperFunction((float) v)) * pos.actZ, 0, 0.4f - scale, 0.4f
+                    - scale);
+                renderer->RenderQuad(manaIcons[h->color1].get(), x + (-12 * j + 75 + 3 * SineHelperFunction((float) t)) * pos.actZ,
+                    pos.actY + (yOffset + 3 * CosineHelperFunction((float) t)) * pos.actZ, 0, 0.4f + scale, 0.4f
+                    + scale);
             }
             ++j;
         }
@@ -403,8 +448,8 @@ void CardGui::AlternateRender(MTGCard * card, const Pos& pos)
         {
             for (int cost = manacost->getCost(i); cost > 0; --cost)
             {
-                renderer->RenderQuad(manaIcons[i], x + (-12 * j + 75) * pos.actZ, pos.actY + (yOffset) * pos.actZ, 0, 0.4f
-                                * pos.actZ, 0.4f * pos.actZ);
+                renderer->RenderQuad(manaIcons[i].get(), x + (-12 * j + 75) * pos.actZ, pos.actY + (yOffset) * pos.actZ, 0, 0.4f
+                    * pos.actZ, 0.4f * pos.actZ);
                 ++j;
             }
         }
@@ -413,19 +458,19 @@ void CardGui::AlternateRender(MTGCard * card, const Pos& pos)
         {
             char buffer[10];
             sprintf(buffer, "%d", cost);
-            renderer->RenderQuad(manaIcons[0], x + (-12 * j + 75) * pos.actZ, pos.actY + (yOffset) * pos.actZ, 0, 0.4f * pos.actZ,
-                            0.4f * pos.actZ);
+            renderer->RenderQuad(manaIcons[0].get(), x + (-12 * j + 75) * pos.actZ, pos.actY + (yOffset) * pos.actZ, 0, 0.4f * pos.actZ,
+                0.4f * pos.actZ);
             float w = font->GetStringWidth(buffer);
             font->DrawString(buffer, x + (-12 * j + 76 - w / 2) * pos.actZ, pos.actY + (yOffset - 5) * pos.actZ);
             ++j;
         }
         //Has X?
-        if (int cost = manacost->hasX())
+        if (manacost->hasX())
         {
             char buffer[10];
             sprintf(buffer, "X");
-            renderer->RenderQuad(manaIcons[0], x + (-12 * j + 75) * pos.actZ, pos.actY + (yOffset) * pos.actZ, 0, 0.4f * pos.actZ,
-                            0.4f * pos.actZ);
+            renderer->RenderQuad(manaIcons[0].get(), x + (-12 * j + 75) * pos.actZ, pos.actY + (yOffset) * pos.actZ, 0, 0.4f * pos.actZ,
+                0.4f * pos.actZ);
             float w = font->GetStringWidth(buffer);
             font->DrawString(buffer, x + (-12 * j + 76 - w / 2) * pos.actZ, pos.actY + (yOffset - 5) * pos.actZ);
         }
@@ -436,7 +481,7 @@ void CardGui::AlternateRender(MTGCard * card, const Pos& pos)
         string s = "";
         for (int i = card->data->types.size() - 1; i > 0; --i)
         {
-            if (card->data->basicAbilities[Constants::CHANGELING])
+            if (card->data->basicAbilities[(int)Constants::CHANGELING])
             {// this avoids drawing the list of subtypes on changeling cards.
                 s += _("Shapeshifter - ");
                 break;
@@ -451,117 +496,117 @@ void CardGui::AlternateRender(MTGCard * card, const Pos& pos)
             s += _(Subtypes::subtypesList->find(card->data->types[0]));
         else
         {
-DebugTrace        ("Typeless card: " << setlist[card->setId].c_str() << card->data->getName() << card->getId());
+            DebugTrace("Typeless card: " << setlist[card->setId].c_str() << card->data->getName() << card->getId());
+        }
+
+        font->DrawString(s.c_str(), x + (22 - BigWidth / 2)*pos.actZ, pos.actY + (49 - BigHeight / 2)*pos.actZ);
     }
 
-    font->DrawString(s.c_str(), x + (22 - BigWidth / 2)*pos.actZ, pos.actY + (49 - BigHeight / 2)*pos.actZ);
-}
-
-//expansion and rarity
-font->SetColor(ARGB((int)pos.actA, 0, 0, 0));
-{
-    char buf[512];
-    switch(card->getRarity())
+    //expansion and rarity
+    font->SetColor(ARGB((int)pos.actA, 0, 0, 0));
     {
+        char buf[512];
+        switch(card->getRarity())
+        {
         case Constants::RARITY_M:
-        sprintf(buf,_("%s Mythic").c_str(),setlist[card->setId].c_str());
-        break;
+            sprintf(buf,_("%s Mythic").c_str(),setlist[card->setId].c_str());
+            break;
         case Constants::RARITY_R:
-        sprintf(buf,_("%s Rare").c_str(),setlist[card->setId].c_str());
-        break;
+            sprintf(buf,_("%s Rare").c_str(),setlist[card->setId].c_str());
+            break;
         case Constants::RARITY_U:
-        sprintf(buf,_("%s Uncommon").c_str(),setlist[card->setId].c_str());
-        break;
+            sprintf(buf,_("%s Uncommon").c_str(),setlist[card->setId].c_str());
+            break;
         case Constants::RARITY_C:
-        sprintf(buf,_("%s Common").c_str(),setlist[card->setId].c_str());
-        break;
+            sprintf(buf,_("%s Common").c_str(),setlist[card->setId].c_str());
+            break;
         case Constants::RARITY_L:
-        sprintf(buf,_("%s Land").c_str(),setlist[card->setId].c_str());
-        break;
+            sprintf(buf,_("%s Land").c_str(),setlist[card->setId].c_str());
+            break;
         case Constants::RARITY_T:
-        sprintf(buf,_("%s Token").c_str(),setlist[card->setId].c_str());
-        break;
+            sprintf(buf,_("%s Token").c_str(),setlist[card->setId].c_str());
+            break;
         default:
         case Constants::RARITY_S:
-        sprintf(buf,_("%s Special").c_str(),setlist[card->setId].c_str());
-        break;
-    }
+            sprintf(buf,_("%s Special").c_str(),setlist[card->setId].c_str());
+            break;
+        }
 
-    switch(card->data->getColor())
-    {
+        switch(card->data->getColor())
+        {
         case Constants::MTG_COLOR_BLACK:
         case Constants::MTG_COLOR_GREEN:
         case Constants::MTG_COLOR_BLUE:
         case Constants::MTG_COLOR_LAND:
-        font->SetColor(ARGB((int)pos.actA,255,255,255));
-        font->DrawString(buf, x + (22 - BigWidth / 2)*pos.actZ, pos.actY + (BigHeight / 2 - 30)*pos.actZ);
-        break;
+            font->SetColor(ARGB((int)pos.actA,255,255,255));
+            font->DrawString(buf, x + (22 - BigWidth / 2)*pos.actZ, pos.actY + (BigHeight / 2 - 30)*pos.actZ);
+            break;
         default:
-        font->DrawString(buf, x + (22 - BigWidth / 2)*pos.actZ, pos.actY + (BigHeight / 2 - 30)*pos.actZ);
-        break; //Leave black
+            font->DrawString(buf, x + (22 - BigWidth / 2)*pos.actZ, pos.actY + (BigHeight / 2 - 30)*pos.actZ);
+            break; //Leave black
+        }
+
     }
 
-}
-
-font->SetScale(backup_scale);
+    font->SetScale(backup_scale);
 }
 
 void CardGui::TinyCropRender(MTGCard * card, const Pos& pos, JQuad * quad)
 {
-
     if (!quad)
         return;
 
     JRenderer * renderer = JRenderer::GetInstance();
-    JQuad * q;
+    JQuadPtr q;
 
     float x = pos.actX;
     float displayScale = 250 / BigHeight;
 
     if (card->data->countColors() > 1)
     {
-        q = WResourceManager::Instance()->RetrieveTempQuad("gold.jpg");
+        q = WResourceManager::Instance()->RetrieveTempQuad("gold.jpg", TEXTURE_SUB_5551);
     }
     else
     {
         switch (card->data->getColor())
         {
         case Constants::MTG_COLOR_ARTIFACT:
-            q = WResourceManager::Instance()->RetrieveTempQuad("artifact.jpg");
+            q = WResourceManager::Instance()->RetrieveTempQuad("artifact.jpg", TEXTURE_SUB_5551);
             break;
         case Constants::MTG_COLOR_GREEN:
-            q = WResourceManager::Instance()->RetrieveTempQuad("green.jpg");
+            q = WResourceManager::Instance()->RetrieveTempQuad("green.jpg", TEXTURE_SUB_5551);
             break;
         case Constants::MTG_COLOR_BLUE:
-            q = WResourceManager::Instance()->RetrieveTempQuad("blue.jpg");
+            q = WResourceManager::Instance()->RetrieveTempQuad("blue.jpg", TEXTURE_SUB_5551);
             break;
         case Constants::MTG_COLOR_RED:
-            q = WResourceManager::Instance()->RetrieveTempQuad("red.jpg");
+            q = WResourceManager::Instance()->RetrieveTempQuad("red.jpg", TEXTURE_SUB_5551);
             break;
         case Constants::MTG_COLOR_BLACK:
-            q = WResourceManager::Instance()->RetrieveTempQuad("black.jpg");
+            q = WResourceManager::Instance()->RetrieveTempQuad("black.jpg", TEXTURE_SUB_5551);
             break;
         case Constants::MTG_COLOR_WHITE:
-            q = WResourceManager::Instance()->RetrieveTempQuad("white.jpg");
+            q = WResourceManager::Instance()->RetrieveTempQuad("white.jpg", TEXTURE_SUB_5551);
             break;
         case Constants::MTG_COLOR_LAND:
-            q = WResourceManager::Instance()->RetrieveTempQuad("land.jpg");
+            q = WResourceManager::Instance()->RetrieveTempQuad("land.jpg", TEXTURE_SUB_5551);
             break;
         default:
-            q = WResourceManager::Instance()->RetrieveTempQuad("gold.jpg");
+            q = WResourceManager::Instance()->RetrieveTempQuad("gold.jpg", TEXTURE_SUB_5551);
             break;
         }
     }
-    if (q && q->mTex)
+    if (q.get() && q->mTex)
     {
         q->SetHotSpot(static_cast<float> (q->mTex->mWidth / 2), static_cast<float> (q->mTex->mHeight / 2));
 
         float scale = pos.actZ * displayScale * BigHeight / q->mHeight;
         q->SetColor(ARGB((int)pos.actA,255,255,255));
-        renderer->RenderQuad(q, x, pos.actY, pos.actT, scale, scale);
+        renderer->RenderQuad(q.get(), x, pos.actY, pos.actT, scale, scale);
     }
 
-    const std::vector<string> txt = card->data->formattedText();
+    std::vector<string> txt;
+    FormatText(card->data->getText(), txt);
     size_t nbTextLines = txt.size();
 
     //Render the image on top of that
@@ -623,21 +668,21 @@ void CardGui::TinyCropRender(MTGCard * card, const Pos& pos, JQuad * quad)
 
             if (scale < 0)
             {
-                renderer->RenderQuad(manaIcons[h->color1], x + (-12 * j + 75 + 3 * SineHelperFunction((float) t)) * pos.actZ,
-                                pos.actY + (yOffset + 3 * CosineHelperFunction((float) t)) * pos.actZ, 0, 0.4f + scale, 0.4f
-                                                + scale);
-                renderer->RenderQuad(manaIcons[h->color2], x + (-12 * j + 75 + 3 * SineHelperFunction((float) v)) * pos.actZ,
-                                pos.actY + (yOffset + 3 * CosineHelperFunction((float) v)) * pos.actZ, 0, 0.4f - scale, 0.4f
-                                                - scale);
+                renderer->RenderQuad(manaIcons[h->color1].get(), x + (-12 * j + 75 + 3 * SineHelperFunction((float) t)) * pos.actZ,
+                    pos.actY + (yOffset + 3 * CosineHelperFunction((float) t)) * pos.actZ, 0, 0.4f + scale, 0.4f
+                    + scale);
+                renderer->RenderQuad(manaIcons[h->color2].get(), x + (-12 * j + 75 + 3 * SineHelperFunction((float) v)) * pos.actZ,
+                    pos.actY + (yOffset + 3 * CosineHelperFunction((float) v)) * pos.actZ, 0, 0.4f - scale, 0.4f
+                    - scale);
             }
             else
             {
-                renderer->RenderQuad(manaIcons[h->color2], x + (-12 * j + 75 + 3 * SineHelperFunction((float) v)) * pos.actZ,
-                                pos.actY + (yOffset + 3 * CosineHelperFunction((float) v)) * pos.actZ, 0, 0.4f - scale, 0.4f
-                                                - scale);
-                renderer->RenderQuad(manaIcons[h->color1], x + (-12 * j + 75 + 3 * SineHelperFunction((float) t)) * pos.actZ,
-                                pos.actY + (yOffset + 3 * CosineHelperFunction((float) t)) * pos.actZ, 0, 0.4f + scale, 0.4f
-                                                + scale);
+                renderer->RenderQuad(manaIcons[h->color2].get(), x + (-12 * j + 75 + 3 * SineHelperFunction((float) v)) * pos.actZ,
+                    pos.actY + (yOffset + 3 * CosineHelperFunction((float) v)) * pos.actZ, 0, 0.4f - scale, 0.4f
+                    - scale);
+                renderer->RenderQuad(manaIcons[h->color1].get(), x + (-12 * j + 75 + 3 * SineHelperFunction((float) t)) * pos.actZ,
+                    pos.actY + (yOffset + 3 * CosineHelperFunction((float) t)) * pos.actZ, 0, 0.4f + scale, 0.4f
+                    + scale);
             }
             ++j;
         }
@@ -645,8 +690,8 @@ void CardGui::TinyCropRender(MTGCard * card, const Pos& pos, JQuad * quad)
         {
             for (int cost = manacost->getCost(i); cost > 0; --cost)
             {
-                renderer->RenderQuad(manaIcons[i], x + (-12 * j + 75) * pos.actZ, pos.actY + (yOffset) * pos.actZ, 0, 0.4f
-                                * pos.actZ, 0.4f * pos.actZ);
+                renderer->RenderQuad(manaIcons[i].get(), x + (-12 * j + 75) * pos.actZ, pos.actY + (yOffset) * pos.actZ, 0, 0.4f
+                    * pos.actZ, 0.4f * pos.actZ);
                 ++j;
             }
         }
@@ -655,19 +700,19 @@ void CardGui::TinyCropRender(MTGCard * card, const Pos& pos, JQuad * quad)
         {
             char buffer[10];
             sprintf(buffer, "%d", cost);
-            renderer->RenderQuad(manaIcons[0], x + (-12 * j + 75) * pos.actZ, pos.actY + (yOffset) * pos.actZ, 0, 0.4f * pos.actZ,
-                            0.4f * pos.actZ);
+            renderer->RenderQuad(manaIcons[0].get(), x + (-12 * j + 75) * pos.actZ, pos.actY + (yOffset) * pos.actZ, 0, 0.4f * pos.actZ,
+                0.4f * pos.actZ);
             float w = font->GetStringWidth(buffer);
             font->DrawString(buffer, x + (-12 * j + 76 - w / 2) * pos.actZ, pos.actY + (yOffset - 5) * pos.actZ);
             ++j;
         }
         //Has X?
-        if (int cost = manacost->hasX())
+        if (manacost->hasX())
         {
             char buffer[10];
             sprintf(buffer, "X");
-            renderer->RenderQuad(manaIcons[0], x + (-12 * j + 75) * pos.actZ, pos.actY + (yOffset) * pos.actZ, 0, 0.4f * pos.actZ,
-                            0.4f * pos.actZ);
+            renderer->RenderQuad(manaIcons[0].get(), x + (-12 * j + 75) * pos.actZ, pos.actY + (yOffset) * pos.actZ, 0, 0.4f * pos.actZ,
+                0.4f * pos.actZ);
             float w = font->GetStringWidth(buffer);
             font->DrawString(buffer, x + (-12 * j + 76 - w / 2) * pos.actZ, pos.actY + (yOffset - 5) * pos.actZ);
         }
@@ -685,59 +730,59 @@ void CardGui::TinyCropRender(MTGCard * card, const Pos& pos, JQuad * quad)
             s += _(Subtypes::subtypesList->find(card->data->types[0]));
         else
         {
-DebugTrace        ("Typeless card: " << setlist[card->setId].c_str() << card->data->getName() << card->getId());
+            DebugTrace("Typeless card: " << setlist[card->setId].c_str() << card->data->getName() << card->getId());
+        }
+
+        font->DrawString(s.c_str(), x + (22 - BigWidth / 2)*pos.actZ, pos.actY + (49 - BigHeight / 2)*pos.actZ);
     }
 
-    font->DrawString(s.c_str(), x + (22 - BigWidth / 2)*pos.actZ, pos.actY + (49 - BigHeight / 2)*pos.actZ);
-}
-
-//expansion and rarity
-font->SetColor(ARGB((int)pos.actA, 0, 0, 0));
-{
-    char buf[512];
-    switch(card->getRarity())
+    //expansion and rarity
+    font->SetColor(ARGB((int)pos.actA, 0, 0, 0));
     {
+        char buf[512];
+        switch(card->getRarity())
+        {
         case Constants::RARITY_M:
-        sprintf(buf,_("%s Mythic").c_str(),setlist[card->setId].c_str());
-        break;
+            sprintf(buf,_("%s Mythic").c_str(),setlist[card->setId].c_str());
+            break;
         case Constants::RARITY_R:
-        sprintf(buf,_("%s Rare").c_str(),setlist[card->setId].c_str());
-        break;
+            sprintf(buf,_("%s Rare").c_str(),setlist[card->setId].c_str());
+            break;
         case Constants::RARITY_U:
-        sprintf(buf,_("%s Uncommon").c_str(),setlist[card->setId].c_str());
-        break;
+            sprintf(buf,_("%s Uncommon").c_str(),setlist[card->setId].c_str());
+            break;
         case Constants::RARITY_C:
-        sprintf(buf,_("%s Common").c_str(),setlist[card->setId].c_str());
-        break;
+            sprintf(buf,_("%s Common").c_str(),setlist[card->setId].c_str());
+            break;
         case Constants::RARITY_L:
-        sprintf(buf,_("%s Land").c_str(),setlist[card->setId].c_str());
-        break;
+            sprintf(buf,_("%s Land").c_str(),setlist[card->setId].c_str());
+            break;
         case Constants::RARITY_T:
-        sprintf(buf,_("%s Token").c_str(),setlist[card->setId].c_str());
-        break;
+            sprintf(buf,_("%s Token").c_str(),setlist[card->setId].c_str());
+            break;
         default:
         case Constants::RARITY_S:
-        sprintf(buf,_("%s Special").c_str(),setlist[card->setId].c_str());
-        break;
-    }
+            sprintf(buf,_("%s Special").c_str(),setlist[card->setId].c_str());
+            break;
+        }
 
-    switch(card->data->getColor())
-    {
+        switch(card->data->getColor())
+        {
         case Constants::MTG_COLOR_BLACK:
         case Constants::MTG_COLOR_GREEN:
         case Constants::MTG_COLOR_BLUE:
         case Constants::MTG_COLOR_LAND:
-        font->SetColor(ARGB((int)pos.actA,255,255,255));
-        font->DrawString(buf, x + (22 - BigWidth / 2)*pos.actZ, pos.actY + (BigHeight / 2 - 30)*pos.actZ);
-        break;
+            font->SetColor(ARGB((int)pos.actA,255,255,255));
+            font->DrawString(buf, x + (22 - BigWidth / 2)*pos.actZ, pos.actY + (BigHeight / 2 - 30)*pos.actZ);
+            break;
         default:
-        font->DrawString(buf, x + (22 - BigWidth / 2)*pos.actZ, pos.actY + (BigHeight / 2 - 30)*pos.actZ);
-        break; //Leave black
+            font->DrawString(buf, x + (22 - BigWidth / 2)*pos.actZ, pos.actY + (BigHeight / 2 - 30)*pos.actZ);
+            break; //Leave black
+        }
+
     }
 
-}
-
-font->SetScale(backup_scale);
+    font->SetScale(backup_scale);
 }
 
 //Renders a big card on screen. Defaults to the "alternate" rendering if no image is found
@@ -747,28 +792,20 @@ void CardGui::RenderBig(MTGCard* card, const Pos& pos)
 
     float x = pos.actX;
 
-    JQuad * quad = WResourceManager::Instance()->RetrieveCard(card);
-    if (quad)
+    JQuadPtr quad = WResourceManager::Instance()->RetrieveCard(card);
+    if (quad.get())
     {
         if (quad->mHeight < quad->mWidth)
         {
-            return TinyCropRender(card, pos, quad);
+            return TinyCropRender(card, pos, quad.get());
         }
         quad->SetColor(ARGB(255,255,255,255));
-        float scale = pos.actZ * 257.f / quad->mHeight;
-        renderer->RenderQuad(quad, x, pos.actY, pos.actT, scale, scale);
+        float scale = pos.actZ * 250.f / quad->mHeight;
+        renderer->RenderQuad(quad.get(), x, pos.actY, pos.actT, scale, scale);
         return;
     }
 
-    //No card found, attempt to render the thumbnail instead (better than nothing, even if it gets super stretched)
-    JQuad * q;
-    if ((q = WResourceManager::Instance()->RetrieveCard(card, CACHE_THUMB)))
-    {
-        float scale = pos.actZ * 250 / q->mHeight;
-        q->SetColor(ARGB(255,255,255,255));
-        renderer->RenderQuad(q, x, pos.actY, pos.actT, scale, scale);
-        return;
-    }
+    //DebugTrace("Unable to fetch image: " << card->getImageName());
 
     // If we come here, we do not have the picture.
     AlternateRender(card, pos);
@@ -777,12 +814,14 @@ void CardGui::RenderBig(MTGCard* card, const Pos& pos)
 void CardGui::RenderCountersBig(const Pos& pos)
 {
     // Write Named Counters
-    if (card->counters)
+    if (card->counters && card->counters->mCount > 0)
     {
         WFont * font = WResourceManager::Instance()->GetWFont(Fonts::MAGIC_FONT);
         font->SetColor(ARGB((int)pos.actA, 0, 0, 0));
         font->SetScale(kWidthScaleFactor * pos.actZ);
-        std::vector<string> txt = card->formattedText();
+
+        std::vector<string> txt;
+        FormatText(card->data->getText(), txt);
         unsigned i = txt.size() + 1;
         Counter * c = NULL;
         for (int t = 0; t < card->counters->mCount; t++, i++)
@@ -809,7 +848,7 @@ void CardGui::RenderCountersBig(const Pos& pos)
                     sprintf(buf, _("%i/%i counters: %i").c_str(), c->power, c->toughness, c->nb);
                 }
                 font->DrawString(buf, pos.actX + (22 - BigWidth / 2) * pos.actZ, pos.actY + (-BigHeight / 2 + 80 + 11 * i)
-                                * pos.actZ);
+                    * pos.actZ);
             }
         }
     }
@@ -820,12 +859,13 @@ MTGCardInstance* CardView::getCard()
     return card;
 }
 
-TransientCardView::TransientCardView(MTGCardInstance* card, float x, float y) :
-    CardGui(card, x, y)
+TransientCardView::TransientCardView(MTGCardInstance* card, float x, float y)
+    : CardGui(card, x, y)
 {
 }
-TransientCardView::TransientCardView(MTGCardInstance* card, const Pos& ref) :
-    CardGui(card, ref)
+
+TransientCardView::TransientCardView(MTGCardInstance* card, const Pos& ref)
+    : CardGui(card, ref)
 {
 }
 ;
@@ -833,9 +873,38 @@ TransientCardView::TransientCardView(MTGCardInstance* card, const Pos& ref) :
 ostream& CardView::toString(ostream& out) const
 {
     return (CardGui::toString(out) << " : CardView ::: card : " << card << ";  actX,actY : " << actX << "," << actY << "; t : "
-                    << t << " ; actT : " << actT);
+        << t << " ; actT : " << actT);
 }
 ostream& CardGui::toString(ostream& out) const
 {
     return (out << "CardGui ::: x,y " << x << "," << y);
 }
+
+
+SimpleCardEffectRotate::SimpleCardEffectRotate(float rotation): mRotation(rotation)
+{
+}
+    
+void SimpleCardEffectRotate::doEffect(Pos * card)
+{
+    card->t = mRotation;
+}
+
+void SimpleCardEffectRotate::undoEffect(Pos * card)
+{
+    card->t = 0;
+}
+
+SimpleCardEffectMask::SimpleCardEffectMask(PIXEL_TYPE mask): mMask(mask)
+{
+}
+    
+void SimpleCardEffectMask::doEffect(Pos * card)
+{
+    card->mask = mMask;
+}
+
+void SimpleCardEffectMask::undoEffect(Pos * card)
+{
+    card->mask = 0;
+}
\ No newline at end of file
