@@ -2,7 +2,6 @@
 
 #include "Credits.h"
 #include "GameApp.h"
-#include "GameOptions.h"
 #include "PlayerData.h"
 #include "DeckStats.h"
 #include "Translate.h"
@@ -49,7 +48,7 @@ void Credits::compute(Player * _p1, Player * _p2, GameApp * _app)
     GameObserver * g = GameObserver::GetInstance();
     if (!g->turn)
         return;
-    PlayerData * playerdata = NEW PlayerData(app->collection);
+    PlayerData * playerdata = NEW PlayerData(MTGCollection());
     if (!p1->isAI() && p2->isAI() && p1 != g->gameOver)
     {
         gameLength = time(0) - g->startedAt;
@@ -117,7 +116,7 @@ void Credits::compute(Player * _p1, Player * _p2, GameApp * _app)
         if (unlocked == -1)
         {
             DeckStats * stats = DeckStats::GetInstance();
-            stats->load(p1);
+            stats->load(p1->GetCurrentDeckStatsFile());
             unlocked = isDifficultyUnlocked(stats);
             if (unlocked)
             {
@@ -133,6 +132,24 @@ void Credits::compute(Player * _p1, Player * _p2, GameApp * _app)
                 goa->giveAward();
                 options.save();
             }
+			else if ((unlocked = isStoneHewerUnlocked()))
+			{
+				//unlockedTextureName = "stonehewer_unlocked.png";//until we can find a nice free use font.
+                CreditBonus * b = NEW CreditBonus(0, _("Stone Hewer Basic Unlocked"));
+                bonus.push_back(b);
+				goa = (GameOptionAward*) &options[Options::STONEHEWER_MODE_UNLOCKED];
+				goa->giveAward();
+				options.save();
+			}
+			else if ((unlocked = isHermitUnlocked()))
+			{
+				//unlockedTextureName = "hermit_unlocked.png";//until we can find a nice free use font.
+                CreditBonus * b = NEW CreditBonus(0, _("Hermit Druid Basic Unlocked"));
+                bonus.push_back(b);
+				goa = (GameOptionAward*) &options[Options::HERMIT_MODE_UNLOCKED];
+				goa->giveAward();
+				options.save();
+			}
             else if ((unlocked = isEvilTwinUnlocked()))
             {
                 unlockedTextureName = "eviltwin_unlocked.png";
@@ -201,14 +218,19 @@ void Credits::compute(Player * _p1, Player * _p2, GameApp * _app)
     SAFE_DELETE(playerdata);
 }
 
-JQuad * Credits::GetUnlockedQuad(string textureName)
+JQuadPtr Credits::GetUnlockedQuad(string textureName)
 {
-    if (!textureName.size()) return NULL;
+    if (!textureName.size()) return JQuadPtr();
 
     JTexture * unlockedTex = WResourceManager::Instance()->RetrieveTexture(textureName);
-    if (!unlockedTex) return NULL;
+    if (!unlockedTex) return JQuadPtr();
 
-    return WResourceManager::Instance()->RetrieveQuad(unlockedTextureName, 2, 2, unlockedTex->mWidth - 4, unlockedTex->mHeight - 4);
+    return WResourceManager::Instance()->RetrieveQuad(
+                                            unlockedTextureName,
+                                            2.0f,
+                                            2.0f, 
+                                            static_cast<float>(unlockedTex->mWidth - 4),
+                                            static_cast<float>(unlockedTex->mHeight - 4));
     
 }
 
@@ -239,11 +261,11 @@ void Credits::Render()
             if (g->gameOver != p1)
             {
                 sprintf(buffer, _("Congratulations! You earn %i credits").c_str(), value);
-                JQuad * unlockedQuad = GetUnlockedQuad(unlockedTextureName);
+                JQuadPtr unlockedQuad = GetUnlockedQuad(unlockedTextureName);
                 if (unlockedQuad)
                 {
                     showMsg = 0;
-                    r->RenderQuad(unlockedQuad, 20, 20);
+                    r->RenderQuad(unlockedQuad.get(), 20, 20);
                 }
                 if (unlockedString.size())
                 {
@@ -339,6 +361,24 @@ int Credits::isMomirUnlocked()
     return 0;
 }
 
+int Credits::isStoneHewerUnlocked()
+{
+    if (options[Options::STONEHEWER_MODE_UNLOCKED].number)
+        return 0;
+	if (int(p1->game->inPlay->countByType("equipment") + p1->opponent()->game->inPlay->countByType("equipment")) > 10)
+        return 1;
+    return 0;
+}
+
+int Credits::isHermitUnlocked()
+{
+    if (options[Options::HERMIT_MODE_UNLOCKED].number)
+        return 0;
+	if (int(p1->game->inPlay->countByType("land")) < 10)
+        return 1;
+    return 0;
+}
+
 int Credits::isEvilTwinUnlocked()
 {
     if (options[Options::EVILTWIN_MODE_UNLOCKED].number)
@@ -383,8 +423,7 @@ int Credits::addCardToCollection(int cardId, MTGDeck * collection)
  */
 int Credits::addCardToCollection(int cardId)
 {
-    MTGAllCards * ac = GameApp::collection;
-    PlayerData * playerdata = NEW PlayerData(ac);
+    PlayerData * playerdata = NEW PlayerData(MTGCollection());
     int result = addCardToCollection(cardId, playerdata->collection);
     playerdata->collection->save();
     return result;
@@ -448,7 +487,7 @@ int Credits::IsMoreAIDecksUnlocked(DeckStats * stats) {
         found = 0;
         char buffer[512];
         sprintf(buffer, JGE_GET_RES("ai/baka/deck%i.txt").c_str(), nbdecks + 1);
-        std::ifstream file(buffer);
+        wagic::ifstream file(buffer);
         if (file)
         {
             found = 1;
