@@ -1,5 +1,6 @@
 #include "PrecompiledHeader.h"
 
+#include "DeckManager.h"
 #include "DeckStats.h"
 #include "Player.h"
 #include "GameObserver.h"
@@ -8,7 +9,7 @@
 
 DeckStats * DeckStats::mInstance = NULL;
     
-DeckStat::DeckStat(int _nbgames, int _victories) : nbgames(_nbgames), victories(_victories)
+DeckStat::DeckStat(int nbgames, int victories, string manaColorIndex) : nbgames(nbgames), victories(victories), manaColorIndex( manaColorIndex)
 {
 }
 
@@ -23,30 +24,31 @@ DeckStats * DeckStats::GetInstance()
     if (!mInstance)
     {
         mInstance = NEW DeckStats();
-
     }
     return mInstance;
 }
 
-void DeckStats::cleanStats()
-{
-    map<string, DeckStat *>::iterator it;
-    for (it = stats.begin(); it != stats.end(); it++)
-    {
-        SAFE_DELETE(it->second);
-    }
-
-    stats.clear();
-}
-
 DeckStats::~DeckStats()
 {
-    cleanStats();
+    map<string, map<string,DeckStat*> > ::iterator it;
+    for (it = masterDeckStats.begin(); it != masterDeckStats.end(); ++it)
+    {
+        string key = it->first;
+        map<string, DeckStat *> innerMap = masterDeckStats[key];
+        map<string, DeckStat *>::iterator it2;
+        for (it2 = innerMap.begin(); it2 != innerMap.end(); it2++)
+        {
+            SAFE_DELETE(it2->second);
+        }
+        innerMap.clear();
+    }    
+    masterDeckStats.clear();
 }
 
 
 DeckStat* DeckStats::getDeckStat(string opponentsFile)
 {
+    map<string, DeckStat *> stats = masterDeckStats[currentDeck];
     map<string, DeckStat *>::iterator it = stats.find(opponentsFile);
     if (it == stats.end())
     {
@@ -61,6 +63,7 @@ DeckStat* DeckStats::getDeckStat(string opponentsFile)
 int DeckStats::nbGames()
 {
     int nbgames = 0;
+    map<string, DeckStat *> stats = masterDeckStats[currentDeck];
     map<string, DeckStat *>::iterator it;
     for (it = stats.begin(); it != stats.end(); it++)
     {
@@ -72,6 +75,7 @@ int DeckStats::nbGames()
 
 int DeckStats::percentVictories(string opponentsFile)
 {
+    map<string, DeckStat *> stats = masterDeckStats[currentDeck];
     map<string, DeckStat *>::iterator it = stats.find(opponentsFile);
     if (it == stats.end())
     {
@@ -87,6 +91,7 @@ int DeckStats::percentVictories()
 {
     int victories = 0;
     int nbgames = 0;
+    map<string, DeckStat *> stats = masterDeckStats[currentDeck];
     map<string, DeckStat *>::iterator it;
     for (it = stats.begin(); it != stats.end(); it++)
     {
@@ -101,21 +106,41 @@ int DeckStats::percentVictories()
     return 50;
 }
 
-void DeckStats::load(Player * player)
+void DeckStats::load(const std::string& filename)
 {
-    char filename[512];
-    sprintf(filename, "stats/%s.txt", player->deckFileSmall.c_str());
-    load(options.profileFile(filename).c_str());
-}
 
-void DeckStats::load(const char * filename)
-{
-    cleanStats();
-    std::ifstream file(filename);
+    currentDeck = filename;
+    if ( masterDeckStats.find(filename) != masterDeckStats.end() )
+    {
+        return;
+    }
+    wagic::ifstream file(filename.c_str());
     std::string s;
 
     if (file)
     {
+        // get the associated player deck file:
+        int deckId = atoi(filename.substr(filename.find("_deck") + 5, filename.find(".txt")).c_str());
+        char buffer[512];
+        sprintf(buffer, "deck%i.txt", deckId);
+        string playerDeckFilePath = options.profileFile( buffer);
+        DeckMetaData *playerDeckMetaData = DeckManager::GetInstance()->getDeckMetaDataByFilename( playerDeckFilePath, false);
+        if (!playerDeckMetaData)
+        {
+            DebugTrace("DeckStats.cpp:CONSISTENCY ERROR: DeckStats are set, but no deck meta data");
+            file.close();
+            return;
+        }
+        // check if this player deck has already been profiled for manacolors
+        char next = file.peek();
+        string manaColorIndex = "";
+        if ( next == 'M')
+        {
+            std::getline(file, s );
+            manaColorIndex = s.substr( s.find(":") + 1);
+            playerDeckMetaData->setColorIndex( manaColorIndex );
+        }
+
         while (std::getline(file, s))
         {
             string deckfile = s;
@@ -123,30 +148,46 @@ void DeckStats::load(const char * filename)
             int games = atoi(s.c_str());
             std::getline(file, s);
             int victories = atoi(s.c_str());
-            map<string, DeckStat *>::iterator it = stats.find(deckfile);
-            if (it == stats.end())
+            next = file.peek();
+            
+            if ( next == 'M')
             {
-                stats[deckfile] = NEW DeckStat(games, victories);
+                std::getline(file, s );
+                manaColorIndex = s.substr( s.find(":") + 1);
             }
+            if ( masterDeckStats[filename].find(deckfile) != masterDeckStats[filename].end())
+            {
+                SAFE_DELETE( masterDeckStats[filename][deckfile] );
+            }
+            DeckStat * newDeckStat = NEW DeckStat(games, victories, manaColorIndex);
+            (masterDeckStats[filename])[deckfile] = newDeckStat;
         }
         file.close();
     }
 }
 
-void DeckStats::save(Player * player)
+void DeckStats::save(const std::string& filename)
 {
-    char filename[512];
-    sprintf(filename, "stats/%s.txt", player->deckFileSmall.c_str());
-    save(options.profileFile(filename).c_str());
-}
-
-void DeckStats::save(const char * filename)
-{
-    std::ofstream file(filename);
+    std::ofstream file(filename.c_str());
     char writer[512];
     if (file)
     {
+        map<string, DeckStat *> stats = masterDeckStats[currentDeck];
         map<string, DeckStat *>::iterator it;
+        string manaColorIndex = "";
+        int deckId = atoi(filename.substr(filename.find("_deck") + 5, filename.find(".txt")).c_str());
+        char buffer[512];
+        sprintf(buffer, "deck%i.txt", deckId);
+        string playerDeckFilePath= options.profileFile( buffer);
+        DeckManager *deckManager = DeckManager::GetInstance();
+        DeckMetaData *playerDeckMeta = deckManager->getDeckMetaDataByFilename(playerDeckFilePath, false);
+        if ( playerDeckMeta->getColorIndex() == "" )
+        {
+            StatsWrapper *stw = deckManager->getExtendedDeckStats( playerDeckMeta, MTGAllCards::getInstance(), false);
+            manaColorIndex = stw->getManaColorIndex();
+            playerDeckMeta->setColorIndex( manaColorIndex );
+        }
+        file << "MANA:" << manaColorIndex << endl;
         for (it = stats.begin(); it != stats.end(); it++)
         {
             sprintf(writer, "%s\n", it->first.c_str());
@@ -155,8 +196,11 @@ void DeckStats::save(const char * filename)
             file << writer;
             sprintf(writer, "%i\n", it->second->victories);
             file << writer;
+            file << "MANA:" << it->second->manaColorIndex <<endl;
         }
         file.close();
+
+        playerDeckMeta->Invalidate();
     }
 }
 
@@ -172,20 +216,47 @@ void DeckStats::saveStats(Player *player, Player *opponent, GameObserver * game)
     {
         victory = 0;
     }
-    load(player);
-    map<string, DeckStat *>::iterator it = stats.find(opponent->deckFileSmall);
-    if (it == stats.end())
+    load(currentDeck);
+    map<string, DeckStat *> *stats = &masterDeckStats[currentDeck];
+    map<string, DeckStat *>::iterator it = stats->find(opponent->deckFileSmall);
+    string manaColorIndex = "";
+    DeckManager *deckManager = DeckManager::GetInstance();
+    DeckMetaData *aiDeckMeta = deckManager->getDeckMetaDataByFilename( opponent->deckFile, true);
+    StatsWrapper *stw = deckManager->getExtendedDeckStats( aiDeckMeta, MTGAllCards::getInstance(), true);
+    manaColorIndex = stw->getManaColorIndex();
+    if (it == stats->end())
     {
-        stats[opponent->deckFileSmall] = NEW DeckStat(1, victory);
+        stats->insert( make_pair( opponent->deckFileSmall, NEW DeckStat(1, victory, manaColorIndex) ));
     }
     else
     {
         it->second->victories += victory;
         it->second->nbgames += 1;
+        if ( it->second->manaColorIndex == "" )
+        {
+            it->second->manaColorIndex = manaColorIndex;
+        }
     }
-    save(player);
+    save(currentDeck);
+    
+    DeckMetaData* playerMeta = DeckManager::GetInstance()->getDeckMetaDataByFilename(player->deckFile, false);
+
+    // metadata caches its internal data (number of games, victories, etc)
+    // tell it to refresh when stats are updated
+    if (playerMeta)
+        playerMeta->Invalidate();
+
+    DeckMetaData* aiMeta = DeckManager::GetInstance()->getDeckMetaDataByFilename(opponent->deckFile, true);
+    if (aiMeta)
+        aiMeta->Invalidate();
 }
 
+void DeckStats::EndInstance()
+{
+    SAFE_DELETE( mInstance );
+}
+
+
 // StatsWrapper
 
     float noLandsProbInTurn[Constants::STATS_FOR_TURNS] = {0.0f};
@@ -244,7 +315,7 @@ void StatsWrapper::initStatistics(string deckstats)
     aiDeckNames.clear();
     aiDeckStats.clear();
 
-    if (fileExists(deckstats.c_str()))
+    if (FileExists(deckstats))
     {
         stats->load(deckstats.c_str());
         percentVictories = stats->percentVictories();
@@ -256,10 +327,13 @@ void StatsWrapper::initStatistics(string deckstats)
         found = 0;
         char buffer[512];
         char smallDeckName[512];
-        sprintf(buffer, "%s/deck%i.txt", RESPATH"/ai/baka", nbDecks + 1);
+        ostringstream oss;
+        oss << "deck" << (nbDecks + 1);
+        string bakaDir = JGE_GET_RES("/ai/baka");
+        string deckFilename = oss.str();
+        sprintf(buffer, "%s/%s.txt", bakaDir.c_str(), deckFilename.c_str());
         if (fileExists(buffer))
         {
-            MTGDeck * mtgd = NEW MTGDeck(buffer, NULL, 1);
             found = 1;
             nbDecks++;
 
@@ -268,12 +342,10 @@ void StatsWrapper::initStatistics(string deckstats)
 
             if ((deckStat != NULL) && (deckStat->nbgames > 0))
             {
-                int percentVictories = stats->percentVictories(string(smallDeckName));
-                aiDeckNames.push_back(string(mtgd->meta_name));
+                aiDeckNames.push_back(deckFilename);
                 aiDeckStats.push_back(deckStat);
             }
 
-            delete mtgd;
         }
     }
     else
@@ -283,9 +355,21 @@ void StatsWrapper::initStatistics(string deckstats)
     }
 }
 
+string StatsWrapper::getManaColorIndex()
+{
+    ostringstream oss;
+    for (int i = Constants::MTG_COLOR_ARTIFACT; i < Constants::MTG_COLOR_LAND; ++i)
+        if (totalCostPerColor[i] != 0)
+            oss << "1";
+        else
+            oss <<"0";
+    return oss.str();
+
+}
+
 void StatsWrapper::updateStats(string filename, MTGAllCards *collection)
 {
-    if (fileExists(filename.c_str()))
+    if (FileExists(filename))
     {
         MTGDeck * mtgd = NEW MTGDeck(filename.c_str(), collection);
         DeckDataWrapper *deckDataWrapper = NEW DeckDataWrapper(mtgd);
@@ -367,6 +451,28 @@ void StatsWrapper::updateStats(DeckDataWrapper *myDeck)
 
         // Lets look for mana producing abilities
 
+        //http://code.google.com/p/wagic/issues/detail?id=650
+        //Basic lands are not producing their mana through regular abilities anymore,
+        //but through a rule that is outside of the primitives. This block is a hack to address this
+        const int colors[] = {Constants::MTG_COLOR_GREEN, Constants::MTG_COLOR_BLUE, Constants::MTG_COLOR_RED, Constants::MTG_COLOR_BLACK, Constants::MTG_COLOR_WHITE};
+        const string lands[] = { "forest", "island", "mountain", "swamp", "plains" };
+        for (unsigned int i = 0; i < sizeof(colors)/sizeof(colors[0]); ++i)
+        {
+            int colorId = colors[i];
+            string type = lands[i];
+            if (current->data->hasType(type.c_str()))
+            {
+                if (current->data->hasType("Basic"))
+                {
+                    this->countBasicLandsPerColor[colorId] += currentCount;
+                }
+                else
+                {
+                    this->countLandsPerColor[colorId] += currentCount;
+                }
+            }
+        }
+
         vector<string> abilitiesVector;
         string thisstring = current->data->magicText;
         abilitiesVector = split(thisstring, '\n');
