@@ -23,7 +23,9 @@ void DuelLayers::init()
     action = NEW ActionLayer();
     action->Add(NEW MTGGamePhase(action->getMaxId()));
     //Add Magic Specific Rules
+    action->Add(NEW MTGEventBonus(-1));
     action->Add(NEW MTGPutInPlayRule(-1));
+    action->Add(NEW MTGKickerRule(-1));
     action->Add(NEW MTGAlternativeCostRule(-1));
     action->Add(NEW MTGBuyBackRule(-1));
     action->Add(NEW MTGFlashBackRule(-1));
@@ -36,6 +38,7 @@ void DuelLayers::init()
     action->Add(NEW MTGPlaneWalkerRule(-1));
     action->Add(NEW MTGTokensCleanup(-1)); // needs to be before persist
     action->Add(NEW MTGPersistRule(-1));
+    action->Add(NEW MTGVampireRule(-1));
     action->Add(NEW MTGUnearthRule(-1));
     action->Add(NEW MTGLifelinkRule(-1));
     action->Add(NEW MTGDeathtouchRule(-1));
@@ -63,34 +66,36 @@ void DuelLayers::CheckUserInput(int isAI)
 {
     JButton key;
     int x, y;
-    while ((key = JGE::GetInstance()->ReadButton()))
+    while ((key = JGE::GetInstance()->ReadButton()) || JGE::GetInstance()->GetLeftClickCoordinates(x, y))
     {
-        if ((!isAI) && (0 != key))
+        if ((!isAI) && ((0 != key) ||  JGE::GetInstance()->GetLeftClickCoordinates(x, y)))
         {
-            if (stack->CheckUserInput(key))
+            if (stack->CheckUserInput(key)) {
+                JGE::GetInstance()->LeftClickedProcessed();
                 break;
-            if (combat->CheckUserInput(key))
+            }
+            if (combat->CheckUserInput(key)) {
+                JGE::GetInstance()->LeftClickedProcessed();
                 break;
-            if (avatars->CheckUserInput(key))
+            }
+            if (avatars->CheckUserInput(key)) {
+                JGE::GetInstance()->LeftClickedProcessed();
                 break; //avatars need to check their input before action (CTRL_CROSS)
-            if (action->CheckUserInput(key))
+            }
+            if (action->CheckUserInput(key)) {
+                JGE::GetInstance()->LeftClickedProcessed();
                 break;
-            if (hand->CheckUserInput(key))
+            }
+            if (hand->CheckUserInput(key)) {
+                JGE::GetInstance()->LeftClickedProcessed();
                 break;
-            if (CardSelectorSingleton::Instance()->CheckUserInput(key))
+            }
+            if (CardSelectorSingleton::Instance()->CheckUserInput(key)) {
+                JGE::GetInstance()->LeftClickedProcessed();
                 break;
+            }
         }
-    }
-    if ((!isAI) && JGE::GetInstance()->GetLeftClickCoordinates(x, y))
-    {
-        if (avatars->CheckUserInput(x, y))
-        {
-            JGE::GetInstance()->LeftClickedProcessed();
-        }
-        else if (CardSelectorSingleton::Instance()->CheckUserInput(x, y))
-        {
-            JGE::GetInstance()->LeftClickedProcessed();
-        }
+        JGE::GetInstance()->LeftClickedProcessed();
     }
 }
 
