@@ -1,11 +1,11 @@
 #include "PrecompiledHeader.h"
 
 #include "GameObserver.h"
-#include "GameOptions.h"
 #include "CardGui.h"
 #include "Damage.h"
 #include "Rules.h"
 #include "ExtraCost.h"
+#include "Subtypes.h"
 #include <JLogger.h>
 #include <JRenderer.h>
 
@@ -13,7 +13,6 @@ GameObserver * GameObserver::mInstance = NULL;
 
 GameObserver* GameObserver::GetInstance()
 {
-
     return mInstance;
 }
 
@@ -108,38 +107,34 @@ void GameObserver::nextGamePhase()
     if (currentGamePhase == Constants::MTG_PHASE_BEFORE_BEGIN)
     {
         cleanupPhase();
-        currentPlayer->canPutLandsIntoPlay = true;
-        currentPlayer->landsPlayerCanStillPlay = 1;
-        currentPlayer->castedspellsthisturn = 0;
-        currentPlayer->opponent()->castedspellsthisturn = 0;
-        currentPlayer->castcount = 0;
-        currentPlayer->nocreatureinstant = false;
-        currentPlayer->nospellinstant = false;
-        currentPlayer->onlyoneinstant = false;
         currentPlayer->damageCount = 0;
         currentPlayer->preventable = 0;
+        currentPlayer->isPoisoned = false;
         mLayers->actionLayer()->cleanGarbage(); //clean abilities history for this turn;
         mLayers->stackLayer()->garbageCollect(); //clean stack history for this turn;
         mLayers->actionLayer()->Update(0);
         for (int i = 0; i < 2; i++)
         {
-            delete (players[i]->game->garbage);
-            players[i]->game->garbage = NEW MTGGameZone();
-            players[i]->game->garbage->setOwner(players[i]);
+            //Cleanup of each player's gamezones
+            players[i]->game->beforeBeginPhase();
         }
         combatStep = BLOCKERS;
         return nextGamePhase();
     }
 
-    for (int i = 0; i < 2; ++i)
-        players[i]->getManaPool()->init();
-
     if (currentGamePhase == Constants::MTG_PHASE_AFTER_EOT)
     {
         //Auto Hand cleaning, in case the player didn't do it himself
-        while (currentPlayer->game->hand->nb_cards > 7 && currentPlayer->nomaxhandsize == false)
+		while (currentPlayer->game->hand->nb_cards > currentPlayer->handsize && currentPlayer->nomaxhandsize == false)
+        {
+            WEvent * e = NEW WEventCardDiscard(currentPlayer->game->hand->cards[0]);
+            GameObserver * game = GameObserver::GetInstance();
+            game->receiveEvent(e);
             currentPlayer->game->putInGraveyard(currentPlayer->game->hand->cards[0]);
+        }
         mLayers->actionLayer()->Update(0);
+        currentPlayer->lifeLostThisTurn = 0;
+        currentPlayer->opponent()->lifeLostThisTurn = 0;
         return nextGamePhase();
     }
 
@@ -150,9 +145,6 @@ void GameObserver::nextGamePhase()
         DebugTrace("Untap Phase -------------   Turn " << turn );
         untapPhase();
         break;
-    case Constants::MTG_PHASE_DRAW:
-        //mLayers->stackLayer()->addDraw(currentPlayer,1);
-        break;
     case Constants::MTG_PHASE_COMBATBLOCKERS:
         receiveEvent(NEW WEventAttackersChosen());
         break;
@@ -210,26 +202,20 @@ void GameObserver::userRequestNextGamePhase()
     if (WaitForExtraPayment(NULL)) 
     	return;
 
-    bool executeNextPhaseImmediately = true;
-
     Phase * cPhaseOld = phaseRing->getCurrentPhase();
     if ((cPhaseOld->id == Constants::MTG_PHASE_COMBATBLOCKERS && combatStep == ORDER) 
     	|| (cPhaseOld->id == Constants::MTG_PHASE_COMBATBLOCKERS && combatStep == TRIGGERS) 
     	|| (cPhaseOld->id == Constants::MTG_PHASE_COMBATDAMAGE)
         || opponent()->isAI() 
         || options[Options::optionInterrupt(currentGamePhase)].number
+		|| currentPlayer->offerInterruptOnPhase - 1 == currentGamePhase
     )
     {
-        executeNextPhaseImmediately = false;
-    }
-
-    if (executeNextPhaseImmediately)
-    {
-        nextGamePhase();
+        mLayers->stackLayer()->AddNextGamePhase();
     }
     else
     {
-        mLayers->stackLayer()->AddNextGamePhase();
+       nextGamePhase(); 
     }
 
 }
@@ -237,18 +223,16 @@ void GameObserver::userRequestNextGamePhase()
 int GameObserver::forceShuffleLibraries()
 {
     int result = 0;
-    if (players[0]->game->library->needShuffle)
+    for (int i = 0; i < 2 ; ++i)
     {
-        players[0]->game->library->shuffle();
-        players[0]->game->library->needShuffle = false;
-        ++result;
-    }
-    if (players[1]->game->library->needShuffle)
-    {
-        players[1]->game->library->shuffle();
-        players[1]->game->library->needShuffle = false;
-        ++result;
+        if (players[i]->game->library->needShuffle)
+        {
+            players[i]->game->library->shuffle();
+            players[i]->game->library->needShuffle = false;
+            ++result;
+        }
     }
+
     return result;
 }
 
@@ -297,7 +281,7 @@ void GameObserver::startGame(Rules * rules)
                 for (int j = 0; j < z->nb_cards; j++)
                 {
                     MTGCardInstance * _card = z->cards[j];
-                    if (_card->hasType("land"))
+                    if (_card->isLand())
                     {
                         card = _card;
                         j = z->nb_cards;
@@ -320,14 +304,15 @@ void GameObserver::addObserver(MTGAbility * observer)
     mLayers->actionLayer()->Add(observer);
 }
 
-void GameObserver::removeObserver(ActionElement * observer)
+//Returns true if the Ability was correctly removed from the game, false otherwise
+//Main (valid) reason of returning false is an attempt at removing an Ability that has already been removed
+bool GameObserver::removeObserver(ActionElement * observer)
 {
-    if (observer)
-        mLayers->actionLayer()->moveToGarbage(observer);
+    if (!observer)
+        return false;
+
+    return mLayers->actionLayer()->moveToGarbage(observer);
 
-    else
-    {
-    } //TODO log error
 }
 
 GameObserver::~GameObserver()
@@ -373,16 +358,7 @@ void GameObserver::gameStateBasedEffects()
     //check land playability at start; as we want this effect to happen reguardless of unresolved
     //effects or menus actions
     for (int i = 0; i < 2; i++)
-    {
-        if (players[i]->landsPlayerCanStillPlay <= 0)
-        {
-            players[i]->canPutLandsIntoPlay = false;
-        }
-        else
-        {
-            players[i]->canPutLandsIntoPlay = true;
-        }
-    }
+        players[i]->isPoisoned = (players[i]->poisonCount > 0);
     if (mLayers->stackLayer()->count(0, NOT_RESOLVED) != 0)
     	return;
     if (mLayers->actionLayer()->menuObject) 
@@ -401,12 +377,78 @@ void GameObserver::gameStateBasedEffects()
         {
             MTGCardInstance * card = zone->cards[j];
             card->afterDamage();
+            card->mPropertiesChangedSinceLastUpdate = false;
 
-            //Remove auras that don't have a valid target anymore
-            if (card->target && !isInPlay(card->target) && !card->hasType("equipment"))
+            ///////////////////////////////////////////////////////
+            //Remove auras that don't have a valid target anymore//
+            ///////////////////////////////////////////////////////
+            if (card->target && !isInPlay(card->target) && !card->hasType(Subtypes::TYPE_EQUIPMENT))
             {
                 players[i]->game->putInGraveyard(card);
             }
+            card->enchanted = false;
+            if (card->target && isInPlay(card->target) && !card->hasType(Subtypes::TYPE_EQUIPMENT) && card->hasSubtype(Subtypes::TYPE_AURA))
+            {
+                card->target->enchanted = true;
+            }
+            ///////////////////////////
+            //reset extracost shadows//
+            ///////////////////////////
+            card->isExtraCostTarget = false;
+            if(mExtraPayment != NULL)
+            {
+                for(unsigned int ec = 0;ec < mExtraPayment->costs.size();ec++)
+                {
+                    if( mExtraPayment->costs[ec]->target)
+                        mExtraPayment->costs[ec]->target->isExtraCostTarget = true;
+                }
+            }
+            //////////////////////
+            //reset morph hiding//
+            //////////////////////
+            if((card->previous && card->previous->morphed && !card->turningOver) || (card->morphed && !card->turningOver))
+            {
+                card->morphed = true;
+                card->isMorphed = true;
+            }
+            else
+            {
+                card->isMorphed = false;
+                card->morphed = false;
+            }
+            //////////////////////////
+            //handles phasing events//
+            //////////////////////////
+            if(card->has(Constants::PHASING)&& currentGamePhase == Constants::MTG_PHASE_UNTAP && currentPlayer == card->controller() && card->phasedTurn != turn && !card->isPhased)
+            {
+                card->isPhased = true;
+                card->phasedTurn = turn;
+                if(card->view)
+                    card->view->alpha = 50;
+                card->initAttackersDefensers();
+            }
+            else if((card->has(Constants::PHASING) || card->isPhased)&& currentGamePhase == Constants::MTG_PHASE_UNTAP && currentPlayer == card->controller() && card->phasedTurn != turn)
+            {
+                card->isPhased = false;
+                card->phasedTurn = turn;
+                if(card->view)
+                    card->view->alpha = 255;
+            }
+            if (card->target && isInPlay(card->target) && (card->hasSubtype(Subtypes::TYPE_EQUIPMENT) || card->hasSubtype(Subtypes::TYPE_AURA)))
+            {
+                card->isPhased = card->target->isPhased;
+                card->phasedTurn = card->target->phasedTurn;
+                if(card->view && card->target->view)
+                    card->view->alpha = card->target->view->alpha;
+            }
+            //////////////////////////  
+            //forceDestroy over ride//
+            //////////////////////////
+            if(card->isInPlay())
+            {
+                card->graveEffects = false;
+                card->exileEffects = false;
+            }
         }
     }
     //-------------------------------------
@@ -424,7 +466,7 @@ void GameObserver::gameStateBasedEffects()
             for (int j = 0; j < nbcards; ++j)
             {
                 MTGCardInstance * c = z->cards[j];
-                if (c->has(Constants::CANTLOSE) || c->has(Constants::CANTLIFELOSE))
+                if (c->has(Constants::CANTLOSE) || (c->has(Constants::CANTLIFELOSE) && players[i]->poisonCount < 10))
                 {
                     cantlosers++;
                 }
@@ -457,94 +499,13 @@ void GameObserver::gameStateBasedEffects()
         Player * p = players[i];
         MTGGameZone * z = players[i]->game->inPlay;
         int nbcards = z->nb_cards;
-        p->onlyonecast = false;
-        p->opponent()->onlyonecast = false;
         //------------------------------
-        if (z->hasAbility(Constants::NOMAXHAND))
-        {
-            p->nomaxhandsize = true;
-        }
-        else
-        {
-            p->nomaxhandsize = false;
-        }
-        //------------------------------
-        if (z->hasAbility(Constants::CANTCASTCREATURE))
-        {
-            p->castrestrictedcreature = true;
-        }
-        else
-        {
-            p->castrestrictedcreature = false;
-        }
-        //------------------------------
-        if (z->hasAbility(Constants::CANTCAST))
-        {
-            p->castrestrictedspell = true;
-        }
-        else
-        {
-            p->castrestrictedspell = false;
-        }
-        //------------------------------
-        if (z->hasAbility(Constants::CANTCASTTWO))
-        {
-            p->onlyonecast = true;
-        }
-        else
-        {
-            p->onlyonecast = false;
-        }
-        //--------------------------------
-        if (z->hasAbility(Constants::BOTHCANTCAST))
-        {
-            p->bothrestrictedspell = true;
-        }
-        else
-        {
-            p->bothrestrictedspell = false;
-        }
-        //---------------------------------
-        if (z->hasAbility(Constants::BOTHNOCREATURE))
-        {
-            p->bothrestrictedcreature = true;
-        }
-        else
-        {
-            p->bothrestrictedcreature = false;
-        }
-        //-----------------------------------
-        if (z->hasAbility(Constants::ONLYONEBOTH))
-        {
-            p->onlyoneboth = true;
-        }
-        else
-        {
-            p->onlyoneboth = false;
-        }
-        //------------------------------------
-        if (players[0]->bothrestrictedcreature) 
-        	players[1]->castrestrictedcreature = true;
-        //------------------------------------
-        if (players[0]->bothrestrictedspell) 
-        	players[1]->castrestrictedspell = true;
-        //------------------------------------
-        if (players[0]->onlyoneboth) 
-        	players[1]->onlyoneboth = true;
-        //------------------------------------
-        if (players[1]->bothrestrictedcreature) 
-        	players[0]->castrestrictedcreature = true;
-        //------------------------------------
-        if (players[1]->bothrestrictedspell) 
-        	players[0]->castrestrictedspell = true;
-        //------------------------------------
-        if (players[1]->onlyoneboth) 
-        	players[0]->onlyoneboth = true;
-        //------------------------------------
+        p->nomaxhandsize = (z->hasAbility(Constants::NOMAXHAND));
+
         /////////////////////////////////////////////////
         //handle end of turn effects while we're at it.//
         /////////////////////////////////////////////////
-        if (currentGamePhase == Constants::MTG_PHASE_ENDOFTURN)
+        if (currentGamePhase == Constants::MTG_PHASE_ENDOFTURN+1)
         {
             for (int j = 0; j < nbcards; ++j)
             {
@@ -552,17 +513,19 @@ void GameObserver::gameStateBasedEffects()
 
                 if(!c)break;
                 while (c->flanked)
-                {//undoes the flanking on a card
+                {
+                    /////////////////////////////////
+                    //undoes the flanking on a card//
+                    /////////////////////////////////
                     c->power += 1;
                     c->addToToughness(1);
                     c->flanked -= 1;
                 }
-                if (c->fresh) c->fresh = 0;
-                if (c->has(Constants::ONLYONEBOTH))
-                {
-                    c->controller()->castcount = 0;
-                    c->controller()->opponent()->castcount = 0;
-                }
+                c->fresh = 0;
+                if(c->wasDealtDamage && c->isInPlay())
+                    c->wasDealtDamage = false;
+                c->damageToController = false;
+                c->damageToOpponent = false;
 
             }
             for (int t = 0; t < nbcards; t++)
@@ -582,14 +545,13 @@ void GameObserver::gameStateBasedEffects()
                     p->game->putInExile(c);
 
                 }
-             if(nbcards > z->nb_cards)
-             {
-              t = 0;
-              nbcards = z->nb_cards;
-              }
+                if(nbcards > z->nb_cards)
+                {
+                    t = 0;
+                    nbcards = z->nb_cards;
+                }
             }
 
-
             MTGGameZone * f = p->game->graveyard;
             for (int k = 0; k < f->nb_cards; k++)
             {
@@ -600,11 +562,48 @@ void GameObserver::gameStateBasedEffects()
         if (z->nb_cards == 0)
         {
             p->nomaxhandsize = false;
-            if (!p->bothrestrictedcreature && !p->opponent()->bothrestrictedcreature)
-            	p->castrestrictedcreature = false;
-            if (!p->bothrestrictedspell && !p->opponent()->bothrestrictedspell) 
-            	p->castrestrictedspell = false;
-            p->onlyonecast = false;
+        }
+        //////////////////////////
+        // Check auras on a card//
+        //////////////////////////
+        enchantmentStatus();
+        /////////////////////////////
+        // Check affinity on a card//
+        /////////////////////////////
+        Affinity();
+        /////////////////////////////////////
+        // Check colored statuses on cards //
+        /////////////////////////////////////
+        for(int w = 0;w < z->nb_cards;w++)
+        {  
+            int colored = 0;
+            for (int i = Constants::MTG_COLOR_GREEN; i <= Constants::MTG_COLOR_WHITE; ++i)
+            {
+                if (z->cards[w]->hasColor(i))
+                    ++colored;
+            }
+            z->cards[w]->isMultiColored = (colored > 1) ? 1 : 0;
+
+            if(z->cards[w]->hasColor(Constants::MTG_COLOR_WHITE) && z->cards[w]->hasColor(Constants::MTG_COLOR_BLACK))
+                z->cards[w]->isBlackAndWhite = 1;
+            else
+                z->cards[w]->isBlackAndWhite = 0;
+            if(z->cards[w]->hasColor(Constants::MTG_COLOR_RED) && z->cards[w]->hasColor(Constants::MTG_COLOR_BLUE))
+                z->cards[w]->isRedAndBlue = 1;
+            else
+                z->cards[w]->isRedAndBlue = 0;
+            if(z->cards[w]->hasColor(Constants::MTG_COLOR_GREEN) && z->cards[w]->hasColor(Constants::MTG_COLOR_BLACK))
+                z->cards[w]->isBlackAndGreen = 1;
+            else
+                z->cards[w]->isBlackAndGreen = 0;
+            if(z->cards[w]->hasColor(Constants::MTG_COLOR_BLUE) && z->cards[w]->hasColor(Constants::MTG_COLOR_GREEN))
+                z->cards[w]->isBlueAndGreen = 1;
+            else
+                z->cards[w]->isBlueAndGreen = 0;
+            if(z->cards[w]->hasColor(Constants::MTG_COLOR_RED) && z->cards[w]->hasColor(Constants::MTG_COLOR_WHITE))
+                z->cards[w]->isRedAndWhite = 1;
+            else
+                z->cards[w]->isRedAndWhite = 0;
         }
     }
     ///////////////////////////////////
@@ -613,33 +612,150 @@ void GameObserver::gameStateBasedEffects()
     if (combatStep == TRIGGERS)
     {
         if (!mLayers->stackLayer()->getNext(NULL, 0, NOT_RESOLVED) && !targetChooser
-                && !mLayers->actionLayer()->isWaitingForAnswer()) 
-			mLayers->stackLayer()->AddNextCombatStep();
+            && !mLayers->actionLayer()->isWaitingForAnswer()) 
+            mLayers->stackLayer()->AddNextCombatStep();
     }
 
     //Auto skip Phases
     GameObserver * game = game->GetInstance();
     int skipLevel = (game->currentPlayer->playMode == Player::MODE_TEST_SUITE) ? Constants::ASKIP_NONE
-            : options[Options::ASPHASES].number;
+        : options[Options::ASPHASES].number;
     int nrCreatures = currentPlayer->game->inPlay->countByType("Creature");
 
     if (skipLevel == Constants::ASKIP_SAFE || skipLevel == Constants::ASKIP_FULL)
     {
         if ((opponent()->isAI() && !(isInterrupting)) && ((currentGamePhase == Constants::MTG_PHASE_UNTAP)
-        		|| (currentGamePhase == Constants::MTG_PHASE_DRAW) || (currentGamePhase == Constants::MTG_PHASE_COMBATBEGIN) 
-        		|| ((currentGamePhase == Constants::MTG_PHASE_COMBATATTACKERS) && (nrCreatures == 0)) 
-        		|| currentGamePhase == Constants::MTG_PHASE_COMBATEND || currentGamePhase == Constants::MTG_PHASE_ENDOFTURN 
-                || ((currentGamePhase == Constants::MTG_PHASE_CLEANUP) && (currentPlayer->game->hand->nb_cards < 8)))) 
-			userRequestNextGamePhase();
+            || (currentGamePhase == Constants::MTG_PHASE_DRAW) || (currentGamePhase == Constants::MTG_PHASE_COMBATBEGIN) 
+            || ((currentGamePhase == Constants::MTG_PHASE_COMBATATTACKERS) && (nrCreatures == 0)) 
+            || currentGamePhase == Constants::MTG_PHASE_COMBATEND || currentGamePhase == Constants::MTG_PHASE_ENDOFTURN 
+            || ((currentGamePhase == Constants::MTG_PHASE_CLEANUP) && (currentPlayer->game->hand->nb_cards < 8)))) 
+            userRequestNextGamePhase();
     }
     if (skipLevel == Constants::ASKIP_FULL)
     {
         if ((opponent()->isAI() && !(isInterrupting)) && (currentGamePhase == Constants::MTG_PHASE_UPKEEP 
-        		|| currentGamePhase == Constants::MTG_PHASE_COMBATDAMAGE)) 
-        	userRequestNextGamePhase();
+            || currentGamePhase == Constants::MTG_PHASE_COMBATDAMAGE)) 
+            userRequestNextGamePhase();
     }
 }
 
+void GameObserver::enchantmentStatus()
+{
+    for (int i = 0; i < 2; i++)
+    {
+        MTGGameZone * zone = players[i]->game->inPlay;
+        for (int k = zone->nb_cards - 1; k >= 0; k--)
+        {
+            MTGCardInstance * card = zone->cards[k];
+            if (card && !card->hasType(Subtypes::TYPE_EQUIPMENT) && !card->hasSubtype(Subtypes::TYPE_AURA))
+            {
+                card->enchanted = false;
+                card->auras = 0;
+            }
+        }
+        for (int j = zone->nb_cards - 1; j >= 0; j--)
+        {
+            MTGCardInstance * card = zone->cards[j];
+            if (card->target && isInPlay(card->target) && !card->hasType(Subtypes::TYPE_EQUIPMENT) && card->hasSubtype(Subtypes::TYPE_AURA))
+            {
+                card->target->enchanted = true;
+                card->target->auras += 1;
+            }
+        }
+    }
+}
+
+void GameObserver::Affinity()
+{
+    for (int i = 0; i < 2; i++)
+    {
+        MTGGameZone * zone = players[i]->game->hand;
+        for (int k = zone->nb_cards - 1; k >= 0; k--)
+        {
+            MTGCardInstance * card = zone->cards[k];
+            if (!card)
+                continue;
+
+            int color = 0;
+            string type = "";
+            //only do any of the following if a card with the stated ability is in your hand.
+            ManaCost * original = NEW ManaCost();
+            original->copy(card->model->data->getManaCost());
+            //have to run alter cost before affinity or the 2 cancel each other out.
+            if(card->getIncreasedManaCost()->getConvertedCost()||card->getReducedManaCost()->getConvertedCost())
+            {
+                if(card->getIncreasedManaCost()->getConvertedCost())
+                    original->add(card->getIncreasedManaCost());
+                if(card->getReducedManaCost()->getConvertedCost())
+                    original->remove(card->getReducedManaCost());
+                card->getManaCost()->copy(original);
+                if(card->getManaCost()->extraCosts)
+                {
+                    for(unsigned int i = 0; i < card->getManaCost()->extraCosts->costs.size();i++)
+                    {
+                        card->getManaCost()->extraCosts->costs[i]->setSource(card);
+                    }
+                }
+            }
+            if(card->has(Constants::AFFINITYARTIFACTS)||
+                card->has(Constants::AFFINITYFOREST)||
+                card->has(Constants::AFFINITYGREENCREATURES)||
+                card->has(Constants::AFFINITYISLAND)||
+                card->has(Constants::AFFINITYMOUNTAIN)||
+                card->has(Constants::AFFINITYPLAINS)||
+                card->has(Constants::AFFINITYSWAMP)){
+                    if (card->has(Constants::AFFINITYARTIFACTS))
+                    {
+                        type = "artifact";
+                    }
+                    else if (card->has(Constants::AFFINITYSWAMP))
+                    {
+                        type = "swamp";
+                    }
+                    else if (card->has(Constants::AFFINITYMOUNTAIN))
+                    {
+                        type = "mountain";
+                    }
+                    else if (card->has(Constants::AFFINITYPLAINS))
+                    {
+                        type = "plains";
+                    }
+                    else if (card->has(Constants::AFFINITYISLAND))
+                    {
+                        type = "island";
+                    }
+                    else if (card->has(Constants::AFFINITYFOREST))
+                    {
+                        type = "forest";
+                    }
+                    else if (card->has(Constants::AFFINITYGREENCREATURES))
+                    {
+                        color = 1;
+                        type = "creature";
+                    }
+                    card->getManaCost()->copy(original);
+                    int reduce = 0;
+                    if(card->has(Constants::AFFINITYGREENCREATURES))
+                    {
+                        TargetChooserFactory tf;
+                        TargetChooser * tc = tf.createTargetChooser("creature[green]",NULL);
+                        reduce = card->controller()->game->battlefield->countByCanTarget(tc);
+                        SAFE_DELETE(tc);
+                    }
+                    else
+                    {
+                        reduce = card->controller()->game->battlefield->countByType(type.c_str());
+                    }
+                    for(int i = 0; i < reduce;i++)
+                    {
+                        if(card->getManaCost()->getCost(color) > 0)
+                            card->getManaCost()->remove(color,1);
+                    }
+            }
+            SAFE_DELETE(original);
+        }
+    }
+}
 void GameObserver::Render()
 {
     mLayers->Render();
@@ -769,6 +885,8 @@ int GameObserver::cardClick(MTGCardInstance * card, Targetable * object)
             else
             {
                 result = targetChooser->toggleTarget(card);
+                WEvent * e = NEW WEventTarget(card,cardWaitingForTargets);
+                receiveEvent(e);
             }
         }
         else
@@ -796,7 +914,7 @@ int GameObserver::cardClick(MTGCardInstance * card, Targetable * object)
     {
         //card played as normal, alternative cost, buyback, flashback, retrace.
 
-        //the varible "paymenttype = int" only serves one purpose, to tell this bug fix what menu item you clicked on...
+        //the variable "paymenttype = int" only serves one purpose, to tell this bug fix what menu item you clicked on...
         // all alternative cost or play methods suffered from the fix because if the card contained "target=" 
         // it would automatically force the play method to putinplayrule...even charge you the original mana cost.
 
@@ -826,8 +944,11 @@ int GameObserver::cardClick(MTGCardInstance * card, Targetable * object)
 
     //Current player's hand
     if (currentPlayer->game->hand->hasCard(card) && currentGamePhase == Constants::MTG_PHASE_CLEANUP
-            && currentPlayer->game->hand->nb_cards > 7 && currentPlayer->nomaxhandsize == false)
+		&& currentPlayer->game->hand->nb_cards > currentPlayer->handsize && currentPlayer->nomaxhandsize == false)
     {
+        WEvent * e = NEW WEventCardDiscard(currentPlayer->game->hand->cards[0]);
+        GameObserver * game = GameObserver::GetInstance();
+        game->receiveEvent(e);
         currentPlayer->game->putInGraveyard(card);
     }
     else if (reaction)
@@ -883,10 +1004,27 @@ int GameObserver::isInPlay(MTGCardInstance * card)
     }
     return 0;
 }
-
-void GameObserver::draw()
+int GameObserver::isInGrave(MTGCardInstance * card)
 {
-    currentPlayer->game->drawFromLibrary();
+
+    for (int i = 0; i < 2; i++)
+    {
+        MTGGameZone * graveyard = players[i]->game->graveyard;
+        if (players[i]->game->isInZone(card,graveyard)) 
+            return 1;
+    }
+    return 0;
+}
+int GameObserver::isInExile(MTGCardInstance * card)
+{
+
+    for (int i = 0; i < 2; i++)
+    {
+        MTGGameZone * exile = players[i]->game->exile;
+        if (players[i]->game->isInZone(card,exile)) 
+            return 1;
+    }
+    return 0;
 }
 
 void GameObserver::cleanupPhase()
