@@ -2,7 +2,6 @@
 
 #include "utils.h"
 #include "MTGDeck.h"
-#include "GameOptions.h"
 #include "Translate.h"
 #include "OptionItem.h"
 #include "StyleManager.h"
@@ -28,26 +27,27 @@ const string Options::optionNames[] = {
   "disable_cards",
   "maxGrade",
   "ASPhases",
+  "FirstPlayer",
+  "KickerPay",
   "economic_difficulty",
   "transitions",
   "bgStyle",
   "interruptSeconds",
-#if defined(QT_CONFIG)
+#if defined(SDL_CONFIG)
+  "keybindings_sdl",
+#elif defined(QT_CONFIG)
   "keybindings_qt",
-#else
-#if defined(WIN32)
+#elif defined(WIN32)
   "keybindings_win",
-#else
-#if defined(LINUX)
+#elif defined(LINUX)
   "keybindings_x",
 #else
   "keybindings_psp",
-#endif
-#endif
 #endif
   "aidecks",
   "interruptMySpells",
   "interruptMyAbilities",
+  "saveDetailedDeckInfo",
 //General interrupts
   "interruptBeforeBegin",
   "interruptUntap",
@@ -66,6 +66,8 @@ const string Options::optionNames[] = {
 //Unlocked modes
   "prx_handler",
   "prx_rimom",
+  "prx_rewehenots",
+  "prx_timreh",
   "prx_eviltwin",
   "prx_rnddeck",
   "aw_collector",
@@ -89,10 +91,9 @@ int Options::getID(string name)
     }
 
     //Is it an unlocked set?
-    size_t un = strlen("unlocked_");
-    if (un < name.size())
+    if (name.find("unlocked_") == 0)
     {
-        string setname = name.substr(un);
+        string setname = name.substr(strlen("unlocked_"));
         if (setlist.size())
         {
             int unlocked = setlist[setname];
@@ -193,7 +194,7 @@ GameOption::GameOption(int value) :
 {
 }
 GameOption::GameOption(string value) :
-    number(0), str(value)
+    number(atoi(value.c_str())), str(value)
 {
 }
 GameOption::GameOption(int num, string str) :
@@ -316,7 +317,7 @@ GameOptions::GameOptions(string filename)
 
 int GameOptions::load()
 {
-    std::ifstream file(mFilename.c_str());
+    wagic::ifstream file(mFilename.c_str());
     std::string s;
 
     if (file)
@@ -333,7 +334,7 @@ int GameOptions::load()
             int id = Options::getID(name);
             if (id == INVALID_OPTION)
             {
-                unknown.push_back(s);
+                if (!unknownMap[name]) unknownMap[name] = NEW GameOption(val);
                 continue;
             }
 
@@ -383,8 +384,16 @@ int GameOptions::save()
             opt->write(&file, name);
         }
 
-        for (vector<string>::size_type t = 0; t < unknown.size(); t++)
-            file << unknown[t] << "\n";
+        for (map<string, GameOption *>::iterator it = unknownMap.begin(); it != unknownMap.end(); it++)
+        {
+            if (it->second)
+            {
+                if (it->second->str.size())
+                    file << it->first << "=" <<  it->second->str << "\n";
+                else if (it->second->number)
+                    file << it->first << "=" <<  it->second->number << "\n";
+            }
+        }
         file.close();
     }
     return 1;
@@ -399,6 +408,26 @@ GameOption& GameOptions::operator[](int optionID)
     return *go;
 }
 
+GameOption& GameOptions::operator[](string optionName)
+{
+    int id = Options::getID(optionName);
+    if (id != INVALID_OPTION)
+        return operator[](id);
+
+    GameOption * go =  get(optionName);
+
+    return * go;
+
+}
+
+GameOption * GameOptions::get(string optionName)
+{
+   if (!unknownMap[optionName])
+        unknownMap[optionName] = NEW GameOption(0);
+
+    return unknownMap[optionName];
+}
+
 GameOption * GameOptions::get(int optionID)
 {
     //Invalid options!
@@ -441,6 +470,16 @@ GameOption * GameOptions::get(int optionID)
             goEnum->def = OptionASkipPhase::getInstance();
             go = goEnum;
             break;
+        case Options::FIRSTPLAYER:
+            goEnum = NEW GameOptionEnum();
+            goEnum->def = OptionWhosFirst::getInstance();
+            go = goEnum;
+            break;
+        case Options::KICKERPAYMENT:
+            goEnum = NEW GameOptionEnum();
+            goEnum->def = OptionKicker::getInstance();
+            go = goEnum;
+            break;
         case Options::KEY_BINDINGS:
             go = NEW GameOptionKeyBindings();
             break;
@@ -468,6 +507,10 @@ GameOptions::~GameOptions()
     for (vector<GameOption*>::iterator it = values.begin(); it != values.end(); it++)
         SAFE_DELETE(*it);
     values.clear();
+
+    for (map<string, GameOption*>::iterator it = unknownMap.begin(); it != unknownMap.end(); it++)
+        SAFE_DELETE(it->second);
+    unknownMap.clear();
 }
 
 GameSettings options;
@@ -483,8 +526,10 @@ GameSettings::GameSettings()
 
 WStyle * GameSettings::getStyle()
 {
-    if (!styleMan)
-        styleMan = new StyleManager();
+    if (!styleMan){
+          styleMan = new StyleManager();
+          styleMan->determineActive(NULL,NULL);
+    }
     return styleMan->get();
 }
 
@@ -502,7 +547,7 @@ void GameSettings::automaticStyle(Player * p1, Player * p2)
     MTGDeck * decks[2];
     for (int i = 0; i < 2; i++)
     {
-        decks[i] = new MTGDeck(GameApp::collection);
+        decks[i] = new MTGDeck(MTGCollection());
         Player * p;
         if (i == 0)
             p = p1;
@@ -556,9 +601,27 @@ GameOption& GameSettings::operator[](int optionID)
     return *go;
 }
 
+GameOption& GameSettings::operator[](string optionName)
+{
+    int id = Options::getID(optionName);
+    if (id != INVALID_OPTION)
+        return operator[](id);
+
+    if (!profileOptions)
+        return invalid_option;
+    
+    GameOption * go =  profileOptions->get(optionName);
+
+    assert(go);
+
+    return *go;
+}
+
 GameOption* GameSettings::get(int optionID)
 {
+#ifdef DEBUG
     string option_name = Options::getName(optionID);
+#endif
 
     if (optionID < 0)
         return &invalid_option;
@@ -668,7 +731,7 @@ void GameSettings::checkProfile()
     }
 
     //Validation of collection, etc, only happens if the game is up.
-    if (theGame == NULL || theGame->collection == NULL)
+    if (theGame == NULL || MTGCollection() == NULL)
         return;
 
     string pcFile = profileFile(PLAYER_COLLECTION, "", false);
@@ -698,7 +761,7 @@ void GameSettings::checkProfile()
     int ok = 0;
     for (int i = 0; i < setlist.size(); i++)
     {
-        int value = theGame->collection->countBySet(i);
+        int value = MTGCollection()->countBySet(i);
         if (value > maxcards)
         {
             maxcards = value;
@@ -725,10 +788,10 @@ void GameSettings::checkProfile()
 void GameSettings::createUsersFirstDeck(int setId)
 {
 
-    if (theGame == NULL || theGame->collection == NULL)
+    if (theGame == NULL || MTGCollection() == NULL)
         return;
 
-    MTGDeck *mCollection = NEW MTGDeck(options.profileFile(PLAYER_COLLECTION, "", false).c_str(), theGame->collection);
+    MTGDeck *mCollection = NEW MTGDeck(options.profileFile(PLAYER_COLLECTION, "", false).c_str(), MTGCollection());
     if (mCollection->totalCards() > 0)
         return;
 
@@ -760,12 +823,14 @@ void GameSettings::createUsersFirstDeck(int setId)
     mCollection->save();
     SAFE_DELETE(mCollection);
 }
+
 void GameSettings::keypadTitle(string set)
 {
     if (keypad != NULL)
         keypad->title = set;
 }
-SimplePad * GameSettings::keypadStart(string input, string * _dest, bool _cancel, bool _numpad, int _x, int _y)
+
+SimplePad * GameSettings::keypadStart(string input, string * _dest, bool _cancel, bool _numpad, float _x, float _y)
 {
     if (keypad == NULL)
         keypad = NEW SimplePad();
@@ -870,6 +935,16 @@ OptionASkipPhase::OptionASkipPhase()
     mDef.values.push_back(EnumDefinition::assoc(Constants::ASKIP_FULL, "Full"));
 }
 ;
+
+OptionWhosFirst OptionWhosFirst::mDef;
+OptionWhosFirst::OptionWhosFirst()
+{
+    mDef.values.push_back(EnumDefinition::assoc(Constants::WHO_P, "Player"));
+    mDef.values.push_back(EnumDefinition::assoc(Constants::WHO_O, "Opponent"));
+    mDef.values.push_back(EnumDefinition::assoc(Constants::WHO_R, "Random"));
+}
+;
+
 OptionClosedHand OptionClosedHand::mDef;
 OptionClosedHand::OptionClosedHand()
 {
@@ -920,7 +995,13 @@ OptionEconDifficulty::OptionEconDifficulty()
     mDef.values.push_back(EnumDefinition::assoc(Constants::ECON_EASY, "Easy"));
 }
 ;
-
+OptionKicker OptionKicker::mDef;
+OptionKicker::OptionKicker()
+{
+    mDef.values.push_back(EnumDefinition::assoc(Constants::KICKER_ALWAYS, "Always Pay"));
+    mDef.values.push_back(EnumDefinition::assoc(Constants::KICKER_CHOICE, "Offer Choice"));
+}
+;
 //GameOptionAward
 GameOptionAward::GameOptionAward()
 {
@@ -931,7 +1012,7 @@ GameOptionAward::GameOptionAward()
 bool GameOptionAward::read(string input)
 {
     //This is quick and dirty.
-    bool bNumeric = true;
+
     achieved = time(NULL);
     tm * at = localtime(&achieved);
     viewed = false;
@@ -1040,7 +1121,7 @@ string GameOptionAward::menuStr()
         return _("Unlocked.");
 
     char buf[256];
-    Translator * t = Translator::GetInstance();
+
     tm * lt = localtime(&achieved);
     if (!lt)
         return "Error";
