@@ -6,6 +6,7 @@
 #include "SimpleMenu.h"
 #include "DeckStats.h"
 #include "DeckMetaData.h"
+#include "DeckManager.h"
 #include "Player.h"
 
 // The purpose of this method is to create a listing of decks to be used for the input menu
@@ -16,8 +17,8 @@
 vector<DeckMetaData *> GameState::fillDeckMenu(SimpleMenu * _menu, const string& path, const string& smallDeckPrefix,
                 Player * statsPlayer)
 {
-    bool translate = _menu->autoTranslate;
-    vector<DeckMetaData *> deckMetaDataVector = getValidDeckMetaData(path, smallDeckPrefix, statsPlayer);
+
+    vector<DeckMetaData *> deckMetaDataVector = BuildDeckList(path, smallDeckPrefix, statsPlayer);
     renderDeckMenu(_menu, deckMetaDataVector);
 
     return deckMetaDataVector;
@@ -26,46 +27,68 @@ vector<DeckMetaData *> GameState::fillDeckMenu(SimpleMenu * _menu, const string&
 vector<DeckMetaData *> GameState::fillDeckMenu(DeckMenu * _menu, const string& path, const string& smallDeckPrefix,
                 Player * statsPlayer, int maxDecks)
 {
-    bool translate = _menu->mAutoTranslate;
-    vector<DeckMetaData *> deckMetaDataVector = getValidDeckMetaData(path, smallDeckPrefix, statsPlayer, maxDecks);
+
+    vector<DeckMetaData *> deckMetaDataVector = BuildDeckList(path, smallDeckPrefix, statsPlayer, maxDecks);
     renderDeckMenu(_menu, deckMetaDataVector);
 
     return deckMetaDataVector;
 }
 
-vector<DeckMetaData *> GameState::getValidDeckMetaData(const string& path, const string& smallDeckPrefix, Player * statsPlayer, int maxDecks)
+vector<DeckMetaData *> GameState::BuildDeckList(const string& path, const string& smallDeckPrefix, Player * statsPlayer, int maxDecks)
 {
     vector<DeckMetaData*> retList;
 
-    DeckMetaDataList * metas = DeckMetaDataList::decksMetaData;
     int found = 1;
     int nbDecks = 1;
+    DeckManager *deckManager = DeckManager::GetInstance();
+    bool isAI = path.find("baka") != string::npos;
     while (found && (!maxDecks || nbDecks <= maxDecks))
     {
         found = 0;
         std::ostringstream filename;
         filename << path << "/deck" << nbDecks << ".txt";
-        DeckMetaData * meta = metas->get(filename.str(), statsPlayer);
-		string deckStatsFileName;
+        DeckMetaData * meta = deckManager->getDeckMetaDataByFilename(filename.str(), isAI);
+
         if (meta)
         {
-            found = 1;
-            if (statsPlayer)
+            //Check if the deck is unlocked based on sets etc...
+            bool unlocked = true;
+            vector<int> unlockRequirements = meta->getUnlockRequirements();
+            for (size_t i = 0; i < unlockRequirements.size(); ++i)
             {
-                std::ostringstream aiStatsDeckName;
-                aiStatsDeckName << smallDeckPrefix << "_deck" << nbDecks;
-				deckStatsFileName = aiStatsDeckName.str();
-            }
-            else
-            {
-                std::ostringstream playerStatsDeckName;
-                playerStatsDeckName << "stats/player_deck" << nbDecks << ".txt";
-                deckStatsFileName = options.profileFile(playerStatsDeckName.str());
+                    if (! options[unlockRequirements[i]].number)
+                    {
+                        unlocked = false;
+                        break;
+                    }
             }
 
-			meta->loadStatsForPlayer(statsPlayer, deckStatsFileName);
-            retList.push_back(meta);
-            nbDecks++;
+            if (unlocked)
+            {
+                found = 1;
+                if (statsPlayer)
+                {
+                    std::ostringstream aiStatsDeckName;
+                    aiStatsDeckName << smallDeckPrefix << "_deck" << nbDecks;
+				    meta->mStatsFilename = aiStatsDeckName.str();
+                    meta->mIsAI = true;
+                    if (meta->mPlayerDeck != statsPlayer->GetCurrentDeckStatsFile())
+                    {
+                        meta->mPlayerDeck = statsPlayer->GetCurrentDeckStatsFile();
+                        meta->Invalidate();
+                    }
+                }
+                else
+                {
+                    std::ostringstream playerStatsDeckName;
+                    playerStatsDeckName << "stats/player_deck" << nbDecks << ".txt";
+                    meta->mStatsFilename = options.profileFile(playerStatsDeckName.str());
+                    meta->mIsAI = false;
+                }
+
+                retList.push_back(meta);
+                nbDecks++;
+            }
         }
         meta = NULL;
     }
