@@ -9,7 +9,6 @@
 #include "MTGDeck.h"
 #include "Translate.h"
 #include "OptionItem.h"
-#include "GameOptions.h"
 #include "DeckDataWrapper.h"
 
 enum ENUM_AWARDS_STATE
@@ -72,6 +71,14 @@ void GameStateAwards::Start()
     btn = NEW WGuiButton(aw, -103, Options::MOMIR_MODE_UNLOCKED, this);
     listview->Add(btn);
 
+	aw = NEW WGuiAward(Options::STONEHEWER_MODE_UNLOCKED, "Stone Hewer Mode", "Won with more than 10 equipments.");
+	btn = NEW WGuiButton(aw, -103, Options::STONEHEWER_MODE_UNLOCKED, this);
+	listview->Add(btn);
+
+	aw = NEW WGuiAward(Options::HERMIT_MODE_UNLOCKED, "Hermit Druid Mode", "Won with less than 10 lands.");
+	btn = NEW WGuiButton(aw, -103, Options::HERMIT_MODE_UNLOCKED, this);
+	listview->Add(btn);
+
     aw = NEW WGuiAward(Options::EVILTWIN_MODE_UNLOCKED, "Evil Twin Mode", "Won with same army size.");
     btn = NEW WGuiButton(aw, -103, Options::EVILTWIN_MODE_UNLOCKED, this);
     listview->Add(btn);
@@ -137,9 +144,9 @@ void GameStateAwards::Render()
     JRenderer * r = JRenderer::GetInstance();
     r->ClearScreen(ARGB(0,0,0,0));
 
-    JQuad * mBg = WResourceManager::Instance()->RetrieveTempQuad("awardback.jpg", TEXTURE_SUB_5551);
-    if (mBg)
-        r->RenderQuad(mBg, 0, 0);
+    JQuadPtr background = WResourceManager::Instance()->RetrieveTempQuad("awardback.jpg", TEXTURE_SUB_5551);
+    if (background.get())
+        r->RenderQuad(background.get(), 0, 0);
 
     switch (mState)
     {
@@ -168,8 +175,9 @@ void GameStateAwards::Update(float dt)
     }
     else
     {
-        JButton key;
-        while ((key = JGE::GetInstance()->ReadButton()))
+        JButton key = JGE_BTN_NONE;
+        int x, y;
+        while ((key = JGE::GetInstance()->ReadButton()) || JGE::GetInstance()->GetLeftClickCoordinates(x,y))
         {
             switch (key)
             {
@@ -226,7 +234,7 @@ bool GameStateAwards::enterSet(int setid)
 
     setSrc = NEW WSrcCards();
     setSrc->addFilter(NEW WCFilterSet(setid));
-    setSrc->loadMatches(mParent->collection);
+    setSrc->loadMatches(MTGCollection());
     setSrc->bakeFilters();
     setSrc->Sort(WSrcCards::SORT_COLLECTOR);
 
@@ -255,7 +263,7 @@ bool GameStateAwards::enterStats(int option)
 {
     if (option != Options::AWARD_COLLECTOR)
         return false;
-    DeckDataWrapper* ddw = NEW DeckDataWrapper(NEW MTGDeck(options.profileFile(PLAYER_COLLECTION).c_str(), mParent->collection));
+    DeckDataWrapper* ddw = NEW DeckDataWrapper(NEW MTGDeck(options.profileFile(PLAYER_COLLECTION).c_str(), MTGCollection()));
     if (!ddw)
         return false;
 
@@ -360,7 +368,7 @@ void GameStateAwards::ButtonPressed(int controllerId, int controlId)
             SAFE_DELETE(detailview);
             showMenu = false;
             break;
-        case 3:
+        case kCancelMenuID:
             showMenu = false;
             break;
         }
