@@ -8,7 +8,6 @@
 #include <math.h>
 #include <iomanip>
 
-#include "DeckManager.h"
 #include "GameStateDuel.h"
 #include "GameStateDeckViewer.h"
 #include "Translate.h"
@@ -16,10 +15,14 @@
 #include "MTGCardInstance.h"
 #include "WFilter.h"
 #include "WDataSrc.h"
+#include "DeckManager.h"
+#include "DeckMetaData.h"
 #include "DeckEditorMenu.h"
 #include "SimpleMenu.h"
 #include "utils.h"
 
+
+
 //!! helper function; this is probably handled somewhere in the code already.
 // If not, should be placed in general library
 void StringExplode(string str, string separator, vector<string>* results)
@@ -39,8 +42,6 @@ GameStateDeckViewer::GameStateDeckViewer(GameApp* parent) :
     GameState(parent)
 {
     bgMusic = NULL;
-    nbDecks = 0;
-    deckNum = 0;
     useFilter = 0;
     isAIDeckSave = false;
     mSwitching = false;
@@ -71,7 +72,6 @@ GameStateDeckViewer::~GameStateDeckViewer()
         SAFE_DELETE(myCollection->parent);
         SAFE_DELETE(myCollection);
     }
-    SAFE_DELETE(stw);
     SAFE_DELETE(filterMenu);
 }
 
@@ -90,13 +90,14 @@ void GameStateDeckViewer::rotateCards(int direction)
         lastPos = getCurrentPos();
         lastTotal = total;
     }
-    int i = 0;
+
 }
+
 void GameStateDeckViewer::rebuildFilters()
 {
     if (!filterMenu) filterMenu = NEW WGuiFilters("Filter by...", NULL);
     if (source)
-    SAFE_DELETE(source);
+        SAFE_DELETE(source);
     source = NEW WSrcDeckViewer(myDeck, myCollection);
     filterMenu->setSrc(source);
     if (displayed_deck != myDeck) source->swapSrc();
@@ -106,6 +107,7 @@ void GameStateDeckViewer::rebuildFilters()
 	if (stw && myDeck) 
 		stw->updateStats( myDeck );;
 }
+
 void GameStateDeckViewer::updateFilters()
 {
     if (!displayed_deck) return;
@@ -136,6 +138,7 @@ void GameStateDeckViewer::updateFilters()
     stw->updateStats( myDeck );;
     return;
 }
+
 void GameStateDeckViewer::loadIndexes()
 {
     for (int i = 0; i < 7; i++)
@@ -166,9 +169,7 @@ void GameStateDeckViewer::updateDecks()
     DeckManager * deckManager = DeckManager::GetInstance();
     vector<DeckMetaData *> playerDeckList = fillDeckMenu(welcome_menu, options.profileFile());
 
-    deckNum = 0;
     newDeckname = "";
-    nbDecks = playerDeckList.size() + 1;
     welcome_menu->Add(MENU_ITEM_NEW_DECK, "--NEW--");
     if (options[Options::CHEATMODE].number && (!myCollection || myCollection->getCount(WSrcDeck::UNFILTERED_MIN_COPIES) < 4)) welcome_menu->Add(
             MENU_ITEM_CHEAT_MODE, "--UNLOCK CARDS--");
@@ -186,7 +187,7 @@ void GameStateDeckViewer::buildEditorMenu()
     deckSummaryInformation << "All changes are final." << endl;
 
     if (menu)
-    SAFE_DELETE( menu );
+        SAFE_DELETE( menu );
     //Build menu.
     JRenderer::GetInstance()->FillRoundRect(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, 100, ARGB(0, 0, 0, 0) );
     menu = NEW DeckEditorMenu(MENU_DECK_BUILDER, this, Fonts::OPTION_FONT, "Deck Editor", myDeck, stw);
@@ -217,8 +218,8 @@ void GameStateDeckViewer::Start()
     lastPos = 0;
     lastTotal = 0;
 
-    pricelist = NEW PriceList(JGE_GET_RES("settings/prices.dat").c_str(), mParent->collection);
-    playerdata = NEW PlayerData(mParent->collection);
+    pricelist = NEW PriceList(JGE_GET_RES("settings/prices.dat").c_str(), MTGCollection());
+    playerdata = NEW PlayerData(MTGCollection());
     myCollection = NEW DeckDataWrapper(playerdata->collection);
     myCollection->Sort(WSrcCards::SORT_ALPHA);
     displayed_deck = myCollection;
@@ -237,7 +238,7 @@ void GameStateDeckViewer::Start()
     }
 
     //Grab a texture in VRAM.
-    pspIconsTexture = WResourceManager::Instance()->RetrieveTexture("iconspsp.png", RETRIEVE_LOCK);
+    pspIconsTexture = WResourceManager::Instance()->RetrieveTexture("iconspsp.png", RETRIEVE_MANAGE);
 
     char buf[512];
     for (int i = 0; i < 8; i++)
@@ -247,8 +248,6 @@ void GameStateDeckViewer::Start()
         pspIcons[i]->SetHotSpot(16, 16);
     }
 
-    backQuad = WResourceManager::Instance()->GetQuad("back");
-
     //init welcome menu
     updateDecks();
 
@@ -306,6 +305,11 @@ void GameStateDeckViewer::addRemove(MTGCard * card)
 
 void GameStateDeckViewer::saveDeck()
 {
+    //update the corresponding meta data object
+    DeckMetaData *metaData = DeckManager::GetInstance()->getDeckMetaDataById( myDeck->parent->meta_id, false );
+    if ( newDeckname.length() > 0 )
+        metaData->setDeckName( newDeckname );
+    mSwitching = true;
     myDeck->save();
     playerdata->save();
     pricelist->save();
@@ -316,8 +320,8 @@ void GameStateDeckViewer::saveDeck()
  */
 void GameStateDeckViewer::saveAsAIDeck(string deckName)
 {
-    DeckManager * deckManager = DeckManager::GetInstance();
-    vector<DeckMetaData *> aiDecks = GameState::getValidDeckMetaData(JGE_GET_RES("ai/baka"), "ai_baka", NULL);
+
+    vector<DeckMetaData *> aiDecks = GameState::BuildDeckList(JGE_GET_RES("ai/baka"), "ai_baka", NULL);
     int nbAiDecks = aiDecks.size() + 1;
     aiDecks.clear();
 
@@ -340,8 +344,6 @@ void GameStateDeckViewer::saveAsAIDeck(string deckName)
 void GameStateDeckViewer::Update(float dt)
 {
 
-    int myD = (displayed_deck == myDeck);
-
     int x, y;
     unsigned int distance2;
     unsigned int minDistance2 = -1;
@@ -366,6 +368,7 @@ void GameStateDeckViewer::Update(float dt)
                 {
                     myDeck->parent->meta_name = newDeckname;
                     saveDeck();
+                    updateDecks();
                 }
                 mStage = STAGE_WAITING;
             }
@@ -453,7 +456,7 @@ void GameStateDeckViewer::Update(float dt)
             {
                 filterMenu = NEW WGuiFilters("Filter by...", NULL);
                 if (source)
-                SAFE_DELETE(source);
+                    SAFE_DELETE(source);
                 source = NEW WSrcDeckViewer(myDeck, myCollection);
                 filterMenu->setSrc(source);
                 if (displayed_deck != myDeck) source->swapSrc();
@@ -475,7 +478,7 @@ void GameStateDeckViewer::Update(float dt)
             {
               for(int i=0; i < CARDS_DISPLAYED; i++)
               {
-                distance2 = (cardsCoordinates[i].second - y) * (cardsCoordinates[i].second - y) + (cardsCoordinates[i].first - x) * (cardsCoordinates[i].first - x);
+                distance2 = static_cast<unsigned int>((cardsCoordinates[i].second - y) * (cardsCoordinates[i].second - y) + (cardsCoordinates[i].first - x) * (cardsCoordinates[i].first - x));
                 if (distance2 < minDistance2)
                 {
                     minDistance2 = distance2;
@@ -619,7 +622,6 @@ void GameStateDeckViewer::renderOnScreenBasicInfo()
     JRenderer *renderer = JRenderer::GetInstance();
     WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT);
     char buffer[256];
-    int myD = (displayed_deck == myDeck);
 
     float y = 0;
     int allCopies, nowCopies;
@@ -638,7 +640,7 @@ void GameStateDeckViewer::renderOnScreenBasicInfo()
     renderer->FillRoundRect(SCREEN_WIDTH - (w + 27), y + 5, w + 10, 15, 5, ARGB(128,0,0,0));
 
     mFont->DrawString(buffer, SCREEN_WIDTH - 22, y + 15, JGETEXT_RIGHT);
-    if (useFilter != 0) renderer->RenderQuad(mIcons[useFilter - 1], SCREEN_WIDTH - 10, y + 15, 0.0f, 0.5, 0.5);
+    if (useFilter != 0) renderer->RenderQuad(mIcons[useFilter - 1].get(), SCREEN_WIDTH - 10, y + 15, 0.0f, 0.5, 0.5);
 }
 
 //returns position of the current card (cusor) in the currently viewed color/filter
@@ -748,10 +750,10 @@ void GameStateDeckViewer::renderOnScreenMenu()
         //LEFT PSP CIRCLE render
         r->FillCircle(leftPspX, leftPspY, 40, ARGB(128,50,50,50));
 
-        r->RenderQuad(pspIcons[0], leftPspX, leftPspY - 20, 0, pspIconsSize, pspIconsSize);
-        r->RenderQuad(pspIcons[1], leftPspX, leftPspY + 20, 0, pspIconsSize, pspIconsSize);
-        r->RenderQuad(pspIcons[2], leftPspX - 20, leftPspY, 0, pspIconsSize, pspIconsSize);
-        r->RenderQuad(pspIcons[3], leftPspX + 20, leftPspY, 0, pspIconsSize, pspIconsSize);
+        r->RenderQuad(pspIcons[0].get(), leftPspX, leftPspY - 20, 0, pspIconsSize, pspIconsSize);
+        r->RenderQuad(pspIcons[1].get(), leftPspX, leftPspY + 20, 0, pspIconsSize, pspIconsSize);
+        r->RenderQuad(pspIcons[2].get(), leftPspX - 20, leftPspY, 0, pspIconsSize, pspIconsSize);
+        r->RenderQuad(pspIcons[3].get(), leftPspX + 20, leftPspY, 0, pspIconsSize, pspIconsSize);
 
         font->DrawString(_("Prev."), leftPspX - 35, leftPspY - 15);
         font->DrawString(_("Next"), leftPspX + 15, leftPspY - 15);
@@ -762,10 +764,10 @@ void GameStateDeckViewer::renderOnScreenMenu()
 
         //RIGHT PSP CIRCLE render
         r->FillCircle(rightPspX + (onScreenTransition * 204), rightPspY, 40, ARGB(128,50,50,50));
-        r->RenderQuad(pspIcons[4], rightPspX + 20, rightPspY, 0, pspIconsSize, pspIconsSize);
-        r->RenderQuad(pspIcons[5], rightPspX, rightPspY - 20, 0, pspIconsSize, pspIconsSize);
-        r->RenderQuad(pspIcons[6], rightPspX - 20, rightPspY, 0, pspIconsSize, pspIconsSize);
-        r->RenderQuad(pspIcons[7], rightPspX, rightPspY + 20, 0, pspIconsSize, pspIconsSize);
+        r->RenderQuad(pspIcons[4].get(), rightPspX + 20, rightPspY, 0, pspIconsSize, pspIconsSize);
+        r->RenderQuad(pspIcons[5].get(), rightPspX, rightPspY - 20, 0, pspIconsSize, pspIconsSize);
+        r->RenderQuad(pspIcons[6].get(), rightPspX - 20, rightPspY, 0, pspIconsSize, pspIconsSize);
+        r->RenderQuad(pspIcons[7].get(), rightPspX, rightPspY + 20, 0, pspIconsSize, pspIconsSize);
 
         font->DrawString(_("Toggle Images"), rightPspX - 35, rightPspY - 40);
 
@@ -794,8 +796,8 @@ void GameStateDeckViewer::renderOnScreenMenu()
             {
                 sprintf(buffer, "%i", value);
                 font->DrawString(buffer, SCREEN_WIDTH - 190 + rightTransition + nb_letters * 13, SCREEN_HEIGHT / 2 + 40);
-                r->RenderQuad(mIcons[j], SCREEN_WIDTH - 197 + rightTransition + nb_letters * 13, SCREEN_HEIGHT / 2 + 46, 0, 0.5,
-                        0.5);
+                r->RenderQuad(mIcons[j].get(), SCREEN_WIDTH - 197 + rightTransition + nb_letters * 13, SCREEN_HEIGHT / 2 + 46, 0, 0.5,
+                                0.5);
                 if (value > 9)
                 {
                     nb_letters += 3;
@@ -868,7 +870,7 @@ void GameStateDeckViewer::renderOnScreenMenu()
                 {
                     sprintf(buffer, "%i", value);
                     font->DrawString(buffer, 38 + nb_letters * 13 + leftTransition, posY + 5);
-                    r->RenderQuad(mIcons[j], 30 + nb_letters * 13 + leftTransition, posY + 11, 0, 0.5, 0.5);
+                    r->RenderQuad(mIcons[j].get(), 30 + nb_letters * 13 + leftTransition, posY + 11, 0, 0.5, 0.5);
                     if (value > 9)
                     {
                         nb_letters += 3;
@@ -981,7 +983,7 @@ void GameStateDeckViewer::renderOnScreenMenu()
             // Column titles
             for (int j = 0; j < Constants::MTG_NB_COLORS - 1; j++)
             {
-                r->RenderQuad(mIcons[j], 52 + j * 15 + leftTransition, posY - 10, 0, 0.5, 0.5);
+                r->RenderQuad(mIcons[j].get(), 52 + j * 15 + leftTransition, posY - 10, 0, 0.5, 0.5);
             }
 
             //font->DrawString(_("C"), 30 + leftTransition, posY-16);
@@ -1065,7 +1067,7 @@ void GameStateDeckViewer::renderOnScreenMenu()
                     posX = 72;
                     for (int j = 0; j < stw->countLandsPerColor[i] + stw->countBasicLandsPerColor[i]; j++)
                     {
-                        r->RenderQuad(mIcons[i], posX + leftTransition, posY + 6, 0, 0.5, 0.5);
+                        r->RenderQuad(mIcons[i].get(), posX + leftTransition, posY + 6, 0, 0.5, 0.5);
                         posX += ((j + 1) % 10 == 0) ? 17 : 13;
                         if ((((j + 1) % 30) == 0) && (j < stw->countLandsPerColor[i] + stw->countBasicLandsPerColor[i] - 1))
                         {
@@ -1127,7 +1129,7 @@ void GameStateDeckViewer::renderOnScreenMenu()
             // Column titles
             for (int j = 0; j < Constants::MTG_NB_COLORS - 1; j++)
             {
-                r->RenderQuad(mIcons[j], 67 + j * 15 + leftTransition, posY - 10, 0, 0.5, 0.5);
+                r->RenderQuad(mIcons[j].get(), 67 + j * 15 + leftTransition, posY - 10, 0, 0.5, 0.5);
             }
 
             font->DrawString(_("C"), 30 + leftTransition, posY - 16);
@@ -1239,7 +1241,7 @@ void GameStateDeckViewer::renderOnScreenMenu()
                     posX = 72;
                     for (int j = 0; j < stw->totalCostPerColor[i]; j++)
                     {
-                        r->RenderQuad(mIcons[i], posX + leftTransition, posY + 6, 0, 0.5, 0.5);
+                        r->RenderQuad(mIcons[i].get(), posX + leftTransition, posY + 6, 0, 0.5, 0.5);
                         posX += ((j + 1) % 10 == 0) ? 17 : 13;
                         if ((((j + 1) % 30) == 0) && (j < stw->totalCostPerColor[i] - 1))
                         {
@@ -1311,53 +1313,59 @@ void GameStateDeckViewer::renderCard(int id, float rotation)
     int alpha = (int) (255 * (scale + 1.0 - max_scale));
 
     if (!card) return;
-    JQuad * quad = NULL;
-
-    int cacheError = CACHE_ERROR_NONE;
-
-    if (!options[Options::DISABLECARDS].number)
+    
+    if (!WResourceManager::Instance()->IsThreaded())
     {
-        quad = WResourceManager::Instance()->RetrieveCard(card, RETRIEVE_EXISTING);
-        cacheError = WResourceManager::Instance()->RetrieveError();
-        if (!quad && cacheError != CACHE_ERROR_404)
+        JQuadPtr backQuad = WResourceManager::Instance()->GetQuad(kGenericCardID);
+        JQuadPtr quad;
+
+        int cacheError = CACHE_ERROR_NONE;
+
+        if (!options[Options::DISABLECARDS].number)
         {
-            if (last_user_activity > (abs(2 - id) + 1) * NO_USER_ACTIVITY_SHOWCARD_DELAY)
-                quad = WResourceManager::Instance()->RetrieveCard(card);
-            else
+            quad = WResourceManager::Instance()->RetrieveCard(card, RETRIEVE_EXISTING);
+            cacheError = WResourceManager::Instance()->RetrieveError();
+            if (!quad.get() && cacheError != CACHE_ERROR_404)
             {
-                quad = backQuad;
+                if (last_user_activity > (abs(2 - id) + 1) * NO_USER_ACTIVITY_SHOWCARD_DELAY)
+                    quad = WResourceManager::Instance()->RetrieveCard(card);
+                else
+                {
+                    quad = backQuad;
+                }
             }
         }
-    }
 
-    int quadAlpha = alpha;
-    if (!displayed_deck->count(card)) quadAlpha /= 2;
-    if (quad)
-    {
-        if (quad == backQuad)
+        if (quad.get())
         {
-            quad->SetColor(ARGB(255,255,255,255));
-            float _scale = scale * (285 / quad->mHeight);
-            JRenderer::GetInstance()->RenderQuad(quad, x, y, 0.0f, _scale, _scale);
+            if (quad == backQuad)
+            {
+                quad->SetColor(ARGB(255,255,255,255));
+                float _scale = scale * (285 / quad->mHeight);
+                JRenderer::GetInstance()->RenderQuad(quad.get(), x, y, 0.0f, _scale, _scale);
+            }
+            else
+            {
+                Pos pos = Pos(x, y, scale * 285 / 250, 0.0, 255);
+                CardGui::DrawCard(card, pos);
+            }
         }
         else
         {
             Pos pos = Pos(x, y, scale * 285 / 250, 0.0, 255);
-            CardGui::DrawCard(card, pos);
+            CardGui::DrawCard(card, pos, DrawMode::kText);
         }
     }
     else
     {
+        int mode = !options[Options::DISABLECARDS].number ? DrawMode::kNormal : DrawMode::kText;
+
         Pos pos = Pos(x, y, scale * 285 / 250, 0.0, 255);
-        CardGui::DrawCard(card, pos, DrawMode::kText);
-        if (!options[Options::DISABLECARDS].number) quad = WResourceManager::Instance()->RetrieveCard(card, CACHE_THUMB);
-        if (quad)
-        {
-            float _scale = 285 * scale / quad->mHeight;
-            quad->SetColor(ARGB(40,255,255,255));
-            JRenderer::GetInstance()->RenderQuad(quad, x, y, 0, _scale, _scale);
-        }
+        CardGui::DrawCard(card, pos, mode);
     }
+
+    int quadAlpha = alpha;
+    if (!displayed_deck->count(card)) quadAlpha /= 2;
     quadAlpha = 255 - quadAlpha;
     if (quadAlpha > 0)
     {
@@ -1390,9 +1398,8 @@ void GameStateDeckViewer::Render()
 {
 
     WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT);
-    JRenderer * r = JRenderer::GetInstance();
 
-    r->ClearScreen(ARGB(0,0,0,0));
+    JRenderer::GetInstance()->ClearScreen(ARGB(0,0,0,0));
     if (displayed_deck == myDeck && mStage != STAGE_MENU) 
 		renderDeckBackground();
     int order[3] = { 1, 2, 3 };
@@ -1407,6 +1414,19 @@ void GameStateDeckViewer::Render()
         order[2] = 1;
     }
 
+    // even though we want to draw the cards in a particular z order for layering, we want to prefetch them 
+    // in a different order, ie the center card should appear first, then the adjacent ones
+    if (WResourceManager::Instance()->IsThreaded())
+    {
+        WResourceManager::Instance()->RetrieveCard(cardIndex[0]);
+        WResourceManager::Instance()->RetrieveCard(cardIndex[3]);
+        WResourceManager::Instance()->RetrieveCard(cardIndex[4]);
+        WResourceManager::Instance()->RetrieveCard(cardIndex[2]);
+        WResourceManager::Instance()->RetrieveCard(cardIndex[5]);
+        WResourceManager::Instance()->RetrieveCard(cardIndex[1]);
+        WResourceManager::Instance()->RetrieveCard(cardIndex[6]);
+    }
+
     renderCard(6, mRotation);
     renderCard(5, mRotation);
     renderCard(4, mRotation);
@@ -1453,13 +1473,17 @@ void GameStateDeckViewer::Render()
 int GameStateDeckViewer::loadDeck(int deckid)
 {
 
-    if (!stw) stw = new StatsWrapper(deckid);
-
+    if (!stw) 
+    {
+        DeckManager *deckManager = DeckManager::GetInstance();
+        stw = deckManager->getExtendedStatsForDeckId( deckid, MTGCollection(), false );
+    }
+    
     stw->currentPage = 0;
     stw->pageCount = 9;
     stw->needUpdate = true;
 
-    if (!playerdata) playerdata = NEW PlayerData(mParent->collection);
+    if (!playerdata) playerdata = NEW PlayerData(MTGCollection());
     SAFE_DELETE(myCollection);
     myCollection = NEW DeckDataWrapper(playerdata->collection);
     myCollection->Sort(WSrcCards::SORT_ALPHA);
@@ -1472,7 +1496,7 @@ int GameStateDeckViewer::loadDeck(int deckid)
         SAFE_DELETE(myDeck->parent);
         SAFE_DELETE(myDeck);
     }
-    myDeck = NEW DeckDataWrapper(NEW MTGDeck(options.profileFile(deckname, "", false, false).c_str(), mParent->collection));
+    myDeck = NEW DeckDataWrapper(NEW MTGDeck(options.profileFile(deckname, "", false, false).c_str(), MTGCollection()));
 
     // Check whether the cards in the deck are actually available in the player's collection:
     int cheatmode = options[Options::CHEATMODE].number;
@@ -1567,7 +1591,6 @@ void GameStateDeckViewer::ButtonPressed(int controllerId, int controlId)
 
         loadDeck(deckIdNumber);
         mStage = STAGE_WAITING;
-        deckNum = controlId;
         break;
 
     case MENU_DECK_BUILDER: //Save / exit menu
@@ -1602,7 +1625,6 @@ void GameStateDeckViewer::ButtonPressed(int controllerId, int controlId)
             break;
 
         case MENU_ITEM_SWITCH_DECKS_NO_SAVE:
-            //updateDecks();
             mStage = STAGE_WELCOME;
             mSwitching = true;
             break;
@@ -1646,3 +1668,12 @@ void GameStateDeckViewer::ButtonPressed(int controllerId, int controlId)
     }
 }
 
+void GameStateDeckViewer::OnScroll(int inXVelocity, int inYVelocity)
+{
+    if (abs(inYVelocity) > 300)
+    {
+        bool flickUpwards = (inYVelocity < 0);
+        mEngine->HoldKey_NoRepeat(flickUpwards ? JGE_BTN_DOWN : JGE_BTN_UP);
+
+    }
+}
