@@ -2,7 +2,6 @@
 
 #include "DeckMenu.h"
 #include "GameStateDuel.h"
-#include "GameOptions.h"
 #include "utils.h"
 #include "AIPlayer.h"
 #include "AIMomirPlayer.h"
@@ -15,6 +14,7 @@
 #include "Credits.h"
 #include "Translate.h"
 #include "Rules.h"
+#include "ModRules.h"
 
 #ifdef TESTSUITE
 #include "TestSuiteAI.h"
@@ -82,7 +82,6 @@ GameState(parent)
 #endif
 
     credits = NULL;
-    rules = NULL;
 
 #ifdef NETWORK_SUPPORT
     RegisterNetworkPlayers();
@@ -116,6 +115,7 @@ void GameStateDuel::Start()
     {
         if (mParent->players[i] == PLAYER_TYPE_HUMAN)
         {
+            //DeckManager::EndInstance();
             decksneeded = 1;
 
             deckmenu = NEW DeckMenu(DUEL_MENU_CHOOSE_DECK, this, Fonts::OPTION_FONT, "Choose a Deck",
@@ -126,10 +126,10 @@ void GameStateDuel::Start()
             int nbDecks = playerDeckList.size();
 
             if (nbDecks)
-            {
                 decksneeded = 0;
+            
+             if (nbDecks > 1)   
                 deckmenu->Add(MENUITEM_RANDOM_PLAYER, "Random", "Play with a random deck.");
-            }
 
             renderDeckMenu(deckmenu, playerDeckList);
             // save the changes to the player deck list maintained in DeckManager
@@ -144,17 +144,21 @@ void GameStateDuel::Start()
     {
         if (decksneeded)
         {
-            //translate deck creating desc
-            Translator * t = Translator::GetInstance();
-            map<string, string>::iterator it = t->deckValues.find("Create your Deck!");
-            if (it != t->deckValues.end())
-                deckmenu->Add(MENUITEM_NEW_DECK, "Create your Deck!", it->second);
-            else
-                deckmenu->Add(MENUITEM_NEW_DECK, "Create your Deck!", "Highly recommended to get\nthe full Wagic experience!");
+            if (gModRules.general.hasDeckEditor())
+            {
+                //translate deck creating desc
+                Translator * t = Translator::GetInstance();
+                string desc =  "Highly recommended to get\nthe full Wagic experience!";
+                map<string, string>::iterator it = t->deckValues.find("Create your Deck!");
+                if (it != t->deckValues.end())
+                    desc = it->second;
+
+                deckmenu->Add(MENUITEM_NEW_DECK, "Create your Deck!", desc);
+            }
             premadeDeck = true;
             fillDeckMenu(deckmenu, JGE_GET_RES("player/premade"));
         }
-        if (!decksneeded)
+        else if (gModRules.general.hasDeckEditor())
         {
             deckmenu->Add(MENUITEM_NEW_DECK, "New Deck...", "Create a new deck to play with.");
         }
@@ -215,7 +219,8 @@ void GameStateDuel::loadPlayer(int playerId, int decknb, bool isAI, bool isNetwo
 
 void GameStateDuel::initRand(unsigned int seed)
 {
-    if (!seed) seed = time(0);
+    if (!seed)
+        seed = static_cast<unsigned int>(time(0));
     srand(seed);
 }
 
@@ -233,7 +238,7 @@ void GameStateDuel::loadTestSuitePlayers()
 
     GameObserver::Init(mPlayers, 2);
     game = GameObserver::GetInstance();
-    game->startGame(rules);
+    game->startGame(mParent->rules);
     if (mParent->gameType == GAME_TYPE_MOMIR)
     {
         game->addObserver(NEW MTGMomirRule(-1, MTGCollection()));
@@ -245,6 +250,10 @@ void GameStateDuel::End()
 {
     DebugTrace("Ending GameStateDuel");
 
+#ifdef TRACK_OBJECT_USAGE
+    ObjectAnalytics::DumpStatistics();
+#endif
+
     JRenderer::GetInstance()->EnableVSync(false);
     if (!premadeDeck && mPlayers[0] && mPlayers[1])
     { // save the stats for the game
@@ -263,7 +272,6 @@ void GameStateDuel::End()
         mPlayers[i] = NULL;
     }
     SAFE_DELETE(credits);
-    SAFE_DELETE(rules);
 
     SAFE_DELETE(menu);
     SAFE_DELETE(opponentMenu);
@@ -322,19 +330,8 @@ void GameStateDuel::Update(float dt)
         break;
 
     case DUEL_STATE_CHOOSE_DECK1:
-        if (mParent->gameType == GAME_TYPE_MOMIR)
+        if (!mParent->rules->canChooseDeck())
         {
-            rules = NEW Rules("momir.txt");
-            mGamePhase = DUEL_STATE_PLAY;
-        }
-        else if (mParent->gameType == GAME_TYPE_RANDOM1)
-        {
-            rules = NEW Rules("random1.txt");
-            mGamePhase = DUEL_STATE_PLAY;
-        }
-        else if (mParent->gameType == GAME_TYPE_RANDOM2)
-        {
-            rules = NEW Rules("random2.txt");
             mGamePhase = DUEL_STATE_PLAY;
         }
 #ifdef TESTSUITE
@@ -342,7 +339,6 @@ void GameStateDuel::Update(float dt)
         {
             if (testSuite && testSuite->loadNext())
             {
-                rules = NEW Rules("testsuite.txt");
                 loadTestSuitePlayers();
                 mGamePhase = DUEL_STATE_PLAY;
                 testSuite->pregameTests();
@@ -363,7 +359,6 @@ void GameStateDuel::Update(float dt)
 #endif
         else
         {
-            if (!rules) rules = NEW Rules("mtg.txt");
             if (mParent->players[0] == PLAYER_TYPE_HUMAN)
             {
                 if (!popupScreen || popupScreen->isClosed()) deckmenu->Update(dt);
@@ -420,11 +415,19 @@ void GameStateDuel::Update(float dt)
         {
             GameObserver::Init(mPlayers, 2);
             game = GameObserver::GetInstance();
-            game->startGame(rules);
+            game->startGame(mParent->rules);
             if (mParent->gameType == GAME_TYPE_MOMIR)
             {
                 game->addObserver(NEW MTGMomirRule(-1, MTGCollection()));
             }
+			if (mParent->gameType == GAME_TYPE_STONEHEWER)
+			{
+				game->addObserver(NEW MTGStoneHewerRule(-1,MTGCollection()));
+			}
+			if (mParent->gameType == GAME_TYPE_HERMIT)
+			{
+				game->addObserver(NEW MTGHermitRule(-1));
+			}
 
             //start of in game music code
             musictrack = "";
@@ -447,6 +450,9 @@ void GameStateDuel::Update(float dt)
             GameApp::playMusic(musictrack);
         }
         game->Update(dt);
+        //run a "post update" init call in the rules. This is for things such as Manapool, which gets emptied in the update
+        // That's mostly because of a legacy bug, where we use the update sequence for some things when we should use events (such as phase changes)
+        mParent->rules->postUpdateInit();
         if (game->gameOver)
         {
             if (game->players[1]->playMode != Player::MODE_TEST_SUITE) credits->compute(game->players[0], game->players[1], mParent);
@@ -477,12 +483,12 @@ void GameStateDuel::Update(float dt)
             {
                 menu = NEW SimpleMenu(DUEL_MENU_GAME_MENU, this, Fonts::MENU_FONT, SCREEN_WIDTH / 2 - 100, 25,
                     game->players[1]->deckName.c_str());
-                int cardsinhand = game->players[0]->game->hand->nb_cards;
+                int cardsinhand = game->currentPlayer->game->hand->nb_cards;
 
                 //almosthumane - mulligan
                 if ((game->turn < 1) && (cardsinhand != 0) && game->currentGamePhase == Constants::MTG_PHASE_FIRSTMAIN
-                    && game->players[0]->game->inPlay->nb_cards == 0 && game->players[0]->game->graveyard->nb_cards == 0
-                    && game->players[0]->game->exile->nb_cards == 0) //1st Play Check
+                    && game->currentPlayer->game->inPlay->nb_cards == 0 && game->currentPlayer->game->graveyard->nb_cards == 0
+					&& game->currentPlayer->game->exile->nb_cards == 0 && game->currentlyActing() == (Player*)game->currentPlayer) //1st Play Check
                     //IF there was no play at the moment automatically mulligan
                 {
                     menu->Add(MENUITEM_MULLIGAN, "Mulligan");
@@ -561,7 +567,6 @@ void GameStateDuel::Render()
     {
     case DUEL_STATE_END:
         {
-            JRenderer * r = JRenderer::GetInstance();
             r->FillRect(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, ARGB(200,0,0,0));
             credits->Render();
 #ifdef TESTSUITE
@@ -615,7 +620,8 @@ void GameStateDuel::Render()
 #ifdef NETWORK_SUPPORT
                 && mParent->gameType != GAME_TYPE_SLAVE
 #endif //NETWORK_SUPPORT
-                )
+				&& mParent->gameType != GAME_TYPE_STONEHEWER
+				&& mParent->gameType != GAME_TYPE_HERMIT)
             mFont->DrawString(_("LOADING DECKS").c_str(), 0, SCREEN_HEIGHT / 2);
         else
         {
@@ -753,7 +759,7 @@ void GameStateDuel::ButtonPressed(int controllerId, int controlId)
         if (controlId == MENUITEM_RANDOM_PLAYER) // Random Player Deck Selection
         {
             vector<DeckMetaData *> * playerDeckList = deckManager->getPlayerDeckOrderList();
-            deckNumber = playerDeckList->at(WRand() * 1001 % (playerDeckList->size()))->getDeckId();
+            deckNumber = playerDeckList->at(WRand() % (playerDeckList->size()))->getDeckId();
             loadPlayer(0, deckNumber);
             deckmenu->Close();
             mGamePhase = DUEL_STATE_CHOOSE_DECK2_TO_PLAY;
@@ -849,3 +855,13 @@ void GameStateDuel::ButtonPressed(int controllerId, int controlId)
 
     }
 }
+
+void GameStateDuel::OnScroll(int inXVelocity, int inYVelocity)
+{
+    if (abs(inYVelocity) > 300)
+    {
+        bool flickUpwards = (inYVelocity < 0);
+        mEngine->HoldKey_NoRepeat(flickUpwards ? JGE_BTN_PREV : JGE_BTN_SEC);
+    }
+}
+
