@@ -10,7 +10,6 @@
 #include "GameStateMenu.h"
 #include "DeckManager.h"
 #include "MenuItem.h"
-#include "GameOptions.h"
 #include "GameApp.h"
 #include "MTGCard.h"
 #include "Translate.h"
@@ -19,13 +18,14 @@
 #include "utils.h"
 #include "WFont.h"
 #include <JLogger.h>
+#include "Rules.h"
+#include "ModRules.h"
 
-static const char* GAME_VERSION = "WTH?! 0.14.1 - wololo.net";
+#ifdef NETWORK_SUPPORT
+#include <JNetwork.h>
+#endif//NETWORK_SUPPORT
 
-#define DEFAULT_ANGLE_MULTIPLIER 0.4f
-#define MAX_ANGLE_MULTIPLIER (3*M_PI)
-#define MIN_ANGLE_MULTIPLIER 0.4f
-static const float STEP_ANGLE_MULTIPLIER = 0.0002f;
+static const char* GAME_VERSION = "WTH?! 0.16.0 - wololo.net";
 
 enum ENUM_MENU_STATE_MAJOR
 {
@@ -36,7 +36,10 @@ enum ENUM_MENU_STATE_MAJOR
     MENU_STATE_MAJOR_FIRST_TIME = 0x05,
     MENU_STATE_MAJOR_DUEL = 0x06,
     MENU_STATE_MAJOR_LANG = 0x07,
-
+#ifdef NETWORK_SUPPORT
+    MENU_STATE_NETWORK_DEFINE = 0x08,
+    MENU_STATE_NETWORK_WAIT = 0x09,
+#endif //NETWORK_SUPPORT
     MENU_STATE_MAJOR = 0xFF
 };
 
@@ -48,37 +51,14 @@ enum ENUM_MENU_STATE_MINOR
     MENU_STATE_MINOR = 0xF00
 };
 
-enum
-{
-    SUBMENUITEM_CANCEL = kCancelMenuID,
-    MENUITEM_PLAY,
-    MENUITEM_DECKEDITOR,
-    MENUITEM_SHOP,
-    MENUITEM_OPTIONS,
-    MENUITEM_EXIT,
-    SUBMENUITEM_1PLAYER,
-    SUBMENUITEM_2PLAYER,
-    SUBMENUITEM_DEMO,
-    SUBMENUITEM_TESTSUITE,
-    SUBMENUITEM_MOMIR,
-    SUBMENUITEM_CLASSIC,
-    SUBMENUITEM_RANDOM1,
-    SUBMENUITEM_RANDOM2,
-    SUBMENUITEM_STORY,
-};
-
 GameStateMenu::GameStateMenu(GameApp* parent) :
     GameState(parent)
 {
     mGuiController = NULL;
     subMenuController = NULL;
     gameTypeMenu = NULL;
-    mSplash = NULL;
-    mBg = NULL;
     //bgMusic = NULL;
     timeIndex = 0;
-    angleMultiplier = MIN_ANGLE_MULTIPLIER;
-    yW = 55;
     mVolume = 0;
     scroller = NULL;
     langChoices = false;
@@ -129,7 +109,6 @@ void GameStateMenu::Create()
     scrollerSet = 0;
 
     splashTex = NULL;
-    mSplash = NULL;
 
 }
 
@@ -151,13 +130,13 @@ void GameStateMenu::Start()
 
     GameApp::playMusic("Track0.mp3");
 
-    hasChosenGameType = 0;
+    hasChosenGameType = false;
     mParent->gameType = GAME_TYPE_CLASSIC;
 
-    /*
-     if (options[Options::MOMIR_MODE_UNLOCKED].number) hasChosenGameType = 0;
-     if (options[Options::RANDOMDECK_MODE_UNLOCKED].number) hasChosenGameType = 0;
-     */
+    //Manual clean up of some cache Data. Ideally those should clean themselves up, so this is kind of a hack for now
+    JFileSystem::GetInstance()->clearZipCache();
+    WResourceManager::Instance()->ClearUnlocked();
+
     bgTexture = WResourceManager::Instance()->RetrieveTexture("menutitle.png", RETRIEVE_LOCK);
     mBg = WResourceManager::Instance()->RetrieveQuad("menutitle.png", 0, 0, 256, 166); // Create background quad for rendering.
 
@@ -173,14 +152,26 @@ void GameStateMenu::Start()
 void GameStateMenu::genNbCardsStr()
 {
     //How many cards total ?
-    PlayerData * playerdata = NEW PlayerData(mParent->collection);
-    if (playerdata && !options[Options::ACTIVE_PROFILE].isDefault())
-        sprintf(nbcardsStr, _("%s: %i cards (%i) (%i unique)").c_str(), options[Options::ACTIVE_PROFILE].str.c_str(),
-                        playerdata->collection->totalCards(), mParent->collection->totalCards(),
-                        mParent->collection->primitives.size());
+    PlayerData * playerdata = NEW PlayerData(MTGCollection());
+    size_t totalUnique =  MTGCollection()->primitives.size();
+    size_t totalPrints = MTGCollection()->totalCards();
+
+    if (totalUnique != totalPrints)
+    {
+        if (playerdata && !options[Options::ACTIVE_PROFILE].isDefault())
+            sprintf(nbcardsStr, _("%s: %i cards (%i) (%i unique)").c_str(), options[Options::ACTIVE_PROFILE].str.c_str(),
+                            playerdata->collection->totalCards(), totalPrints,totalUnique);
+        else
+            sprintf(nbcardsStr, _("%i cards (%i unique)").c_str(),totalPrints,totalUnique);
+    }
     else
-        sprintf(nbcardsStr, _("%i cards (%i unique)").c_str(), mParent->collection->totalCards(),
-                        mParent->collection->primitives.size());
+    {
+        if (playerdata && !options[Options::ACTIVE_PROFILE].isDefault())
+            sprintf(nbcardsStr, _("%s: %i cards (%i)").c_str(), options[Options::ACTIVE_PROFILE].str.c_str(),
+                            playerdata->collection->totalCards(), totalPrints);
+        else
+            sprintf(nbcardsStr, _("%i cards").c_str(),totalPrints);
+    }
 
     SAFE_DELETE(playerdata);
 }
@@ -192,12 +183,13 @@ void GameStateMenu::fillScroller()
     char buff2[512];
 
     DeckStats * stats = DeckStats::GetInstance();
-	DeckManager *deckManager = DeckManager::GetInstance();
-	vector<DeckMetaData *> playerDecks = getValidDeckMetaData(options.profileFile(), "", NULL, 6);
+	vector<DeckMetaData *> playerDecks = BuildDeckList(options.profileFile(), "", NULL, 6);
     int totalGames = 0;
 	for (size_t j = 0; j < playerDecks.size(); j++)
     {
 		DeckMetaData* meta = playerDecks[j];
+        if (meta)
+            meta->LoadStats();
 		sprintf(buffer, "stats/player_deck%i.txt", meta->getDeckId());
         string deckstats = options.profileFile(buffer);
         if (fileExists(deckstats.c_str()))
@@ -222,6 +214,8 @@ void GameStateMenu::fillScroller()
         scroller->Add(_("Unlock the difficult mode for more challenging duels!"));
     if (!options[Options::MOMIR_MODE_UNLOCKED].number)
         scroller->Add(_("Interested in playing Momir Basic? You'll have to unlock it first :)"));
+	if (!options[Options::STONEHEWER_MODE_UNLOCKED].number)
+		scroller->Add(_("Love Equipment and want a real challenge? Unlock Stone Hewer Basic:)"));
     if (!options[Options::RANDOMDECK_MODE_UNLOCKED].number)
         scroller->Add(_("You haven't unlocked the random deck mode yet"));
     if (!options[Options::EVILTWIN_MODE_UNLOCKED].number)
@@ -241,19 +235,23 @@ void GameStateMenu::fillScroller()
     sprintf(buff2, _("You have unlocked %i expansions out of %i").c_str(), nbunlocked, setlist.size());
     scroller->Add(buff2);
 
-    PlayerData * playerdata = NEW PlayerData(mParent->collection);
-    int totalCards = playerdata->collection->totalCards();
-    if (totalCards)
+    PlayerData * playerdata = NEW PlayerData(MTGCollection());
+
+    if (gModRules.general.hasDeckEditor() && gModRules.general.hasShop())
     {
-        sprintf(buff2, _("You have a total of %i cards in your collection").c_str(), totalCards);
-        scroller->Add(buff2);
+        int totalCards = playerdata->collection->totalCards();
+        if (totalCards)
+        {
+            sprintf(buff2, _("You have a total of %i cards in your collection").c_str(), totalCards);
+            scroller->Add(buff2);
 
-        int estimatedValue = playerdata->collection->totalPrice();
-        sprintf(buff2, _("The shopkeeper would buy your entire collection for around %i credits").c_str(), estimatedValue / 2);
-        scroller->Add(buff2);
+            int estimatedValue = playerdata->collection->totalPrice();
+            sprintf(buff2, _("The shopkeeper would buy your entire collection for around %i credits").c_str(), estimatedValue / 2);
+            scroller->Add(buff2);
 
-        sprintf(buff2, _("The cards in your collection have an average value of %i credits").c_str(), estimatedValue / totalCards);
-        scroller->Add(buff2);
+            sprintf(buff2, _("The cards in your collection have an average value of %i credits").c_str(), estimatedValue / totalCards);
+            scroller->Add(buff2);
+        }
     }
 
     sprintf(buff2, _("You currently have %i credits").c_str(), playerdata->credits);
@@ -286,7 +284,7 @@ int GameStateMenu::nextDirectory(const char * root, const char * file)
     while (!found && (mDit = readdir(mDip)))
     {
         sprintf(mCurrentSetFileName, "%s/%s/%s", root, mDit->d_name, file);
-        std::ifstream file(mCurrentSetFileName);
+        wagic::ifstream file(mCurrentSetFileName);
         if (file)
         {
             sprintf(mCurrentSetName, "%s", mDit->d_name);
@@ -313,7 +311,7 @@ string GameStateMenu::loadRandomWallpaper()
         return wallpaper;
 
     vector<string> wallpapers;
-    std::ifstream file(JGE_GET_RES("graphics/wallpapers.txt").c_str());
+    wagic::ifstream file(JGE_GET_RES("graphics/wallpapers.txt").c_str());
 
     if (!file)
         return wallpaper;
@@ -371,7 +369,7 @@ void GameStateMenu::loadLangMenu()
     {
         string filename = JGE_GET_RES("lang/");
         filename += mDit->d_name;
-        std::ifstream file(filename.c_str());
+        wagic::ifstream file(filename.c_str());
         string s;
         string lang;
         if (file)
@@ -403,11 +401,18 @@ void GameStateMenu::listPrimitives()
         mDip = opendir(JGE_GET_RES("sets/primitives/").c_str());
     }
 
+    if (!mDip)
+    {
+        DebugTrace("GameStateMenu.cpp:WARNING:Primitives folder is missing");
+        primitivesLoadCounter = 0;
+        return;
+    }
+
     while ((mDit = readdir(mDip)))
     {
         string filename = JGE_GET_RES("sets/primitives/");
         filename += mDit->d_name;
-        std::ifstream file(filename.c_str());
+        wagic::ifstream file(filename.c_str());
         if (!file)
             continue;
         file.close();
@@ -427,16 +432,28 @@ void GameStateMenu::ensureMGuiController()
         {
             WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::MENU_FONT);
             mFont->SetColor(ARGB(255,255,255,255));
-            mGuiController->Add(NEW MenuItem(MENUITEM_PLAY, mFont, "Play", 80, 50 + SCREEN_HEIGHT / 2, mIcons[8], mIcons[9],
-                            "particle1.psi", WResourceManager::Instance()->GetQuad("particles"), true));
-            mGuiController->Add(NEW MenuItem(MENUITEM_DECKEDITOR, mFont, "Deck Editor", 160, 50 + SCREEN_HEIGHT / 2, mIcons[2],
-                            mIcons[3], "particle2.psi", WResourceManager::Instance()->GetQuad("particles")));
-            mGuiController->Add(NEW MenuItem(MENUITEM_SHOP, mFont, "Shop", 240, 50 + SCREEN_HEIGHT / 2, mIcons[0], mIcons[1],
-                            "particle3.psi", WResourceManager::Instance()->GetQuad("particles")));
-            mGuiController->Add(NEW MenuItem(MENUITEM_OPTIONS, mFont, "Options", 320, 50 + SCREEN_HEIGHT / 2, mIcons[6], mIcons[7],
-                            "particle4.psi", WResourceManager::Instance()->GetQuad("particles")));
-            mGuiController->Add(NEW MenuItem(MENUITEM_EXIT, mFont, "Exit", 400, 50 + SCREEN_HEIGHT / 2, mIcons[4], mIcons[5],
-                            "particle5.psi", WResourceManager::Instance()->GetQuad("particles")));
+            vector<ModRulesMainMenuItem *>items = gModRules.menu.main;
+
+            int numItems = (int)items.size();
+            float startX = 80.f;
+            float totalSize = SCREEN_WIDTH_F - (2 * startX);
+            float space = 0;
+            if (numItems < 2)
+                startX = SCREEN_WIDTH_F/2;
+            else 
+                space = totalSize/(numItems - 1);
+
+            for (size_t i = 0; i < items.size(); ++i) {
+                ModRulesMainMenuItem * item = items[i];
+                int iconId = (item->mIconId - 1) * 2;
+                mGuiController->Add(NEW MenuItem(
+                    item->mActionId, 
+                    mFont, item->mDisplayName, 
+                    startX + (i * space), 50 + SCREEN_HEIGHT / 2, 
+                    mIcons[iconId].get(), mIcons[iconId + 1].get(),
+                    item->mParticleFile.c_str(), WResourceManager::Instance()->GetQuad("particles").get(),
+                    (i == 0)));
+            }
         }
     }
 }
@@ -464,18 +481,26 @@ void GameStateMenu::Update(float dt)
         if (primitivesLoadCounter == -1)
         {
             listPrimitives();
-            Translator::GetInstance()->init();
         }
         if (primitivesLoadCounter < (int) (primitives.size()))
         {
-            mParent->collection->load(primitives[primitivesLoadCounter].c_str());
+#ifdef _DEBUG
+            int startTime = JGEGetTime();
+#endif
+            MTGCollection()->load(primitives[primitivesLoadCounter].c_str());
+#if _DEBUG
+            int endTime = JGEGetTime();
+            int elapsedTime = (endTime - startTime);
+            DebugTrace("Time elapsed while loading " << primitives[primitivesLoadCounter] << " : " << elapsedTime << " ms");
+#endif
+
             primitivesLoadCounter++;
             break;
         }
         primitivesLoadCounter = primitives.size() + 1;
         if (mReadConf)
         {
-            mParent->collection->load(mCurrentSetFileName, mCurrentSetName);
+            MTGCollection()->load(mCurrentSetFileName, mCurrentSetName);
         }
         else
         {
@@ -487,8 +512,8 @@ void GameStateMenu::Update(float dt)
             Translator::GetInstance()->tempValues.clear();
 
             DebugTrace(std::endl << "==" << std::endl <<
-                            "Total MTGCards: " << mParent->collection->collection.size() << std::endl <<
-                            "Total CardPrimitives: " << mParent->collection->primitives.size() << std::endl << "==");
+                            "Total MTGCards: " << MTGCollection()->collection.size() << std::endl <<
+                            "Total CardPrimitives: " << MTGCollection()->primitives.size() << std::endl << "==");
 
             //Force default, if necessary.
             if (options[Options::ACTIVE_PROFILE].str == "")
@@ -497,10 +522,9 @@ void GameStateMenu::Update(float dt)
             //Release splash texture
             WResourceManager::Instance()->Release(splashTex);
             splashTex = NULL;
-            mSplash = NULL;
 
             //check for deleted collection / first-timer
-            std::ifstream file(options.profileFile(PLAYER_COLLECTION).c_str());
+            wagic::ifstream file(options.profileFile(PLAYER_COLLECTION).c_str());
             if (file)
             {
                 file.close();
@@ -523,15 +547,61 @@ void GameStateMenu::Update(float dt)
         currentState &= MENU_STATE_MAJOR_MAINMENU;
         options.reloadProfile(); //Handles building a new deck, if needed.
         break;
-    case MENU_STATE_MAJOR_MAINMENU:
-        if (!scrollerSet)
-            fillScroller();
-        ensureMGuiController();
-        if (mGuiController)
-            mGuiController->Update(dt);
-        if (mEngine->GetButtonState(JGE_BTN_NEXT)) //Hook for GameStateAward state
-            mParent->DoTransition(TRANSITION_FADE, GAME_STATE_AWARDS); //TODO: A slide transition would be nice.
+    case MENU_STATE_MAJOR_MAINMENU: 
+        {
+            if (!scrollerSet)
+                fillScroller();
+            ensureMGuiController();
+            if (mGuiController)
+                mGuiController->Update(dt);
+
+            //Hook for Top Menu actions
+            vector<ModRulesOtherMenuItem *>items = gModRules.menu.other;
+            for (size_t i = 0; i < items.size(); ++i)
+            {
+                if (mEngine->GetButtonState(items[i]->mKey) && items[i]->getMatchingGameState())
+                     mParent->DoTransition(TRANSITION_FADE, items[i]->getMatchingGameState()); //TODO: Add the transition as a parameter in the rules file
+            }
+            break;
+         }
+#ifdef NETWORK_SUPPORT
+    case MENU_STATE_NETWORK_DEFINE:
+        currentState = MENU_STATE_MAJOR_SUBMENU;
+        subMenuController = NEW SimpleMenu(MENU_FIRST_DUEL_SUBMENU, this, Fonts::MENU_FONT, 150, 60);
+        if (subMenuController)
+        {
+            subMenuController->Add(SUBMENUITEM_HOST_GAME, "Host a game");
+            subMenuController->Add(SUBMENUITEM_JOIN_GAME, "Join a game");
+            subMenuController->Add(SUBMENUITEM_CANCEL, "Cancel");
+        }
+    break;
+    case MENU_STATE_NETWORK_WAIT:
+        if(MENU_STATE_MINOR_NONE == (currentState & MENU_STATE_MINOR))
+        {
+            if(mParent->mpNetwork->isConnected())
+            {
+                if(subMenuController) subMenuController->Close();
+                currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
+            }
+            else if(!subMenuController)
+            {
+//                currentState = MENU_STATE_MAJOR_SUBMENU;
+                subMenuController = NEW SimpleMenu(MENU_FIRST_DUEL_SUBMENU, this, Fonts::MENU_FONT, 150, 60);
+                if (subMenuController)
+                {
+                    subMenuController->Add(SUBMENUITEM_CANCEL, "Cancel connection");
+                }
+            }
+            else{
+                if (subMenuController)
+                    subMenuController->Update(dt);
+                ensureMGuiController();
+                mGuiController->Update(dt);
+                break;
+            }
+        }
         break;
+#endif //NETWORK_SUPPORT
     case MENU_STATE_MAJOR_SUBMENU:
         if (subMenuController)
             subMenuController->Update(dt);
@@ -547,15 +617,14 @@ void GameStateMenu::Update(float dt)
                 subMenuController = NEW SimpleMenu(MENU_FIRST_DUEL_SUBMENU, this, Fonts::MENU_FONT, 150, 60);
                 if (subMenuController)
                 {
-                    subMenuController->Add(SUBMENUITEM_CLASSIC, "Classic");
-                    if (options[Options::MOMIR_MODE_UNLOCKED].number)
-                        subMenuController->Add(SUBMENUITEM_MOMIR, "Momir Basic");
-                    if (options[Options::RANDOMDECK_MODE_UNLOCKED].number)
+                    for (size_t i = 0; i < Rules::RulesList.size(); ++i)
                     {
-                        subMenuController->Add(SUBMENUITEM_RANDOM1, "Random 1 Color");
-                        subMenuController->Add(SUBMENUITEM_RANDOM2, "Random 2 Colors");
+                        Rules * rules = Rules::RulesList[i];
+                        if (!rules->hidden && (rules->unlockOption == INVALID_OPTION || options[rules->unlockOption].number))
+                        {
+                            subMenuController->Add(SUBMENUITEM_END_OFFSET + i, rules->displayName.c_str());
+                        }
                     }
-                    subMenuController->Add(SUBMENUITEM_STORY, "Story");
                     subMenuController->Add(SUBMENUITEM_CANCEL, "Cancel");
                 }
             }
@@ -590,38 +659,6 @@ void GameStateMenu::Update(float dt)
         ;// Nothing to do.
     }
 
-    if (mEngine->GetButtonState(JGE_BTN_PREV))
-    {
-        //Reset deck of cards
-        angleMultiplier = MIN_ANGLE_MULTIPLIER;
-        yW = 55;
-    }
-
-    if (yW <= 55)
-    {
-        if (mEngine->GetButtonState(JGE_BTN_PRI))
-            angleMultiplier += STEP_ANGLE_MULTIPLIER;
-        else
-            angleMultiplier *= 0.9999f;
-        if (angleMultiplier > MAX_ANGLE_MULTIPLIER)
-            angleMultiplier = MAX_ANGLE_MULTIPLIER;
-        else if (angleMultiplier < MIN_ANGLE_MULTIPLIER)
-            angleMultiplier = MIN_ANGLE_MULTIPLIER;
-
-        if (mEngine->GetButtonState(JGE_BTN_CANCEL) && (dt != 0))
-        {
-            angleMultiplier = (cos(timeIndex) * angleMultiplier - M_PI / 3.0f - 0.1f - angleW) / dt;
-            yW = yW + 5 * dt + (yW - 45) * 5 * dt;
-        }
-        else
-            angleW = cos(timeIndex) * angleMultiplier - M_PI / 3.0f - 0.1f;
-    }
-    else
-    {
-        angleW += angleMultiplier * dt;
-        yW = yW + 5 * dt + (yW - 55) * 5 * dt;
-    }
-
     scroller->Update(dt);
     if ((currentState & MENU_STATE_MINOR) == MENU_STATE_MINOR_FADEIN)
     {
@@ -630,6 +667,83 @@ void GameStateMenu::Update(float dt)
     }
 }
 
+//Renders the "sub" menu with shoulder button links
+void GameStateMenu::RenderTopMenu()
+{
+    float leftTextPos = 10;
+    float rightTextPos = SCREEN_WIDTH - 10;
+
+    vector<ModRulesOtherMenuItem *>items = gModRules.menu.other;
+    for (size_t i = 0; i < items.size(); ++i)
+    {
+        switch(items[i]->mKey)
+        {
+        case JGE_BTN_PREV:
+            leftTextPos += 64;
+            break;
+        case JGE_BTN_NEXT:
+            rightTextPos -= 64;
+            break;
+        default:
+            DebugTrace("not supported yet!");
+            break;
+        }
+    }
+
+    WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT);
+    mFont->SetScale(DEFAULT_MAIN_FONT_SCALE);
+    mFont->SetColor(ARGB(128,255,255,255));
+    mFont->DrawString(GAME_VERSION, rightTextPos, 5, JGETEXT_RIGHT);
+    mFont->DrawString(nbcardsStr, leftTextPos, 5);
+    mFont->SetScale(1.f);
+    mFont->SetColor(ARGB(255,255,255,255));
+
+    if (!items.size())
+        return;
+
+    JQuadPtr jq = WResourceManager::Instance()->RetrieveTempQuad("button_shoulder.png");
+    if (!jq.get())
+        return;
+
+    mFont = WResourceManager::Instance()->GetWFont(Fonts::OPTION_FONT);
+    float olds = mFont->GetScale();
+
+    for (size_t i = 0; i < items.size(); ++i)
+    {
+        ModRulesOtherMenuItem * item = items[i];
+
+        int alpha = 255;
+        if (item->mActionId == MENUITEM_TROPHIES && options.newAward())
+            alpha = (int) (sin(timeIndex) * 255);
+        
+        float xPos = SCREEN_WIDTH - 64;
+        float xTextPos = xPos + 54;
+        int textAlign = JGETEXT_RIGHT;
+        jq->SetHFlip(false);
+
+        switch(item->mKey)
+        {
+        case JGE_BTN_PREV:
+            xPos = 5;
+            xTextPos = xPos + 10;
+            textAlign = JGETEXT_LEFT;
+            jq->SetHFlip(true);
+            break;
+        default:
+            break;
+        }
+
+        jq->SetColor(ARGB(abs(alpha),255,255,255));
+        mFont->SetColor(ARGB(abs(alpha),0,0,0));
+        string s = _(item->mDisplayName);
+        mFont->SetScale(1.0f);
+        mFont->SetScale(50.0f / mFont->GetStringWidth(s.c_str()));
+        JRenderer::GetInstance()->RenderQuad(jq.get(), xPos, 2);
+        mFont->DrawString(s, xTextPos, 9, textAlign);
+        mFont->SetScale(olds);
+    }
+}
+
 void GameStateMenu::Render()
 {
     if ((currentState & MENU_STATE_MINOR) == MENU_STATE_MINOR_FADEIN)
@@ -642,26 +756,17 @@ void GameStateMenu::Render()
     }
     else if ((currentState & MENU_STATE_MAJOR) == MENU_STATE_MAJOR_LOADING_CARDS)
     {
-        if (!splashTex)
+        string wp = loadRandomWallpaper();
+        if (wp.size())
         {
-            splashTex = WResourceManager::Instance()->RetrieveTexture("splash.jpg", RETRIEVE_LOCK);
-            mSplash = WResourceManager::Instance()->RetrieveTempQuad("splash.jpg");
-        }
-        if (mSplash)
-            renderer->RenderQuad(mSplash, 0, 0);
-        else
-        {
-            string wp = loadRandomWallpaper();
-            if (wp.size())
+            JTexture * wpTex = WResourceManager::Instance()->RetrieveTexture(wp);
+            if (wpTex)
             {
-                JTexture * wpTex = WResourceManager::Instance()->RetrieveTexture(wp);
-                if (wpTex)
-                {
-                    JQuad * wpQuad = WResourceManager::Instance()->RetrieveTempQuad(wp);
-                    renderer->RenderQuad(wpQuad, 0, 0, 0, SCREEN_WIDTH_F / wpQuad->mWidth, SCREEN_HEIGHT_F / wpQuad->mHeight);
-                }
+                JQuadPtr wpQuad = WResourceManager::Instance()->RetrieveTempQuad(wp);
+                renderer->RenderQuad(wpQuad.get(), 0, 0, 0, SCREEN_WIDTH_F / wpQuad->mWidth, SCREEN_HEIGHT_F / wpQuad->mHeight);
             }
         }
+
         char text[512];
         if (mCurrentSetName[0])
         {
@@ -690,38 +795,14 @@ void GameStateMenu::Render()
         if (mGuiController)
             mGuiController->Render();
 
-        mFont->SetScale(DEFAULT_MAIN_FONT_SCALE);
-        mFont->SetColor(ARGB(128,255,255,255));
-        mFont->DrawString(GAME_VERSION, SCREEN_WIDTH - 74, 5, JGETEXT_RIGHT);
-        mFont->DrawString(nbcardsStr, 10, 5);
-        mFont->SetScale(1.f);
-        mFont->SetColor(ARGB(255,255,255,255));
-
         renderer->FillRoundRect(SCREEN_WIDTH / 2 - 100, SCREEN_HEIGHT, 191, 6, 5, ARGB(100,10,5,0));
         scroller->Render();
 
-        if (mBg)
-            renderer->RenderQuad(mBg, SCREEN_WIDTH / 2, 50);
+        if (mBg.get())
+            renderer->RenderQuad(mBg.get(), SCREEN_WIDTH / 2, 50);
+
+        RenderTopMenu();
 
-        JQuad * jq = WResourceManager::Instance()->RetrieveTempQuad("button_shoulder.png");
-        if (jq)
-        {
-            int alp = 255;
-            if (options.newAward())
-                alp = (int) (sin(timeIndex) * 255);
-            float olds = mFont->GetScale();
-            mFont = WResourceManager::Instance()->GetWFont(Fonts::OPTION_FONT);
-            jq->SetColor(ARGB(abs(alp),255,255,255));
-            mFont->SetColor(ARGB(abs(alp),0,0,0));
-            string s = _("Trophy Room");
-            ;
-            mFont->SetScale(1.0f);
-            mFont->SetScale(50.0f / mFont->GetStringWidth(s.c_str()));
-            renderer->RenderQuad(jq, SCREEN_WIDTH - 64, 2);
-            mFont->DrawString(s, SCREEN_WIDTH - 10, 9, JGETEXT_RIGHT);
-            mFont = WResourceManager::Instance()->GetWFont(Fonts::MENU_FONT);
-            mFont->SetScale(olds);
-        }
     }
     if (subMenuController)
     {
@@ -731,12 +812,12 @@ void GameStateMenu::Render()
 
 void GameStateMenu::ButtonPressed(int controllerId, int controlId)
 {
-    WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::MENU_FONT);
-
     DebugTrace("GameStateMenu: controllerId " << controllerId << " selected");
     switch (controllerId)
     {
     case MENU_LANGUAGE_SELECTION:
+        if ( controlId == kInfoMenuID )
+            break;
         setLang(controlId);
         WResourceManager::Instance()->ReloadWFonts(); // Fix for choosing Chinese language at first time.
         subMenuController->Close();
@@ -756,24 +837,26 @@ void GameStateMenu::ButtonPressed(int controllerId, int controlId)
                 subMenuController->Add(SUBMENUITEM_1PLAYER, "1 Player");
                 // TODO Put 2 players mode back
                 // This requires to fix the hand (to accept 2 players) OR to implement network game
-                //subMenuController->Add(SUBMENUITEM_2PLAYER, "2 Players");
+#ifdef NETWORK_SUPPORT
+                subMenuController->Add(SUBMENUITEM_2PLAYERS, "2 Players");
+#endif //NETWORK_SUPPORT
                 subMenuController->Add(SUBMENUITEM_DEMO, "Demo");
                 subMenuController->Add(SUBMENUITEM_CANCEL, "Cancel");
 #ifdef TESTSUITE
-                subMenuController->Add(SUBMENUITEM_TESTSUITE, "Test Suite");
+                if (Rules::getRulesByFilename("testsuite.txt"))
+                    subMenuController->Add(SUBMENUITEM_TESTSUITE, "Test Suite");
 #endif
                 currentState = MENU_STATE_MAJOR_SUBMENU | MENU_STATE_MINOR_NONE;
             }
             break;
+
         case MENUITEM_DECKEDITOR:
-            mParent->DoTransition(TRANSITION_FADE, GAME_STATE_DECK_VIEWER);
-            break;
         case MENUITEM_SHOP:
-            mParent->DoTransition(TRANSITION_FADE, GAME_STATE_SHOP);
-            break;
         case MENUITEM_OPTIONS:
-            mParent->DoTransition(TRANSITION_FADE, GAME_STATE_OPTIONS);
+        case MENUITEM_TROPHIES:
+            mParent->DoTransition(TRANSITION_FADE,  ModRulesMenuItem::getMatchingGameState(controlId));
             break;
+
         case MENUITEM_EXIT:
             mEngine->End();
             break;
@@ -783,69 +866,81 @@ void GameStateMenu::ButtonPressed(int controllerId, int controlId)
             subMenuController->Close();
             currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
             break;
-        case SUBMENUITEM_2PLAYER:
+#ifdef NETWORK_SUPPORT
+        case SUBMENUITEM_2PLAYERS:
             mParent->players[0] = PLAYER_TYPE_HUMAN;
-            mParent->players[1] = PLAYER_TYPE_HUMAN;
+            mParent->players[1] = PLAYER_TYPE_REMOTE;
             subMenuController->Close();
-            currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
+            currentState = MENU_STATE_NETWORK_DEFINE | MENU_STATE_MINOR_SUBMENU_CLOSING;
             break;
+        case SUBMENUITEM_HOST_GAME:
+        {
+            if(!mParent->mpNetwork)
+            {
+                mParent->mpNetwork = new JNetwork();
+            }
+            mParent->mpNetwork->connect();
+            subMenuController->Close();
+            currentState = MENU_STATE_NETWORK_WAIT | MENU_STATE_MINOR_SUBMENU_CLOSING;
+            break;
+        }
+        case SUBMENUITEM_JOIN_GAME:
+        {
+            if(!mParent->mpNetwork)
+            {
+                mParent->mpNetwork = new JNetwork();
+            }
+            // FIXME needs to be able to specify the server ip
+            mParent->mpNetwork->connect("127.0.0.1");
+            // we let the server choose the game mode
+            mParent->gameType = GAME_TYPE_SLAVE;
+            hasChosenGameType = true;
+            subMenuController->Close();
+//            currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
+            currentState = MENU_STATE_NETWORK_WAIT;
+            break;
+        }
+#endif //NETWORK_SUPPORT
         case SUBMENUITEM_DEMO:
             mParent->players[0] = PLAYER_TYPE_CPU;
             mParent->players[1] = PLAYER_TYPE_CPU;
+            mParent->gameType = GAME_TYPE_DEMO;
             subMenuController->Close();
             currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
             break;
         case SUBMENUITEM_CANCEL:
+        case kInfoMenuID: // Triangle button
             if (subMenuController != NULL)
             {
                 subMenuController->Close();
             }
+#ifdef NETWORK_SUPPORT
+            if(mParent->mpNetwork)
+            {
+              SAFE_DELETE(mParent->mpNetwork);
+            }
+#endif //NETWORK_SUPPORT
             currentState = MENU_STATE_MAJOR_MAINMENU | MENU_STATE_MINOR_SUBMENU_CLOSING;
             break;
 
-        case SUBMENUITEM_CLASSIC:
-            this->hasChosenGameType = 1;
-            mParent->gameType = GAME_TYPE_CLASSIC;
-            subMenuController->Close();
-            currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
-            break;
-
-        case SUBMENUITEM_MOMIR:
-            this->hasChosenGameType = 1;
-            mParent->gameType = GAME_TYPE_MOMIR;
-            subMenuController->Close();
-            currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
-            break;
-
-        case SUBMENUITEM_RANDOM1:
-            this->hasChosenGameType = 1;
-            mParent->gameType = GAME_TYPE_RANDOM1;
-            subMenuController->Close();
-            currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
-            break;
-
-        case SUBMENUITEM_RANDOM2:
-            this->hasChosenGameType = 1;
-            mParent->gameType = GAME_TYPE_RANDOM2;
-            subMenuController->Close();
-            currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
-            break;
-
-        case SUBMENUITEM_STORY:
-            this->hasChosenGameType = 1;
-            mParent->gameType = GAME_TYPE_STORY;
-            subMenuController->Close();
-            currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
-            break;
-
 #ifdef TESTSUITE
             case SUBMENUITEM_TESTSUITE:
+            mParent->rules = Rules::getRulesByFilename("testsuite.txt");
+            this->hasChosenGameType = true;
+            mParent->gameType = GAME_TYPE_CLASSIC;
             mParent->players[0] = PLAYER_TYPE_TESTSUITE;
             mParent->players[1] = PLAYER_TYPE_TESTSUITE;
             subMenuController->Close();
             currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
             break;
 #endif
+            default: //Game modes
+            this->hasChosenGameType = true;
+            mParent->rules = Rules::RulesList[controlId - SUBMENUITEM_END_OFFSET];
+            mParent->gameType = (mParent->rules->gamemode); //TODO can we get rid of gameType in the long run, since it is also stored in the rules object ?
+            subMenuController->Close();
+            currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
+            break;
         }
         break;
     }
@@ -871,8 +966,5 @@ ostream& GameStateMenu::toString(ostream& out) const
                  << " ; mCurrentSetName : " << mCurrentSetName
                  << " ; mCurrentSetFileName : " << mCurrentSetFileName
                  << " ; mReadConf : " << mReadConf
-                 << " ; timeIndex : " << timeIndex
-                 << " ; angleMultiplier : " << angleMultiplier
-                 << " ; angleW : " << angleW
-                 << " ; yW : " << yW;
+                 << " ; timeIndex : " << timeIndex;
 }
