@@ -5,7 +5,6 @@
 #include "OptionItem.h"
 #include "SimpleMenu.h"
 #include "SimplePad.h"
-#include "GameOptions.h"
 #include "Translate.h"
 
 namespace
@@ -106,6 +105,10 @@ void GameStateOptions::Start()
     WDecoEnum * oFirstPlayer = NEW WDecoEnum(NEW OptionInteger(Options::FIRSTPLAYER, "First Turn Player", Constants::WHO_R, 1,
                     Constants::WHO_P, "", Constants::WHO_P));
     optionsList->Add(oFirstPlayer);
+    
+    WDecoEnum * oKickerPay = NEW WDecoEnum(NEW OptionInteger(Options::KICKERPAYMENT, "Kicker Cost", Constants::KICKER_CHOICE, 1,
+        Constants::KICKER_ALWAYS, "", Constants::KICKER_ALWAYS));
+    optionsList->Add(oKickerPay);
 
     optionsList = NEW WGuiKeyBinder("Key Bindings", this);
     optionsTabs->Add(optionsList);
@@ -174,7 +177,8 @@ void GameStateOptions::Update(float dt)
         case SHOW_OPTIONS:
         {
             JGE* j = JGE::GetInstance();
-            JButton key;
+            JButton key = JGE_BTN_NONE;
+            int x, y;
             if (grabber)
             {
                 LocalKeySym sym;
@@ -182,7 +186,7 @@ void GameStateOptions::Update(float dt)
                     grabber->KeyPressed(sym);
             }
             else
-                while ((key = JGE::GetInstance()->ReadButton()))
+                while ((key = JGE::GetInstance()->ReadButton()) || JGE::GetInstance()->GetLeftClickCoordinates(x,y))
                 {
                     if (!optionsTabs->CheckUserInput(key) && key == JGE_BTN_MENU)
                         mState = SHOW_OPTIONS_MENU;
