@@ -5,6 +5,8 @@
 #include "GameObserver.h"
 #include "Rules.h"
 
+const std::string kBackdropFile = "backdrop.jpg";
+
 GuiBackground::GuiBackground()
 {
 }
@@ -16,17 +18,18 @@ GuiBackground::~GuiBackground()
 void GuiBackground::Render()
 {
     JRenderer* renderer = JRenderer::GetInstance();
-    JQuad * quad = NULL;
+    JQuadPtr quad;
     GameObserver * go = GameObserver::GetInstance();
     if (go && go->mRules && go->mRules->bg.size())
     {
         quad = WResourceManager::Instance()->RetrieveTempQuad(go->mRules->bg);
     }
-    if (!quad)
+    if (!quad.get())
     {
-        quad = WResourceManager::Instance()->RetrieveTempQuad("backdrop.jpg");
+        quad = WResourceManager::Instance()->RetrieveTempQuad(kBackdropFile);
+    }
+    if (quad.get())
+    {
+        renderer->RenderQuad(quad.get(), 0, 18);
     }
-    if (!quad)
-        return;
-    renderer->RenderQuad(quad, 0, 18);
 }
