@@ -13,6 +13,8 @@
 
 using std::string;
 
+SUPPORT_OBJECT_ANALYTICS(MTGCard)
+
 MTGCard::MTGCard()
 {
     init();
@@ -23,9 +25,9 @@ MTGCard::MTGCard(int set_id)
     init();
     setId = set_id;
 }
+
 MTGCard::MTGCard(MTGCard * source)
 {
-
     strcpy(image_name, source->image_name);
     rarity = source->rarity;
     mtgid = source->mtgid;
@@ -33,6 +35,10 @@ MTGCard::MTGCard(MTGCard * source)
     data = source->data;
 }
 
+MTGCard::~MTGCard()
+{
+}
+
 int MTGCard::init()
 {
     setId = 0;
