@@ -14,6 +14,8 @@
 
 using namespace std;
 
+SUPPORT_OBJECT_ANALYTICS(MTGCardInstance)
+
 MTGCardInstance MTGCardInstance::AnyCard = MTGCardInstance();
 MTGCardInstance MTGCardInstance::NoCard = MTGCardInstance();
 
@@ -31,6 +33,8 @@ MTGCardInstance::MTGCardInstance(MTGCard * card, MTGPlayerCards * arg_belongs_to
     model = card;
     attacker = 0;
     lifeOrig = life;
+    origpower = power;
+    origtoughness = toughness;
     belongs_to = arg_belongs_to;
     owner = NULL;
     if (arg_belongs_to)
@@ -40,26 +44,24 @@ MTGCardInstance::MTGCardInstance(MTGCard * card, MTGPlayerCards * arg_belongs_to
     banding = NULL;
     life = toughness;
     preventable = 0;
+    thatmuch = 0;
     flanked = 0;
+    castMethod = Constants::NOT_CAST;
 }
 
 void MTGCardInstance::copy(MTGCardInstance * card)
 {
     MTGCard * source = card->model;
     CardPrimitive * data = source->data;
-    for (map<int, int>::const_iterator it = data->basicAbilities.begin(); it != data->basicAbilities.end(); ++it)
-    {
-        int i = it->first;
-        basicAbilities[i] = data->basicAbilities[i];
-    }
+
+    basicAbilities = card->basicAbilities;
     for (size_t i = 0; i < data->types.size(); i++)
     {
         types.push_back(data->types[i]);
     }
-    for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
-    {
-        colors[i] = data->colors[i];
-    }
+
+    colors = data->colors;
+
     manaCost.copy(data->getManaCost());
 
     text = data->text;
@@ -69,7 +71,6 @@ void MTGCardInstance::copy(MTGCardInstance * card)
     toughness = data->toughness;
     life = toughness;
     lifeOrig = life;
-
     magicText = data->magicText;
     spellTargetType = data->spellTargetType;
     alias = data->alias;
@@ -101,7 +102,6 @@ int MTGCardInstance::init()
     CardPrimitive::init();
     data = this;
     X = 0;
-    XX = 0;
     return 1;
 }
 
@@ -117,17 +117,45 @@ void MTGCardInstance::initMTGCI()
     untapping = 0;
     frozen = 0;
     fresh = 0;
+    isMultiColored = 0;
+    isBlackAndWhite = 0;
+    isRedAndBlue = 0;
+    isBlackAndGreen = 0;
+    isBlueAndGreen = 0;
+    isRedAndWhite = 0;
+    isLeveler = 0;
+    enchanted = false;
+    CDenchanted = 0;
+    CDdamaged = 0;
+    blinked = false;
+    isExtraCostTarget = false;
+    morphed = false;
+    turningOver = false;
+    isMorphed = false;
+    isPhased = false;
+    phasedTurn = -1;
     didattacked = 0;
     didblocked = 0;
     notblocked = 0;
-    sunburst = NULL;
-    equipment = NULL;
-    boughtback = 0;
-    flashedback = 0;
+    sunburst = 0;
+    equipment = 0;
+    auras = 0;
+    damageToOpponent = false;
+    damageToController = false;
+    wasDealtDamage = false;
+    suspended = false;
+    castMethod = Constants::NOT_CAST;
+    mPropertiesChangedSinceLastUpdate = false;
+
+
+    for (int i = 0; i < ManaCost::MANA_PAID_WITH_RETRACE +1; i++)
+        alternateCostPaid[i] = 0;
+
     paymenttype = MTGAbility::PUT_INTO_PLAY;
     reduxamount = 0;
     summoningSickness = 1;
     preventable = 0;
+    thatmuch = 0;
     flanked = 0;
     target = NULL;
     type_as_damageable = DAMAGEABLE_MTGCARDINSTANCE;
@@ -143,29 +171,47 @@ void MTGCardInstance::initMTGCI()
     currentZone = NULL;
     data = this; //an MTGCardInstance point to itself for data, allows to update it without killing the underlying database item
 
-    if (basicAbilities[Constants::CHANGELING])
-    {//if the card is a changeling.
-        for (int i = Subtypes::LAST_TYPE + 1;; i++)
+    if (basicAbilities[(int)Constants::CHANGELING])
+    {//if the card is a changeling, it gains all creature subtypes
+        const vector<string> values = Subtypes::subtypesList->getValuesById();
+        for (size_t i = 0; i < values.size(); ++i)
         {
-            string s = Subtypes::subtypesList->find(i);
-            if (!hasSubtype(i))
-            {
-                if (s == "")
-                    break;
-                if (s.find(" ") != string::npos)
-                    continue;
-                if (s == "Nothing" || s == "Swamp" || s == "Plains" || s == "Mountain" || s == "Forest" ||
-                        s == "Island" || s == "Shrine" || s == "Basic" || s == "Colony" || s == "Desert" ||
-                        s == "Dismiss" || s == "Equipment" || s == "Everglades" || s == "Grasslands" || s == "Lair" ||
-                        s == "Level" || s == "Levelup" || s == "Mine" || s == "Oasis" || s == "World" || s == "Aura"
-                )
-                    continue;
+            if (!Subtypes::subtypesList->isSubtypeOfType(i,Subtypes::TYPE_CREATURE))
+                continue;
 
-                addType(i);
-            }
+            //Don' want to send any event to the gameObserver inside of initMCGI, so calling the parent addType method instead of mine
+            CardPrimitive::addType(i);
         }
     }
 
+    int colored = 0;
+
+    for (int i = Constants::MTG_COLOR_GREEN; i <= Constants::MTG_COLOR_WHITE; ++i)
+    {
+        if (this->hasColor(i))
+            ++colored;
+    }
+    if (colored > 1)
+    {
+        isMultiColored = 1;
+
+        if(this->hasColor(Constants::MTG_COLOR_WHITE) && this->hasColor(Constants::MTG_COLOR_BLACK))
+            isBlackAndWhite = 1;
+        if(this->hasColor(Constants::MTG_COLOR_RED) && this->hasColor(Constants::MTG_COLOR_BLUE))
+            isRedAndBlue = 1;
+        if(this->hasColor(Constants::MTG_COLOR_GREEN) && this->hasColor(Constants::MTG_COLOR_BLACK))
+            isBlackAndGreen = 1;
+        if(this->hasColor(Constants::MTG_COLOR_BLUE) && this->hasColor(Constants::MTG_COLOR_GREEN))
+            isBlueAndGreen = 1;
+        if(this->hasColor(Constants::MTG_COLOR_RED) && this->hasColor(Constants::MTG_COLOR_WHITE))
+            isRedAndWhite = 1;
+    }
+
+    if(previous && previous->morphed && !turningOver)
+    {
+        morphed = true;
+        isMorphed = true;
+    }
 }
 
 const string MTGCardInstance::getDisplayName() const
@@ -177,6 +223,16 @@ void MTGCardInstance::addType(int type)
 {
     bool before = hasType(type);
     CardPrimitive::addType(type);
+
+    if (!before)
+        mPropertiesChangedSinceLastUpdate = true;
+
+    // If the card name is not set, set it to the type.
+    //This is a hack for "transform", used in combination with "losesubtypes" on Basic Lands
+    //See removeType below
+    if (!name.length())
+        setName(Subtypes::subtypesList->find(type));
+
     WEvent * e = NEW WEventCardChangeType(this, type, before, true);
     GameObserver * go = GameObserver::GetInstance();
     if (go)
@@ -211,6 +267,16 @@ int MTGCardInstance::removeType(int id, int removeAll)
     bool before = hasType(id);
     int result = CardPrimitive::removeType(id, removeAll);
     bool after = hasType(id);
+    if (before != after)
+    {
+        mPropertiesChangedSinceLastUpdate = true;
+        // Basic lands have the same name as their subtypes, and TargetChoosers don't make a distinction between name and type,
+        // so if we remove a subtype "Forest", we also need to remove its name.
+        //This means the card might lose its name, but usually when we force remove a type, we add another one just after that.
+        //see "AddType" above which force sets a name if necessary
+        if (name.compare(Subtypes::subtypesList->find(id)) == 0)
+            setName("");
+    }
     WEvent * e = NEW WEventCardChangeType(this, id, before, after);
     GameObserver * go = GameObserver::GetInstance();
     if (go)
@@ -249,12 +315,12 @@ int MTGCardInstance::afterDamage()
 int MTGCardInstance::bury()
 {
     Player * p = controller();
-    if (basicAbilities[Constants::EXILEDEATH])
+    if (basicAbilities[(int)Constants::EXILEDEATH])
     {
         p->game->putInZone(this, p->game->inPlay, owner->game->exile);
         return 1;
     }
-    if (!basicAbilities[Constants::INDESTRUCTIBLE])
+    if (!basicAbilities[(int)Constants::INDESTRUCTIBLE])
     {
         p->game->putInZone(this, p->game->inPlay, owner->game->graveyard);
         return 1;
@@ -278,6 +344,15 @@ int MTGCardInstance::has(int basicAbility)
     return basicAbilities[basicAbility];
 }
 
+ManaCost* MTGCardInstance::getReducedManaCost()
+{
+    return &reducedCost;
+}
+ManaCost* MTGCardInstance::getIncreasedManaCost()
+{
+    return &increasedCost;
+}
+
 //sets card as attacked and sends events
 void MTGCardInstance::eventattacked()
 {
@@ -305,19 +380,19 @@ void MTGCardInstance::eventattackednotblocked()
 }
 
 //sets card as attacked and sends events
-void MTGCardInstance::eventattackedblocked()
+void MTGCardInstance::eventattackedblocked(MTGCardInstance * opponent)
 {
     didattacked = 1;
-    WEvent * e = NEW WEventCardAttackedBlocked(this);
+    WEvent * e = NEW WEventCardAttackedBlocked(this,opponent);
     GameObserver * game = GameObserver::GetInstance();
     game->receiveEvent(e);
 }
 
 //sets card as blocking and sends events
-void MTGCardInstance::eventblocked()
+void MTGCardInstance::eventblocked(MTGCardInstance * opponent)
 {
     didblocked = 1;
-    WEvent * e = NEW WEventCardBlocked(this);
+    WEvent * e = NEW WEventCardBlocked(this,opponent);
     GameObserver * game = GameObserver::GetInstance();
     game->receiveEvent(e);
 }
@@ -384,10 +459,13 @@ int MTGCardInstance::triggerRegenerate()
         return 0;
     regenerateTokens--;
     tap();
-    life = toughness;
-    initAttackersDefensers();
-    if (life < 1)
-        return 0; //regeneration didn't work (wither ?)
+    if(isCreature())
+    {
+        life = toughness;
+        initAttackersDefensers();
+        if (life < 1)
+            return 0; //regeneration didn't work (wither ?)
+    }
     return 1;
 }
 
@@ -397,17 +475,16 @@ int MTGCardInstance::initAttackersDefensers()
     setDefenser(NULL);
     banding = NULL;
     blockers.clear();
-    blocked = false;
+    blocked = 0;
     didattacked = 0;
     didblocked = 0;
     return 1;
 }
 
-//Function to call to remove all damages, etc to a card (generally at the end of the turn)
+//Function to cleanup flags on a card (generally at the end of the turn)
 int MTGCardInstance::cleanup()
 {
     initAttackersDefensers();
-    life = toughness;
     GameObserver * game = GameObserver::GetInstance();
     if (!game || game->currentPlayer == controller())
     {
@@ -420,6 +497,7 @@ int MTGCardInstance::cleanup()
     }
     regenerateTokens = 0;
     preventable = 0;
+    thatmuch = 0;
     return 1;
 }
 
@@ -444,7 +522,7 @@ int MTGCardInstance::hasSummoningSickness()
 {
     if (!summoningSickness)
         return 0;
-    if (basicAbilities[Constants::HASTE])
+    if (basicAbilities[(int)Constants::HASTE])
         return 0;
     if (!isCreature())
         return 0;
@@ -472,7 +550,7 @@ int MTGCardInstance::canAttack()
         return 0;
     if (hasSummoningSickness())
         return 0;
-    if (basicAbilities[Constants::DEFENSER] || basicAbilities[Constants::CANTATTACK])
+    if ((basicAbilities[(int)Constants::DEFENSER] || basicAbilities[(int)Constants::CANTATTACK]) && !basicAbilities[(int)Constants::CANATTACK])
         return 0;
     if (!isCreature())
         return 0;
@@ -501,7 +579,7 @@ int MTGCardInstance::canBlock()
 {
     if (tapped)
         return 0;
-    if (basicAbilities[Constants::CANTBLOCK])
+    if (basicAbilities[(int)Constants::CANTBLOCK])
         return 0;
     if (!isCreature())
         return 0;
@@ -523,20 +601,24 @@ int MTGCardInstance::canBlock(MTGCardInstance * opponent)
         return 0;
     if (opponent->cantBeBlockedBy(this))
         return 0;
-    if (opponent->basicAbilities[Constants::UNBLOCKABLE])
+    if (opponent->basicAbilities[(int)Constants::UNBLOCKABLE])
         return 0;
-    if (opponent->basicAbilities[Constants::ONEBLOCKER] && opponent->blocked)
+    if (opponent->basicAbilities[(int)Constants::ONEBLOCKER] && opponent->blocked)
         return 0;
-    if (opponent->basicAbilities[Constants::FEAR] && !(hasType(Subtypes::TYPE_ARTIFACT) || hasColor(Constants::MTG_COLOR_BLACK)))
+    if(opponent->basicAbilities[(int)Constants::STRONG] && power < opponent->power)
+        return 0;
+    if(this->basicAbilities[(int)Constants::WEAK] && power < opponent->power)
+        return 0;
+    if (opponent->basicAbilities[(int)Constants::FEAR] && !(this->hasType(Subtypes::TYPE_ARTIFACT) || this->hasColor(Constants::MTG_COLOR_BLACK)))
         return 0;
 
     //intimidate
-    if (opponent->basicAbilities[Constants::INTIMIDATE] && !(hasType(Subtypes::TYPE_ARTIFACT)))
+    if (opponent->basicAbilities[(int)Constants::INTIMIDATE] && !(this->hasType(Subtypes::TYPE_ARTIFACT)))
     {
         int canblock = 0;
         for (int i = Constants::MTG_COLOR_GREEN; i <= Constants::MTG_COLOR_WHITE; ++i)
         {
-            if (hasColor(i) && opponent->hasColor(i))
+            if (this->hasColor(i) && opponent->hasColor(i))
             {
                 canblock = 1;
                 break;
@@ -546,34 +628,52 @@ int MTGCardInstance::canBlock(MTGCardInstance * opponent)
             return 0;
     }
 
-    if (opponent->basicAbilities[Constants::FLYING] && !(basicAbilities[Constants::FLYING] || basicAbilities[Constants::REACH]))
+    if (opponent->basicAbilities[(int)Constants::FLYING] && !(basicAbilities[(int)Constants::FLYING] || basicAbilities[(int)Constants::REACH]))
         return 0;
     //Can block only creatures with flying if has cloud
-    if (basicAbilities[Constants::CLOUD] && !(opponent->basicAbilities[Constants::FLYING]))
+    if (basicAbilities[(int)Constants::CLOUD] && !(opponent->basicAbilities[(int)Constants::FLYING]))
         return 0;
     // If opponent has shadow and a creature does not have either shadow or reachshadow it cannot be blocked
-    if (opponent->basicAbilities[Constants::SHADOW] && !(basicAbilities[Constants::SHADOW]
-                    || basicAbilities[Constants::REACHSHADOW]))
+    if (opponent->basicAbilities[(int)Constants::SHADOW] && !(basicAbilities[(int)Constants::SHADOW]
+                    || basicAbilities[(int)Constants::REACHSHADOW]))
         return 0;
     // If opponent does not have shadow and a creature has shadow it cannot be blocked
-    if (!opponent->basicAbilities[Constants::SHADOW] && basicAbilities[Constants::SHADOW])
+    if (!opponent->basicAbilities[(int)Constants::SHADOW] && basicAbilities[(int)Constants::SHADOW])
         return 0;
-    if (opponent->basicAbilities[Constants::HORSEMANSHIP] && !basicAbilities[Constants::HORSEMANSHIP])
+    if (opponent->basicAbilities[(int)Constants::HORSEMANSHIP] && !basicAbilities[(int)Constants::HORSEMANSHIP])
         return 0;
-    if (opponent->basicAbilities[Constants::SWAMPWALK] && controller()->game->inPlay->hasType("swamp"))
+    if (opponent->basicAbilities[(int)Constants::SWAMPWALK] && controller()->game->inPlay->hasType("swamp"))
         return 0;
-    if (opponent->basicAbilities[Constants::FORESTWALK] && controller()->game->inPlay->hasType("forest"))
+    if (opponent->basicAbilities[(int)Constants::FORESTWALK] && controller()->game->inPlay->hasType("forest"))
         return 0;
-    if (opponent->basicAbilities[Constants::ISLANDWALK] && controller()->game->inPlay->hasType("island"))
+    if (opponent->basicAbilities[(int)Constants::ISLANDWALK] && controller()->game->inPlay->hasType("island"))
         return 0;
-    if (opponent->basicAbilities[Constants::MOUNTAINWALK] && controller()->game->inPlay->hasType("mountain"))
+    if (opponent->basicAbilities[(int)Constants::MOUNTAINWALK] && controller()->game->inPlay->hasType("mountain"))
         return 0;
-    if (opponent->basicAbilities[Constants::PLAINSWALK] && controller()->game->inPlay->hasType("plains"))
+    if (opponent->basicAbilities[(int)Constants::PLAINSWALK] && controller()->game->inPlay->hasType("plains"))
+        return 0;
+    if (opponent->basicAbilities[(int)Constants::LEGENDARYWALK] && controller()->game->inPlay->hasPrimaryType("legendary","land"))
+        return 0;
+    if (opponent->basicAbilities[(int)Constants::DESERTWALK] && controller()->game->inPlay->hasSpecificType("land","desert"))
+        return 0;
+    if (opponent->basicAbilities[(int)Constants::SNOWSWAMPWALK] && controller()->game->inPlay->hasSpecificType("snow","swamp"))
+        return 0;
+    if (opponent->basicAbilities[(int)Constants::SNOWFORESTWALK] && controller()->game->inPlay->hasSpecificType("snow","forest"))
+        return 0;
+    if (opponent->basicAbilities[(int)Constants::SNOWISLANDWALK] && controller()->game->inPlay->hasSpecificType("snow","island"))
+        return 0;
+    if (opponent->basicAbilities[(int)Constants::SNOWMOUNTAINWALK] && controller()->game->inPlay->hasSpecificType("snow","mountain"))
+        return 0;
+    if (opponent->basicAbilities[(int)Constants::SNOWPLAINSWALK] && controller()->game->inPlay->hasSpecificType("snow","plains"))
+        return 0;
+    if (opponent->basicAbilities[(int)Constants::SNOWWALK] && controller()->game->inPlay->hasPrimaryType("snow","land"))
+        return 0;
+    if (opponent->basicAbilities[(int)Constants::NONBASICWALK] && controller()->game->inPlay->hasTypeButNotType("land","basic"))
         return 0;
     return 1;
 }
 
-JQuad * MTGCardInstance::getIcon()
+JQuadPtr MTGCardInstance::getIcon()
 {
     return WResourceManager::Instance()->RetrieveCard(this, CACHE_THUMB);
 }
@@ -584,7 +684,7 @@ MTGCardInstance * MTGCardInstance::getNextPartner()
     MTGCardInstance * bandingPartner = inplay->getNextAttacker(banding);
     while (bandingPartner)
     {
-        if (basicAbilities[Constants::BANDING] || bandingPartner->basicAbilities[Constants::BANDING])
+        if (basicAbilities[(int)Constants::BANDING] || bandingPartner->basicAbilities[(int)Constants::BANDING])
             return bandingPartner;
         bandingPartner = inplay->getNextAttacker(bandingPartner);
     }
@@ -806,10 +906,11 @@ int MTGCardInstance::toggleDefenser(MTGCardInstance * opponent)
         {
             setDefenser(opponent);
             didblocked = 1;
-            if (opponent && opponent->controller()->isAI())
+            if (opponent && opponent->controller()->isAI() && opponent->controller()->playMode != Player::MODE_TEST_SUITE)
             {
                 if(opponent->view != NULL)
                 {
+                //todo: quote wololo "change this into a cool blinking effects when opposing creature has cursor focus."
                     opponent->view->actZ += .8f;
                     opponent->view->actT -= .2f;
                 }
@@ -822,6 +923,16 @@ int MTGCardInstance::toggleDefenser(MTGCardInstance * opponent)
     return 0;
 }
 
+bool MTGCardInstance::matchesCastFilter(int castFilter) {
+    if(castFilter == Constants::CAST_DONT_CARE)
+        return true; //everything
+    if(castFilter == Constants::CAST_ALL)
+        return (castMethod != Constants::NOT_CAST); //everything except "not cast"
+    if (castFilter == Constants::CAST_ALTERNATE && castMethod > Constants::CAST_NORMALLY)
+        return true; //all alternate casts
+    return (castFilter == castMethod);
+};
+
 int MTGCardInstance::addProtection(TargetChooser * tc)
 {
     tc->targetter = NULL;
@@ -862,6 +973,38 @@ int MTGCardInstance::protectedAgainst(MTGCardInstance * card)
     return 0;
 }
 
+int MTGCardInstance::addCantBeTarget(TargetChooser * tc)
+{
+    tc->targetter = NULL;
+    canttarget.push_back(tc);
+    return canttarget.size();
+}
+
+int MTGCardInstance::removeCantBeTarget(TargetChooser * tc, int erase)
+{
+    for (size_t i = 0; i < canttarget.size(); i++)
+    {
+        if (canttarget[i] == tc)
+        {
+            if (erase)
+                delete (canttarget[i]);
+            canttarget.erase(canttarget.begin() + i);
+            return 1;
+        }
+    }
+    return 0;
+}
+
+int MTGCardInstance::CantBeTargetby(MTGCardInstance * card)
+{
+    for (size_t i = 0; i < canttarget.size(); i++)
+    {
+        if (canttarget[i]->canTarget(card))
+            return 1;
+    }
+    return 0;
+}
+
 int MTGCardInstance::addCantBeBlockedBy(TargetChooser * tc)
 {
     cantBeBlockedBys.push_back(tc);
@@ -913,23 +1056,28 @@ JSample * MTGCardInstance::getSample()
         }
     }
 
-    for (map<int, int>::const_iterator it = basicAbilities.begin(); it != basicAbilities.end(); ++it)
+    if (basicAbilities.any())
     {
-        int i = it->first;
-        if (!basicAbilities[i])
-            continue;
-        string type = Constants::MTGBasicAbilities[i];
-        type = type + ".wav";
-        js = WResourceManager::Instance()->RetrieveSample(type);
-        if (js)
+        for (size_t x = 0; x < basicAbilities.size(); ++x)
         {
-            sample = string(type);
-            return js;
+            if (!basicAbilities.test(x))
+                continue;
+            string type = Constants::MTGBasicAbilities[x];
+            type = type + ".wav";
+            js = WResourceManager::Instance()->RetrieveSample(type);
+            if (js)
+            {
+                sample = string(type);
+                return js;
+            }
         }
     }
 
-    string type = Subtypes::subtypesList->find(types[0]);
-    type = type + ".wav";
+    string type = "";
+    if(!types.size())
+        return NULL;   
+    type = Subtypes::subtypesList->find(types[0]);
+    type.append(".wav");
     js = WResourceManager::Instance()->RetrieveSample(type);
     if (js)
     {
