@@ -9,6 +9,8 @@
 #include "WDataSrc.h"
 #include "MTGPack.h"
 #include "utils.h"
+#include "DeckManager.h"
+#include <iomanip>
 
 #if defined (WIN32) || defined (LINUX)
 #include <time.h>
@@ -43,13 +45,10 @@ static inline int getGrade(int v)
 //MTGAllCards
 int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primitive)
 {
-    if ('#' == s[0]) return 0;
+    if ('#' == s[0]) return 1; // a comment shouldn't be treated as an error condition
     size_t i = s.find_first_of('=');
     if (i == string::npos || 0 == i)
-    {
-        DebugTrace("MTGDECK: Bad Line:\n\t" << s);
         return 0;
-    }
 
     char* key = const_cast<char*> (s.c_str()); // I know what I'm doing, let me do it
     key[i] = 0;
@@ -79,25 +78,13 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
             string value = val;
             //Specific Abilities
             std::transform(value.begin(), value.end(), value.begin(), ::tolower);
-            while (value.size())
+            vector<string> values = split(value, ',');
+            for (size_t values_i = 0; values_i < values.size(); ++values_i)
             {
-                string attribute;
-                size_t found2 = value.find(',');
-                if (found2 != string::npos)
-                {
-                    attribute = value.substr(0, found2);
-                    value = value.substr(found2 + 1);
-                }
-                else
-                {
-                    attribute = value;
-                    value = "";
-                }
-                
+
                 for (int j = Constants::NB_BASIC_ABILITIES - 1; j >= 0; --j)
                 {
-                    size_t found = attribute.find(Constants::MTGBasicAbilities[j]);
-                    if (found != string::npos)
+                    if (values[values_i].find(Constants::MTGBasicAbilities[j]) != string::npos)
                     {
                         primitive->basicAbilities[j] = 1;
                         break;
@@ -138,11 +125,29 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
 
     case 'o': //othercost
         if (!primitive) primitive = NEW CardPrimitive();
-        if (ManaCost * cost = primitive->getManaCost())
+        if(key[5] == 'r')//otherrestrictions
         {
             string value = val;
-            std::transform(value.begin(), value.end(), value.begin(), ::tolower);
-            cost->alternative = ManaCost::parseManaCost(value);
+            primitive->setOtherRestrictions(value);
+        }
+        else
+        {
+            if (ManaCost * cost = primitive->getManaCost())
+            {
+                string value = val;
+                std::transform(value.begin(), value.end(), value.begin(), ::tolower);
+                size_t name = value.find("name(");
+                string theName = "";
+                    if(name != string::npos)
+                    {
+                        size_t endName = value.find(")",name);
+                        theName = value.substr(name + 5,endName - name - 5);
+                        s.erase(name, endName - name + 1);
+                    }
+                    cost->alternative = ManaCost::parseManaCost(value);
+                    if(theName.size())
+                        cost->alternative->alternativeName.append(theName);
+            }
         }
         break;
 
@@ -155,16 +160,26 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
             cost->BuyBack = ManaCost::parseManaCost(value);
         }
         break;
-    case 'f': //flashback
-        if (!primitive) primitive = NEW CardPrimitive();
-        if (ManaCost * cost = primitive->getManaCost())
+    case 'f': //flashback//morph
         {
-            string value = val;
-            std::transform(value.begin(), value.end(), value.begin(), ::tolower);
-            cost->FlashBack = ManaCost::parseManaCost(value);
+            if (!primitive) primitive = NEW CardPrimitive();
+            if(ManaCost * cost = primitive->getManaCost())
+            {
+                if( s.find("facedown") != string::npos)//morph
+                {
+                    string value = val;
+                    std::transform(value.begin(), value.end(), value.begin(), ::tolower);
+                    cost->morph = ManaCost::parseManaCost(value);
+                }
+                else
+                {
+                    string value = val;
+                    std::transform(value.begin(), value.end(), value.begin(), ::tolower);
+                    cost->FlashBack = ManaCost::parseManaCost(value);
+                }
+            }
+            break;
         }
-        break;
-
     case 'i': //id
         if (!card) card = NEW MTGCard();
         card->setMTGId(atoi(val));
@@ -181,7 +196,6 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
 
     case 'n': //name
         if (!primitive) primitive = NEW CardPrimitive();
-        if (0 == strcmp("Bloodrock Cyclops", val)) cout << "val" << endl;
         primitive->setName(val);
         break;
 
@@ -200,7 +214,14 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
         break;
 
     case 'r': //retrace/rarity
-        if ('e' == key[1])
+        if('s' == key[2] && 't' == key[3])//restrictions
+        {
+            if (!primitive) primitive = NEW CardPrimitive();
+            string value = val;
+            primitive->setRestrictions(value);
+            primitive->hasRestriction = true;
+        }
+        else if ('e' == key[1] && 't' == key[2])
         { //retrace
             if (!primitive) primitive = NEW CardPrimitive();
             if (ManaCost * cost = primitive->getManaCost())
@@ -210,32 +231,38 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
                 cost->Retrace = ManaCost::parseManaCost(value);
             }
         }
-        else
+        else if (s.find("rar") != string::npos)
         {//rarity
             if (!card) card = NEW MTGCard();
             card->setRarity(val[0]);
         }
         break;
-
-    case 's': //subtype
-        if (!primitive) primitive = NEW CardPrimitive();
-        while (true)
+    case 's': //subtype, suspend
         {
-            char* found = strchr(val, ' ');
-            if (found)
+            if (s.find("suspend") != string::npos)
             {
-                string value(val, found - val);
-                primitive->setSubtype(value);
-                val = found + 1;
+            size_t time = s.find("suspend(");
+            size_t end = s.find(")=");
+            int suspendTime = atoi(s.substr(time + 8,end - 2).c_str());
+                if (!primitive) primitive = NEW CardPrimitive();
+                if (ManaCost * cost = primitive->getManaCost())
+                {
+                    string value = val;
+                    std::transform(value.begin(), value.end(), value.begin(), ::tolower);
+                    cost->suspend = ManaCost::parseManaCost(value);
+                    primitive->suspendedTime = suspendTime;
+                }
+                
             }
             else
             {
-                primitive->setSubtype(val);
-                break;
+                if (!primitive) primitive = NEW CardPrimitive();
+                vector<string> values = split(val, ' ');
+                for (size_t values_i = 0; values_i < values.size(); ++values_i)
+                    primitive->setSubtype(values[values_i]);
             }
+            break;
         }
-        break;
-
     case 't':
         if (!primitive) primitive = NEW CardPrimitive();
         if (0 == strcmp("target", key))
@@ -248,21 +275,9 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
             primitive->setText(val);
         else if (0 == strcmp("type", key))
         {
-            while (true)
-            {
-                char* found = strchr(val, ' ');
-                if (found)
-                {
-                    string value(val, found - val);
-                    primitive->setType(value);
-                    val = found + 1;
-                }
-                else
-                {
-                      primitive->setType(val);
-                    break;
-                }
-            }
+            vector<string> values = split(val, ' ');
+            for (size_t values_i = 0; values_i < values.size(); ++values_i)
+                    primitive->setType(values[values_i]);
         }
         else if (0 == strcmp("toughness", key)) primitive->setToughness(atoi(val));
         break;
@@ -282,9 +297,7 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
 void MTGAllCards::initCounters()
 {
     for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
-    {
-        colorsCount[i] = NULL;
-    }
+        colorsCount[i] = 0;
 }
 
 void MTGAllCards::init()
@@ -295,13 +308,49 @@ void MTGAllCards::init()
     initCounters();
 }
 
+void MTGAllCards::loadPrimitives(const char *root_directory)
+{
+    DIR *dir = opendir(root_directory);
+    struct dirent *dit;
+
+    while ((dit = readdir(dir)))
+    {
+        char fullname[1000];
+        sprintf(fullname, "%s/%s", root_directory, dit->d_name);
+        wagic::ifstream file(fullname);
+        if (!file)
+            continue;
+        file.close();
+        MTGCollection()->load(fullname);
+    }
+    closedir(dir);
+}
+
+void MTGAllCards::loadSets(const char *root_directory, const char* filename)
+{
+    DIR *dir = opendir(root_directory);
+    struct dirent *dit;
+
+    while ((dit = readdir(dir)))
+    {
+        char fullname[1000];
+        sprintf(fullname, "%s/%s/%s", root_directory, dit->d_name, filename);
+        wagic::ifstream file(fullname);
+        if (!file)
+            continue;
+        file.close();
+        MTGCollection()->load(fullname, dit->d_name);
+    }
+    closedir(dir);}
+
 int MTGAllCards::load(const char * config_file, const char * set_name, int autoload)
 {
     conf_read_mode = 0;
     const int set_id = set_name ? setlist.Add(set_name) : MTGSets::INTERNAL_SET;
     MTGSetInfo *si = setlist.getInfo(set_id);
 
-    std::ifstream setFile(config_file, ios::in | ios::ate);
+    int lineNumber = 0;
+    wagic::ifstream setFile(config_file, ios::in | ios::ate);
     if (!setFile) return total_cards;
 
     streampos fileSize = setFile.tellg();
@@ -317,6 +366,7 @@ int MTGAllCards::load(const char * config_file, const char * set_name, int autol
     while (true)
     {
         if (!std::getline(stream, s)) return total_cards;
+        lineNumber++;
         if (!s.size()) continue;
 
         if (s[s.size() - 1] == '\r') s.erase(s.size() - 1); // Handle DOS files
@@ -363,7 +413,8 @@ int MTGAllCards::load(const char * config_file, const char * set_name, int autol
             }
             else
             {
-                processConfLine(s, tempCard, tempPrimitive);
+                if (!processConfLine(s, tempCard, tempPrimitive))
+                    DebugTrace("MTGDECK: BAD Line: \n[" << lineNumber << "]: " << s );
             }
             continue;
         }
@@ -371,19 +422,21 @@ int MTGAllCards::load(const char * config_file, const char * set_name, int autol
     return total_cards;
 }
 
+MTGAllCards* MTGAllCards::instance = NULL;
+
 MTGAllCards::MTGAllCards()
 {
     init();
 }
 
-MTGAllCards::~MTGAllCards()
+MTGAllCards::MTGAllCards(const char * config_file, const char * set_name)
 {
-    //Why don't we call destroyAllCards from here ???
+    init();
+    load(config_file, set_name, 0);
 }
 
-void MTGAllCards::destroyAllCards()
+MTGAllCards::~MTGAllCards()
 {
-
     for (map<int, MTGCard *>::iterator it = collection.begin(); it != collection.end(); it++)
         delete (it->second);
     collection.clear();
@@ -392,13 +445,20 @@ void MTGAllCards::destroyAllCards()
     for (map<string, CardPrimitive *>::iterator it = primitives.begin(); it != primitives.end(); it++)
         delete (it->second);
     primitives.clear();
-
 }
 
-MTGAllCards::MTGAllCards(const char * config_file, const char * set_name)
+void MTGAllCards::loadInstance()
 {
-    init();
-    load(config_file, set_name, 0);
+    if(!instance)
+        instance = new MTGAllCards();
+}
+
+void MTGAllCards::unloadAll()
+{
+    if(instance) {
+        delete instance;
+        instance = NULL;
+    }
 }
 
 int MTGAllCards::randomCardId()
@@ -558,10 +618,21 @@ MTGCard * MTGAllCards::getCardByName(string name)
     if (!name.size()) return NULL;
     if (name[0] == '#') return NULL;
 
+    map<string, MTGCard * >::iterator cached = mtgCardByNameCache.end();
+    if (mtgCardByNameCache.size() > 0)
+        cached = mtgCardByNameCache.find(name);
+    
+    if (cached!= mtgCardByNameCache.end())
+    {
+        return cached->second;
+    }
+
     int cardnb = atoi(name.c_str());
     if (cardnb)
     {
-        return getCardById(cardnb);
+        MTGCard * result = getCardById(cardnb);
+        mtgCardByNameCache[name] = result;
+        return result;
     }
 
     std::transform(name.begin(), name.end(), name.begin(), ::tolower);
@@ -583,9 +654,13 @@ MTGCard * MTGAllCards::getCardByName(string name)
         if (setId != -1 && setId != c->setId) continue;
         string cardName = c->data->name;
         std::transform(cardName.begin(), cardName.end(), cardName.begin(), ::tolower);
-        if (cardName.compare(name) == 0) return c;
+        if (cardName.compare(name) == 0) {
+            mtgCardByNameCache[name] = c;
+            return c;
+        }
 
     }
+    mtgCardByNameCache[name] = NULL;
     return NULL;
 }
 
@@ -601,7 +676,7 @@ MTGDeck::MTGDeck(MTGAllCards * _allcards)
 int MTGDeck::totalPrice()
 {
     int total = 0;
-    PriceList * pricelist = NEW PriceList(JGE_GET_RES("settings/prices.dat").c_str(), GameApp::collection);
+    PriceList * pricelist = NEW PriceList(JGE_GET_RES("settings/prices.dat").c_str(), MTGCollection());
     map<int, int>::iterator it;
     for (it = cards.begin(); it != cards.end(); it++)
     {
@@ -612,7 +687,7 @@ int MTGDeck::totalPrice()
     return total;
 }
 
-MTGDeck::MTGDeck(const char * config_file, MTGAllCards * _allcards, int meta_only)
+MTGDeck::MTGDeck(const char * config_file, MTGAllCards * _allcards, int meta_only,int difficultyRating)
 {
     total_cards = 0;
     database = _allcards;
@@ -620,9 +695,9 @@ MTGDeck::MTGDeck(const char * config_file, MTGAllCards * _allcards, int meta_onl
     size_t slash = filename.find_last_of("/");
     size_t dot = filename.find(".");
     meta_name = filename.substr(slash + 1, dot - slash - 1);
-    std::ifstream file(config_file);
+    meta_id = atoi(meta_name.substr(4).c_str());
+    wagic::ifstream file(config_file);
     std::string s;
-
     if (file)
     {
         while (std::getline(file, s))
@@ -644,36 +719,56 @@ MTGDeck::MTGDeck(const char * config_file, MTGAllCards * _allcards, int meta_onl
                     meta_desc.append(s.substr(found + 5));
                     continue;
                 }
+                found = s.find("HINT:");
+                if (found != string::npos)
+                {
+                    meta_AIHints.push_back(s.substr(found + 5));
+                    continue;
+                }
+                found = s.find("UNLOCK:");
+                if (found != string::npos)
+                {
+                    meta_unlockRequirements = s.substr(found + 7);
+                    continue;
+                }
                 continue;
             }
             if (meta_only) break;
-            int cardnb = atoi(s.c_str());
-            if (cardnb)
+            int numberOfCopies = 1;
+            size_t found = s.find(" *");
+            if (found != string::npos)
             {
-                add(cardnb);
+                numberOfCopies = atoi(s.substr(found + 2).c_str());
+                s = s.substr(0, found);
             }
-            else
+            size_t diff = s.find("toggledifficulty:");
+            if(diff != string::npos)
             {
-                int nb = 1;
-                size_t found = s.find(" *");
-                if (found != string::npos)
+                string cards = s.substr(diff + 17);
+                size_t separator = cards.find("|");
+                string cardeasy = cards.substr(0,separator);
+                string cardhard = cards.substr(separator + 1);
+                if(difficultyRating == HARD)
                 {
-                    nb = atoi(s.substr(found + 2).c_str());
-                    s = s.substr(0, found);
-                }
-                MTGCard * card = database->getCardByName(s);
-                if (card)
-                {
-                    for (int i = 0; i < nb; i++)
-                    {
-                        add(card);
-                    }
+                    s = cardhard;
                 }
                 else
                 {
-                    DebugTrace("could not find Card matching name: " << s);
+                    s = cardeasy;
                 }
             }
+            MTGCard * card = database->getCardByName(s);
+            if (card)
+            {
+                for (int i = 0; i < numberOfCopies; i++)
+                {
+                    add(card);
+                }
+            }
+            else
+            {
+                DebugTrace("could not find Card matching name: " << s);
+            }
         }
         file.close();
     }
@@ -688,6 +783,11 @@ int MTGDeck::totalCards()
     return total_cards;
 }
 
+string MTGDeck::getFilename()
+{
+    return filename;
+}
+
 MTGCard * MTGDeck::getCardById(int mtgId)
 {
     return database->getCardById(mtgId);
@@ -723,8 +823,8 @@ int MTGDeck::addRandomCards(int howmany, int * setIds, int nbSets, int rarity, c
         MTGCard * card = database->_(i);
         int r = card->getRarity();
         if (r != Constants::RARITY_T && (rarity == -1 || r == rarity) && // remove tokens
-                card->setId != MTGSets::INTERNAL_SET && //remove cards that are defined in primitives. Those are workarounds (usually tokens) and should only be used internally
-                (!_subtype || card->data->hasSubtype(subtype)))
+            card->setId != MTGSets::INTERNAL_SET && //remove cards that are defined in primitives. Those are workarounds (usually tokens) and should only be used internally
+            (!_subtype || card->data->hasSubtype(subtype)))
         {
             int ok = 0;
 
@@ -759,8 +859,7 @@ int MTGDeck::addRandomCards(int howmany, int * setIds, int nbSets, int rarity, c
     }
     if (subtotal == 0)
     {
-        if (rarity == Constants::RARITY_M) return addRandomCards(howmany, setIds, nbSets, Constants::RARITY_R, _subtype, colors,
-                nbcolors);
+        if (rarity == Constants::RARITY_M) return addRandomCards(howmany, setIds, nbSets, Constants::RARITY_R, _subtype, colors, nbcolors);
         return 0;
     }
     for (int i = 0; i < howmany; i++)
@@ -786,7 +885,8 @@ int MTGDeck::add(MTGDeck * deck)
 
 int MTGDeck::add(int cardid)
 {
-    if (!database->getCardById(cardid)) return 0;
+    if (!database->getCardById(cardid))
+        return 0;
     if (cards.find(cardid) == cards.end())
     {
         cards[cardid] = 1;
@@ -802,15 +902,16 @@ int MTGDeck::add(int cardid)
 
 int MTGDeck::add(MTGCard * card)
 {
-    if (!card) return 0;
+    if (!card) 
+        return 0;
     return (add(card->getId()));
 }
 
 int MTGDeck::complete()
 {
     /* (PSY) adds cards to the deck/collection. Makes sure that the deck
-     or collection has at least 4 of every implemented card. Does not
-     change the number of cards of which already 4 or more are present. */
+    or collection has at least 4 of every implemented card. Does not
+    change the number of cards of which already 4 or more are present. */
     int id, n;
     bool StypeIsNothing;
     size_t databaseSize = database->ids.size();
@@ -822,7 +923,7 @@ int MTGDeck::complete()
         {
             StypeIsNothing = true;
         }
-        if (!StypeIsNothing == true)
+        if (!StypeIsNothing )
         {
             if (cards.find(id) == cards.end())
             {
@@ -871,7 +972,7 @@ int MTGDeck::save()
     return save(filename, false, meta_name, meta_desc);
 }
 
-int MTGDeck::save(string destFileName, bool useExpandedDescriptions, string &deckTitle, string &deckDesc)
+int MTGDeck::save(const string& destFileName, bool useExpandedDescriptions, const string& deckTitle, const string& deckDesc)
 {
     string tmp = destFileName;
     tmp.append(".tmp"); //not thread safe
@@ -899,23 +1000,14 @@ int MTGDeck::save(string destFileName, bool useExpandedDescriptions, string &dec
             file << "#DESC:" << desc << "\n";
         }
 
-        if (useExpandedDescriptions)
+        bool saveDetailedDeckInfo = options.get( Options::SAVEDETAILEDDECKINFO )->number == 1;
+
+        if ( filename.find("collection.dat") != string::npos )
+            saveDetailedDeckInfo = false;
+
+        if (useExpandedDescriptions || saveDetailedDeckInfo)
         {
-            map<int, int>::iterator it;
-            for (it = cards.begin(); it != cards.end(); it++)
-            {
-                int nbCards = it->second;
-                MTGCard *card = this->getCardById(it->first);
-                if (card == NULL)
-                {
-                    continue;
-                }
-                MTGSetInfo *setInfo = setlist.getInfo(card->setId);
-                string setName = setInfo->id;
-                string cardName = card->data->getName();
-                file << cardName << "\t " << "(" << setName << ") *" << nbCards << endl;
-                setInfo = NULL;
-            }
+            printDetailedDeckText(file);
         }
         else
         {
@@ -933,10 +1025,97 @@ int MTGDeck::save(string destFileName, bool useExpandedDescriptions, string &dec
         std::remove(destFileName.c_str());
         rename(tmp.c_str(), destFileName.c_str());
     }
-    DeckMetaDataList::decksMetaData->invalidate(destFileName);
     return 1;
 }
 
+/***
+    print out an expanded version of the deck to file.  
+    This save meta data about each card to allow easy reading of the deck file.  It will
+    also save each card by id, to speed up the loading of the deck next time.
+*/
+void MTGDeck::printDetailedDeckText(std::ofstream& file )
+{
+    ostringstream currentCard, creatures, lands, spells, types;
+    map<int, int>::iterator it;
+    for (it = cards.begin(); it != cards.end(); it++)
+    {
+        int cardId = it->first;
+        int nbCards = it->second;
+        MTGCard *card = this->getCardById( cardId );
+        if (card == NULL)
+        {
+            continue;
+        }
+        MTGSetInfo *setInfo = setlist.getInfo(card->setId);
+        string setName = setInfo->id;
+        string cardName = card->data->getName();
+        string description = card->data->getText();
+
+        currentCard << "#" << nbCards << " x " << cardName << " (" << setName << "), ";
+
+        if ( !card->data->isLand() )
+            currentCard << card->data->getManaCost() << ", ";
+
+        // Add the card's types 
+        vector<int>::iterator typeIter;
+        for ( typeIter = card->data->types.begin(); typeIter != card->data->types.end(); ++typeIter )
+            types << Subtypes::subtypesList->find( *typeIter ) << " ";
+
+        currentCard << trim(types.str()) << ", ";
+        types.str(""); // reset the buffer.
+
+        // Add P/T if a creature
+        if ( card->data->isCreature() )
+            currentCard << card->data->getPower() << "/" << card->data->getToughness() << ", ";
+
+
+        if ( card->data->hasRestriction )
+            currentCard << ", " << card->data->otherrestriction;
+
+        for (size_t x = 0; x < card->data->basicAbilities.size(); ++x)
+        {
+            if ( card->data->basicAbilities[x] == 1 )
+                currentCard <<  Constants::MTGBasicAbilities[x] << "; ";
+        }
+        currentCard <<endl;
+
+        for ( int i = 0; i < nbCards; i++ )
+            currentCard << cardId << endl;
+
+        currentCard <<endl;
+        setInfo = NULL;
+        if ( card->data->isLand() )
+            lands<< currentCard.str();
+        else if ( card->data->isCreature() )
+            creatures << currentCard.str();
+        else 
+            spells << currentCard.str();
+        currentCard.str("");
+    }
+    file << getCardBlockText( "Creatures", creatures.str() ) << endl;
+    file << getCardBlockText( "Spells", spells.str() ) << endl;
+    file << getCardBlockText( "Lands", lands.str() ) << endl;
+    creatures.str("");
+    spells.str("");
+    lands.str("");
+}
+
+/***
+* Convience method to print out blocks of card descriptions
+*/
+string MTGDeck::getCardBlockText( const string& title, const string& text )
+{
+    ostringstream oss;
+    string textBlock (text);
+
+    oss << setfill('#') << setw( 40 ) << "#" << endl;
+    oss << "#    " << setfill(' ') << setw(34) << left << title << "#" << endl;
+    oss << setfill('#') << setw( 40 ) << "#" << endl;
+    oss << trim(textBlock) << endl;
+
+    return oss.str();    
+}
+
 //MTGSets
 MTGSets setlist; //Our global.
 
@@ -962,7 +1141,7 @@ MTGSetInfo* MTGSets::getInfo(int setID)
 MTGSetInfo* MTGSets::randomSet(int blockId, int atleast)
 {
     char * unlocked = (char *) calloc(size(), sizeof(char));
-    int attempts = 50;
+
     //Figure out which sets are available.
     for (int i = 0; i < size(); i++)
     {
@@ -976,7 +1155,7 @@ MTGSetInfo* MTGSets::randomSet(int blockId, int atleast)
         for (int i = a; i < size(); i++)
         {
             if (unlocked[i] && (blockId == -1 || setinfo[i]->block == blockId) &&
-                    (atleast == -1 || setinfo[i]->totalCards() >= atleast))
+                (atleast == -1 || setinfo[i]->totalCards() >= atleast))
             {
                 free(unlocked);
                 return setinfo[i];
@@ -985,7 +1164,7 @@ MTGSetInfo* MTGSets::randomSet(int blockId, int atleast)
         for (int i = 0; i < a; i++)
         {
             if (unlocked[i] && (blockId == -1 || setinfo[i]->block == blockId) &&
-                    (atleast == -1 || setinfo[i]->totalCards() >= atleast))
+                (atleast == -1 || setinfo[i]->totalCards() >= atleast))
             {
                 free(unlocked);
                 return setinfo[i];
