@@ -24,6 +24,7 @@ const string Constants::kBuyBackKeyword = "buyback";
 const string Constants::kFlashBackKeyword = "flashback";
 const string Constants::kRetraceKeyword = "retrace";
 const string Constants::kKickerKeyword = "kicker";
+const string Constants::kMorphKeyword = "facedown";
 
 const char* Constants::MTGBasicAbilities[] = {
     "trample",
@@ -88,13 +89,13 @@ const char* Constants::MTGBasicAbilities[] = {
     "cantlose",
     "cantlifelose",
     "cantmilllose",
-    "cantcreaturecast",
-    "cantspellcast",
-    "onlyonecast",
+    "snowlandwalk",
+    "nonbasiclandwalk",
+    "strong",//cant be blocked by creature with less power
     "storm",
-    "bothcantcast",
-    "bothnocreature",
-    "oneboth",
+    "phasing",
+    "split second",
+    "weak",//cant block creatures with more power
     "affinityartifacts",
     "affinityplains",
     "affinityforests",
@@ -110,6 +111,14 @@ const char* Constants::MTGBasicAbilities[] = {
     "sunburst",
     "flanking",
     "exiledeath",
+    "legendarylandwalk",
+    "desertlandwalk",
+    "snowforestlandwalk",
+    "snowplainslandwalk",
+    "snowmountainlandwalk",
+    "snowislandlandwalk",
+    "snowswamplandwalk",
+    "canattack"
 };
 
 map<string,int> Constants::MTGBasicAbilitiesMap;
