@@ -1,8 +1,8 @@
 #include "PrecompiledHeader.h"
 
+#include "CardSelectorSingleton.h"
 #include "MTGGameZones.h"
 #include "Player.h"
-#include "GameOptions.h"
 #include "WEvent.h"
 #include "MTGDeck.h"
 #include "Subtypes.h"
@@ -238,6 +238,23 @@ void MTGPlayerCards::drawFromLibrary()
     }
     MTGCardInstance * toMove = library->cards[library->nb_cards - 1];
     library->lastCardDrawn = toMove;
+
+    // useability tweak - assume that the user is probably going to want to see the new card,
+    // so prefetch it.
+
+    // if we're not in text mode, always get the thumb
+    if (CardSelectorSingleton::Instance()->GetDrawMode() != DrawMode::kText)
+    {
+        DebugTrace("Prefetching AI card going into play: " << toMove->getImageName());
+        WResourceManager::Instance()->RetrieveCard(toMove, RETRIEVE_THUMB);
+
+        // also cache the large image if we're using kNormal mode
+        if (CardSelectorSingleton::Instance()->GetDrawMode() == DrawMode::kNormal)
+        {
+            WResourceManager::Instance()->RetrieveCard(toMove);
+        }
+    }
+
     putInZone(toMove, library, hand);
 }
 
@@ -450,7 +467,6 @@ MTGCardInstance * MTGGameZone::removeCard(MTGCardInstance * card, int createCopy
                 copy->view = card->view;
                 copy->isToken = card->isToken;
                 copy->X = card->X;
-                copy->XX = card->X/2;
 
                 //stupid bug with tokens...
                 if (card->model == card)
@@ -516,79 +532,79 @@ MTGCardInstance * MTGGameZone::findByName(string name)
     return NULL;
 }
 
-int MTGGameZone::hasType(const char * value)
+bool MTGGameZone::hasType(const char * value)
 {
     for (int i = 0; i < (nb_cards); i++)
     {
         if (cards[i]->hasType(value))
         {
-            return 1;
+            return true;
         }
     }
-    return 0;
+    return false;
 }
 
-int MTGGameZone::hasPrimaryType(const char * value,const char * secondvalue)
+bool MTGGameZone::hasPrimaryType(const char * value,const char * secondvalue)
 {
     for (int i = 0; i < (nb_cards); i++)
     {
         if (cards[i]->hasType(value) && cards[i]->hasType(secondvalue))
         {
-            return 1;
+            return true;
         }
     }
     return 0;
 }
 
-int MTGGameZone::hasSpecificType(const char * value,const char * secondvalue)
+bool MTGGameZone::hasSpecificType(const char * value,const char * secondvalue)
 {
     for (int i = 0; i < (nb_cards); i++)
     {
         if (cards[i]->hasType(value) && cards[i]->hasSubtype(secondvalue))
         {
-            return 1;
+            return true;
         }
     }
-    return 0;
+    return false;
 }
 
-int MTGGameZone::hasTypeButNotType(const char * value,const char * secondvalue)
+bool MTGGameZone::hasTypeButNotType(const char * value,const char * secondvalue)
 {
     for (int i = 0; i < (nb_cards); i++)
     {
         if (cards[i]->hasType(value) && cards[i]->hasSubtype(value) && !cards[i]->hasType(secondvalue) && !cards[i]->hasSubtype(secondvalue))
         {
-            return 1;
+            return true;
         }
     }
-    return 0;
+    return false;
 }
 
-int MTGGameZone::hasName(string value)
+bool MTGGameZone::hasName(string value)
 {
     for (int i = 0; i < (nb_cards); i++)
     {
         if (cards[i]->name == value)
         {
-            return 1;
+            return true;
         }
     }
-    return 0;
+    return false;
 }
 
-int MTGGameZone::hasColor(int value)
+bool MTGGameZone::hasColor(int value)
 {
     for (int i = 0; i < (nb_cards); i++)
     {
 			if (cards[i]->getManaCost()->hasColor(value) && cards[i]->getManaCost()->getConvertedCost() > 0)
         {
-            return 1;
+            return true;
         }
     }
-    return 0;
+    return false;
 }
 
-int MTGGameZone::hasX()
+bool MTGGameZone::hasX()
 {
     for (int i = 0; i < (nb_cards); i++)
     {
@@ -600,16 +616,16 @@ int MTGGameZone::hasX()
     return false;
 }
 
-int MTGGameZone::hasAbility(int ability)
+bool MTGGameZone::hasAbility(int ability)
 {
     for (int i = 0; i < (nb_cards); i++)
     {
         if (cards[i]->basicAbilities[ability])
         {
-            return 1;
+            return true;
         }
     }
-    return 0;
+    return false;
 }
 
 int MTGGameZone::seenThisTurn(TargetChooser * tc, int castMethod)
@@ -698,7 +714,7 @@ void MTGInPlay::untapAll()
     {
         MTGCardInstance * card = cards[i];
         card->setUntapping();
-        if (!card->basicAbilities[Constants::DOESNOTUNTAP])
+        if (!card->basicAbilities[(int)Constants::DOESNOTUNTAP])
         {
             if (card->frozen < 1)
             {
