@@ -4,8 +4,286 @@
 #include "MTGRules.h"
 #include "Translate.h"
 #include "Subtypes.h"
-#include "GameOptions.h"
+#include "Credits.h"
+#include "AllAbilities.h"
 
+MTGEventBonus::MTGEventBonus(int _id) :
+MTGAbility(_id,NULL)
+{
+    textAlpha = 0;
+    text = "";
+    for(int i = 0;i < 2;i++)
+    {
+        chain[i] = 0;
+        highestChain[i] = 0;
+        //-----------
+        army[i] = 0;
+        army1[i] = false;
+        army2[i] = false;
+        army3[i] = false;
+        //--------
+
+        toybonusgranted[i] = false;
+        toys[i] = 0;
+        beastbonusgranted[i] = false;
+        beast[i] = 0;
+        zombiebonusgranted[i] = false;
+        zombie[i] = 0;
+        knightbonusgranted[i] = false;
+        knight[i] = 0;
+        insectbonusgranted[i] = false;
+        insect[i] = 0;
+        elementalbonusgranted[i] = false;
+        elemental[i] = 0;
+        vampirebonusgranted[i] = false;
+        vampire[i] = 0;
+        clericbonusgranted[i] = false;
+        cleric[i] = 0;
+        elfbonusgranted[i] = false;
+        elf[i] = 0;
+        Angelbonusgranted[i] = false;
+        Angel[i] = 0;
+        dragonbonusgranted[i] = false;
+        dragon[i] = 0;
+    
+    }
+}
+int MTGEventBonus::receiveEvent(WEvent * event)
+{
+    Player * currentPlayer = game->currentPlayer;
+    //bonus for chain chain casting without tapping for mana or being interupted;
+    //note gaining mana from other sources is still possible.
+    //only spells going to the stack are counted.
+	if(game->turn <2)//this shouldnt trigger on first turn, chances are they are cheating.
+		return 0;
+    if (WEventCardTappedForMana* e = dynamic_cast<WEventCardTappedForMana*>(event))
+    {
+        if(e)
+        {
+            if(chain[currentPlayer->getId()]/5 > 0)
+            {
+                text = "Chain Broken!";
+                textAlpha = 255;
+            }
+            chain[currentPlayer->getId()] = 0;
+        }
+    }
+    if (event->type == WEvent::CHANGE_ZONE && !currentPlayer->isAI())
+    {
+        WEventZoneChange * e = (WEventZoneChange *) event;
+        if (e->to == currentPlayer->game->stack)
+        {
+            chain[currentPlayer->getId()]++;
+            if(chain[currentPlayer->getId()] > highestChain[currentPlayer->getId()])
+                highestChain[currentPlayer->getId()] = chain[currentPlayer->getId()];
+            if(chain[currentPlayer->getId()] > 4)
+            {
+
+                if(highestChain[currentPlayer->getId()] > 14)
+                {
+                    char buffer3[20];
+                    sprintf(buffer3,"Killer!-Combo %i",chain[currentPlayer->getId()]);
+                    grantAward(buffer3,100);
+                    //increase the chains bonus by 100 for every card after playing a chain of 15 in a match.
+                    //this is almost impossible would require superior draw and mana production.
+                }
+                else if(highestChain[currentPlayer->getId()] > 9)
+                {
+                    char buffer2[30];
+                    sprintf(buffer2,"Abundant Resources-Combo %i",chain[currentPlayer->getId()]);
+                    grantAward(buffer2,50);
+                    //increase the chains bonus by 50 for every card after playing a chain of 10 in a match.
+                    //this is extremely hard to do. would require a very well built deck an an abundence of mana
+                    //to spend in a single go combined with decent card drawing.
+                }
+                else if(highestChain[currentPlayer->getId()] > 4)
+                {
+                    char buffer[20];
+                    sprintf(buffer,"Chained-Combo %i",chain[currentPlayer->getId()]);
+                    grantAward(buffer,chain[currentPlayer->getId()]);
+                    //gain credits for every card played after you played a chain of 5
+                    //during the match. this would require a very decent hand to do
+                    //and good mana production.
+                }
+            }
+        }
+        //end of chain bonuses
+        //==========================
+        //creatures entering play consecutively will allow you a chance 
+        //to gain a bonus for maintaining force sizes, it will trigger every 10th
+        //creature which enters play consecutively.
+        if (e->to == currentPlayer->game->inPlay && !currentPlayer->isAI())
+        {
+            if(e->card->hasType(Subtypes::TYPE_CREATURE))
+                army[currentPlayer->getId()]++;
+            else
+                army[currentPlayer->getId()] = 0;
+            if(army[currentPlayer->getId()] > 9)
+            {
+                //this might seem easy at first glance, but you have to both maintain a high
+                //creature count, and triggers when 10 or more enter consecutively. if any thing else
+                //enters play the count is reset.
+                army[currentPlayer->getId()] = 0;
+                int forceSize = currentPlayer->inPlay()->countByType("creature");
+                if(forceSize > 40 && !army3[currentPlayer->getId()])
+                {
+                    grantAward("Malignant Conqueror Bonus!",1000);
+                    army3[currentPlayer->getId()] = true;
+                }
+                else if(forceSize > 19 && !army2[currentPlayer->getId()])
+                {
+                    grantAward("Extreme Infantry Bonus!",500);
+                    army2[currentPlayer->getId()] = true;
+                }
+                else if(forceSize > 9 && !army1[currentPlayer->getId()])
+                {
+                    grantAward("Deadly Force Bonus!",250);
+                    army1[currentPlayer->getId()] = true;
+                }
+            }
+            //////bonus for having a LOT of specific type.
+            //not else'd becuase it is possible for a card to contain
+            //more then one of the types, and for more then one to trigger.
+            if(e->card->hasType(Subtypes::TYPE_ARTIFACT))
+                toys[currentPlayer->getId()]++;
+            if(e->card->isCreature())
+            {
+                if(e->card->hasType("beast"))
+                    beast[currentPlayer->getId()]++;
+                if(e->card->hasType("vampire"))
+                    vampire[currentPlayer->getId()]++;
+                if(e->card->hasType("insect"))
+                    insect[currentPlayer->getId()]++;
+                if(e->card->hasType("elemental"))
+                    elemental[currentPlayer->getId()]++;
+                if(e->card->hasType("zombie"))
+                    zombie[currentPlayer->getId()]++;
+                if(e->card->hasType("soldier")||e->card->hasType("knight")||e->card->hasType("warrior"))
+                    knight[currentPlayer->getId()]++;
+                if(e->card->hasType("cleric")||e->card->hasType("shaman")||e->card->hasType("druid"))
+                    cleric[currentPlayer->getId()]++;
+                if(e->card->hasType("elf"))
+                    elf[currentPlayer->getId()]++;
+                if(e->card->hasType("angel")||e->card->hasType("spirit"))
+                    Angel[currentPlayer->getId()]++;
+                if(e->card->hasType("dragon")||e->card->hasType("wurm")||e->card->hasType("drake")||e->card->hasType("snake")||e->card->hasType("hydra"))
+                    dragon[currentPlayer->getId()]++;
+            }
+            if(toys[currentPlayer->getId()] > 30 && !toybonusgranted[currentPlayer->getId()])
+            {
+                grantAward("Toy Collector!",300);
+                toybonusgranted[currentPlayer->getId()] = true;
+            }
+            if(beast[currentPlayer->getId()] > 30 && !beastbonusgranted[currentPlayer->getId()])
+            {
+                grantAward("Beast Tamer!",300);
+                beastbonusgranted[currentPlayer->getId()] = true;
+            }
+            if(vampire[currentPlayer->getId()] > 30 && !vampirebonusgranted[currentPlayer->getId()])
+            {
+                grantAward("Vampire King!",300);
+                vampirebonusgranted[currentPlayer->getId()] = true;
+            }
+            if(insect[currentPlayer->getId()] > 30 && !insectbonusgranted[currentPlayer->getId()])
+            {
+                grantAward("Lord of Swarms!",300);
+                insectbonusgranted[currentPlayer->getId()] = true;
+            }
+            if(elemental[currentPlayer->getId()] > 30 && !elementalbonusgranted[currentPlayer->getId()])
+            {
+                grantAward("Master of Elements!",300);
+                elementalbonusgranted[currentPlayer->getId()] = true;
+            }
+            if(zombie[currentPlayer->getId()] > 30 && !zombiebonusgranted[currentPlayer->getId()])
+            {
+                grantAward("Zombie Apocalypse!",300);
+                zombiebonusgranted[currentPlayer->getId()] = true;
+            }
+            if(knight[currentPlayer->getId()] > 30 && !knightbonusgranted[currentPlayer->getId()])
+            {
+                grantAward("Sword And Shield!",300);
+                knightbonusgranted[currentPlayer->getId()] = true;
+            }
+            if(cleric[currentPlayer->getId()] > 30 && !clericbonusgranted[currentPlayer->getId()])
+            {
+                grantAward("Medic!",300);
+                clericbonusgranted[currentPlayer->getId()] = true;
+            }
+
+            if(elf[currentPlayer->getId()] > 30 && !elfbonusgranted[currentPlayer->getId()])
+            {
+                grantAward("The Promenade!",300);
+                elfbonusgranted[currentPlayer->getId()] = true;
+            }
+            if(Angel[currentPlayer->getId()] > 30 && !Angelbonusgranted[currentPlayer->getId()])
+            {
+                grantAward("Heavenly Host!",300);
+                Angelbonusgranted[currentPlayer->getId()] = true;
+            }
+            if(dragon[currentPlayer->getId()] > 30 && !dragonbonusgranted[currentPlayer->getId()])
+            {
+                grantAward("Teeth And Scales!",300);
+                dragonbonusgranted[currentPlayer->getId()] = true;
+            }
+        }
+    }
+    //bonus for dealing 100+ damage from a single source
+    WEventDamage * damageEvent = dynamic_cast<WEventDamage *> (event);
+    if(damageEvent && !currentPlayer->isAI())
+    {
+        MTGCardInstance * damageSource = (MTGCardInstance*)damageEvent->getTarget(damageEvent->TARGET_FROM);
+        if(damageSource && damageSource->controller() == currentPlayer && damageEvent->damage->damage > 99)
+            grantAward("Overkill!",500);
+    }
+    return 1;
+}
+
+void MTGEventBonus::grantAward(string awardName,int amount)
+{
+    JSample * sample = WResourceManager::Instance()->RetrieveSample("bonus.wav");
+    if (sample)
+    {
+        JSoundSystem::GetInstance()->PlaySample(sample);
+    }
+    text = awardName;
+    textAlpha = 255;
+    Credits::addCreditBonus(amount);
+}
+
+int MTGEventBonus::testDestroy()
+{
+    return 0;
+}
+
+void MTGEventBonus::Update(float dt)
+{
+    if (textAlpha)
+    {
+        textAlpha -= static_cast<int> (200 * dt);
+        if (textAlpha < 0)
+            textAlpha = 0;
+    }
+    MTGAbility::Update(dt);
+}
+
+void MTGEventBonus::Render()
+{
+    if (!textAlpha)
+        return;
+    WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::OPTION_FONT/*MENU_FONT*/);
+    mFont->SetScale(2 - (float) textAlpha / 130);
+    mFont->SetColor(ARGB(255,255,255,255));
+    mFont->DrawString(text.c_str(), SCREEN_WIDTH / 2, SCREEN_HEIGHT / 2, JGETEXT_CENTER);
+}
+
+MTGEventBonus * MTGEventBonus::clone() const
+{
+    MTGEventBonus * a = NEW MTGEventBonus(*this);
+    a->isClone = 1;
+    return a;
+}
+
+//
 MTGPutInPlayRule::MTGPutInPlayRule(int _id) :
 MTGAbility(_id, NULL)
 {
@@ -19,14 +297,14 @@ int MTGPutInPlayRule::isReactingToClick(MTGCardInstance * card, ManaCost * mana)
     Player * currentPlayer = game->currentPlayer;
     if (!player->game->hand->hasCard(card))
         return 0;
-    if ((game->turn < 1) && (cardsinhand != 0) && (card->basicAbilities[Constants::LEYLINE])
+    if ((game->turn < 1) && (cardsinhand != 0) && (card->basicAbilities[(int)Constants::LEYLINE])
         && game->currentGamePhase == Constants::MTG_PHASE_FIRSTMAIN
         && game->players[0]->game->graveyard->nb_cards == 0
         && game->players[0]->game->exile->nb_cards == 0
         )
     {
 
-        if (card->basicAbilities[Constants::LEYLINE])
+        if (card->basicAbilities[(int)Constants::LEYLINE])
         {
             MTGCardInstance * copy = player->game->putInZone(card, player->game->hand, player->game->temp);
             Spell * spell = NEW Spell(copy);
@@ -40,7 +318,7 @@ int MTGPutInPlayRule::isReactingToClick(MTGCardInstance * card, ManaCost * mana)
 
     if (card->isLand())
     {
-        if (currentPlayer->game->playRestrictions->canPutIntoZone(card, currentPlayer->game->inPlay) == PlayRestriction::CANT_PLAY)
+        if (game->currentActionPlayer->game->playRestrictions->canPutIntoZone(card, game->currentActionPlayer->game->inPlay) == PlayRestriction::CANT_PLAY)
             return 0;
         if (player == currentPlayer
             && (game->currentGamePhase == Constants::MTG_PHASE_FIRSTMAIN || game->currentGamePhase == Constants::MTG_PHASE_SECONDMAIN)
@@ -56,7 +334,7 @@ int MTGPutInPlayRule::isReactingToClick(MTGCardInstance * card, ManaCost * mana)
         )
     {
 
-        if (currentPlayer->game->playRestrictions->canPutIntoZone(card, currentPlayer->game->stack) == PlayRestriction::CANT_PLAY)
+        if (game->currentActionPlayer->game->playRestrictions->canPutIntoZone(card, game->currentActionPlayer->game->stack) == PlayRestriction::CANT_PLAY)
             return 0;
         ManaCost * playerMana = player->getManaPool();
         ManaCost * cost = card->getManaCost();
@@ -64,8 +342,6 @@ int MTGPutInPlayRule::isReactingToClick(MTGCardInstance * card, ManaCost * mana)
 #ifdef WIN32
         cost->Dump();
 #endif
-        if(!cost->getConvertedCost() && card->getManaCost()->suspend)
-            return 0;
         //cost of card.
         if (playerMana->canAfford(cost))
         {
@@ -123,6 +399,17 @@ int MTGPutInPlayRule::reactToClick(MTGCardInstance * card)
 
     ManaCost * previousManaPool = NEW ManaCost(player->getManaPool());
     int payResult = player->getManaPool()->pay(card->getManaCost());
+    if (card->getManaCost()->kicker && OptionKicker::KICKER_ALWAYS == options[Options::KICKERPAYMENT].number)
+    {
+        ManaCost * withKickerCost= NEW ManaCost(card->getManaCost());
+        withKickerCost->add(withKickerCost->kicker);
+        if (previousManaPool->canAfford(withKickerCost))
+        {
+            player->getManaPool()->pay(card->getManaCost()->kicker);
+            payResult = ManaCost::MANA_PAID_WITH_KICKER;
+        }
+        delete withKickerCost;
+    }
     card->getManaCost()->doPayExtra();
     ManaCost * spellCost = previousManaPool->Diff(player->getManaPool());
 
@@ -162,7 +449,6 @@ int MTGPutInPlayRule::reactToClick(MTGCardInstance * card)
         if (!card->has(Constants::STORM))
         {
             copy->X = spell->computeX(copy);
-            copy->XX = spell->computeXX(copy);
         }
     }
 
@@ -188,8 +474,134 @@ MTGPutInPlayRule * MTGPutInPlayRule::clone() const
     return a;
 }
 
-//cast from anywhere possible with this??
 
+//kicker
+MTGKickerRule::MTGKickerRule(int _id) :
+MTGPutInPlayRule(_id)
+{
+    aType = MTGAbility::PUT_INTO_PLAY_WITH_KICKER;
+}
+int MTGKickerRule::isReactingToClick(MTGCardInstance * card, ManaCost * mana)
+{
+    if(OptionKicker::KICKER_ALWAYS == options[Options::KICKERPAYMENT].number)
+        return 0;
+    Player * player = game->currentlyActing();
+    if(!player->game->hand->hasCard(card))
+    return 0;
+    ManaCost * kicker = card->getManaCost()->kicker;
+    if(!kicker)
+    {
+    SAFE_DELETE(kicker);
+        return 0;
+    }
+    ManaCost * playerMana = player->getManaPool();
+    ManaCost * withKickerCost= NEW ManaCost(card->getManaCost());
+    withKickerCost->add(withKickerCost->kicker);
+    if(!playerMana->canAfford(withKickerCost))
+    {
+        delete withKickerCost;
+        return 0;
+    }
+    delete withKickerCost;
+    
+    
+    return 1;
+}
+
+int MTGKickerRule::reactToClick(MTGCardInstance * card)
+{
+    if(!isReactingToClick(card, NULL))
+        return 0;
+        
+    Player * player = game->currentlyActing();
+    ManaCost * withKickerCost= NEW ManaCost(card->getManaCost());//using pointers here alters the real cost of the card.
+    withKickerCost->add(withKickerCost->kicker);
+    card->paymenttype = MTGAbility::PUT_INTO_PLAY_WITH_KICKER;
+    if (withKickerCost->isExtraPaymentSet())
+    {
+        if (!game->targetListIsSet(card))
+        {
+        delete withKickerCost;
+            return 0;
+        }
+    }
+    else
+    {
+        withKickerCost->setExtraCostsAction(this, card);
+        game->mExtraPayment = withKickerCost->extraCosts;
+        delete withKickerCost;
+        return 0;
+    }
+
+    ManaCost * previousManaPool = NEW ManaCost(player->getManaPool());
+    player->getManaPool()->pay(withKickerCost);
+    withKickerCost->doPayExtra();
+    ManaCost * spellCost = previousManaPool->Diff(player->getManaPool());
+    delete withKickerCost;
+    delete previousManaPool;
+    if (card->isLand())
+    {
+        MTGCardInstance * copy = player->game->putInZone(card, player->game->hand, player->game->temp);
+        Spell * spell = NEW Spell(0,copy,NULL,NULL, ManaCost::MANA_PAID_WITH_KICKER);
+        spell->resolve();
+        delete spellCost;
+        delete spell;
+    }
+    else
+    {
+        Spell * spell = NULL;
+        MTGCardInstance * copy = player->game->putInZone(card, player->game->hand, player->game->stack);
+        if (game->targetChooser)
+        {
+            spell = game->mLayers->stackLayer()->addSpell(copy, game->targetChooser, spellCost, ManaCost::MANA_PAID_WITH_KICKER, 0);
+            game->targetChooser = NULL;
+        }
+        else
+        {
+            spell = game->mLayers->stackLayer()->addSpell(copy, NULL, spellCost, ManaCost::MANA_PAID_WITH_KICKER, 0);
+        }
+
+        if (card->has(Constants::STORM))
+        {
+            int storm = player->game->stack->seenThisTurn("*", Constants::CAST_ALL) + player->opponent()->game->stack->seenThisTurn("*", Constants::CAST_ALL);
+            ManaCost * stormSpellCost = player->getManaPool();
+            for (int i = storm; i > 1; i--)
+            {
+                spell = game->mLayers->stackLayer()->addSpell(copy, NULL, stormSpellCost, ManaCost::MANA_PAID_WITH_KICKER, 1);
+
+            }
+        }//end of storm
+        if (!card->has(Constants::STORM))
+        {
+            copy->X = spell->computeX(copy);
+        }
+    }
+
+    return 1;
+}
+
+int MTGKickerRule::testDestroy()
+{
+    return 0;
+}
+
+ostream& MTGKickerRule::toString(ostream& out) const
+{
+    out << "MTGKickerRule ::: (";
+    return MTGAbility::toString(out) << ")";
+}
+
+MTGKickerRule * MTGKickerRule::clone() const
+{
+    MTGKickerRule * a = NEW MTGKickerRule(*this);
+    a->isClone = 1;
+    return a;
+}
+
+
+
+
+//cast from anywhere possible with this??
 //Alternative cost rules
 //-------------------------------------------------------------------------
 //-------------------------------------------------------------------------
@@ -218,15 +630,15 @@ int MTGAlternativeCostRule::isReactingToClick(MTGCardInstance * card, ManaCost *
 
     if (!alternateManaCost)
         return 0;
-
-    if(!allowedToCast(card,player))
-        return 0;
     if(!allowedToAltCast(card,player))
         return 0;
 
+    if(card->model->data->getManaCost()->alternative && card->model->data->getManaCost()->alternative->alternativeName.size())
+        alternativeName = card->model->data->getManaCost()->alternative->alternativeName;
+
     if (card->isLand())
     {
-        if (currentPlayer->game->playRestrictions->canPutIntoZone(card, currentPlayer->game->inPlay) == PlayRestriction::CANT_PLAY)
+        if (game->currentActionPlayer->game->playRestrictions->canPutIntoZone(card, game->currentActionPlayer->game->inPlay) == PlayRestriction::CANT_PLAY)
             return 0;
         if (player == currentPlayer
             && (game->currentGamePhase == Constants::MTG_PHASE_FIRSTMAIN
@@ -240,7 +652,7 @@ int MTGAlternativeCostRule::isReactingToClick(MTGCardInstance * card, ManaCost *
         || game->currentGamePhase == Constants::MTG_PHASE_SECONDMAIN))
         )
     {
-        if (currentPlayer->game->playRestrictions->canPutIntoZone(card, currentPlayer->game->stack) == PlayRestriction::CANT_PLAY)
+        if (game->currentActionPlayer->game->playRestrictions->canPutIntoZone(card, game->currentActionPlayer->game->stack) == PlayRestriction::CANT_PLAY)
             return 0;
         ManaCost * playerMana = player->getManaPool();
 
@@ -263,11 +675,9 @@ int MTGAlternativeCostRule::reactToClick(MTGCardInstance * card)
 		return 0;
 
 	ManaCost *alternateCost = card->getManaCost()->alternative;
-    Player * player = game->currentlyActing();
-    ManaCost * playerMana = player->getManaPool();
 	card->paymenttype = MTGAbility::ALTERNATIVE_COST;
 
-    return reactToClick(card, card->getManaCost()->alternative, ManaCost::MANA_PAID_WITH_ALTERNATIVE);
+    return reactToClick(card, alternateCost, ManaCost::MANA_PAID_WITH_ALTERNATIVE);
 }
 
 int MTGAlternativeCostRule::reactToClick(MTGCardInstance * card, ManaCost *alternateCost, int alternateCostType){
@@ -325,7 +735,6 @@ int MTGAlternativeCostRule::reactToClick(MTGCardInstance * card, ManaCost *alter
         else
         {
             copy->X = spell->computeX(copy);
-            copy->XX = copy->X/2;
         }
     }
 
@@ -368,10 +777,10 @@ MTGAlternativeCostRule(_id)
 int MTGBuyBackRule::isReactingToClick(MTGCardInstance * card, ManaCost * mana)
 {
     Player * player = game->currentlyActing();
-    if(!allowedToCast(card,player))
-        return 0;
     if (!player->game->hand->hasCard(card))
         return 0;
+    if(!allowedToCast(card,player))
+        return 0;
     return MTGAlternativeCostRule::isReactingToClick( card, mana, card->getManaCost()->BuyBack );
 }
 
@@ -380,8 +789,6 @@ int MTGBuyBackRule::reactToClick(MTGCardInstance * card)
     if (!isReactingToClick(card))
         return 0;
 
-    Player *player = game->currentlyActing();
-    ManaCost * playerMana = player->getManaPool();
     ManaCost * alternateCost = card->getManaCost()->BuyBack;
     
     card->paymenttype = MTGAbility::BUYBACK_COST;
@@ -429,9 +836,8 @@ int MTGFlashBackRule::isReactingToClick(MTGCardInstance * card, ManaCost * mana)
 
 int MTGFlashBackRule::reactToClick(MTGCardInstance * card) 
 {
-    Player *player = game->currentlyActing();
     ManaCost * alternateCost = card->getManaCost()->FlashBack;
-    ManaCost * playerMana = game->currentlyActing()->getManaPool();
+    
     if (!isReactingToClick(card))
         return 0;
 
@@ -487,8 +893,7 @@ int MTGRetraceRule::reactToClick(MTGCardInstance * card)
 {
     if (!isReactingToClick(card))
         return 0;
-    Player *player = game->currentlyActing();
-    ManaCost * playerMana = player->getManaPool();
+    
     ManaCost * alternateCost = card->getManaCost()->Retrace;
     
     card->paymenttype = MTGAbility::RETRACE_COST;
@@ -630,15 +1035,13 @@ MTGMorphCostRule::MTGMorphCostRule(int _id) :
 }
 int MTGMorphCostRule::isReactingToClick(MTGCardInstance * card, ManaCost * mana)
 {
-    int cardsinhand = game->players[0]->game->hand->nb_cards;
+
     Player * player = game->currentlyActing();
     Player * currentPlayer = game->currentPlayer;
     if (!player->game->hand->hasCard(card))
         return 0;
     if (!card->getManaCost()->morph)
         return 0;
-    if(!allowedToCast(card,player))
-        return 0;
     if(!allowedToAltCast(card,player))
         return 0;
     //note lands can morph too, this is different from other cost types.
@@ -648,14 +1051,16 @@ int MTGMorphCostRule::isReactingToClick(MTGCardInstance * card, ManaCost * mana)
         || game->currentGamePhase == Constants::MTG_PHASE_SECONDMAIN))
         )
     {
-        if (currentPlayer->game->playRestrictions->canPutIntoZone(card, currentPlayer->game->stack) == PlayRestriction::CANT_PLAY)
+        if (game->currentActionPlayer->game->playRestrictions->canPutIntoZone(card, game->currentActionPlayer->game->stack) == PlayRestriction::CANT_PLAY)
             return 0;
         ManaCost * playerMana = player->getManaPool();
-        ManaCost * cost = card->getManaCost();
         ManaCost * morph = card->getManaCost()->morph;
+
 #ifdef WIN32
+        ManaCost * cost = card->getManaCost();
         cost->Dump();
 #endif
+        
         //cost of card.
         if (morph && playerMana->canAfford(morph))
         {
@@ -723,7 +1128,6 @@ int MTGMorphCostRule::reactToClick(MTGCardInstance * card)
     if (!card->has(Constants::STORM))
     {
         copy->X = spell->computeX(copy);
-        copy->XX = copy->X/2;
     }
     return 1;
 }
@@ -919,6 +1323,15 @@ int MTGCombatTriggersRule::receiveEvent(WEvent *e)
     }
     if (dynamic_cast<WEventBlockersChosen*>(e))
     {
+        MTGGameZone* opponentZone = game->currentPlayer->opponent()->game->inPlay;
+        for (int i = 0; i < opponentZone->nb_cards; i++)
+        {
+            MTGCardInstance* card = opponentZone->cards[i];
+            if (card && card->didblocked)
+            {
+                card->eventblocked(card->getNextOpponent());
+            }
+        }
         Player * p = game->currentPlayer;
         MTGGameZone * z = p->game->inPlay;
         for (int i = 0; i < z->nb_cards; i++)
@@ -927,21 +1340,18 @@ int MTGCombatTriggersRule::receiveEvent(WEvent *e)
             if (card && card->isAttacker() && !card->blocked)
             {
                 card->eventattackednotblocked();
-                card->notblocked += 1;
+                card->notblocked = 1;
             }
             if (card && card->isAttacker() && card->blocked)
             {
-                card->eventattackedblocked();
-            }
-        }
 
-        MTGGameZone* opponentZone = game->currentPlayer->opponent()->game->inPlay;
-        for (int i = 0; i < opponentZone->nb_cards; i++)
-        {
-            MTGCardInstance* card = opponentZone->cards[i];
-            if (card && card->didblocked > 0)
-            {
-                card->eventblocked();
+                MTGCardInstance * opponent = card->getNextOpponent();
+                while (opponent)
+                {
+                    card->eventattackedblocked(opponent);
+                    opponent = card->getNextOpponent(opponent);
+                }
+
             }
         }
     }
@@ -1220,6 +1630,164 @@ MTGMomirRule * MTGMomirRule::clone() const
     return a;
 }
 
+//stone hewer game mode
+//in stonehewer when ever a creature enters the battlefield
+//it enters play with a equipment choosen at random with a converted manacost
+//less than or equal to the creature.
+//note this can kill your creature if the equipment contains negitive toughness
+
+int MTGStoneHewerRule::initialized = 0;
+vector<int> MTGStoneHewerRule::pool[20];
+
+MTGStoneHewerRule::MTGStoneHewerRule(int _id, MTGAllCards * _collection) :
+MTGAbility(_id, NULL)
+{
+    collection = _collection;
+    if (!initialized)
+    {
+        for (size_t i = 0; i < collection->ids.size(); i++)
+        {
+            MTGCard * card = collection->collection[collection->ids[i]];
+			if (card->data->hasSubtype("equipment") && (card->getRarity() != Constants::RARITY_T) && //remove tokens
+                card->setId != MTGSets::INTERNAL_SET //remove cards that are defined in primitives. Those are workarounds (usually tokens) and should only be used internally
+                )
+            {
+                int convertedCost = card->data->getManaCost()->getConvertedCost();
+                if (convertedCost > 20)
+                    continue;
+                pool[convertedCost].push_back(card->getMTGId());
+            }
+        }
+        initialized = 1;
+    }
+}
+
+int MTGStoneHewerRule::receiveEvent(WEvent * event)
+{
+	WEventZoneChange * e = (WEventZoneChange *) event;
+	if (e->to == game->currentlyActing()->game->inPlay && e->card->isCreature())
+	{
+		int eId = genRandomEquipId(e->card->getManaCost()->getConvertedCost());
+		MTGCardInstance * card = genEquip(eId);
+		if(card)
+		{
+			game->currentlyActing()->game->temp->addCard(card);
+			Spell * spell = NEW Spell(card);
+			spell->resolve();
+			spell->source->isToken = 1;
+			GameObserver * g = g->GetInstance();
+			for (size_t i = 1; i < g->mLayers->actionLayer()->mObjects.size(); i++)
+			{
+				MTGAbility * a = ((MTGAbility *) g->mLayers->actionLayer()->mObjects[i]);
+				AEquip * eq = dynamic_cast<AEquip*> (a);
+				if (eq && eq->source == spell->source)
+				{
+					((AEquip*)a)->unequip();
+					((AEquip*)a)->equip(e->card);
+				}
+			}
+			SAFE_DELETE(spell);
+		}
+		return 1;
+	}
+	return 0;
+}
+
+MTGCardInstance * MTGStoneHewerRule::genEquip(int id)
+{
+    if (!id)
+        return NULL;
+    Player * p = game->currentlyActing();
+    MTGCard * card = collection->getCardById(id);
+    return NEW MTGCardInstance(card, p->game);
+}
+
+int MTGStoneHewerRule::genRandomEquipId(int convertedCost)
+{
+	if (convertedCost >= 20)
+		convertedCost = 19;
+    int total_cards = 0;
+    int i = (WRand() % int(convertedCost+1));//+1 becuase we want to generate a random "<=" the coverted.
+    while (!total_cards && i >= 0)
+    {
+        total_cards = pool[i].size();
+        convertedCost = i;
+        i--;
+    }
+    if (!total_cards)
+        return 0;
+    int start = (WRand() % total_cards);
+    return pool[convertedCost][start];
+}
+
+//The StoneHewer is never destroyed
+int MTGStoneHewerRule::testDestroy()
+{
+    return 0;
+}
+
+ostream& MTGStoneHewerRule::toString(ostream& out) const
+{
+    out << "MTGStoneHewerRule ::: pool : " << pool << " ; initialized : " << initialized 
+        << " ; collection : " << collection << "(";
+    return MTGAbility::toString(out) << ")";
+}
+
+MTGStoneHewerRule * MTGStoneHewerRule::clone() const
+{
+    MTGStoneHewerRule * a = NEW MTGStoneHewerRule(*this);
+    a->isClone = 1;
+    return a;
+}
+
+//------------------
+//Hermit druid mode places a random land from your deck into play during each of your upkeeps
+MTGHermitRule::MTGHermitRule(int _id) :
+MTGAbility(_id, NULL)
+{
+}
+
+int MTGHermitRule::receiveEvent(WEvent * event)
+{
+	WEventPhaseChange * e = dynamic_cast<WEventPhaseChange*>(event);
+	if (e && e->from->id == Constants::MTG_PHASE_UNTAP)
+	{
+		MTGCardInstance * lcard = NULL;
+		vector<MTGCardInstance*>lands = vector<MTGCardInstance*>();
+		for(int i = 0; i < game->currentPlayer->game->library->nb_cards-1; i++)
+		{
+			MTGCardInstance * temp = game->currentPlayer->game->library->cards[i];
+			if(temp && temp->isLand())
+				lands.push_back(temp);
+		}
+		if(lands.size())
+			lcard = lands[WRand() % lands.size()];
+		if(lcard)
+		{
+			MTGCardInstance * copy = game->currentPlayer->game->putInZone(lcard,game->currentPlayer->game->library, game->currentPlayer->game->temp);
+            Spell * spell = NEW Spell(copy);
+            spell->resolve();
+            delete spell;
+		}
+		return 1;
+	}
+	return 0;
+}
+
+//The hermit is never destroyed
+int MTGHermitRule::testDestroy()
+{
+    return 0;
+}
+
+MTGHermitRule * MTGHermitRule::clone() const
+{
+    MTGHermitRule * a = NEW MTGHermitRule(*this);
+    a->isClone = 1;
+    return a;
+}
+//--------------------
+
 //HUDDisplay
 int HUDDisplay::testDestroy()
 {
@@ -1353,7 +1921,7 @@ int MTGPersistRule::receiveEvent(WEvent * event)
     {
         WEventZoneChange * e = (WEventZoneChange *) event;
         MTGCardInstance * card = e->card->previous;
-        if (card && card->basicAbilities[Constants::PERSIST] && !card->counters->hasCounter(-1, -1))
+        if (card && card->basicAbilities[(int)Constants::PERSIST] && !card->counters->hasCounter(-1, -1))
         {
             int ok = 0;
             for (int i = 0; i < 2; i++)
@@ -1369,7 +1937,7 @@ int MTGPersistRule::receiveEvent(WEvent * event)
                 Player * p = game->players[i];
                 if (e->to == p->game->graveyard)
                 {
-                    MTGCardInstance * copy = p->game->putInZone(e->card, p->game->graveyard, e->card->owner->game->stack);
+					MTGCardInstance * copy = p->game->putInZone(e->card, p->game->graveyard, e->card->owner->game->temp);
                     if (!copy)
                     {
                         DebugTrace("MTGRULES: couldn't move card for persist");
@@ -1403,6 +1971,84 @@ MTGPersistRule * MTGPersistRule::clone() const
     return a;
 }
 
+//vampires rule
+//handled seperately as a rule since we only want one object to send out events that a card was "vampired".
+//otherwise vampire event is sent per instance of @vampired on the battlefield, multipling the results.
+MTGVampireRule::MTGVampireRule(int _id) :
+MTGAbility(_id, NULL)
+{
+}
+;
+
+int MTGVampireRule::receiveEvent(WEvent * event)
+{
+    WEventDamage * e = dynamic_cast<WEventDamage *> (event);
+    WEventZoneChange * z = dynamic_cast<WEventZoneChange *> (event);
+    WEventPhaseChange * pe = dynamic_cast<WEventPhaseChange*>(event);
+    if (e)
+    {
+        if(!e->damage->damage)
+            return 0;
+        if (!e->damage->target) 
+            return 0;
+        if(e->damage->target->typeAsTarget() != TARGET_CARD)
+            return 0;
+        MTGCardInstance * newVictem = (MTGCardInstance*)(e->damage->target);
+        MTGCardInstance * vampire = (MTGCardInstance*)(e->damage->source);
+
+        victems[newVictem].push_back(vampire);
+
+    }
+    else if (z)
+    {
+        MTGCardInstance * card = z->card->previous;
+        if(card && victems[card].empty())
+            return 0;
+        std::sort(victems[card].begin(), victems[card].end());
+        victems[card].erase(std::unique(victems[card].begin(), victems[card].end()), victems[card].end());
+        //sort and remove duplicates, we only want one event of a vampire damaging a card stored per victem.
+        for(unsigned int w = 0;w < victems[card].size();w++)
+        {
+            if(victems[card].at(w) == NULL) 
+                continue;
+            Player * p = card->controller();
+            if (z->from == p->game->inPlay && z->to == p->game->graveyard)
+            {
+                if(card == z->card->previous)
+                {
+                    WEvent * e = NEW WEventVampire(card,victems[card].at(w),card);
+                    game->receiveEvent(e);
+                }
+            }
+        }
+        return 0;
+    }
+    else if (pe)
+    {
+        if( pe->from->id == Constants::MTG_PHASE_ENDOFTURN)
+        {
+            victems.clear();
+        }
+    }
+    return 0;
+}
+
+ostream& MTGVampireRule::toString(ostream& out) const
+{
+    out << "MTGVampireRule ::: (";
+    return MTGAbility::toString(out) << ")";
+}
+int MTGVampireRule::testDestroy()
+{
+    return 0;
+}
+MTGVampireRule * MTGVampireRule::clone() const
+{
+    MTGVampireRule * a = NEW MTGVampireRule(*this);
+    a->isClone = 1;
+    return a;
+}
+/////////////////////////////////////////////////
 //unearth rule----------------------------------
 //if the card leaves play, exile it instead.
 MTGUnearthRule::MTGUnearthRule(int _id) :
@@ -1426,7 +2072,7 @@ int MTGUnearthRule::receiveEvent(WEvent * event)
             e->card->fresh = 1;
         }
 
-        if (card && card->basicAbilities[Constants::UNEARTH])
+        if (card && card->basicAbilities[(int)Constants::UNEARTH])
         {
             int ok = 0;
             for (int i = 0; i < 2; i++)
@@ -1633,7 +2279,7 @@ int MTGLifelinkRule::receiveEvent(WEvent * event)
         WEventDamage * e = (WEventDamage *) event;
         Damage * d = e->damage;
         MTGCardInstance * card = d->source;
-        if (d->damage > 0 && card && card->basicAbilities[Constants::LIFELINK])
+        if (d->damage > 0 && card && card->basicAbilities[(int)Constants::LIFELINK])
         {
             card->controller()->gainLife(d->damage);
             return 1;
@@ -1684,7 +2330,7 @@ int MTGDeathtouchRule::receiveEvent(WEvent * event)
             return 0;
         MTGCardInstance * _target = (MTGCardInstance *) (d->target);
 
-        if (card->basicAbilities[Constants::DEATHTOUCH])
+        if (card->basicAbilities[(int)Constants::DEATHTOUCH])
         {
             _target->destroy();
             return 1;
