@@ -9,6 +9,10 @@
 #include "WEvent.h"
 #include "MTGAbility.h"
 
+#include "iterator"
+
+SUPPORT_OBJECT_ANALYTICS(ManaCost)
+
 ManaCost * ManaCost::parseManaCost(string s, ManaCost * _manaCost, MTGCardInstance * c)
 {
     ManaCost * manaCost;
@@ -92,7 +96,7 @@ ManaCost * ManaCost::parseManaCost(string s, ManaCost * _manaCost, MTGCardInstan
                     case 't': //Tap
                         if (value == "t")
                         {
-                            //default Tap is handled outside of Manacost
+                            manaCost->addExtraCost(NEW TapCost);
                         }
                         else
                         {
@@ -143,6 +147,20 @@ ManaCost * ManaCost::parseManaCost(string s, ManaCost * _manaCost, MTGCardInstan
                         tc = tcf.createTargetChooser("creature|myBattlefield", c);
                         manaCost->addExtraCost(NEW Ninja(tc));
                         break;
+                    case 'p' :
+                        {
+                            SAFE_DELETE(tc);
+                            size_t start = value.find("(");
+                            size_t end = value.rfind(")");
+                            string manaType = value.substr(start + 1, end - start - 1);
+                            manaCost->addExtraCost(NEW LifeorManaCost(NULL,manaType));
+                            break;
+                        }
+                    case 'q':
+                        {
+                            manaCost->addExtraCost(NEW UnTapCost);
+                            break;
+                        }
                     case 'c': //Counters
                     {
                         size_t counter_start = value.find("(");
@@ -198,7 +216,8 @@ ManaCost * ManaCost::parseManaCost(string s, ManaCost * _manaCost, MTGCardInstan
                                     }
                                 }
                             }
-                            manaCost->addHybrid(colors[0], values[0], colors[1], values[1]);
+                            if (values[0] > 0 || values[1] > 0)
+                                manaCost->addHybrid(colors[0], values[0], colors[1], values[1]);
                         }
                         else
                         {
@@ -223,41 +242,100 @@ ManaCost::ManaCost()
 {
     init();
 }
-ManaCost::ManaCost(int _cost[], int nb_elems)
+
+ManaCost::ManaCost(int8_t _cost[], int nb_elems)
 {
     init();
-    int i;
-    int total = nb_elems;
-    for (i = 0; i < total; i++)
+    for (int i = 0; i < nb_elems; i++)
     {
         cost[_cost[i * 2]] = _cost[i * 2 + 1];
     }
 
 }
 
-ManaCost::ManaCost(ManaCost * _manaCost)
+// pointer copy constructor 
+
+ManaCost::ManaCost(ManaCost * manaCost)
 {
     init();
-    int i;
-    for (i = 0; i <= Constants::MTG_NB_COLORS; i++)
+    if ( !manaCost ) 
+        return;
+    for (int i = 0; i <= Constants::MTG_NB_COLORS; i++)
     {
-        cost[i] = _manaCost->getCost(i);
+        cost[i] = manaCost->getCost(i);
     }
+    hybrids = manaCost->hybrids;
+
+    kicker = NEW ManaCost( manaCost->kicker );
+    Retrace = NEW ManaCost( manaCost->Retrace );
+    BuyBack = NEW ManaCost( manaCost->BuyBack );
+    alternative = NEW ManaCost( manaCost->alternative );
+    FlashBack = NEW ManaCost( manaCost->FlashBack );
+    morph = NEW ManaCost( manaCost->morph );
+    suspend = NEW ManaCost( manaCost->suspend );
+
+    // TODO: Need to figure out if a deep copy is necessary
+    extraCosts = manaCost->extraCosts;
+}
+
+// Copy Constructor 
+
+ManaCost::ManaCost(const ManaCost& manaCost)
+#ifdef TRACK_OBJECT_USAGE
+    : InstanceCounter<ManaCost>(manaCost)
+#endif
+{
+    for (int i = 0; i <= Constants::MTG_NB_COLORS; i++)
+    {
+        cost[i] = manaCost.cost[i];      
+    }
+ 
+    hybrids = manaCost.hybrids;
+
+    // make new copies of the pointers for the deep copy
+    kicker = NEW ManaCost( manaCost.kicker );
+    Retrace = NEW ManaCost( manaCost.Retrace );
+    BuyBack = NEW ManaCost( manaCost.BuyBack );
+    alternative = NEW ManaCost( manaCost.alternative );
+    FlashBack = NEW ManaCost( manaCost.FlashBack );
+    morph = NEW ManaCost( manaCost.morph );
+    suspend = NEW ManaCost( manaCost.suspend );
+    
+    // TODO: Need to figure out if a deep copy is necessary
+    extraCosts = manaCost.extraCosts;
+}
+
+// operator=
+ManaCost & ManaCost::operator= (const ManaCost & manaCost)
+{
+    if ( this != &manaCost )
+    {
+        for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
+            cost[i] = manaCost.cost[i];
+
+        hybrids = manaCost.hybrids;
+        extraCosts = manaCost.extraCosts;
+        kicker = manaCost.kicker;
+        Retrace = manaCost.Retrace;
+        BuyBack = manaCost.BuyBack;
+        alternative = manaCost.alternative;
+        FlashBack = manaCost.FlashBack;
+        morph = manaCost.morph;
+        suspend = manaCost.suspend;
+    }
+    return *this;
 }
 
 ManaCost::~ManaCost()
 {
-    for (unsigned int i = 0; i < nbhybrids; i++)
-    {
-        SAFE_DELETE(hybrids[i]);
-    }
-
     SAFE_DELETE(extraCosts);
     SAFE_DELETE(kicker);
     SAFE_DELETE(alternative);
     SAFE_DELETE(BuyBack);
     SAFE_DELETE(FlashBack);
     SAFE_DELETE(Retrace);
+    SAFE_DELETE(morph);
+    SAFE_DELETE(suspend);
 }
 
 void ManaCost::x()
@@ -277,14 +355,31 @@ void ManaCost::init()
     {
         cost[i] = 0;
     }
-    nbhybrids = 0;
     extraCosts = NULL;
-    extraCostsIsCopy = 0;
     kicker = NULL;
     alternative = NULL;
     BuyBack = NULL;
     FlashBack = NULL;
     Retrace = NULL;
+    morph = NULL;
+    suspend = NULL;
+}
+
+void ManaCost::reinit()
+{
+    int i;
+    for (i = 0; i <= Constants::MTG_NB_COLORS; i++)
+    {
+        cost[i] = 0;
+    }
+    SAFE_DELETE(extraCosts);
+    SAFE_DELETE(kicker);
+    SAFE_DELETE(alternative);
+    SAFE_DELETE(BuyBack);
+    SAFE_DELETE(FlashBack);
+    SAFE_DELETE(Retrace);
+    SAFE_DELETE(morph);
+    SAFE_DELETE(suspend);
 }
 
 void ManaCost::copy(ManaCost * _manaCost)
@@ -295,15 +390,8 @@ void ManaCost::copy(ManaCost * _manaCost)
     {
         cost[i] = _manaCost->getCost(i);
     }
-    for (unsigned int i = 0; i < nbhybrids; i++)
-    {
-        SAFE_DELETE(hybrids[i]);
-    }
-    for (unsigned int i = 0; i < _manaCost->nbhybrids; i++)
-    {
-        hybrids[i] = NEW ManaCostHybrid((*_manaCost->hybrids[i]));
-    }
-    nbhybrids = _manaCost->nbhybrids;
+
+    hybrids = _manaCost->hybrids;
 
     SAFE_DELETE(extraCosts);
     if (_manaCost->extraCosts)
@@ -341,6 +429,18 @@ void ManaCost::copy(ManaCost * _manaCost)
         Retrace = NEW ManaCost();
         Retrace->copy(_manaCost->Retrace);
     }
+    SAFE_DELETE(morph);
+    if (_manaCost->morph)
+    {
+        morph = NEW ManaCost();
+        morph->copy(_manaCost->morph);
+    }
+    SAFE_DELETE(suspend);
+    if (_manaCost->suspend)
+    {
+        suspend = NEW ManaCost();
+        suspend->copy(_manaCost->suspend);
+    }
 }
 
 int ManaCost::getCost(int color)
@@ -350,18 +450,18 @@ int ManaCost::getCost(int color)
 
 ManaCostHybrid * ManaCost::getHybridCost(unsigned int i)
 {
-    if (nbhybrids <= i)
+    if (hybrids.size() <= i)
         return NULL;
-    return hybrids[i];
+    return &hybrids[i];
 }
 
 int ManaCost::hasColor(int color)
 {
     if (cost[color])
         return 1;
-    for (unsigned int i = 0; i < nbhybrids; i++)
+    for (size_t i = 0; i < hybrids.size(); i++)
     {
-        if (hybrids[i]->hasColor(color))
+        if (hybrids[i].hasColor(color))
             return 1;
     }
     return 0;
@@ -383,20 +483,28 @@ int ManaCost::getConvertedCost()
     {
         result += cost[i];
     }
-    for (unsigned int i = 0; i < nbhybrids; i++)
+    for (size_t i = 0; i < hybrids.size(); i++)
     {
-        result += hybrids[i]->getConvertedCost();
+        result += hybrids[i].getConvertedCost();
     }
+	if(extraCosts && extraCosts->costs.size())
+	{
+		for(unsigned int i = 0; i < extraCosts->costs.size();i++)
+		{
+			ExtraCost * pMana = dynamic_cast<LifeorManaCost*>(extraCosts->costs[i]);
+			if(pMana)
+				result++;
+		}
+	}
+
     return result;
 }
 
 int ManaCost::remove(int color, int value)
 {
-    cost[color] -= value;
-    if (cost[color] < 0)
-    {
-        cost[color] = 0;
-    }
+    assert (value >= 0);
+    int8_t toRemove = min(cost[color], (int8_t)value);
+    cost[color] -= toRemove;
     return 1;
 }
 
@@ -416,20 +524,35 @@ int ManaCost::add(ManaCost * _cost)
     {
         cost[i] += _cost->getCost(i);
     }
-    for (unsigned int i = 0; i < _cost->nbhybrids; i++)
+
+    std::copy(_cost->hybrids.begin(), _cost->hybrids.end(), std::back_inserter(hybrids));
+
+    return 1;
+}
+
+int ManaCost::remove(ManaCost * _cost)
+{
+    if (!_cost)
+        return 0;
+    for (unsigned int i = 0; i < Constants::MTG_NB_COLORS; i++)
     {
-        hybrids[nbhybrids] = NEW ManaCostHybrid((*_cost->hybrids[i]));
-        nbhybrids++;
+        int8_t toRemove = min(cost[i], (int8_t)_cost->getCost(i)); //we don't want to be negative
+        cost[i] -= toRemove;
+        assert(cost[i] >= 0);
     }
     return 1;
 }
 
+int ManaCost::removeAll(int color)
+{
+    cost[color] = 0;
+    return 1;
+}
+
 int ManaCost::addHybrid(int c1, int v1, int c2, int v2)
 {
-    ManaCostHybrid * h = NEW ManaCostHybrid(c1, v1, c2, v2);
-    hybrids[nbhybrids] = h;
-    nbhybrids++;
-    return nbhybrids;
+    hybrids.push_back(ManaCostHybrid(c1, v1, c2, v2));
+    return hybrids.size();
 }
 
 int ManaCost::addExtraCost(ExtraCost * _cost)
@@ -473,20 +596,6 @@ int ManaCost::pay(ManaCost * _cost)
     int result = MANA_PAID;
     ManaCost * toPay = NEW ManaCost();
     toPay->copy(_cost);
-
-    if (toPay->kicker)
-    {
-        toPay->add(toPay->kicker);
-        if (!canAfford(toPay))
-        {
-            toPay->copy(_cost);
-        }
-        else
-        {
-            result = MANA_PAID_WITH_KICKER;
-        }
-    }
-
     ManaCost * diff = Diff(toPay);
     for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
     {
@@ -525,37 +634,39 @@ int ManaCost::isPositive()
 
 }
 
-void ManaCost::randomDiffHybrids(ManaCost * _cost, int diff[])
+void ManaCost::randomDiffHybrids(ManaCost * _cost, int8_t diff[])
 {
-    int _nbhybrids = _cost->nbhybrids;
-    for (int i = 0; i < _nbhybrids; i++)
+    for (size_t i = 0; i < _cost->hybrids.size(); i++)
     {
-        ManaCostHybrid * h = _cost->hybrids[i];
-        diff[h->color1 * 2 + 1] -= h->value1;
+        ManaCostHybrid& h = _cost->hybrids[i];
+        diff[h.color1 * 2 + 1] -= h.value1;
     }
 }
 
-int ManaCost::tryToPayHybrids(ManaCostHybrid * _hybrids[], int _nbhybrids, int diff[])
+/**
+    starting from the end of the array (diff) 
+*/
+int ManaCost::tryToPayHybrids(std::vector<ManaCostHybrid>& _hybrids, int _nbhybrids, int8_t diff[])
 {
     if (!_nbhybrids)
         return 1;
     int result = 0;
-    ManaCostHybrid * h = _hybrids[_nbhybrids - 1];
-    if (diff[h->color1 * 2 + 1] >= h->value1)
+    ManaCostHybrid& h = _hybrids[_nbhybrids - 1];
+    if (diff[h.color1 * 2 + 1] >= h.value1)
     {
-        diff[h->color1 * 2 + 1] -= h->value1;
+        diff[h.color1 * 2 + 1] -= h.value1;
         result = tryToPayHybrids(_hybrids, _nbhybrids - 1, diff);
         if (result)
             return 1;
-        diff[h->color1 * 2 + 1] += h->value1;
+        diff[h.color1 * 2 + 1] += h.value1;
     }
-    if (diff[h->color2 * 2 + 1] >= h->value2)
+    if (diff[h.color2 * 2 + 1] >= h.value2)
     {
-        diff[h->color2 * 2 + 1] -= h->value2;
+        diff[h.color2 * 2 + 1] -= h.value2;
         result = tryToPayHybrids(_hybrids, _nbhybrids - 1, diff);
         if (result)
             return 1;
-        diff[h->color2 * 2 + 1] += h->value2;
+        diff[h.color2 * 2 + 1] += h.value2;
     }
     return 0;
 }
@@ -563,14 +674,14 @@ int ManaCost::tryToPayHybrids(ManaCostHybrid * _hybrids[], int _nbhybrids, int d
 //compute the difference between two mana costs
 ManaCost * ManaCost::Diff(ManaCost * _cost)
 {
-    int diff[(Constants::MTG_NB_COLORS + 1) * 2];
+    int8_t diff[(Constants::MTG_NB_COLORS + 1) * 2];
     diff[Constants::MTG_NB_COLORS * 2] = Constants::MTG_NB_COLORS;
     for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
     {
         diff[i * 2] = i;
         diff[i * 2 + 1] = cost[i] - _cost->getCost(i);
     }
-    int hybridResult = tryToPayHybrids(_cost->hybrids, _cost->nbhybrids, diff);
+    int hybridResult = tryToPayHybrids(_cost->hybrids, _cost->hybrids.size(), diff);
     if (!hybridResult)
         randomDiffHybrids(_cost, diff);
 
@@ -619,36 +730,48 @@ ManaCost * ManaCost::Diff(ManaCost * _cost)
 string ManaCost::toString()
 {
     ostringstream oss;
-    oss << "\n===ManaCost===\n";
     for (int i = 0; i <= Constants::MTG_NB_COLORS; i++)
     {
         if (cost[i])
         {
-            oss << Constants::MTGColorChars[i] << ":" << cost[i] << " - ";
+            if ( i == Constants::MTG_COLOR_ARTIFACT)
+                oss << "{" << getCost(i) << "}";
+            else
+                for (int colorCount = 0; colorCount < getCost(i); colorCount++ )
+                    oss << "{" << Constants::MTGColorChars[i] << "}";
         }
     }
 
-    for (unsigned int i = 0; i < nbhybrids; i++)
+    for (size_t i = 0; i < hybrids.size(); i++)
     {
-        ManaCostHybrid * h = hybrids[i];
-        oss << "H:{" << Constants::MTGColorChars[h->color1] << ":" << h->value1 << "}/{" << Constants::MTGColorChars[h->color2]
-                << ":" << h->value2 << "}";
+        oss << hybrids[i];
     }
-    oss << "\n=============\n";
     return oss.str();
 }
 
 #ifdef WIN32
 void ManaCost::Dump()
 {
+    DebugTrace( "\n===ManaCost===" );
     DebugTrace( this->toString() );
+    DebugTrace( "\n=============" );
 }
 
 #endif
 
-ostream& operator<<(ostream& out, const ManaCost& m)
+ostream& operator<<(ostream& out, ManaCost& m)
 {
-    return out << "(manacost)";
+    return out << m.toString();
+}
+
+ostream& operator<<(ostream& out, ManaCost* m)
+{
+    return out << m->toString();
+}
+
+ostream& operator<<(ostream& out, ManaCost m)
+{
+    return out << m.toString();
 }
 
 void ManaPool::init()
