@@ -21,10 +21,9 @@ void OptionInteger::Render()
 {
     WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::OPTION_FONT);
     mFont->SetColor(getColor(WGuiColor::TEXT));
-    JRenderer * renderer = JRenderer::GetInstance();
-
     mFont->DrawString(_(displayValue).c_str(), x + 2, y + 3);
     char buf[512];
+
     if (maxValue == 1)
     {
         if (value)
@@ -80,8 +79,6 @@ void OptionSelect::Render()
 {
     WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::OPTION_FONT);
     mFont->SetColor(getColor(WGuiColor::TEXT));
-
-    JRenderer * renderer = JRenderer::GetInstance();
     mFont->DrawString(_(displayValue).c_str(), x, y + 2);
 
     if (value < selections.size())
@@ -165,10 +162,10 @@ void OptionProfile::populate()
         return;
     }
     options[Options::ACTIVE_PROFILE].str = selections[value];
-    PlayerData * pdata = NEW PlayerData(app->collection);
+    PlayerData * pdata = NEW PlayerData(MTGCollection());
 
     int unlocked = 0, sets = setlist.size();
-    std::ifstream file(options.profileFile(PLAYER_SETTINGS).c_str());
+    wagic::ifstream file(options.profileFile(PLAYER_SETTINGS).c_str());
     std::string s;
     if (file)
     {
@@ -206,11 +203,11 @@ void OptionProfile::Render()
     else
         sprintf(buf, "profiles/%s/avatar.jpg", selections[value].c_str());
     string filename = buf;
-    JQuad * mAvatar = WResourceManager::Instance()->RetrieveTempQuad(filename, TEXTURE_SUB_EXACT);
+    JQuadPtr avatar = WResourceManager::Instance()->RetrieveTempQuad(filename, TEXTURE_SUB_EXACT);
 
-    if (mAvatar)
+    if (avatar)
     {
-        renderer->RenderQuad(mAvatar, x, pY);
+        renderer->RenderQuad(avatar.get(), x, pY);
         pX += 40;
     }
 
@@ -333,7 +330,7 @@ void OptionLanguage::Reload()
     {
         string filename = JGE_GET_RES("lang/");
         filename += mDit->d_name;
-        std::ifstream file(filename.c_str());
+        wagic::ifstream file(filename.c_str());
         string s;
         string lang;
         if (file)
@@ -412,7 +409,7 @@ void OptionDirectory::Reload()
     while ((mDit = readdir(mDip)))
     {
         sprintf(buf, "%s/%s/%s", root.c_str(), mDit->d_name, type.c_str());
-        std::ifstream file(buf);
+        wagic::ifstream file(buf);
         if (!file)
             continue;
         file.close();
@@ -439,7 +436,7 @@ OptionDirectory::OptionDirectory(string root, int id, string displayValue, strin
     while ((mDit = readdir(mDip)))
     {
         sprintf(buf, "%s/%s/%s", root.c_str(), mDit->d_name, type.c_str());
-        std::ifstream file(buf);
+        wagic::ifstream file(buf);
         if (!file)
             continue;
         file.close();
@@ -463,7 +460,7 @@ OptionTheme::OptionTheme(OptionThemeStyle * style) :
     ts = style;
 }
 
-JQuad * OptionTheme::getImage()
+JQuadPtr OptionTheme::getImage()
 {
     char buf[512];
     string val = selections[value];
@@ -498,7 +495,7 @@ void OptionTheme::Render()
             sprintf(buf, "%s", JGE_GET_RES("graphics/themeinfo.txt").c_str());
         else
             sprintf(buf, "%s%s", JGE_GET_RES("themes/%s/themeinfo.txt").c_str(), selections[value].c_str());
-        std::ifstream file(buf);
+        wagic::ifstream file(buf);
         if (file)
         {
             string temp;
@@ -513,11 +510,11 @@ void OptionTheme::Render()
     }
     sprintf(buf, _("Theme: %s").c_str(), selections[value].c_str());
 
-    JQuad * q = getImage();
+    JQuadPtr q = getImage();
     if (q)
     {
         float scale = 128 / q->mHeight;
-        renderer->RenderQuad(q, x, y, 0, scale, scale);
+        renderer->RenderQuad(q.get(), x, y, 0, scale, scale);
     }
 
     WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::OPTION_FONT);
@@ -551,6 +548,7 @@ void OptionTheme::confirmChange(bool confirmed)
     {
         setData();
         options.getStyleMan()->loadRules();
+        options.getStyleMan()->determineActive(NULL, NULL);
         if (ts)
             ts->Reload();
 
