@@ -13,6 +13,59 @@
 #include "AIPlayer.h"
 #include <JLogger.h>
 
+
+vector<Rules *> Rules::RulesList = vector<Rules *>();
+
+//Sorting by dissplayName
+struct RulesMenuCmp{
+	bool operator()(const Rules * a,const Rules * b) const{
+        return a->displayName < b->displayName;
+	}
+} RulesMenuCmp_;
+
+Rules * Rules::getRulesByFilename(string _filename)
+{
+    for (size_t i = 0; i < RulesList.size(); ++i)
+    {
+        if (RulesList[i]->filename == _filename)
+            return RulesList[i];
+    }
+    return NULL;
+}
+
+int Rules::loadAllRules()
+{
+    DIR *dip = opendir(JGE_GET_RES("rules").c_str());
+    struct dirent *dit;
+
+    while ((dit = readdir(dip)))
+    {
+        Rules * rules = NEW Rules();
+        if (rules->load(dit->d_name))
+        {
+            RulesList.push_back(rules);
+        }
+        else
+        {
+            SAFE_DELETE(rules);
+        }
+    }
+    //Kind of a hack here, we sort Rules alphabetically because it turns out to be matching
+    // The historical order of Game modes: Classic, Momir Basic, Random 1, Random 2, Story
+    std::sort(RulesList.begin(),RulesList.end(),RulesMenuCmp_);
+    closedir(dip);
+    return 1;
+}
+
+void Rules::unloadAllRules()
+{
+    for (size_t i = 0; i < RulesList.size(); ++i)
+    {
+        SAFE_DELETE(RulesList[i]);
+    }
+    RulesList.clear();
+}
+
 int Rules::getMTGId(string cardName)
 {
     int cardnb = atoi(cardName.c_str());
@@ -80,7 +133,7 @@ void RulesState::parsePlayerState(int playerId, string s)
     size_t limiter = s.find("=");
     if (limiter == string::npos) limiter = s.find(":");
     string areaS;
-    int area;
+    int area = -1;
     if (limiter != string::npos)
     {
         areaS = s.substr(0, limiter);
@@ -136,6 +189,19 @@ void RulesState::parsePlayerState(int playerId, string s)
             playerData[playerId].phaseRing = s.substr(limiter + 1);
             return;
         }
+		else if (areaS.compare("offerinterruptonphase") == 0)
+		{
+			  for (int i = 0; i < Constants::NB_MTG_PHASES; i++)
+            {
+                string phaseStr = Constants::MTGPhaseCodeNames[i];
+                if (s.find(phaseStr) != string::npos)
+                {
+                    playerData[playerId].offerInterruptOnPhase = PhaseRing::phaseStrToInt(phaseStr);
+                    break;
+                }
+
+            }  
+		}
         else if (areaS.compare("auto") == 0)
         {
             playerData[playerId].extraRules.push_back(s.substr(limiter + 1));
@@ -195,13 +261,15 @@ void Rules::addExtraRules()
 #ifdef NETWORK_SUPPORT
                 && !g->players[1] == PLAYER_TYPE_REMOTE
 #endif //NETWORK_SUPPORT
-                    )//keep this out of mimor and other game modes.
+                    )//keep this out of momir and other game modes.
             {
                 difficultyRating = DeckManager::getDifficultyRating(g->players[0], g->players[1]);
             }
 
             if (a)
             {
+                //We make those non interruptible, so that they don't appear on the player's stack
+                a->canBeInterrupted = false;
                 if (a->oneShot)
                 {
                     if (((p->isAI() && p->playMode
@@ -281,7 +349,6 @@ Player * Rules::loadPlayerMomir(int isAI)
     else
         player = NEW AIMomirPlayer(options.profileFile("momir.txt", "", true).c_str(), deckFileSmall, empty, tempDeck);
 
-    delete tempDeck;
     return player;
 }
 
@@ -316,7 +383,6 @@ Player * Rules::loadPlayerRandom(int isAI, int mode)
     else
         player = NEW AIPlayerBaka(deckFile, deckFileSmall, "", tempDeck);
 
-    delete tempDeck;
     return player;
 }
 
@@ -341,6 +407,7 @@ Player * Rules::initPlayer(int playerId)
         }
     }
     p->phaseRing = initState.playerData[playerId].phaseRing;
+	p->offerInterruptOnPhase = initState.playerData[playerId].offerInterruptOnPhase;
     return p;
 }
 
@@ -389,10 +456,7 @@ void Rules::initGame()
     DebugTrace("RULES Init Game\n");
 
     //Set the current player/phase
-    if (g->currentPlayer->playMode
-        != Player::MODE_TEST_SUITE && /*g->mRules->gamemode != GAME_TYPE_MOMIR && g->mRules->gamemode
-        != GAME_TYPE_RANDOM1 && g->mRules->gamemode != GAME_TYPE_RANDOM2 &&*/ g->mRules->gamemode
-        != GAME_TYPE_STORY)
+    if (g->currentPlayer->playMode!= Player::MODE_TEST_SUITE &&  g->mRules->gamemode!= GAME_TYPE_STORY)
     {
         if(OptionWhosFirst::WHO_R == options[Options::FIRSTPLAYER].number)
             initState.player = WRand() % 2;
@@ -413,7 +477,6 @@ void Rules::initGame()
         p->poisonCount = initState.playerData[i].poisonCount;
         p->damageCount = initState.playerData[i].damageCount;
         p->preventable = initState.playerData[i].preventable;
-        p->getManaPool()->copy(initState.playerData[i].manapool);
         if (initState.playerData[i].avatar.size())
         {
             p->loadAvatar(initState.playerData[i].avatar);
@@ -454,9 +517,24 @@ void Rules::initGame()
         }
     }
     addExtraRules();
+
+    postUpdateInitDone = false;
 DebugTrace("RULES Init Game Done !\n");
 }
 
+//This function has all iitialization that can't be done in the "real" init function,
+// because the first update call messes things up.
+//It's a hack, ideally, the first update call shouldn't mess the init parameters...
+void Rules::postUpdateInit()
+{
+    if (postUpdateInitDone)
+        return;
+    for (int i = 0; i < 2; ++ i)
+        GameObserver::GetInstance()->players[i]->getManaPool()->add(initState.playerData[i].manapool);
+       // GameObserver::GetInstance()->players[i]->getManaPool()->copy(initState.playerData[i].manapool);
+    postUpdateInitDone = true;
+}
+
 void RulesPlayerZone::cleanup()
 {
     cards.clear();
@@ -490,25 +568,38 @@ void Rules::cleanup()
     initState.cleanup();
 }
 
-Rules::Rules(string filename, string _bg)
+Rules::Rules(string _bg)
 {
     bg = _bg;
-    load(filename);
+    unlockOption = INVALID_OPTION;
+    hidden = false;
+    filename = "";
+    postUpdateInitDone = false;
+}
+
+bool Rules::canChooseDeck() 
+{
+    return (gamemode == GAME_TYPE_CLASSIC || gamemode == GAME_TYPE_STONEHEWER || gamemode == GAME_TYPE_HERMIT); 
 }
 
 int Rules::load(string _filename)
 {
+    //avoid non .txt files
+    if (_filename.size() < 5 || (_filename.find(".txt") == string::npos && _filename.find(".TXT") == string::npos))
+        return 0;
 
-    char filename[4096];
+    if (!filename.size()) //this check is necessary because of the recursive calls (a file loads other files)
+        filename = _filename;
+    char c_filename[4096];
     if (fileExists(_filename.c_str()))
     {
-        sprintf(filename, "%s", _filename.c_str());
+        sprintf(c_filename, "%s", _filename.c_str());
     }
     else
     {
-        sprintf(filename, JGE_GET_RES("rules/%s").c_str(), _filename.c_str());
+        sprintf(c_filename, JGE_GET_RES("rules/%s").c_str(), _filename.c_str());
     }
-    wagic::ifstream file(filename);
+    wagic::ifstream file(c_filename);
     std::string s;
 
     int state = PARSE_UNDEFINED;
@@ -517,15 +608,18 @@ int Rules::load(string _filename)
     if (!file) return 0;
 
     cleanup();
+
     while (std::getline(file, s))
     {
         if (!s.size()) continue;
         if (s[s.size() - 1] == '\r') s.erase(s.size() - 1); //Handle DOS files
         if (s[0] == '#') continue;
+        string scopy = s;
         std::transform(s.begin(), s.end(), s.begin(), ::tolower);
         if (s.find("include ") == 0)
         {
             load(s.substr(8));
+            hidden = false; //To avoid transmitting the hidden param to children
             continue;
         }
         if (s.compare("[init]") == 0)
@@ -552,6 +646,18 @@ int Rules::load(string _filename)
         switch (state)
         {
         case PARSE_UNDEFINED:
+            if (s.find("name=") == 0)
+            {
+                displayName = scopy.substr(5);
+            }
+            else if (s.find("unlock=") == 0)
+            {
+                unlockOption = Options::getID(s.substr(7));
+            }
+            else if (s.find("hidden") == 0)
+            {
+                hidden = true;
+            }
             break;
         case PARSE_INIT:
             if (s.find("auto=") == 0)
@@ -592,5 +698,8 @@ int Rules::strToGameMode(string s)
     if (s.compare("momir") == 0) return GAME_TYPE_MOMIR;
     if (s.compare("random1") == 0) return GAME_TYPE_RANDOM1;
     if (s.compare("random2") == 0) return GAME_TYPE_RANDOM2;
+    if (s.compare("story") == 0) return GAME_TYPE_STORY;
+	if (s.compare("stonehewer") == 0) return GAME_TYPE_STONEHEWER;
+	if (s.compare("hermit") == 0) return GAME_TYPE_HERMIT;
     return GAME_TYPE_CLASSIC;
 }
