@@ -126,7 +126,7 @@ void StoryReward::Update(float dt)
         MTGCard * card = NULL;
         if (value.size())
         {
-            card = GameApp::collection->getCardByName(value);
+            card = MTGCollection()->getCardByName(value);
             if (card)
             {
                 cardId = card->getId();
@@ -134,15 +134,15 @@ void StoryReward::Update(float dt)
         }
         else
         {
-            cardId = GameApp::collection->randomCardId();
-            card = GameApp::collection->getCardById(cardId);
+            cardId = MTGCollection()->randomCardId();
+            card = MTGCollection()->getCardById(cardId);
         }
 
         if (!cardId) break;
 
         if (!collection)
         {
-            collection = NEW MTGDeck(options.profileFile(PLAYER_COLLECTION).c_str(), GameApp::collection);
+            collection = NEW MTGDeck(options.profileFile(PLAYER_COLLECTION).c_str(), MTGCollection());
         }
 
         result = Credits::addCardToCollection(cardId, collection);
@@ -193,7 +193,7 @@ StoryImage::StoryImage(string img, float mX, float mY) :
 }
 void StoryImage::Render()
 {
-    JQuad * quad = WResourceManager::Instance()->RetrieveTempQuad(img);
+    JQuadPtr quad = WResourceManager::Instance()->RetrieveTempQuad(img);
     if (quad)
     {
         float x = mX;
@@ -202,13 +202,13 @@ void StoryImage::Render()
             x = SCREEN_WIDTH / 2;
             quad->SetHotSpot(quad->mWidth / 2, 0);
         }
-        JRenderer::GetInstance()->RenderQuad(quad, x, mY);
+        JRenderer::GetInstance()->RenderQuad(quad.get(), x, mY);
     }
 }
 
 float StoryImage::getHeight()
 {
-    JQuad * quad = WResourceManager::Instance()->RetrieveQuad(img);
+    JQuadPtr quad = WResourceManager::Instance()->RetrieveQuad(img);
     if (quad)
     {
         return quad->mHeight;
@@ -305,20 +305,17 @@ void StoryDuel::init()
     sprintf(folder, JGE_GET_RES(CAMPAIGNS_FOLDER"%s/%s").c_str(), mParent->folder.c_str(), pageId.c_str());
 
     sprintf(deckFile, "%s/deck.txt", folder);
-    MTGDeck * tempDeck = NEW MTGDeck(deckFile, GameApp::collection);
     sprintf(deckFileSmall, "campaign_%s", mParent->folder.c_str());
-    players[0] = NEW HumanPlayer(tempDeck, deckFile, deckFileSmall);
-    SAFE_DELETE(tempDeck);
+    players[0] = NEW HumanPlayer(deckFile, deckFileSmall);
 
     sprintf(deckFile, "%s/opponent_deck.txt", folder);
-    tempDeck = NEW MTGDeck(deckFile, GameApp::collection);
     sprintf(deckFileSmall, "campaign_ennemy_%s_%s", mParent->folder.c_str(), pageId.c_str());
-    players[1] = NEW AIPlayerBaka(tempDeck, deckFile, deckFileSmall, "baka.jpg");
-    SAFE_DELETE(tempDeck);
+    players[1] = NEW AIPlayerBaka(deckFile, deckFileSmall, "baka.jpg");
 
     string rulesFile = folder;
     rulesFile.append("/rules.txt");
-    rules = NEW Rules(rulesFile, bg);
+    rules = NEW Rules(bg);
+    rules->load(rulesFile);
 
     GameObserver::Init(players, 2);
     game = GameObserver::GetInstance();
@@ -531,6 +528,11 @@ void StoryDialog::Render()
 
 void StoryDialog::ButtonPressed(int controllerid, int controlid)
 {
+    if ( controlid == kInfoMenuID )
+        return;
+    if ( controlid == kCancelMenuID )
+        return;
+
     mParent->gotoPage(((StoryChoice *) mObjects[controlid])->pageId);
 }
 
