@@ -7,6 +7,7 @@
 #include "Subtypes.h"
 #include "Counters.h"
 #include "WEvent.h"
+#include "AllAbilities.h"
 
 TargetChooser * TargetChooserFactory::createTargetChooser(string s, MTGCardInstance * card, MTGAbility * ability)
 {
@@ -96,50 +97,49 @@ TargetChooser * TargetChooserFactory::createTargetChooser(string s, MTGCardInsta
 
             if (zoneName.compare("*") == 0)
             {
-                zones[nbzones] = MTGGameZone::ALL_ZONES;
+                zones[nbzones++] = MTGGameZone::ALL_ZONES;
             }
             else if (zoneName.compare("graveyard") == 0)
             {
-                zones[nbzones] = MTGGameZone::MY_GRAVEYARD;
-                nbzones++;
-                zones[nbzones] = MTGGameZone::OPPONENT_GRAVEYARD;
+                zones[nbzones++] = MTGGameZone::MY_GRAVEYARD;
+                zones[nbzones++] = MTGGameZone::OPPONENT_GRAVEYARD;
             }
             else if (zoneName.compare("battlefield") == 0 || zoneName.compare("inplay") == 0)
             {
-                zones[nbzones] = MTGGameZone::MY_BATTLEFIELD;
-                nbzones++;
-                zones[nbzones] = MTGGameZone::OPPONENT_BATTLEFIELD;
+                zones[nbzones++] = MTGGameZone::MY_BATTLEFIELD;
+                zones[nbzones++] = MTGGameZone::OPPONENT_BATTLEFIELD;
+            }
+            else if (zoneName.compare("hand") == 0)
+            {
+                zones[nbzones++] = MTGGameZone::MY_HAND;
+                zones[nbzones++] = MTGGameZone::OPPONENT_HAND;
+            }
+            else if (zoneName.compare("library") == 0)
+            {
+                zones[nbzones++] = MTGGameZone::MY_LIBRARY;
+                zones[nbzones++] = MTGGameZone::OPPONENT_LIBRARY;
             }
             else if (zoneName.compare("nonbattlezone") == 0)
             {
-                zones[nbzones] = MTGGameZone::MY_GRAVEYARD;
-                nbzones++;
-                zones[nbzones] = MTGGameZone::OPPONENT_GRAVEYARD;
-                nbzones++;
-                zones[nbzones] = MTGGameZone::MY_LIBRARY;
-                nbzones++;
-                zones[nbzones] = MTGGameZone::OPPONENT_LIBRARY;
-                nbzones++;
-                zones[nbzones] = MTGGameZone::MY_HAND;
-                nbzones++;
-                zones[nbzones] = MTGGameZone::OPPONENT_HAND;
-                nbzones++;
-                zones[nbzones] = MTGGameZone::MY_EXILE;
-                nbzones++;
-                zones[nbzones] = MTGGameZone::OPPONENT_EXILE;
+                zones[nbzones++] = MTGGameZone::MY_GRAVEYARD;
+                zones[nbzones++] = MTGGameZone::OPPONENT_GRAVEYARD;
+                zones[nbzones++] = MTGGameZone::MY_LIBRARY;
+                zones[nbzones++] = MTGGameZone::OPPONENT_LIBRARY;
+                zones[nbzones++] = MTGGameZone::MY_HAND;
+                zones[nbzones++] = MTGGameZone::OPPONENT_HAND;
+                zones[nbzones++] = MTGGameZone::MY_EXILE;
+                zones[nbzones++] = MTGGameZone::OPPONENT_EXILE;
             }
             else if (zoneName.compare("stack") == 0)
             {
-                zones[nbzones] = MTGGameZone::MY_STACK;
-                nbzones++;
-                zones[nbzones] = MTGGameZone::OPPONENT_STACK;
+                zones[nbzones++] = MTGGameZone::MY_STACK;
+                zones[nbzones++] = MTGGameZone::OPPONENT_STACK;
             }
             else
             {
                 int zone = MTGGameZone::zoneStringToId(zoneName);
-                if (zone) zones[nbzones] = zone;
+                if (zone) zones[nbzones++] = zone;
             }
-            nbzones++;
         }
     }
     else
@@ -153,6 +153,21 @@ TargetChooser * TargetChooserFactory::createTargetChooser(string s, MTGCardInsta
     TargetChooser * tc = NULL;
     int maxtargets = 1;
     CardDescriptor * cd = NULL;
+    //max targets allowed
+    size_t limit = s1.find('<');
+    if (limit != string::npos) 
+    {
+        size_t end = s1.find(">", limit);
+        string howmany;
+        if (end != string::npos)
+        {
+            howmany = s1.substr(limit + 1, end - limit - 1);
+            WParsedInt * howmuch = NEW WParsedInt(howmany, NULL, card);
+            maxtargets = howmuch->getValue();
+            delete howmuch;
+            s1 = s1.substr(end + 1);
+        }
+    }
 
     while (s1.size())
     {
@@ -206,8 +221,11 @@ TargetChooser * TargetChooserFactory::createTargetChooser(string s, MTGCardInsta
                     size_t operatorPosition = attribute.find("=", 1);
                     if (operatorPosition != string::npos)
                     {
-                        comparisonCriterion = atoi(
-                                        attribute.substr(operatorPosition + 1, attribute.size() - operatorPosition - 1).c_str());
+                        string numberCD = attribute.substr(operatorPosition + 1, attribute.size() - operatorPosition - 1);
+                        WParsedInt * val = NEW WParsedInt(numberCD,NULL, card);
+                        comparisonCriterion = val->getValue();
+                        /*atoi(attribute.substr(operatorPosition + 1, attribute.size() - operatorPosition - 1).c_str());*/
+                        delete val;           
                         switch (attribute[operatorPosition - 1])
                         {
                         case '<':
@@ -305,10 +323,148 @@ TargetChooser * TargetChooserFactory::createTargetChooser(string s, MTGCardInsta
                     {
                         cd->unsecuresetfresh(1);
                     }
-                    //Power restrictions
+                }
+                //creature is a level up creature
+                else if (attribute.find("leveler") != string::npos)
+                {
+                    if (minus)
+                    {
+                        cd->isLeveler = -1;
+                    }
+                    else
+                    {
+                        cd->isLeveler = 1;
+                    }
+                }
+                //creature is enchanted
+                else if (attribute.find("enchanted") != string::npos)
+                {
+                    if (minus)
+                    {
+                        cd->CDenchanted = -1;
+                    }
+                    else
+                    {
+                        cd->CDenchanted = 1;
+                    }
+                }
+                //creature was damaged
+                else if (attribute.find("damaged") != string::npos)
+                {
+                    if (minus)
+                    {
+                        cd->CDdamaged = -1;
+                    }
+                    else
+                    {
+                        cd->CDdamaged = 1;
+                    }
+                }
+                //creature dealt damage to opponent
+                else if (attribute.find("opponentdamager") != string::npos)
+                {
+                    if (minus)
+                    {
+                        cd->CDopponentDamaged = -1;
+                    }
+                    else
+                    {
+                        cd->CDopponentDamaged = 1;
+                    }
+                }
+                //creature dealt damage to controller
+                else if (attribute.find("controllerdamager") != string::npos)
+                {
+                    if (minus)
+                    {
+                        cd->CDcontrollerDamaged = -1;
+                    }
+                    else
+                    {
+                        cd->CDcontrollerDamaged = 1;
+                    }
+                }
+                else if (attribute.find("multicolor") != string::npos)
+                {
+                    //card is multicolored?
+                    if (minus)
+                    {
+                        cd->setisMultiColored(-1);
+                    }
+                    else
+                    {
+                        cd->setisMultiColored(1);
+                    }
+
+                }
+                else if (attribute.find("blackandgreen") != string::npos)
+                {
+                    //card is both colors?
+                    if (minus)
+                    {
+                        cd->setisBlackAndGreen(-1);
+                    }
+                    else
+                    {
+                        cd->setisBlackAndGreen(1);
+                    }
+
+                }
+                else if (attribute.find("blackandwhite") != string::npos)
+                {
+                    //card is both colors?
+                    if (minus)
+                    {
+                        cd->setisBlackAndWhite(-1);
+                    }
+                    else
+                    {
+                        cd->setisBlackAndWhite(1);
+                    }
+
+                }
+                else if (attribute.find("redandblue") != string::npos)
+                {
+                    //card is both colors?
+                    if (minus)
+                    {
+                        cd->setisRedAndBlue(-1);
+                    }
+                    else
+                    {
+                        cd->setisRedAndBlue(1);
+                    }
+
+                }
+                else if (attribute.find("blueandgreen") != string::npos)
+                {
+                    //card is both colors?
+                    if (minus)
+                    {
+                        cd->setisBlueAndGreen(-1);
+                    }
+                    else
+                    {
+                        cd->setisBlueAndGreen(1);
+                    }
+
+                }
+                else if (attribute.find("redandwhite") != string::npos)
+                {
+                    //card is both colors?
+                    if (minus)
+                    {
+                        cd->setisRedAndWhite(-1);
+                    }
+                    else
+                    {
+                        cd->setisRedAndWhite(1);
+                    }
+
                 }
                 else if (attribute.find("power") != string::npos)
                 {
+                    //Power restrictions
                     cd->setPower(comparisonCriterion);
                     cd->powerComparisonMode = comparisonMode;
                     //Toughness restrictions
@@ -325,6 +481,60 @@ TargetChooser * TargetChooserFactory::createTargetChooser(string s, MTGCardInsta
                     cd->manacostComparisonMode = comparisonMode;
                     //Counter Restrictions
                 }
+                else if (attribute.find("share!") != string::npos)
+                {
+                    size_t start = attribute.find("share!");
+                    size_t end = attribute.rfind("!");
+                    string CDtype = attribute.substr(start + 6,end - start);
+                    if( card && card->isSpell() && card->backupTargets.size() && card->backupTargets[0]->typeAsTarget() == TARGET_STACKACTION)
+                    {
+                    //spells always store their targets in :targets[]
+                    //however they are all erased as the spell resolves
+                    //added a array to store these backups incase theyre needed
+                    //again for effects such as these.
+                        Spell * spell;
+                        spell = (Spell*)card->backupTargets[0];
+                        card->target = spell->source;
+                    }
+                    if( CDtype.find("name") != string::npos )
+                    {
+                        if(card->target)
+                            cd->compareName = card->target->name;
+                        else
+                            cd->compareName = card->name;
+
+                        cd->nameComparisonMode = COMPARISON_EQUAL;
+                    }
+                    else if( CDtype.find("color") != string::npos )
+                    {
+                        if(card->target)
+                            cd->colors = card->target->colors;
+                        else
+                            cd->colors = card->colors;
+                       
+                        cd->mode = CD_OR;
+                    }
+                    else if( CDtype.find("types") != string::npos )
+                    {
+                        if(card && card->target)
+                        {
+                            cd->types = card->target->types;
+                            //remove main types because we only care about subtypes here.
+                            cd->removeType("artifact");
+                            cd->removeType("land");
+                            cd->removeType("enchantment");
+                            cd->removeType("instant");
+                            cd->removeType("sorcery");
+                            cd->removeType("legendary");
+                            cd->removeType("creature");
+                        }
+                        else
+                        {
+                            cd->types = card->types;
+                        }
+                        cd->mode = CD_OR;
+                    }
+                }
                 else if (attribute.find("counter") != string::npos)
                 {
                     if (attribute.find("{any}") != string::npos)
@@ -359,20 +569,16 @@ TargetChooser * TargetChooserFactory::createTargetChooser(string s, MTGCardInsta
                 else
                 {
                     int attributefound = 0;
-                    //Colors
+                    //Colors - remove Artifact and Land from the loop
                     for (int cid = 1; cid < Constants::MTG_NB_COLORS - 1; cid++)
-                    { //remove Artifact and Land from the loop
+                    { 
                         if (attribute.find(Constants::MTGColorStrings[cid]) != string::npos)
                         {
                             attributefound = 1;
                             if (minus)
-                            {
-                                cd->colors[cid] = -1;
-                            }
+                                cd->SetExclusionColor(cid);
                             else
-                            {
-                                cd->colors[cid] = 1;
-                            }
+                                cd->setColor(cid);
                         }
                     }
                     if (!attributefound)
@@ -384,13 +590,9 @@ TargetChooser * TargetChooserFactory::createTargetChooser(string s, MTGCardInsta
                             {
                                 attributefound = 1;
                                 if (minus)
-                                {
-                                    cd->basicAbilities[j] = -1;
-                                }
+                                    cd->mAbilityExclusions.set(j);
                                 else
-                                {
-                                    cd->basicAbilities[j] = 1;
-                                }
+                                    cd->basicAbilities.set(j);
                             }
                         }
                     }
@@ -409,7 +611,8 @@ TargetChooser * TargetChooserFactory::createTargetChooser(string s, MTGCardInsta
                     }
                 }
             }
-            if (nbminuses) cd->mode = CD_AND;
+            if (nbminuses)
+                cd->mode = CD_AND;
             typeName = typeName.substr(0, found);
         }
         //X targets allowed ?
@@ -462,6 +665,8 @@ TargetChooser * TargetChooserFactory::createTargetChooser(string s, MTGCardInsta
 
 TargetChooser * TargetChooserFactory::createTargetChooser(MTGCardInstance * card)
 {
+    if(!card)
+        return NULL;
     int id = card->getId();
     string s = card->spellTargetType;
     if (card->alias)
@@ -525,7 +730,7 @@ TargetChooser::TargetChooser(MTGCardInstance * card, int _maxtargets, bool _othe
 
 //Default targetter : every card can be targetted, unless it is protected from the targetter card
 // For spells that do not "target" a specific card, set targetter to NULL
-bool TargetChooser::canTarget(Targetable * target)
+bool TargetChooser::canTarget(Targetable * target,bool withoutProtections)
 {
     if (!target) return false;
     if (target->typeAsTarget() == TARGET_CARD)
@@ -540,10 +745,16 @@ bool TargetChooser::canTarget(Targetable * target)
                 tempcard = tempcard->previous;
             }
         }
-        if (source && targetter && card->isInPlay()) 
+        if(source && ((source->hasSubtype(Subtypes::TYPE_AURA) || source->hasSubtype(Subtypes::TYPE_EQUIPMENT)) && source->target && source->target == card && source->target->isPhased && targetter->target == card)) 
+        return true;
+        //this is kinda cheating but by default we let auras and equipments always continue to target a phased creature.
+        else if(card->isPhased) 
+        return false;
+        if (source && targetter && card->isInPlay() && !withoutProtections) 
         { 
             if (card->has(Constants::SHROUD)) return false;
             if (card->protectedAgainst(targetter)) return false;
+            if (card->CantBeTargetby(targetter)) return false;
             if ((targetter->controller() != card->controller()) && card->has(Constants::OPPONENTSHROUD)) return false;
         }
         return true;
@@ -574,7 +785,7 @@ int TargetChooser::ForceTargetListReady()
 
 int TargetChooser::targetsReadyCheck()
 {
-    if (cursor == 0)
+    if (!targets.size())
     {
         return TARGET_NOK;
     }
@@ -605,8 +816,8 @@ bool TargetChooser::validTargetsExist()
     {
         Player *p = GameObserver::GetInstance()->players[i];
         if (canTarget(p)) return true;
-        MTGGameZone * zones[] = { p->game->inPlay, p->game->graveyard, p->game->hand, p->game->library };
-        for (int k = 0; k < 4; k++)
+        MTGGameZone * zones[] = { p->game->inPlay, p->game->graveyard, p->game->hand, p->game->library, p->game->exile };
+        for (int k = 0; k < 5; k++)
         {
             MTGGameZone * z = zones[k];
             if (targetsZone(z))
@@ -621,6 +832,36 @@ bool TargetChooser::validTargetsExist()
     return false;
 }
 
+int TargetChooser::countValidTargets()
+{
+    int result = 0;
+    for (int i = 0; i < 2; ++i)
+    {
+        Player *p = GameObserver::GetInstance()->players[i];
+        MTGGameZone * zones[] = { p->game->inPlay, p->game->graveyard, p->game->hand, p->game->library, p->game->exile };
+        for (int k = 0; k < 5; k++)
+        {
+            MTGGameZone * z = zones[k];
+            if (targetsZone(z))
+            {
+                for (int j = 0; j < z->nb_cards; j++)
+                {
+                    if (canTarget(z->cards[j])) result++;
+                }
+            }
+        }
+    }
+    return result;
+}
+
+bool TargetChooser::equals(TargetChooser * tc)
+{
+
+    //This function always return 1 for now, since the default TargetChooser targets everything
+    //In the future we might need to check some of "targetter" settings to take protection into account...
+    return true;
+}
+
 /**
  a specific Card
  **/
@@ -630,12 +871,12 @@ CardTargetChooser::CardTargetChooser(MTGCardInstance * _card, MTGCardInstance *
     validTarget = _card;
 }
 
-bool CardTargetChooser::canTarget(Targetable * target)
+bool CardTargetChooser::canTarget(Targetable * target,bool withoutProtections)
 {
     if (!target) return false;
     if (target->typeAsTarget() != TARGET_CARD) return false;
-    if (!nbzones && !TargetChooser::canTarget(target)) return false;
-    if (nbzones && !TargetZoneChooser::canTarget(target)) return false;
+    if (!nbzones && !TargetChooser::canTarget(target,withoutProtections)) return false;
+    if (nbzones && !TargetZoneChooser::canTarget(target,withoutProtections)) return false;
     MTGCardInstance * card = (MTGCardInstance *) target;
     while (card)
     {
@@ -651,6 +892,20 @@ CardTargetChooser * CardTargetChooser::clone() const
     return a;
 }
 
+bool CardTargetChooser::equals(TargetChooser * tc)
+{
+
+    CardTargetChooser * ctc = dynamic_cast<CardTargetChooser *> (tc);
+    if (!ctc)
+        return false;
+    
+    if (validTarget != ctc->validTarget) //todo, check also previous cards, see "cantarget"...
+        return false;
+
+    return TargetZoneChooser::equals(tc);
+}
+
+
 /**
  Choose anything that has a given list of types
  **/
@@ -694,16 +949,20 @@ void TypeTargetChooser::addType(int type)
     nbtypes++;
 }
 
-bool TypeTargetChooser::canTarget(Targetable * target)
+bool TypeTargetChooser::canTarget(Targetable * target,bool withoutProtections)
 {
-    if (!TargetZoneChooser::canTarget(target)) return false;
+    if (!TargetZoneChooser::canTarget(target,withoutProtections)) return false;
     if (target->typeAsTarget() == TARGET_CARD)
     {
         MTGCardInstance * card = (MTGCardInstance *) target;
         for (int i = 0; i < nbtypes; i++)
         {
+
             if (card->hasSubtype(types[i])) return true;
+			if(card->getLCName().size())
+			{
             if (Subtypes::subtypesList->find(card->getLCName()) == types[i]) return true;
+			}
         }
         return false;
     }
@@ -731,6 +990,34 @@ TypeTargetChooser * TypeTargetChooser::clone() const
     return a;
 }
 
+bool TypeTargetChooser ::equals(TargetChooser * tc)
+{
+
+    TypeTargetChooser  * ttc = dynamic_cast<TypeTargetChooser  *> (tc);
+    if (!ttc)
+        return false;
+    
+    if (nbtypes != ttc->nbtypes)
+        return false;
+
+    map<int,int> counts;
+
+    for (int i = 0; i < nbtypes; ++i)
+    {
+        counts[types[i]] +=1;
+        counts[ttc->types[i]] -=1;
+    }
+
+    for (int i = 0; i < nbtypes; ++i)
+    {
+        if (counts[types[i]] || counts[ttc->types[i]])
+            return false;
+    }
+
+    return TargetZoneChooser::equals(tc);
+}
+
+
 /**
  A Target Chooser associated to a Card Descriptor object, for fine tuning of targets description
  **/
@@ -758,13 +1045,14 @@ DescriptorTargetChooser::DescriptorTargetChooser(CardDescriptor * _cd, int * _zo
     cd = _cd;
 }
 
-bool DescriptorTargetChooser::canTarget(Targetable * target)
+bool DescriptorTargetChooser::canTarget(Targetable * target,bool withoutProtections)
 {
-    if (!TargetZoneChooser::canTarget(target)) return false;
+    if (!TargetZoneChooser::canTarget(target,withoutProtections)) return false;
     if (target->typeAsTarget() == TARGET_CARD)
     {
         MTGCardInstance * _target = (MTGCardInstance *) target;
-        if (cd->match(_target)) return true;
+        if (cd->match(_target)) 
+        return true;
     }
     else if (target->typeAsTarget() == TARGET_STACKACTION)
     {
@@ -791,46 +1079,16 @@ DescriptorTargetChooser * DescriptorTargetChooser::clone() const
     return a;
 }
 
-/**
- Choose a creature
- **/
-
-CreatureTargetChooser::CreatureTargetChooser(MTGCardInstance * card, int _maxtargets, bool other) :
-    TargetZoneChooser(card, _maxtargets, other)
+bool DescriptorTargetChooser::equals(TargetChooser * tc)
 {
-    int default_zones[] = { MTGGameZone::MY_BATTLEFIELD, MTGGameZone::OPPONENT_BATTLEFIELD };
-    init(default_zones, 2);
-}
 
-CreatureTargetChooser::CreatureTargetChooser(int * _zones, int nbzones, MTGCardInstance * card, int _maxtargets, bool other) :
-    TargetZoneChooser(card, _maxtargets, other)
-{
-    if (nbzones == 0)
-    {
-        int default_zones[] = { MTGGameZone::MY_BATTLEFIELD, MTGGameZone::OPPONENT_BATTLEFIELD };
-        init(default_zones, 2);
-    }
-    else
-    {
-        init(_zones, nbzones);
-    }
-}
+    DescriptorTargetChooser  * dtc = dynamic_cast<DescriptorTargetChooser  *> (tc);
+    if (!dtc)
+        return false;
+    
+    //TODO Descriptors need to have an "equals" method too -_-
 
-bool CreatureTargetChooser::canTarget(Targetable * target)
-{
-    if (!TargetZoneChooser::canTarget(target)) return false;
-    if (target->typeAsTarget() == TARGET_CARD)
-    {
-        MTGCardInstance * card = (MTGCardInstance *) target;
-        return card->isCreature();
-    }
-    return false;
-}
-
-CreatureTargetChooser * CreatureTargetChooser::clone() const
-{
-    CreatureTargetChooser * a = NEW CreatureTargetChooser(*this);
-    return a;
+    return TargetZoneChooser::equals(tc);
 }
 
 /* TargetzoneChooser targets everything in a given zone */
@@ -864,9 +1122,9 @@ int TargetZoneChooser::setAllZones()
     return 1;
 }
 
-bool TargetZoneChooser::canTarget(Targetable * target)
+bool TargetZoneChooser::canTarget(Targetable * target,bool withoutProtections)
 {
-    if (!TargetChooser::canTarget(target)) return false;
+    if (!TargetChooser::canTarget(target,withoutProtections)) return false;
     if (target->typeAsTarget() == TARGET_CARD)
     {
         MTGCardInstance * card = (MTGCardInstance *) target;
@@ -897,6 +1155,14 @@ bool TargetZoneChooser::targetsZone(MTGGameZone * z)
         if (MTGGameZone::intToZone(zones[i], source) == z) return true;
     return false;
 }
+bool TargetZoneChooser::targetsZone(MTGGameZone * z,MTGCardInstance * mSource)
+{
+if(mSource)
+source = mSource;
+    for (int i = 0; i < nbzones; i++)
+        if (MTGGameZone::intToZone(zones[i], source) == z) return true;
+    return false;
+}
 
 TargetZoneChooser * TargetZoneChooser::clone() const
 {
@@ -904,13 +1170,42 @@ TargetZoneChooser * TargetZoneChooser::clone() const
     return a;
 }
 
+bool TargetZoneChooser::equals(TargetChooser * tc)
+{
+
+    TargetZoneChooser  * tzc = dynamic_cast<TargetZoneChooser  *> (tc);
+    if (!tzc)
+        return false;
+    
+    if (nbzones!= tzc->nbzones)
+        return false;
+
+    map<int,int> counts;
+
+    for (int i = 0; i < nbzones; ++i)
+    {
+        counts[zones[i]] +=1;
+        counts[tzc->zones[i]] -=1;
+    }
+
+    for (int i = 0; i < nbzones; ++i)
+    {
+        if (counts[zones[i]] || counts[tzc->zones[i]])
+            return false;
+    }
+
+    //TODO: ALL_ZONES should be equivalent to something actually targetting all zones...
+
+    return TargetChooser::equals(tc);
+}
+
 /* Player Target */
 PlayerTargetChooser::PlayerTargetChooser(MTGCardInstance * card, int _maxtargets, Player *p) :
     TargetChooser(card, _maxtargets), p(p)
 {
 }
 
-bool PlayerTargetChooser::canTarget(Targetable * target)
+bool PlayerTargetChooser::canTarget(Targetable * target,bool withoutProtections)
 {
     if (source && targetter && (targetter->controller() != targetter->controller()->opponent())
                     && (targetter->controller()->opponent()->game->inPlay->hasAbility(Constants::CONTROLLERSHROUD))
@@ -930,8 +1225,21 @@ PlayerTargetChooser* PlayerTargetChooser::clone() const
     return a;
 }
 
+bool PlayerTargetChooser::equals(TargetChooser * tc)
+{
+
+    PlayerTargetChooser  * ptc = dynamic_cast<PlayerTargetChooser   *> (tc);
+    if (!ptc)
+        return false;
+    
+    if (p != ptc->p)
+        return false;
+
+    return TargetChooser::equals(tc);
+}
+
 /*Damageable Target */
-bool DamageableTargetChooser::canTarget(Targetable * target)
+bool DamageableTargetChooser::canTarget(Targetable * target,bool withoutProtections)
 {
     if (source && targetter && (targetter->controller() != targetter->controller()->opponent())
                     && (targetter->controller()->opponent()->game->inPlay->hasAbility(Constants::CONTROLLERSHROUD))
@@ -946,7 +1254,7 @@ bool DamageableTargetChooser::canTarget(Targetable * target)
     {
         return true;
     }
-    return CreatureTargetChooser::canTarget(target);
+    return TypeTargetChooser::canTarget(target,withoutProtections);
 }
 
 DamageableTargetChooser* DamageableTargetChooser::clone() const
@@ -955,6 +1263,16 @@ DamageableTargetChooser* DamageableTargetChooser::clone() const
     return a;
 }
 
+bool DamageableTargetChooser::equals(TargetChooser * tc)
+{
+
+    DamageableTargetChooser  * dtc = dynamic_cast<DamageableTargetChooser  *> (tc);
+    if (!dtc)
+        return false;
+    
+    return TypeTargetChooser::equals(tc);
+}
+
 /*Spell */
 
 SpellTargetChooser::SpellTargetChooser(MTGCardInstance * card, int _color, int _maxtargets, bool other) :
@@ -963,7 +1281,7 @@ SpellTargetChooser::SpellTargetChooser(MTGCardInstance * card, int _color, int _
     color = _color;
 }
 
-bool SpellTargetChooser::canTarget(Targetable * target)
+bool SpellTargetChooser::canTarget(Targetable * target,bool withoutProtections)
 {
     MTGCardInstance * card = NULL;
     if (target->typeAsTarget() == TARGET_STACKACTION)
@@ -986,6 +1304,19 @@ SpellTargetChooser* SpellTargetChooser::clone() const
     return a;
 }
 
+bool SpellTargetChooser::equals(TargetChooser * tc)
+{
+
+    SpellTargetChooser  * stc = dynamic_cast<SpellTargetChooser *> (tc);
+    if (!stc)
+        return false;
+    
+    if (color != stc->color)
+        return false;
+
+    return TargetChooser::equals(tc);
+}
+
 /*Spell or Permanent */
 SpellOrPermanentTargetChooser::SpellOrPermanentTargetChooser(MTGCardInstance * card, int _color, int _maxtargets, bool other) :
     TargetZoneChooser(card, _maxtargets, other)
@@ -995,13 +1326,13 @@ SpellOrPermanentTargetChooser::SpellOrPermanentTargetChooser(MTGCardInstance * c
     color = _color;
 }
 
-bool SpellOrPermanentTargetChooser::canTarget(Targetable * target)
+bool SpellOrPermanentTargetChooser::canTarget(Targetable * target,bool withoutProtections)
 {
     MTGCardInstance * card = NULL;
     if (target->typeAsTarget() == TARGET_CARD)
     {
         card = (MTGCardInstance *) target;
-        if (color == -1 || card->hasColor(color)) return TargetZoneChooser::canTarget(target);
+        if (color == -1 || card->hasColor(color)) return TargetZoneChooser::canTarget(target,withoutProtections);
     }
     else if (target->typeAsTarget() == TARGET_STACKACTION)
     {
@@ -1022,6 +1353,19 @@ SpellOrPermanentTargetChooser* SpellOrPermanentTargetChooser::clone() const
     return a;
 }
 
+bool SpellOrPermanentTargetChooser::equals(TargetChooser * tc)
+{
+
+    SpellOrPermanentTargetChooser  * sptc = dynamic_cast<SpellOrPermanentTargetChooser *> (tc);
+    if (!sptc)
+        return false;
+    
+    if (color != sptc->color)
+        return false;
+
+    return TargetZoneChooser::equals(tc);
+}
+
 /*Damage */
 DamageTargetChooser::DamageTargetChooser(MTGCardInstance * card, int _color, int _maxtargets, int _state) :
     TargetChooser(card, _maxtargets)
@@ -1030,7 +1374,7 @@ DamageTargetChooser::DamageTargetChooser(MTGCardInstance * card, int _color, int
     state = _state;
 }
 
-bool DamageTargetChooser::canTarget(Targetable * target)
+bool DamageTargetChooser::canTarget(Targetable * target,bool withoutProtections)
 {
     MTGCardInstance * card = NULL;
     if (target->typeAsTarget() == TARGET_STACKACTION)
@@ -1052,6 +1396,19 @@ DamageTargetChooser* DamageTargetChooser::clone() const
     return a;
 }
 
+bool DamageTargetChooser::equals(TargetChooser * tc)
+{
+
+    DamageTargetChooser * dtc = dynamic_cast<DamageTargetChooser *> (tc);
+    if (!dtc)
+        return false;
+
+    if (color != dtc->color || state != dtc->state)
+        return false;
+
+    return TargetChooser::equals(tc);
+}
+
 TriggerTargetChooser::TriggerTargetChooser(int _triggerTarget)
 {
     triggerTarget = _triggerTarget;
@@ -1063,7 +1420,7 @@ bool TriggerTargetChooser::targetsZone(MTGGameZone * z)
     return true;
 }
 
-bool TriggerTargetChooser::canTarget(Targetable * _target)
+bool TriggerTargetChooser::canTarget(Targetable * _target,bool withoutProtections)
 {
     if (_target == target) return true;
     return false;
@@ -1074,3 +1431,16 @@ TriggerTargetChooser * TriggerTargetChooser::clone() const
     TriggerTargetChooser * a = NEW TriggerTargetChooser(*this);
     return a;
 }
+
+bool TriggerTargetChooser::equals(TargetChooser * tc)
+{
+
+    TriggerTargetChooser * ttc = dynamic_cast<TriggerTargetChooser *> (tc);
+    if (!ttc)
+        return false;
+
+    if (triggerTarget != ttc->triggerTarget)
+        return false;
+
+    return TargetChooser::equals(tc);
+}
\ No newline at end of file
