@@ -16,7 +16,7 @@ using std::string;
 // NULL is sent in place of a MTGDeck since there is no way to create a MTGDeck without a proper deck file.
 // TestSuiteAI will be responsible for managing its own deck state.
 TestSuiteAI::TestSuiteAI(TestSuite * _suite, int playerId) :
-    AIPlayerBaka(NULL, "testsuite", "testsuite", "baka.jpg")
+    AIPlayerBaka("testsuite", "testsuite", "baka.jpg", NULL)
 {
     this->game = _suite->buildDeck(playerId);
     game->setOwner(this);
@@ -78,6 +78,14 @@ int TestSuiteAI::Act(float dt)
 {
     GameObserver * g = GameObserver::GetInstance();
     g->gameOver = NULL; // Prevent draw rule from losing the game
+
+    //Last bits of initialization require to be done here, after the first "update" call of the game
+    if (suite->currentAction == 0)
+    {
+        for (int i = 0; i < 2; ++ i)
+            g->players[i]->getManaPool()->copy(suite->initState.playerData[i].manapool);
+    }
+
     if (playMode == MODE_AI && suite->aiMaxCalls)
     {
         suite->aiMaxCalls--;
@@ -87,6 +95,7 @@ int TestSuiteAI::Act(float dt)
     if (playMode == MODE_HUMAN)
     {
         g->mLayers->CheckUserInput(0);
+        suite->currentAction++; //hack to avoid repeating the initialization of manapool
         return 1;
     }
 
@@ -96,7 +105,6 @@ int TestSuiteAI::Act(float dt)
 
     string action = suite->getNextAction();
     g->mLayers->stackLayer()->Dump();
-    //  DamageResolverLayer * drl = g->mLayers->combatLayer();
     DebugTrace("TESTSUITE command: " << action);
 
     if (g->mLayers->stackLayer()->askIfWishesToInterrupt == this)
@@ -135,13 +143,44 @@ int TestSuiteAI::Act(float dt)
         playMode = MODE_AI;
         return 1;
     }
-    else if (action.compare("next") == 0)
+    else if (action.compare("next") == 0 || action.find("goto") != string::npos)
     {
-        GuiCombat * gc = g->mLayers->combatLayer();
-        if (ORDER == g->combatStep || DAMAGE == g->combatStep)
-            gc->clickOK();
+        if(action.find("goto ")!= string::npos)
+        {
+            size_t found = action.find("goto ");
+            string phase = action.substr(found + 5);
+            int phaseToGo = 0;
+            for (int i = 0; i < Constants::NB_MTG_PHASES; i++)
+            {
+                if (phase.find(Constants::MTGPhaseCodeNames[i]) != string::npos)
+                {
+                    phaseToGo = i;
+                }
+            }
+            if(g->currentGamePhase != phaseToGo)
+                suite->currentAction--;
+                else
+                {
+                return 1;
+                }
+            GuiCombat * gc = g->mLayers->combatLayer();
+            if (ORDER == g->combatStep || DAMAGE == g->combatStep)
+            {
+                gc->clickOK();
+            }
+            else
+            {
+                g->userRequestNextGamePhase();
+            }
+        }
         else
-            g->userRequestNextGamePhase();
+        {
+            GuiCombat * gc = g->mLayers->combatLayer();
+            if (ORDER == g->combatStep || DAMAGE == g->combatStep)
+                gc->clickOK();
+            else
+                g->userRequestNextGamePhase();
+        }
     }
     else if (action.compare("yes") == 0)
         g->mLayers->stackLayer()->setIsInterrupting(this);
@@ -355,7 +394,6 @@ void TestSuite::initGame()
         AIPlayer * p = (AIPlayer *) (g->players[i]);
         p->forceBestAbilityUse = forceAbility;
         p->life = initState.playerData[i].life;
-        p->getManaPool()->copy(initState.playerData[i].manapool);
         MTGGameZone * playerZones[] = { p->game->graveyard, p->game->library, p->game->hand, p->game->inPlay };
         for (int j = 0; j < 4; j++)
         {
@@ -390,7 +428,9 @@ void TestSuite::initGame()
                     }
                 }
             }
+            zone->cardsSeenThisTurn.clear(); //don't consider those cards as having moved in this area during this turn
         }
+        p->game->stack->cardsSeenThisTurn.clear(); //don't consider those cards as having moved in this area during this turn
     }
 DebugTrace("TESTUITE Init Game Done !");
 }
@@ -451,6 +491,12 @@ int TestSuite::assertGame()
             sprintf(result, "<span class=\"error\">==Mana problem. Was expecting %i but got %i for player %i==</span><br />",
                             endState.playerData[i].manapool->getConvertedCost(), p->getManaPool()->getConvertedCost(), i);
             Log(result);
+
+            if ( endState.playerData[i].manapool->getConvertedCost() == p->getManaPool()->getConvertedCost())
+            {
+                sprintf(result, "<span class=\"error\">====(Apparently Mana Color issues since converted cost is the same)==</span><br />");
+                Log(result);
+            }
             error++;
 
         }
@@ -509,7 +555,7 @@ TestSuite::TestSuite(const char * filename, MTGAllCards* _collection)
 {
     collection = _collection;
     timerLimit = 0;
-    std::ifstream file(filename);
+    wagic::ifstream file(filename);
     std::string s;
     nbfiles = 0;
     currentfile = 0;
@@ -617,7 +663,7 @@ int TestSuite::load(const char * _filename)
     gameType = GAME_TYPE_CLASSIC;
     char filename[4096];
     sprintf(filename, JGE_GET_RES("/test/%s").c_str(), _filename);
-    std::ifstream file(filename);
+    wagic::ifstream file(filename);
     std::string s;
     loadRandValues("");
 
