@@ -1,7 +1,7 @@
 #include "PrecompiledHeader.h"
 
 #include "Translate.h"
-#include "GameOptions.h"
+#include "utils.h"
 
 Translator * Translator::mInstance = NULL;
 
