@@ -2,13 +2,15 @@
 
 #include "utils.h"
 #include <JFileSystem.h>
-#include "GameOptions.h"
 #include "WResourceManager.h"
+#include "WCachedResource.h"
 #include <hge/hgeparticle.h>
 
 #ifdef WITH_FMOD
 #endif
 
+const std::string kPlaceholderID("placeholder");
+
 //WResource
 WResource::~WResource()
 {
@@ -39,6 +41,9 @@ void WResource::deadbolt()
 void WResource::lock()
 {
     if (locks < WRES_MAX_LOCK) locks++;
+    // watch out for this - if the lock count increases beyond a reasonable threshold, 
+    // someone is probably not releasing the texture correctly
+    assert(locks < 50);
 }
 
 void WResource::unlock(bool force)
@@ -77,19 +82,7 @@ WCachedTexture::WCachedTexture()
 WCachedTexture::~WCachedTexture()
 {
     if (texture)
-    SAFE_DELETE(texture);
-
-    if (!trackedQuads.size()) return;
-
-    vector<WTrackedQuad*>::iterator it;
-    WTrackedQuad * tq = NULL;
-
-    for (it = trackedQuads.begin(); it != trackedQuads.end(); it++)
-    {
-        tq = (*it);
-        SAFE_DELETE(tq);
-    }
-    trackedQuads.clear();
+    	SAFE_DELETE(texture);
 }
 
 JTexture * WCachedTexture::Actual()
@@ -99,131 +92,55 @@ JTexture * WCachedTexture::Actual()
 
 bool WCachedTexture::isLocked()
 {
-    if (locks != WRES_UNLOCKED) return true;
-
-    for (vector<WTrackedQuad*>::iterator it = trackedQuads.begin(); it != trackedQuads.end(); it++)
-    {
-        if ((*it)->isLocked()) return true;
-    }
-
-    return false;
+    return (locks != WRES_UNLOCKED);
 }
 
-bool WCachedTexture::ReleaseQuad(JQuad* quad)
+JQuadPtr WCachedTexture::GetQuad(float offX, float offY, float width, float height, const string& resname)
 {
-    if (quad == NULL) return false;
-
-    WTrackedQuad * tq = NULL;
-    vector<WTrackedQuad*>::iterator nit;
-    for (vector<WTrackedQuad*>::iterator it = trackedQuads.begin(); it != trackedQuads.end(); it = nit)
-    {
-        nit = it;
-        nit++;
-        if ((*it) && (*it)->quad == quad)
-        {
-            tq = (*it);
-            tq->unlock();
-
-            if (!tq->isLocked())
-            {
-                SAFE_DELETE(tq);
-                trackedQuads.erase(it);
-            }
-
-            return true; //Returns true when found.
-        }
-    }
-    return false;
-}
-
-WTrackedQuad * WCachedTexture::GetTrackedQuad(float offX, float offY, float width, float height, string resname)
-{
-    if (!texture) return NULL;
-
-    bool allocated = false;
-    WTrackedQuad * tq = NULL;
-    JQuad * quad = NULL;
-
-    vector<WTrackedQuad*>::iterator it;
+    if (!texture) return JQuadPtr();
 
     if (width == 0.0f || width > static_cast<float> (texture->mWidth)) width = static_cast<float> (texture->mWidth);
     if (height == 0.0f || height > static_cast<float> (texture->mHeight)) height = static_cast<float> (texture->mHeight);
 
-    for (it = trackedQuads.begin(); it != trackedQuads.end(); it++)
+    // If we're fetching a card resource, but it's not available yet, we'll be attempting to get the Quad from the temporary back image.
+    // If that's the case, don't stash a separate tracked quad entry for each card name in the the Back/BackThumbnail's resource
+    string resource(resname);
+    if (mFilename == kGenericCard || mFilename == kGenericThumbCard)
     {
-        if ((*it) && (*it)->resname == resname)
-        {
-            tq = (*it);
-            break;
-        }
+        // if we're the back or thumb_back file, but we've been asked for a card ID, then assign it
+        // a placeholder ID.  Reason being, hotspots on quads are different (ie centered) for card images, so we'll store a separate quad for cards 
+        if (resname != kGenericCardID && resname != kGenericCardThumbnailID)
+            resource = kPlaceholderID;
     }
 
-    if (!tq)
-    {
-        allocated = true;
-        tq = NEW WTrackedQuad(resname);
-        if (!tq) return NULL;
-    }
+	std::map<string, JQuadPtr>::iterator iter = mTrackedQuads.find(resource);
+	if (iter != mTrackedQuads.end())
+		return iter->second;
 
-    quad = tq->quad;
-
-    if (!quad)
-    {
-        quad = NEW JQuad(texture, offX, offY, width, height);
-        /*
-         There's a risk this erases the texture calling the quad creation.... Erwan 2010/03/13
-         if(!quad) {
-         //Probably out of memory. Try again.
-         WResourceManager::Instance()->Cleanup();
-         quad = NEW JQuad(texture,offX,offY,width,height);
-         }
-         */
-        if (!quad)
-        {
-            if (allocated && tq)
-            SAFE_DELETE(tq);
-            fprintf(stderr, "WCACHEDRESOURCE:GetTrackedQuad -  Quad is null\n");
-            return NULL; //Probably a crash.
-        }
-
-        tq->quad = quad;
-        if (allocated) trackedQuads.push_back(tq);
-        return tq;
-    }
+	JQuadPtr quad(NEW JQuad(texture, offX, offY, width, height));
 
     //Update JQ's values to what we called this with.
     quad->SetTextureRect(offX, offY, width, height);
-    return tq;
+	mTrackedQuads.insert(std::pair<string, JQuadPtr>(resource, quad));
+    return quad;
 
 }
 
-JQuad * WCachedTexture::GetQuad(float offX, float offY, float width, float height, string resname)
+JQuadPtr WCachedTexture::GetQuad(const string& resname)
 {
-    WTrackedQuad * tq = GetTrackedQuad(offX, offY, width, height, resname);
+	JQuadPtr result;
+ 	std::map<string, JQuadPtr>::iterator iter = mTrackedQuads.find(resname);
+	if (iter != mTrackedQuads.end())
+		result = iter->second;
 
-    if (tq) return tq->quad;
-
-    return NULL;
+	return result;
 }
 
-JQuad * WCachedTexture::GetQuad(string resname)
+JQuadPtr WCachedTexture::GetCard(float offX, float offY, float width, float height, const string& resname)
 {
-    vector<WTrackedQuad*>::iterator it;
-
-    for (it = trackedQuads.begin(); it != trackedQuads.end(); it++)
-    {
-        if ((*it) && (*it)->resname == resname)
-        {
-            return (*it)->quad;
-        }
-    }
-
-    return NULL;
-}
-JQuad * WCachedTexture::GetCard(float offX, float offY, float width, float height, string resname)
-{
-    JQuad * jq = GetQuad(offX, offY, width, height, resname);
-    if (jq) jq->SetHotSpot(static_cast<float> (jq->mTex->mWidth / 2), static_cast<float> (jq->mTex->mHeight / 2));
+    JQuadPtr jq = GetQuad(offX, offY, width, height, resname);
+    if (jq.get())
+        jq->SetHotSpot(static_cast<float> (jq->mTex->mWidth / 2), static_cast<float> (jq->mTex->mHeight / 2));
 
     return jq;
 }
@@ -233,8 +150,7 @@ unsigned long WCachedTexture::size()
     if (!texture) return 0;
 
     unsigned int pixel_size = 4;
-#if defined WIN32 || defined LINUX || defined IOS
-#else
+#if defined(PSP)
     pixel_size = JRenderer::GetInstance()->PixelSize(texture->mTextureFormat);
 #endif
     return texture->mTexHeight * texture->mTexWidth * pixel_size;
@@ -261,46 +177,45 @@ void WCachedTexture::Refresh()
 
     JRenderer::GetInstance()->TransferTextureToGLContext(*texture);
 
-    for (vector<WTrackedQuad*>::iterator it = trackedQuads.begin(); it != trackedQuads.end(); it++)
+    for (map<string, JQuadPtr>::iterator it = mTrackedQuads.begin(); it != mTrackedQuads.end(); ++it)
     {
-        if ((*it) && (*it)->quad) (*it)->quad->mTex = texture;
+        if (it->second.get())
+            it->second->mTex = texture;
     }
 }
 
-bool WCachedTexture::Attempt(string filename, int submode, int & error)
+bool WCachedTexture::Attempt(const string& filename, int submode, int & error)
 {
     mFilename = filename;
     int format = TEXTURE_FORMAT;
     loadedMode = submode;
-    string realname;
+    string realname = filename;
 
     //Form correct filename.
-    if (submode & TEXTURE_SUB_EXACT)
-        realname = filename;
-    else if (submode & TEXTURE_SUB_CARD)
+	if (submode & TEXTURE_SUB_CARD)
     {
         if (submode & TEXTURE_SUB_THUMB)
         {
-            for (string::size_type i = 0; i < filename.size(); i++)
+            for (string::size_type i = 0; i < realname.size(); i++)
             {
-                if (filename[i] == '\\' || filename[i] == '/')
+                if (realname[i] == '\\' || realname[i] == '/')
                 {
-                    filename.insert(i + 1, "thumbnails/");
+                    realname.insert(i + 1, "thumbnails/");
                     break;
                 }
             }
 
         }
-        realname = WResourceManager::Instance()->cardFile(filename);
+        realname = WResourceManager::Instance()->cardFile(realname);
     }
     else
     {
-        if (submode & TEXTURE_SUB_THUMB) filename.insert(0, "thumbnails/");
+        if (submode & TEXTURE_SUB_THUMB) realname.insert(0, "thumbnails/");
 
         if (submode & TEXTURE_SUB_AVATAR)
-            realname = WResourceManager::Instance()->avatarFile(filename);
+            realname = WResourceManager::Instance()->avatarFile(realname);
         else
-            realname = WResourceManager::Instance()->graphicsFile(filename);
+            realname = WResourceManager::Instance()->graphicsFile(realname);
     }
 
     //Apply pixel mode
@@ -360,7 +275,7 @@ void WCachedSample::Refresh()
     return;
 }
 
-bool WCachedSample::Attempt(string filename, int submode, int & error)
+bool WCachedSample::Attempt(const string& filename, int submode, int & error)
 {
     loadedMode = submode;
 
@@ -414,7 +329,7 @@ void WCachedParticles::Refresh()
     return;
 }
 
-bool WCachedParticles::Attempt(string filename, int submode, int & error)
+bool WCachedParticles::Attempt(const string& filename, int submode, int & error)
 {
 
     JFileSystem* fileSys = JFileSystem::GetInstance();
@@ -455,26 +370,3 @@ WCachedParticles::~WCachedParticles()
 {
     SAFE_DELETE(particles);
 }
-
-//WTrackedQuad
-unsigned long WTrackedQuad::size()
-{
-    return sizeof(JQuad);
-}
-
-bool WTrackedQuad::isGood()
-{
-    return (quad != NULL);
-}
-
-WTrackedQuad::WTrackedQuad(string _resname)
-{
-    quad = NULL;
-    resname = _resname;
-}
-
-WTrackedQuad::~WTrackedQuad()
-{
-    if (quad)
-    SAFE_DELETE(quad);
-}
