@@ -32,12 +32,13 @@ size_t WCFilterFactory::findNext(string src, size_t start, char open, char close
 }
 WCardFilter * WCFilterFactory::Construct(string src)
 {
-    size_t srcLength = src.size();
     size_t x = 0;
     string whitespaces(" \t\f\v\n\r");
     x = src.find_first_not_of(whitespaces);
-    if (x != string::npos) src = src.substr(x);
+    if (x != string::npos) 
+        src = src.substr(x);
 
+    size_t srcLength = src.size();
     if (!srcLength) return NEW WCFilterNULL(); //Empty string.
 
     for (size_t i = 0; i < srcLength; i++)
@@ -211,7 +212,7 @@ string WCFilterSet::getCode()
 bool WCFilterColor::isMatch(MTGCard * c)
 {
     if (!c || !c->data) return false;
-    return (c->data->hasColor(color) > 0);
+    return (c->data->hasColor(color));
 }
 string WCFilterColor::getCode()
 {
@@ -242,9 +243,9 @@ bool WCFilterOnlyColor::isMatch(MTGCard * c)
     for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
     {
         if (i == color) continue;
-        if (c->data->hasColor(i) > 0) return false;
+        if (c->data->hasColor(i)) return false;
     }
-    return (c->data->hasColor(color) > 0);
+    return (c->data->hasColor(color));
 }
 string WCFilterOnlyColor::getCode()
 {
@@ -259,6 +260,15 @@ bool WCFilterProducesColor::isMatch(MTGCard * c)
 {
     bool bMatch = false;
     if (!c || !c->data) return false;
+
+    //http://code.google.com/p/wagic/issues/detail?id=650
+    //Basic lands are not producing their mana through regular abilities anymore,
+    //but through a rule that is outside of the primitives. This block is a hack to address this
+    const string lands[] = { "dummy(colorless)", "forest", "island", "mountain", "swamp", "plains" };
+    if ((color < (int)(sizeof(lands)/sizeof(lands[0]))) && c->data->hasType(lands[color].c_str()))
+        return true;
+
+    //Retrieve non basic Mana abilities
     string s = c->data->magicText;
     size_t t = s.find("add");
     while (t != string::npos)
@@ -404,14 +414,10 @@ WCFilterRarity::WCFilterRarity(string arg)
 bool WCFilterAbility::isMatch(MTGCard * c)
 {
     if (ability < 0) return false;
-    map<int, int>::iterator it = c->data->basicAbilities.find(ability);
-
-    if (it != c->data->basicAbilities.end())
-    {
-        if (it->second > 0) return true;
-    }
-    return false;
+    
+    return c->data->basicAbilities.test(ability);
 }
+
 WCFilterAbility::WCFilterAbility(string arg)
 {
     std::transform(arg.begin(), arg.end(), arg.begin(), ::tolower);
