@@ -713,31 +713,66 @@ void WGuiSplit::setModal(bool val)
 
 bool WGuiSplit::CheckUserInput(JButton key)
 {
+    bool result = false;
+
     if (hasFocus())
     {
+        int i,j;
+
+        if (key == JGE_BTN_NONE && JGE::GetInstance()->GetLeftClickCoordinates(i, j))
+        {   // a dude clicked somwhere, we're gonna select the closest object from where he clicked
+            unsigned int distanceLeft, distanceRight;
+
+            distanceLeft = static_cast<unsigned int>((left->getY() - j) * (left->getY() - j) + (left->getX() - i) * (left->getX() - i));
+            distanceRight = static_cast<unsigned int>((right->getY() - j) * (right->getY() - j) + (right->getX() - i) * (right->getX() - i));
+
+            if (distanceLeft < distanceRight && bRight)
+            {
+              key = JGE_BTN_LEFT;
+            }
+            else if(!bRight && distanceLeft > distanceRight)
+            {
+              key = JGE_BTN_RIGHT;
+            }
+        }
+
         if (!bRight)
         {
-            if (left->CheckUserInput(key)) return true;
+            if (left->CheckUserInput(key))  {
+                result = true;
+                goto done;
+            }
+
             if (key == JGE_BTN_RIGHT && !isModal() && right->Selectable() && left->Leaving(JGE_BTN_RIGHT))
             {
                 bRight = !bRight;
                 right->Entering(JGE_BTN_RIGHT);
-                return true;
+                result = true;
+                goto done;
             }
         }
         else
         {
-            if (right->CheckUserInput(key)) return true;
+            if (right->CheckUserInput(key)) {
+                result = true;
+                goto done;
+            }
             if (key == JGE_BTN_LEFT && !isModal() && left->Selectable() && right->Leaving(JGE_BTN_LEFT))
             {
                 bRight = !bRight;
                 left->Entering(JGE_BTN_LEFT);
-                return true;
+                result = true;
+                goto done;
             }
         }
-    }
 
-    return false;
+    }
+done:
+    if(result == true)
+    {
+        JGE::GetInstance()->LeftClickedProcessed();
+    }
+    return result;
 }
 
 void WGuiSplit::Update(float dt)
@@ -897,21 +932,22 @@ bool WGuiMenu::CheckUserInput(JButton key)
         int n = currentItem;
         unsigned int distance2;
         unsigned int minDistance2 = -1;
-        for(size_t k=0; k < items.size(); k++)
+        for(size_t k = 0; k < items.size(); k++)
         {
           WGuiItem* pItem = (WGuiItem*)items[k];
-          distance2 = (pItem-> getY() - j) * (pItem-> getY() - j) + (pItem-> getX() - i) * (pItem-> getX() - i);
-          if (distance2 < minDistance2)
+          distance2 = static_cast<unsigned int>((pItem->getY() - j) * (pItem->getY() - j) + (pItem->getX() - i) * (pItem->getX() - i));
+          if (distance2 < minDistance2 && pItem->Selectable())
           {
               minDistance2 = distance2;
               n = k;
           }
         }
 
-        if (n != currentItem)
+        if (n != currentItem && items[n]->Selectable())
         {
             setSelected(n);
             mEngine->LeftClickedProcessed();
+            if (sync) syncMove();
             return true;
         }
     }
@@ -935,7 +971,7 @@ bool WGuiMenu::CheckUserInput(JButton key)
         {
             held = buttonNext;
             duration = 0;
-            if (nextItem()) return true;
+             if (nextItem()) return true;
         }
         else if (held == buttonNext && duration > 1)
         {
@@ -946,6 +982,8 @@ bool WGuiMenu::CheckUserInput(JButton key)
 
     if (currentItem >= 0 && currentItem < nbitems) result = items[currentItem]->CheckUserInput(key);
 
+    mEngine->LeftClickedProcessed();
+
     return result;
 }
 void WGuiMenu::syncMove()
@@ -1117,7 +1155,6 @@ bool WGuiTabMenu::CheckUserInput(JButton key)
 {
     bool result = false;
     bool kidModal = false;
-    bool handledInput = false;
     int nbitems = (int) items.size();
     JGE * mEngine = JGE::GetInstance();
     int i, j;
@@ -1358,14 +1395,14 @@ void WGuiCardImage::Render()
         JQuadPtr q;
         if (bThumb)
         {
-            q = WResourceManager::Instance()->GetQuad("back_thumb");
+            q = WResourceManager::Instance()->GetQuad(kGenericCardThumbnailID);
 #if defined WIN32 || defined LINUX
             if(!q)
-            q = WResourceManager::Instance()->GetQuad("back");
+            q = WResourceManager::Instance()->GetQuad(kGenericCardID);
 #endif
         }
         else
-            q = WResourceManager::Instance()->GetQuad("back");
+            q = WResourceManager::Instance()->GetQuad(kGenericCardID);
         float scale = p.actZ * 257.f / q->mHeight;
         q->SetColor(ARGB(255,255,255,255));
         renderer->RenderQuad(q.get(), p.x, p.y, 0, scale, scale);
@@ -1428,14 +1465,14 @@ void WGuiCardDistort::Render()
         //Default to back.
         if (bThumb)
         {
-            q = WResourceManager::Instance()->GetQuad("back_thumb");
+            q = WResourceManager::Instance()->GetQuad(kGenericCardThumbnailID);
 #if defined WIN32 || defined LINUX
             if(!q)
-            q = WResourceManager::Instance()->GetQuad("back");
+            q = WResourceManager::Instance()->GetQuad(kGenericCardID);
 #endif
         }
         else
-            q = WResourceManager::Instance()->GetQuad("back");
+            q = WResourceManager::Instance()->GetQuad(kGenericCardID);
     }
     else
     {
@@ -1955,12 +1992,17 @@ void WGuiFilterItem::updateValue()
         {
             mParent->addArg("Artifact", "t:Artifact;");
             mParent->addArg("Artifact Creature", "t:Artifact;&t:Creature;");
+            mParent->addArg("Aura", "t:Aura;");
+            mParent->addArg("Basic", "t:Basic;");
             mParent->addArg("Creature", "t:Creature;");
             mParent->addArg("Enchantment", "t:Enchantment;");
+            mParent->addArg("Equipment", "t:Equipment;");
             mParent->addArg("Instant", "t:Instant;");
             mParent->addArg("Land", "t:Land;");
             mParent->addArg("Legendary", "t:Legendary;");
             mParent->addArg("Sorcery", "t:Sorcery;");
+            mParent->addArg("Tribal", "t:Tribal;");
+
         }
         else if (filterType == FILTER_SUBTYPE)
         {
