@@ -1,18 +1,15 @@
 #include "PrecompiledHeader.h"
 
-#include <JFileSystem.h>
-
-#include "utils.h"
-#include "GameOptions.h"
-#include "WResourceManager.h"
+#include "WResourceManagerImpl.h"
 #include "StyleManager.h"
 
+#include "CacheEngine.h"
+
 #if defined (WIN32)
 #include <sys/types.h>
 #include <sys/stat.h>
 #endif
 #include "WFont.h"
-#include <JLogger.h>
 
 //#define FORCE_LOW_CACHE_MEMORY
 const unsigned int kConstrainedCacheLimit = 8 * 1024 * 1024;
@@ -25,16 +22,38 @@ namespace
     const std::string kExtension_png(".png");
     const std::string kExtension_gbk(".gbk");
     const std::string kExtension_font(".font");
+
+    // mutex meant for the cache map
+    boost::mutex sCacheMutex;
+    // mutex meant to protect against unthread-safe calls into JFileSystem, etc.
+    boost::mutex sLoadFunctionMutex;
 }
 
 WResourceManager* WResourceManager::sInstance = NULL;
 
-int WResourceManager::RetrieveError()
+WResourceManager* WResourceManager::Instance()
+{
+	if (sInstance == NULL)
+	{
+		sInstance = NEW ResourceManagerImpl;
+	}
+
+	return sInstance;
+}
+
+void WResourceManager::Terminate()
+{
+	if (sInstance)
+		SAFE_DELETE(sInstance);
+}
+
+
+int ResourceManagerImpl::RetrieveError()
 {
     return lastError;
 }
 
-bool WResourceManager::RemoveOldest()
+bool ResourceManagerImpl::RemoveOldest()
 {
     if (sampleWCache.RemoveOldest()) return true;
     if (textureWCache.RemoveOldest()) return true;
@@ -43,15 +62,14 @@ bool WResourceManager::RemoveOldest()
     return false;
 }
 
-//WResourceManager
-void WResourceManager::DebugRender()
+//ResourceManagerImpl
+void ResourceManagerImpl::DebugRender()
 {
     JRenderer* renderer = JRenderer::GetInstance();
-    WFont * font = WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT);
-    font->SetColor(ARGB(255,255,255,255));
-
+    WFont * font = ResourceManagerImpl::Instance()->GetWFont(Fonts::MAIN_FONT);
     if (!font || !renderer) return;
 
+    font->SetColor(ARGB(255,255,255,255));
     font->SetScale(DEFAULT_MAIN_FONT_SCALE);
     renderer->FillRect(0, 0, SCREEN_WIDTH, 40, ARGB(128,155,0,0));
 
@@ -70,12 +88,13 @@ void WResourceManager::DebugRender()
         textureWCache.cacheSize, textureWCache.maxCacheSize, man);
     font->DrawString(buf, 10, 5);
 
-#if defined (WIN32) || defined (LINUX) || defined (IOS)
-#else
-    int maxLinear = ramAvailableLineareMax();
-    int ram = ramAvailable();
-    sprintf(buf, "Ram : linear max: %i - total : %i\n", maxLinear, ram);
-    font->DrawString(buf, 10, 20);
+#ifdef PSP
+    //deliberately off - these functions aren't thread safe!
+    //int maxLinear = ramAvailableLineareMax();
+    //int ram = ramAvailable();
+   
+    //sprintf(buf, "Ram : linear max: %i - total : %i sceSize : %i\n", maxLinear, ram, sceSize);
+    //font->DrawString(buf, 10, 20);
 #endif
 
     sprintf(buf, "Time: %u. Total Size: %lu (%lu cached, %lu managed). ", lastTime, Size(), SizeCached(), SizeManaged());
@@ -88,7 +107,7 @@ void WResourceManager::DebugRender()
 #endif
 }
 
-unsigned long WResourceManager::Size()
+unsigned long ResourceManagerImpl::Size()
 {
     unsigned long res = 0;
     res += textureWCache.totalSize;
@@ -97,7 +116,7 @@ unsigned long WResourceManager::Size()
     return res;
 }
 
-unsigned long WResourceManager::SizeCached()
+unsigned long ResourceManagerImpl::SizeCached()
 {
     unsigned long res = 0;
     res += textureWCache.cacheSize;
@@ -106,7 +125,7 @@ unsigned long WResourceManager::SizeCached()
     return res;
 }
 
-unsigned long WResourceManager::SizeManaged()
+unsigned long ResourceManagerImpl::SizeManaged()
 {
     unsigned long res = 0;
     if (textureWCache.totalSize > textureWCache.cacheSize) res += textureWCache.totalSize - textureWCache.cacheSize;
@@ -118,7 +137,7 @@ unsigned long WResourceManager::SizeManaged()
     return res;
 }
 
-unsigned int WResourceManager::Count()
+unsigned int ResourceManagerImpl::Count()
 {
     unsigned int count = 0;
     count += textureWCache.cacheItems;
@@ -130,7 +149,7 @@ unsigned int WResourceManager::Count()
     return count;
 }
 
-unsigned int WResourceManager::CountCached()
+unsigned int ResourceManagerImpl::CountCached()
 {
     unsigned int count = 0;
     count += textureWCache.cacheItems;
@@ -138,7 +157,7 @@ unsigned int WResourceManager::CountCached()
     count += psiWCache.cacheItems;
     return count;
 }
-unsigned int WResourceManager::CountManaged()
+unsigned int ResourceManagerImpl::CountManaged()
 {
     unsigned int count = 0;
     count += textureWCache.managed.size();
@@ -147,14 +166,14 @@ unsigned int WResourceManager::CountManaged()
     return count;
 }
 
-unsigned int WResourceManager::nowTime()
+unsigned int ResourceManagerImpl::nowTime()
 {
     if (lastTime > MAX_CACHE_TIME) FlattenTimes();
 
     return ++lastTime;
 }
 
-void WResourceManager::FlattenTimes()
+void ResourceManagerImpl::FlattenTimes()
 {
     unsigned int t;
     lastTime = sampleWCache.Flatten();
@@ -166,9 +185,9 @@ void WResourceManager::FlattenTimes()
     if (t > lastTime) lastTime = t;
 }
 
-WResourceManager::WResourceManager()
+ResourceManagerImpl::ResourceManagerImpl()
 {
-    DebugTrace("Init WResourceManager : " << this);
+    DebugTrace("Init ResourceManagerImpl : " << this);
 #ifdef DEBUG_CACHE
     menuCached = 0;
 #endif
@@ -180,26 +199,41 @@ WResourceManager::WResourceManager()
     lastError = CACHE_ERROR_NONE;
 
     bThemedCards = false;
+
+    LOG("Calling CacheEngine::Create");
+
+#ifdef PSP
+    CacheEngine::Create<UnthreadedCardRetriever>(textureWCache);
+#else
+    CacheEngine::Create<ThreadedCardRetriever>(textureWCache);
+#endif
 }
 
-WResourceManager::~WResourceManager()
+ResourceManagerImpl::~ResourceManagerImpl()
 {
-    LOG("==Destroying WResourceManager==");
+    LOG("==Destroying ResourceManagerImpl==");
     RemoveWFonts();
 
-    LOG("==Successfully Destroyed WResourceManager==");
+    CacheEngine::Terminate();
+    LOG("==Successfully Destroyed ResourceManagerImpl==");
 }
 
-JQuadPtr WResourceManager::RetrieveCard(MTGCard * card, int style, int submode)
+bool ResourceManagerImpl::IsThreaded()
+{
+    return CacheEngine::IsThreaded();
+}
+
+JQuadPtr ResourceManagerImpl::RetrieveCard(MTGCard * card, int style, int submode)
 {
     //Cards are never, ever resource managed, so just check cache.
     if (!card || options[Options::DISABLECARDS].number) return JQuadPtr();
 
     submode = submode | TEXTURE_SUB_CARD;
 
-    string filename = setlist[card->setId];
-    filename += "/";
-    string filename1 = filename + card->getImageName();
+    static std::ostringstream filename;
+    filename.str("");
+    filename << setlist[card->setId] << "/" << card->getImageName();
+
     int id = card->getMTGId();
 
     //Aliases.
@@ -209,40 +243,8 @@ JQuadPtr WResourceManager::RetrieveCard(MTGCard * card, int style, int submode)
         style = RETRIEVE_NORMAL;
     }
 
-    //Hack to allow either ID or card name as a filename for a given card.
-    // When missing the first attempt (for example [id].jpg), the cache assigns a "404" to the card's image cache,
-    // Preventing us to try a second time with [name].jpg.
-    //To bypass this, we first check if the card was ever marked as "null". If not, it means it's the first time we're looking for it
-    // In that case, we "unmiss" it after trying the [id].jpg, in order to give a chance to the [name.jpg]
-    bool canUnmiss = false;
-    {
-        JQuadPtr tempQuad = RetrieveQuad(filename1, 0, 0, 0, 0, "", RETRIEVE_EXISTING, submode | TEXTURE_SUB_5551, id);
-        lastError = textureWCache.mError;
-        if (!tempQuad && lastError != CACHE_ERROR_404)
-        {
-            canUnmiss = true;
-        }
-    }
-    JQuadPtr jq = RetrieveQuad(filename1, 0, 0, 0, 0, "", style, submode | TEXTURE_SUB_5551, id);
-    if (!jq)
-    {
-        if (canUnmiss)
-        {
-            int mId = id;
-            //To differentiate between cached thumbnails and the real thing.
-            if (submode & TEXTURE_SUB_THUMB)
-            {
-                if (mId < 0)
-                    mId -= THUMBNAILS_OFFSET;
-                else
-                    mId += THUMBNAILS_OFFSET;
-            }
-            textureWCache.RemoveMiss(mId);
-        }
-        filename1 = filename + card->data->getName() + ".jpg";
-        jq = RetrieveQuad(filename1, 0, 0, 0, 0, "", style, submode | TEXTURE_SUB_5551, id);
+    JQuadPtr jq = RetrieveQuad(filename.str(), 0, 0, 0, 0, "", style, submode | TEXTURE_SUB_5551, id);
 
-    }
     lastError = textureWCache.mError;
     if (jq)
     {
@@ -253,7 +255,7 @@ JQuadPtr WResourceManager::RetrieveCard(MTGCard * card, int style, int submode)
     return JQuadPtr();
 }
 
-int WResourceManager::AddQuadToManaged(const WManagedQuad& inQuad)
+int ResourceManagerImpl::AddQuadToManaged(const WManagedQuad& inQuad)
 {
     int id = mIDLookupMap.size();
     mIDLookupMap.insert(make_pair(id, inQuad.resname));
@@ -262,7 +264,7 @@ int WResourceManager::AddQuadToManaged(const WManagedQuad& inQuad)
     return id;
 }
 
-int WResourceManager::CreateQuad(const string &quadName, const string &textureName, float x, float y, float width, float height)
+int ResourceManagerImpl::CreateQuad(const string &quadName, const string &textureName, float x, float y, float width, float height)
 {
     if (!quadName.size() || !textureName.size()) return INVALID_ID;
 
@@ -297,7 +299,7 @@ int WResourceManager::CreateQuad(const string &quadName, const string &textureNa
     return id;
 }
 
-JQuadPtr WResourceManager::GetQuad(const string &quadName)
+JQuadPtr ResourceManagerImpl::GetQuad(const string &quadName)
 {
     JQuadPtr result;
     ManagedQuadMap::const_iterator found = mManagedQuads.find(quadName);
@@ -309,7 +311,7 @@ JQuadPtr WResourceManager::GetQuad(const string &quadName)
     return result;
 }
 
-JQuadPtr WResourceManager::GetQuad(int id)
+JQuadPtr ResourceManagerImpl::GetQuad(int id)
 {
     JQuadPtr result;
     if (id < 0 || id >= (int) mManagedQuads.size()) return result;
@@ -327,12 +329,12 @@ JQuadPtr WResourceManager::GetQuad(int id)
     return result;
 }
 
-JQuadPtr WResourceManager::RetrieveTempQuad(const string& filename, int submode)
+JQuadPtr ResourceManagerImpl::RetrieveTempQuad(const string& filename, int submode)
 {
     return RetrieveQuad(filename, 0, 0, 0, 0, "temporary", RETRIEVE_NORMAL, submode);
 }
 
-JQuadPtr WResourceManager::RetrieveQuad(const string& filename, float offX, float offY, float width, float height, string resname,
+JQuadPtr ResourceManagerImpl::RetrieveQuad(const string& filename, float offX, float offY, float width, float height, string resname,
     int style, int submode, int id)
 {
     //Lookup managed resources, but only with a real resname.
@@ -353,11 +355,7 @@ JQuadPtr WResourceManager::RetrieveQuad(const string& filename, float offX, floa
     if (!resname.size()) resname = filename;
 
     //No quad, but we have a managed texture for this!
-    WCachedTexture * jtex = NULL;
-    if (style == RETRIEVE_MANAGE || style == RETRIEVE_EXISTING)
-        jtex = textureWCache.Retrieve(id, filename, style, submode);
-    else
-        jtex = textureWCache.Retrieve(id, filename, RETRIEVE_NORMAL, submode);
+    WCachedTexture* jtex = textureWCache.Retrieve(id, filename, style, submode);
 
     lastError = textureWCache.mError;
 
@@ -393,7 +391,7 @@ JQuadPtr WResourceManager::RetrieveQuad(const string& filename, float offX, floa
     return JQuadPtr();
 }
 
-void WResourceManager::Release(JTexture * tex)
+void ResourceManagerImpl::Release(JTexture * tex)
 {
     if (!tex) return;
 
@@ -418,28 +416,21 @@ void WResourceManager::Release(JTexture * tex)
     return; //Released!
 }
 
-void WResourceManager::Unmiss(string filename)
-{
-    map<int, WCachedTexture*>::iterator it;
-    int id = textureWCache.makeID(0, filename, CACHE_NORMAL);
-    textureWCache.RemoveMiss(id);
-}
-
-void WResourceManager::ClearUnlocked()
+void ResourceManagerImpl::ClearUnlocked()
 {
     textureWCache.ClearUnlocked();
     sampleWCache.ClearUnlocked();
     psiWCache.ClearUnlocked();
 }
 
-void WResourceManager::Release(JSample * sample)
+void ResourceManagerImpl::Release(JSample * sample)
 {
     if (!sample) return;
 
     sampleWCache.Release(sample);
 }
 
-JTexture * WResourceManager::RetrieveTexture(const string& filename, int style, int submode)
+JTexture * ResourceManagerImpl::RetrieveTexture(const string& filename, int style, int submode)
 {
     //Aliases.
     if (style == RETRIEVE_THUMB)
@@ -489,7 +480,7 @@ JTexture * WResourceManager::RetrieveTexture(const string& filename, int style,
     return NULL;
 }
 
-int WResourceManager::CreateTexture(const string &textureName)
+int ResourceManagerImpl::CreateTexture(const string &textureName)
 {
     JTexture * jtex = RetrieveTexture(textureName, RETRIEVE_MANAGE);
 
@@ -498,13 +489,13 @@ int WResourceManager::CreateTexture(const string &textureName)
     return INVALID_ID;
 }
 
-JTexture* WResourceManager::GetTexture(const string &textureName)
+JTexture* ResourceManagerImpl::GetTexture(const string &textureName)
 {
     JTexture * jtex = RetrieveTexture(textureName, RETRIEVE_RESOURCE);
     return jtex;
 }
 
-JTexture* WResourceManager::GetTexture(int id)
+JTexture* ResourceManagerImpl::GetTexture(int id)
 {
     map<int, WCachedTexture*>::iterator it;
     JTexture *jtex = NULL;
@@ -523,7 +514,7 @@ JTexture* WResourceManager::GetTexture(int id)
     return jtex;
 }
 
-hgeParticleSystemInfo * WResourceManager::RetrievePSI(const string& filename, JQuad * texture, int style, int submode)
+hgeParticleSystemInfo * ResourceManagerImpl::RetrievePSI(const string& filename, JQuad * texture, int style, int submode)
 {
     if (!texture) return NULL;
 
@@ -540,7 +531,7 @@ hgeParticleSystemInfo * WResourceManager::RetrievePSI(const string& filename, JQ
     return NULL;
 }
 
-JSample * WResourceManager::RetrieveSample(const string& filename, int style, int submode)
+JSample * ResourceManagerImpl::RetrieveSample(const string& filename, int style, int submode)
 {
     WCachedSample * tc = NULL;
     tc = sampleWCache.Retrieve(0, filename, style, submode);
@@ -556,7 +547,7 @@ JSample * WResourceManager::RetrieveSample(const string& filename, int style, in
     return NULL;
 }
 
-string WResourceManager::graphicsFile(const string& filename)
+string ResourceManagerImpl::graphicsFile(const string& filename)
 {
     char buf[512];
 
@@ -607,7 +598,7 @@ string WResourceManager::graphicsFile(const string& filename)
     return graphdir;
 }
 
-string WResourceManager::avatarFile(const string& filename)
+string ResourceManagerImpl::avatarFile(const string& filename)
 {
     char buf[512];
 
@@ -661,7 +652,7 @@ string WResourceManager::avatarFile(const string& filename)
     return "";
 }
 
-string WResourceManager::cardFile(const string& filename)
+string ResourceManagerImpl::cardFile(const string& filename)
 {
     char buf[512];
     string::size_type i = 0;
@@ -734,7 +725,7 @@ string WResourceManager::cardFile(const string& filename)
     return "";
 }
 
-string WResourceManager::musicFile(const string& filename)
+string ResourceManagerImpl::musicFile(const string& filename)
 {
     char buf[512];
 
@@ -771,7 +762,7 @@ string WResourceManager::musicFile(const string& filename)
     return "";
 }
 
-string WResourceManager::sfxFile(const string& filename)
+string ResourceManagerImpl::sfxFile(const string& filename)
 {
     char buf[512];
 
@@ -803,7 +794,7 @@ string WResourceManager::sfxFile(const string& filename)
     return "";
 }
 
-int WResourceManager::dirOK(const string& dirname)
+int ResourceManagerImpl::dirOK(const string& dirname)
 {
     char fname[512];
 
@@ -821,7 +812,7 @@ int WResourceManager::dirOK(const string& dirname)
     return 0;
 }
 
-int WResourceManager::fileOK(const string& filename, bool relative)
+int ResourceManagerImpl::fileOK(const string& filename, bool relative)
 {
     wagic::ifstream * fp = NULL;
     if (relative)
@@ -842,7 +833,7 @@ int WResourceManager::fileOK(const string& filename, bool relative)
     return result;
 }
 
-void WResourceManager::InitFonts(const std::string& inLang)
+void ResourceManagerImpl::InitFonts(const std::string& inLang)
 {
     unsigned int idOffset = 0;
 
@@ -876,7 +867,7 @@ void WResourceManager::InitFonts(const std::string& inLang)
     LoadWFont("smallface", 7, Fonts::SMALLFACE_FONT + idOffset);
 }
 
-int WResourceManager::ReloadWFonts()
+int ResourceManagerImpl::ReloadWFonts()
 {
     RemoveWFonts();
 
@@ -888,7 +879,7 @@ int WResourceManager::ReloadWFonts()
     return 1;
 }
 
-WFont* WResourceManager::LoadWFont(const string& inFontname, int inFontHeight, int inFontID)
+WFont* ResourceManagerImpl::LoadWFont(const string& inFontname, int inFontHeight, int inFontID)
 {
     WFont* font = GetWFont(inFontID);
     if (font)
@@ -910,7 +901,7 @@ WFont* WResourceManager::LoadWFont(const string& inFontname, int inFontHeight, i
     return font;
 }
 
-WFont* WResourceManager::GetWFont(int id)
+WFont* ResourceManagerImpl::GetWFont(int id)
 {
     WFont* font = NULL;
     FontMap::iterator iter = mWFontMap.find(id);
@@ -921,7 +912,7 @@ WFont* WResourceManager::GetWFont(int id)
     return font;
 }
 
-void WResourceManager::RemoveWFonts()
+void ResourceManagerImpl::RemoveWFonts()
 {
     for (FontMap::iterator font = mWFontMap.begin(); font != mWFontMap.end(); ++font)
     {
@@ -930,7 +921,7 @@ void WResourceManager::RemoveWFonts()
     mWFontMap.clear();
 }
 
-void WResourceManager::ResetCacheLimits()
+void ResourceManagerImpl::ResetCacheLimits()
 {
 #if defined WIN32 || defined LINUX || defined (IOS)
 #ifdef FORCE_LOW_CACHE_MEMORY
@@ -943,21 +934,21 @@ void WResourceManager::ResetCacheLimits()
     unsigned int myNewSize = ram - OPERATIONAL_SIZE + textureWCache.totalSize;
     if (myNewSize < TEXTURES_CACHE_MINSIZE)
     {
-        fprintf(stderr, "Error, Not enough RAM for Cache: %i - total Ram: %i\n", myNewSize, ram);
+        DebugTrace( "Error, Not enough RAM for Cache: " << myNewSize << " - total Ram: " << ram);
     }
-    textureWCache.Resize(myNewSize, MAX_CACHE_OBJECTS);
+    textureWCache.Resize(MIN(myNewSize, HUGE_CACHE_LIMIT), MAX_CACHE_OBJECTS);
 #endif
     return;
 }
 
-JMusic * WResourceManager::ssLoadMusic(const char *fileName)
+JMusic * ResourceManagerImpl::ssLoadMusic(const char *fileName)
 {
     string file = musicFile(fileName);
     if (!file.size()) return NULL;
     return JSoundSystem::GetInstance()->LoadMusic(file.c_str());
 }
 
-void WResourceManager::Refresh()
+void ResourceManagerImpl::Refresh()
 {
     //Really easy cache relinking.
     ReloadWFonts();
@@ -990,6 +981,11 @@ bool WCache<cacheItem, cacheActual>::RemoveOldest()
 
     if (oldest != cache.end() && oldest->second && !oldest->second->isLocked())
     {
+#ifdef DEBUG_CACHE
+        std::ostringstream stream;
+        stream << "erasing from cache: "  << oldest->second->mFilename << " " << oldest->first;
+        LOG(stream.str().c_str());
+#endif
         Delete(oldest->second);
         cache.erase(oldest);
         return true;
@@ -1023,6 +1019,13 @@ template<class cacheItem, class cacheActual>
 void WCache<cacheItem, cacheActual>::Resize(unsigned long size, int items)
 {
     maxCacheSize = size;
+    DebugTrace(typeid(cacheActual).name() << " cache resized to " << size << " bytes");
+
+#ifdef DEBUG_CACHE
+    std::ostringstream stream;
+    stream << "Max cache limit resized to " << size << ", items limit reset to " << items;
+    LOG(stream.str().c_str());
+#endif
 
     if (items > MAX_CACHE_OBJECTS || items < 1)
         maxCached = MAX_CACHE_OBJECTS;
@@ -1033,12 +1036,6 @@ void WCache<cacheItem, cacheActual>::Resize(unsigned long size, int items)
 template<class cacheItem, class cacheActual>
 cacheItem* WCache<cacheItem, cacheActual>::AttemptNew(const string& filename, int submode)
 {
-    if (submode & CACHE_EXISTING)
-    { //Should never get this far.
-        mError = CACHE_ERROR_NOT_CACHED;
-        return NULL;
-    }
-
     cacheItem* item = NEW cacheItem;
     if (!item)
     {
@@ -1053,25 +1050,22 @@ cacheItem* WCache<cacheItem, cacheActual>::AttemptNew(const string& filename, in
         //No such file. Fail
         if (mError == CACHE_ERROR_404)
         {
+            DebugTrace("AttemptNew failed to load. Deleting cache item " << ToHex(item));
             SAFE_DELETE(item);
             return NULL;
         }
-        //Probably not enough memory: cleanup and try again
-        Cleanup();
-        mError = CACHE_ERROR_NONE;
-        if (!item->Attempt(filename, submode, mError) || !item->isGood())
+		else
         {
+            DebugTrace("AttemptNew failed to load (not a 404 error). Deleting cache item " << ToHex(item));
             SAFE_DELETE(item);
             mError = CACHE_ERROR_BAD;
-            return NULL;
+        	return NULL;
         }
     }
 
     //Success! Enforce cache limits, then return.
     mError = CACHE_ERROR_NONE;
-    item->lock();
-    Cleanup();
-    item->unlock();
+
     return item;
 }
 
@@ -1079,13 +1073,8 @@ template<class cacheItem, class cacheActual>
 cacheItem* WCache<cacheItem, cacheActual>::Retrieve(int id, const string& filename, int style, int submode)
 {
     //Check cache.
-    cacheItem * tc = NULL;
     mError = CACHE_ERROR_NONE; //Reset error status.
-
-    if (style == RETRIEVE_EXISTING || style == RETRIEVE_RESOURCE)
-        tc = Get(id, filename, style, submode | CACHE_EXISTING);
-    else
-        tc = Get(id, filename, style, submode);
+    cacheItem* tc = Get(id, filename, style, submode);
 
     //Retrieve resource only works on permanent items.
     if (style == RETRIEVE_RESOURCE && tc && !tc->isPermanent())
@@ -1153,14 +1142,9 @@ int WCache<cacheItem, cacheActual>::makeID(int id, const string& filename, int s
         }
     }
 
-    //To differentiate between cached thumbnails and the real thing.
     if (submode & TEXTURE_SUB_THUMB)
-    {
-        if (mId < 0)
-            mId -= THUMBNAILS_OFFSET;
-        else
-            mId += THUMBNAILS_OFFSET;
-    }
+        mId += THUMBNAILS_OFFSET;
+
     return mId;
 }
 
@@ -1179,38 +1163,73 @@ cacheItem* WCache<cacheItem, cacheActual>::Get(int id, const string& filename, i
         return it->second; //A hit.
     }
 
-    //Failed to find managed resource and won't create one. Record a miss.
-    if (style == RETRIEVE_RESOURCE)
-    {
-        managed[lookup] = NULL;
-        return NULL;
-    }
-
     //Not managed, so look in cache.
     if (style != RETRIEVE_MANAGE)
     {
+        boost::mutex::scoped_lock lock(sCacheMutex);
+        //DebugTrace("Cache lock acquired, looking up index " << lookup);
         it = cache.find(lookup);
         //Well, we've found something...
         if (it != cache.end())
         {
-            if (!it->second) mError = CACHE_ERROR_404;
+            if (!it->second)
+            {
+                mError = CACHE_ERROR_404;
+                //DebugTrace("cache hit, no item??");
+                //assert(false);
+            }
             return it->second; //A hit, or maybe a miss.
         }
     }
 
-    //Didn't exist in cache.
-    if (submode & CACHE_EXISTING)
+    // not hit in the cache, respect the RETRIEVE_EXISTING flag if present
+    if (style == RETRIEVE_EXISTING)
     {
-        mError = CACHE_ERROR_NOT_CACHED;
         return NULL;
     }
 
-    //Space in cache, make new texture
-    cacheItem * item = AttemptNew(filename, submode);
+    // no hit in cache, clear space before attempting to load a new one
+    // note: Cleanup() should ONLY be ever called on the main (UI) thread!
+    Cleanup();
 
+    // check if we're doing a card lookup
+    if (submode & TEXTURE_SUB_CARD)
+    {
+        // processing a cache miss, return a generic card & queue up an async read
+
+        // side note:  using a string reference here to a global predefined string, as basic_string is not thread safe for allocations!
+        const std::string& cardPath = (submode & TEXTURE_SUB_THUMB) ? kGenericThumbCard : kGenericCard;
+        int genericCardId = makeID(0, cardPath, CACHE_NORMAL);
+        it = managed.find(genericCardId);
+        assert(it != managed.end());
+
+        CacheEngine::Instance()->QueueRequest(filename, submode, lookup);
+        return it->second;
+    }
+
+    //Space in cache, make new texture
+    return LoadIntoCache(lookup, filename, submode, style);
+}
+
+template<class cacheItem, class cacheActual>
+cacheItem* WCache<cacheItem, cacheActual>::LoadIntoCache(int id, const string& filename, int submode, int style)
+{
+    // note: my original implementation only had one lock (the cache mutex lock) - I eventually
+    // added this second one, as locking at the Get() call means that the main thread is blocked on doing a simple cache
+    // check.  If you're hitting the system hard (like, paging up in the deck editor which forces 7 cards to load simultaneously),
+    // we'd block the UI thread for a long period at this point.  So I moved the cache mutex to lock specifically only attempts to touch
+    // the shared cache container, and this separate lock was added to insulate us against thread safety issues in JGE. In particular,
+    // JFileSystem is particularly unsafe, as it assumes that we have only one zip loaded at a time... rather than add mutexes in JGE,
+    // I've kept it local to here.
+    boost::mutex::scoped_lock functionLock(sLoadFunctionMutex);
+    cacheItem* item = AttemptNew(filename, submode);
+    //assert(item);
     if (style == RETRIEVE_MANAGE)
     {
-        if (mError == CACHE_ERROR_404 || item) managed[lookup] = item; //Record a hit or miss.
+        if (mError == CACHE_ERROR_404 || item)
+        {
+            managed[id] = item; //Record a hit or miss.
+        }
         if (item)
         {
             item->deadbolt(); //Make permanent.
@@ -1218,10 +1237,25 @@ cacheItem* WCache<cacheItem, cacheActual>::Get(int id, const string& filename, i
     }
     else
     {
-        if (mError == CACHE_ERROR_404 || item) cache[lookup] = item;
+        if (mError == CACHE_ERROR_404 || item)
+        {
+            boost::mutex::scoped_lock lock(sCacheMutex);
+            cache[id] = item;
+            DebugTrace("inserted item ptr " << ToHex(item) << " at index " << id);
+        }
     }
 
-    if (!item) return NULL; //Failure
+    if (item == NULL)
+    {
+        DebugTrace("Can't locate ");
+        if (submode & TEXTURE_SUB_THUMB)
+        {
+            DebugTrace("thumbnail ");
+        }
+        DebugTrace(filename);
+        
+        return NULL; //Failure
+    }
 
     //Succeeded in making a new item.
     unsigned long isize = item->size();
@@ -1234,6 +1268,12 @@ cacheItem* WCache<cacheItem, cacheActual>::Get(int id, const string& filename, i
         cacheSize += isize;
     }
 
+#ifdef DEBUG_CACHE
+    std::ostringstream stream;
+    stream << "Cache insert: " << filename << " " << id << ", cacheItem count: " << cacheItems << ", cacheSize is now: " << cacheSize;
+    LOG(stream.str().c_str());
+#endif
+
     return item;
 }
 
@@ -1292,24 +1332,22 @@ WCache<cacheItem, cacheActual>::~WCache()
 template<class cacheItem, class cacheActual>
 bool WCache<cacheItem, cacheActual>::Cleanup()
 {
-    while (cacheItems < cache.size() && cache.size() - cacheItems > MAX_CACHE_MISSES)
+    bool result = true;
+    // this looks redundant, but the idea is, don't grab the mutex if there's no work to do
+    if (RequiresOldItemCleanup())
     {
-        RemoveMiss();
-    }
-
-    while (cacheItems > MAX_CACHE_OBJECTS || cacheItems > maxCached || cacheSize > maxCacheSize
-#if defined WIN32 || defined LINUX || defined (IOS)
-#else
-        || ramAvailableLineareMax() < MIN_LINEAR_RAM
-#endif
-        )
-    {
-        if (!RemoveOldest())
+        boost::mutex::scoped_lock lock(sCacheMutex);
+        while (RequiresOldItemCleanup())
         {
-            return false;
+            if (!RemoveOldest())
+            {
+                result = false;
+                break;
+            }
         }
     }
-    return true;
+
+    return result;
 }
 
 bool WCacheSort::operator()(const WResource * l, const WResource * r)
@@ -1425,6 +1463,7 @@ bool WCache<cacheItem, cacheActual>::Delete(cacheItem * item)
 
     cacheItems--;
 
+    DebugTrace("Deleting cache item " << ToHex(item) << ", cache reduced by " << isize << " bytes");
     SAFE_DELETE(item);
     return true;
 }
